/**
 * This class can access to the NIST Database
 */

package gov.nist.hl7refdb.example;

// JDK
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DriverManager;
import java.sql.ResultSetMetaData;
import java.util.Vector;

public class NISTDatabase {

    private Connection conn;
    private Statement stmt;
    private ResultSet rs;

    /**
     * The constructor creates the connection
     * You have to create a DSN file
     */
    public NISTDatabase() throws SQLException {
	try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver").newInstance(); 
	    conn = DriverManager.getConnection("jdbc:odbc:nist");
	}
	catch(ClassNotFoundException cnfe) {
	    System.err.println("ClassNotFoundException " + cnfe.getMessage());
	}
	catch(InstantiationException ie) {
	    System.err.println("InstantiationException " + ie.getMessage());
	}
	catch(IllegalAccessException iae) {
	    System.err.println("IllegalAccessException " + iae.getMessage());
	}
    }

    /**
     * This method releases all resources
     */
    private void releaseResources() throws SQLException {
	if (rs != null) {
	    rs.close(); 
	    rs = null; 
	}	
	if (stmt != null) {
	    stmt.close(); 
	    stmt = null;
	}
    }
    
    /**
     * This method executes an update(UPDATE, INSERT INTO or DELETE) SQL Query
     * @param query The query
     */
    public void executeUpdate(String query) throws SQLException {
	try {	    
	    stmt = conn.createStatement();
	    stmt.executeUpdate(query);
	    releaseResources();
	}
	catch(SQLException sqle) {
	    throw new SQLException(sqle.getMessage() + "\n" + query);
	}
    }

    /**
     * This method executes a select SQL query
     * @param query The query
     * @return The resultset of the query represented as a Vector
     */
    public Vector executeQuery(String query) throws SQLException {
	Vector vInfo = new Vector();
	ResultSetMetaData rsmd;

	try {
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery(query);
	    rsmd = rs.getMetaData();

	    while(rs.next()) {
		for(int i=1;i<=rsmd.getColumnCount();i++) {
		    // get the type of the column and add the value to the vector
		    switch(rsmd.getColumnType(i)) {
		    case 4: // LONG
		    case -5: //LONGLONG
			vInfo.add(new Long(rs.getLong(i)));
			break;
		    case 1: // CHAR
		    case 12: // VARCHAR
			vInfo.add(rs.getString(i));
			break;
		    case 5: // SHORT
			vInfo.add(new Short(rs.getShort(i)));
			break;
		    case -6: // TINY
			vInfo.add(new Byte(rs.getByte(i)));
			break;
		    case 8: // DOUBLE
			vInfo.add(new Double(rs.getDouble(i)));
			break;
		    case -7: // BIT
			vInfo.add(new Boolean(rs.getBoolean(i)));
			break;
		    default:
			System.out.println("This type is not handle by this class, add a case statement to fix this problem");
			System.out.println("# Type: " + rsmd.getColumnType(i) + " - Type Name: " + rsmd.getColumnTypeName(i));
		    }
		}
	    }
	    releaseResources();
	}
	catch(SQLException sqle) {
	    throw new SQLException(sqle.getMessage() + "\n" + query);
	}
	return vInfo;
    }
}
