/**
 * This class is the code to use NIST database and generate the XML file
 */

package gov.nist.hl7refdb.example;

// Reference Database Generator
import gov.nist.hl7refdb.generator.HL7DataValue;
import gov.nist.hl7refdb.generator.HL7DataElement;
import gov.nist.hl7refdb.generator.DBtoXML;

// JDK
import java.util.Vector;
import java.io.IOException;
import java.sql.SQLException;

// DOM
import javax.xml.parsers.ParserConfigurationException;

// XSLT
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

// SAX
import org.xml.sax.SAXException;

public class NISTImpl {

    private NISTDatabase nDB;

    /**
     * Constructor - Init the database connection
     */
    public NISTImpl() throws SQLException {
	nDB = new NISTDatabase();
    }

    /**
     * Transform the database into an XML file
     * @param filename The name of the XML file
     */
    public void transform(String filename) throws SQLException, TransformerConfigurationException, TransformerException, ParserConfigurationException, SAXException, IOException {
	DBtoXML dbxml = new DBtoXML();
	HL7DataElement hl7de = null;
	HL7DataValue hl7dv = null;

	// Create the HL7DataElement
	System.out.println("Pull out the data from the database");
	String query = "SELECT name, qualifier, segment, datatype, length, table_id, reference, config_ref, status, notes, source, data_value_pointer FROM NIST";
	Vector v = nDB.executeQuery(query);
	for(int i=0;i<v.size()/12;i++) {
	    hl7de = new HL7DataElement();
	    // Name
	    hl7de.setName((String)v.get(i*12+0));
	    // Qualifier
	    hl7de.setQualifier((String)v.get(i*12+1));
	    // Segment
	    hl7de.setSegment((String)v.get(i*12+2));
	    // Datatype
	    hl7de.setDatatype((String)v.get(i*12+3));
	    // Length L
	    hl7de.setLength(((Long)v.get(i*12+4)).longValue());
	    // Table Id L
	    hl7de.setTableId(((Long)v.get(i*12+5)).longValue());
	    // Reference
	    hl7de.setReference((String)v.get(i*12+6));
	    // Config Ref
	    hl7de.setReference((String)v.get(i*12+7));
	    // Status
	    hl7de.setStatus((String)v.get(i*12+8));
	    // Notes
	    hl7de.setNotes((String)v.get(i*12+9));
	    // Source
	    hl7de.setSource((String)v.get(i*12+10));
	    // Data Values
	    query = "SELECT length, value, description, complexity, priority, category FROM NISTValue WHERE mrid = '" + v.get(i*12+11) + "'";
	    Vector vI = nDB.executeQuery(query);
	    for(int j=0;j<vI.size()/6;j++) {
		hl7dv = new HL7DataValue();
		// Value
		hl7dv.setValue((String)vI.get(j*6+1));
		// Description
		hl7dv.setDescription((String)vI.get(j*6+2));
		// Complexity
		hl7dv.setComplexity((String)vI.get(j*6+3));
		// Priority
		hl7dv.setPriority((String)vI.get(j*6+4));
		// Category
		hl7dv.setCategory((String)vI.get(j*6+5));
		// Add this HL7DataValue to the current element
		if(hl7dv.getCategory().startsWith("Valid"))
		    hl7de.addHL7DataValue(hl7dv);
	    }
	    dbxml.addHL7Element(hl7de);
	}
	// Generate the XML file
	System.out.println("Generate the XML file");
	dbxml.generateXMLCR(filename);
    }
    
    public static void main(String[] args) {
	try {
	    NISTImpl main = new NISTImpl();
	    String destFile = "ReferenceData.xml";
	    main.transform(destFile);
	}
	catch(SQLException sqle) {
	    System.err.println("SQLException " + sqle.getMessage());
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
    }
}
