/**
 * This class contains methods to call to generate an XML file to use your own value with MessageMaker
 */

package gov.nist.hl7refdb.generator;

// JDK
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

// DOM
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

// XSLT
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// SAX
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

public class DBtoXML {

    /** A Vector to store all HL7DataElement */
    private Vector vDataElement;

    /**
     * Constructor - init the Vector
     */
    public DBtoXML() {
	vDataElement = new Vector();
    }

    /**
     * Add an HL7 Data Element
     * @param hl7 The HL7 data element to add
     */
    public void addHL7Element(HL7DataElement hl7) {
	if(hl7 != null && !vDataElement.contains(hl7))
	    vDataElement.add(hl7);
    }

    /**
     * Remove an HL7 Data Element
     * @param hl7 The HL7 data element to remove
     */
    public void removeHL7Element(HL7DataElement hl7) {
	if(hl7 != null && vDataElement.contains(hl7))
	    vDataElement.remove(hl7);
    }

    /**
     * Generate the XML file
     * @param filename The name of the file destination
     */
    public void generateXML(String filename) throws TransformerConfigurationException, TransformerException, ParserConfigurationException {
	// Write the DOM document to the file
	Transformer transformer = TransformerFactory.newInstance().newTransformer();
	DOMSource source = new DOMSource(getDocument());
	StreamResult result = new StreamResult(new File(filename));
	transformer.transform(source, result);
    }

    /**
     * Generate the XML file with Carriage Return
     * @param filename The name of the file destination
     */
    public void generateXMLCR(String filename) throws TransformerConfigurationException, TransformerException, ParserConfigurationException, SAXException, IOException {
	XMLFormatter handler = new XMLFormatter();
	StringBuffer buffy = new StringBuffer();
	handler.setStringBuffer(buffy);
	
        SAXParserFactory factory = SAXParserFactory.newInstance();
	SAXParser saxParser = factory.newSAXParser();
	generateXML(filename);
	saxParser.parse(new File(filename), handler);

	FileWriter fw = new FileWriter(filename);
	fw.write(buffy.toString());
	fw.close();
    }

    /**
     * Return a DOM Document of all HL7DataElement
     * @return The DOM Document
     */
    private Document getDocument() throws ParserConfigurationException {
	// Create the document
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	DocumentBuilder builder = factory.newDocumentBuilder();
	Document doc = builder.newDocument();
	// Root element
	Element root = (Element)doc.createElement("NIST_HL7_Reference_Data"); 
	root.setAttribute("xmlns", "http://www.nist.gov/hl7/ReferenceData");
	doc.appendChild(root);

	Element magic = (Element)doc.createElement("MessageMaker");
	root.appendChild(magic);

	// Append all HL7DataElement
	for(int i=0;i<vDataElement.size();i++) {
	    HL7DataElement hl7de = (HL7DataElement)vDataElement.get(i);
	    root.appendChild(hl7de.getDOM(doc));
	}
	return doc;
    }
}
