/**
 * This class represents an HL7 Data Value
 */

package gov.nist.hl7refdb.generator;

// DOM
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

public class HL7DataValue {

    /** Actual value of the data item */
    protected String value;
    /** Actual length of the primitive data element*/
    protected long length;
    /** Description and purpose of the test data */
    protected String description;
    /** Indication as to the normalcy of the data value for this element */
    protected String complexity;
    /** Indicates the priority this data value should be selected in the creation of messages */
    protected String priority;
    /** The test category or type of the data value */
    protected String category;

    // Get Methods
    public String getValue() {
	return value;
    }

    public long getLength() {
	return value.length();
    }

    public String getDescription() {
	return description;
    }

    public String getComplexity() {
	return complexity;
    }

    public String getPriority() {
	return priority;
    }

    public String getCategory() {
	return category;
    }

    // Set Methods
    public void setValue(String value) {
	this.value = value;
    }

    public void setDescription(String description) {
	this.description = description;
    }

    public void setComplexity(String complexity) {
	this.complexity = complexity;
    }

    public void setPriority(String priority) {
	this.priority = priority;
    }

    public void setCategory(String category) {
	this.category = category;
    }

    // Methods

    /**
     * Return true if the two objects are equal
     * means same value
     * @param o The object to be compared to
     * @return true if equals
     */
    public boolean equals(Object o) {
	HL7DataValue hl7dv = (HL7DataValue)o;
	boolean equals = true;

	if(value.compareTo(hl7dv.getValue()) != 0)
	    equals = false;
	return equals;
    }

    /**
     * Create a DOM of this HL7 data value
     * @return a DOM node
     */
    public Node getDOM(Document doc) {
	Element item = null;

	item = (Element)doc.createElement("Item");
	// Length L
	if(length != 0)
	    item.setAttribute("Length", String.valueOf(length));
	// Value
	if(value != null)
	    item.setAttribute("Value", value);
	// Description
	if(description != null)
	    item.setAttribute("Description", description);
	// Complexity
	if(complexity != null)
	    item.setAttribute("Complexity", complexity);
	// Priority
	if(priority != null)
	    item.setAttribute("Priority", priority);
	// Category
	if(category != null)
	    item.setAttribute("Category", category);		    
	return item;
    }
}


