/**
 * This class formats(indentation) an XML file
 */

package gov.nist.hl7refdb.generator;

// SAX
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class XMLFormatter extends DefaultHandler {

    private StringBuffer buffy;
    private int indentLevel = 0;
    private boolean bText = false;

    /**
     * Set the StringBuffer
     * @param buffy The StringBuffer
     */
    public void setStringBuffer(StringBuffer buffy) {
	this.buffy = buffy;
    }

    /**
     * Handle the indentation
     */
    public void nl() {
	buffy.append("\n");
	for(int i=0;i<indentLevel;i++)
	    buffy.append("  ");
    }

    public void startDocument() throws SAXException  {
	buffy.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }

    public void endDocument() throws SAXException {
    }

    public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
	indentLevel++;
	nl();
	buffy.append("<" + qName);
	if(attrs != null) {
	    for (int i=0;i<attrs.getLength();i++) {
		String aName = attrs.getQName(i);
		buffy.append(" ");
		buffy.append(aName + "=\"" + formatString(attrs.getValue(i)) + "\"");
	    }
	}
        buffy.append(">");
    }

    public void endElement(String namespaceURI, String sName, String qName) throws SAXException {
	if(!bText)
	    nl();
	else
	    bText = false;
        buffy.append("</" + qName + ">");
	indentLevel--;
    }

    public void characters(char[] ch, int start, int length) {
        String s = new String(ch, start, length);
        if (!s.trim().equals("")) {
	    buffy.append(formatString(s));
	    bText = true;
	}
    }

    /**
     * Normalize a string, handles special characters
     * @src The source string
     * @return The normalized string
     */
    private String formatString(String src) {
	StringBuffer sb = new StringBuffer();

	for(int i=0;i<src.length();i++) {
	    if(src.charAt(i) == '&') {
		sb.append("&amp;");
	    }
	    else if(src.charAt(i) == '<') {
		sb.append("&lt;");
	    }
	    else if(src.charAt(i) == '>') {
		sb.append("&gt;");
	    }
	    else if(src.charAt(i) == '\'' || src.charAt(i) == '') {
		sb.append("&apos;");
	    }
	    else if(src.charAt(i) == '\"') {
		sb.append("&quot;");
	    }
	    else {
		sb.append(src.charAt(i));
	    }
	}
	return sb.toString();
    }
}
