
package gov.nist.hl7maker.control;

import java.io.*;
import java.util.*;

/*
 * This class internally contains and manages the parameters associated with
 * a particular Message Test Set instance. The name of the Messaqge Test Set
 * is identified by "name". The information in this class is saved as a 
 * Properties file (for persistant) with the name (name.mmk). Typically this
 * class is created upon a "new" operation or recalling a previous saved Test
 * Set. It is also used when saving a Test Set.  
 */

/*
 * TODO: add the remaining variables.
 * TODO: add get/set methods for some of the variables
 * TODO: add validation to set methods, if appropreiate
 * TODO: the defaults settings should be defined and read from a properties
 *       file
 *
 */

/*
 * TODO: It is likey that these variables need to be declared static
 */

public class ConfigParameters {

  public ConfigParameters() {
  }

  /* Name identifying the Message Test Set */
  private static String name;

  /* Name of the XML file containing the Profile used to created messages */
  private static String profile;

  /* Name of the path used to store messages */
  private static String path = ".";

  /* Volume option - How many messages should be created? */
  private static int volume = 100;

  /* Message Validity Type Option - Should Valid, Invalid, or Both types of
     messages be created? */
  private static String type = "Valid Only";

  /* Message Complexity Option - Sets how complex the message structure and
     data is for the messages. */
  private static String complexity = "Typical";

  /* Cardinality Option - Create messages that varies cardinality? This is 
     "Master" variable for cardinality. If set to true, all other cardinality
     options are active, if set to false, all other cardinality options are
     meaningless. */
  private static boolean cardinality = true;

  /* Segment Cardinality Option - Create messages that vares segment 
     cardinality? If cardinality is "false", this option is meaningless. */
  private static boolean segmentCardinality = true;

  /* Field Cardinality Option - Create messages that vares field 
     cardinality? If cardinality is "false", this option is meaningless. */
  private static boolean fieldCardinality = true;

  /* Usage Option - Create messages that varies usage? This is "Master" 
     variable for usage. If set to true, all other usage options are active, 
     if set to false, all other usage options are
     meaningless. */ 
  private static boolean usage = true;

  /* Required Usage Option - Create messages that specifically examine the
     required usage option. */
  private static boolean requiredUsage = true;

  /* Required or Empty Usage Option - Create messages that specifically 
     examine the required or empty usage option. */
  private static boolean requiredEmptyUsage = true;

  /* Conditional Usage Option - Create messages that specifically examine 
     the conditional usage option. */
  private static boolean conditionalUsage = false;

  /* Conditional or Empty Usage Option - Create messages that specifically 
     examine the conditional or Empty usage option. */
  private static boolean conditionalEmptyUsage = false;

  /* Not Supported Usage Option - Create messages that specifically 
     examine the not supported usage option. */
  private static boolean notSupportedUsage = true;

  /* Coded Entry Option - Create messages that specifically examine the 
     coded entry option. */
  private static boolean codedEntry = false;

  /* Length Option - Create messages that specifically examine the length 
     of elements. This is the "master" variable for all other Length test
     options. If set to true, then other Length test can be active, otherwise
     the Field, Component, and SubComponent Length options are meaningless. */
  private static boolean length = false;

  /* fieldLength - Create messages that specifically exmine the field length
     of an element. */
  private static boolean fieldLength= false;

  /* componentLength - Create messages that specifically exmine the field 
     length of an element. */
  private static boolean componentLength = false;

  /* subComponentLength - Create messages that specifically exmine the  
     subComponent length of an element. */
  private static boolean subComponentLength = false;

  /* Content Options - Create messages that contain certain data content. 
     The following options allow for the inclusion or exclusion of these
     content tests. */

  /* Data Value - Include content testing data values. */
  private static boolean dataValue = false;

  /* Data Type - Include content testing the data type of elements. */
  private static boolean dataType = false;

  /* Out of Range - Include content testing "out of range" data values. */
  private static boolean outOfRange = false;

  /* Format - Include content testing data format. */
  private static boolean format = false;

  /* Get and Set methods for all option parameters */

  public String getName() {
    return name;
  }

  public void setName(String n) {
    name = n;
  }

  public String getProfile() {
    return profile;
  }

  public void setProfile(String p) {
    profile = p;
  }

  public String getPath() {
    return path;
  }

  public void setPath(String p) {
    path = p;
  }

  public int getVolume() {
    return volume;
  }

  public void setVolume(int v) {
    volume = v;
  }

  public String getType() {
    return type;
  }

  public void setType(String t) {
    type = t;
  }

  public String getComplexity() {
    return complexity;
  }

  public void setComplexity(String c) {
    complexity = c;
  }

  public boolean getCardinlity() {
    return cardinality;
  }

  public void setCardinality(boolean c) {
    cardinality = c;
  }

  public boolean getSegmentCardinality() {
    return segmentCardinality;
  }

  public void setSegmentCardinality(boolean s) {
    segmentCardinality = s;
  }

  public boolean getFieldCardinality() {
    return fieldCardinality;
  }

  public void setFieldCardinality(boolean f) {
    fieldCardinality = f;
  }

  public boolean getUsage() {
    return usage;
  }

  public void setUsage(boolean u) {
    usage = u;
  }

  public boolean getRequiredUsage() {
    return requiredUsage;
  }

  public void setRequiredUsage(boolean r) {
    requiredUsage = r;
  }

  public boolean getRequiredEmptyUsage() {
    return requiredEmptyUsage;
  }

  public void setRequiredEmptyUsage(boolean r) {
    requiredEmptyUsage = r;
  }

  public boolean getConditionalUsage() {
    return conditionalUsage;
  }

  public void setconditionalUsage(boolean r) {
    conditionalUsage = r;
  }

  public boolean getConditionalEmptyUsage() {
    return conditionalEmptyUsage;
  }

  public void setConditionalEmptyUsage(boolean r) {
    conditionalEmptyUsage = r;
  }

  public boolean getNotSupportedUsage() {
    return notSupportedUsage;
  }

  public void setNotSupportedUsage(boolean n) {
    notSupportedUsage = n;
  }

  public boolean getLength() {
    return length;
  }

  public void setLength(boolean l) {
    length = l;
  }

  public boolean getFieldLength() {
    return fieldLength;
  }

  public void setFieldLength(boolean l) {
    fieldLength = l;
  }

  public boolean getComponentLength() {
    return componentLength;
  }

  public void setComponentLength(boolean l) {
    componentLength = l;
  }

  public boolean getSubComponentLength() {
    return subComponentLength;
  }

  public void setSubComponentLength(boolean l) {
    subComponentLength = l;
  }

  public boolean getDataValue() {
    return dataValue;
  }

  public void setDataValue(boolean d) {
    dataValue = d;
  }

  public boolean getDataType() {
    return dataType;
  }

  public void setDataType(boolean d) {
    dataType = d;
  }

  public boolean getOutOfRange() {
    return outOfRange;
  }

  public void setOutOfRange(boolean o) {
    outOfRange = o;
  }

  public boolean getFormat() {
    return format;
  }

  public void setFormat(boolean f) {
    format = f;
  }

  /*
   * List the value of the configuration parameters to the standard output.
   * TODO: implement this function.
   */

  public void list() {

    System.out.println("Configuration Parameters Dump");
    System.out.println("name= " + name);
    System.out.println("profile= " + profile);
    System.out.println("path= " + path);
    System.out.println("volume= " + volume);
    System.out.println("type= " + type);
    System.out.println("complexity= " + complexity);
    System.out.println("cardinality= " + cardinality);
    System.out.println("segmentCardinality= " + segmentCardinality);
    System.out.println("fieldCardinality= " + fieldCardinality);
    System.out.println("usage= " + usage);
    System.out.println("requiredUsage= " + requiredUsage);
    System.out.println("requiredEmptyUsage= " + requiredEmptyUsage);
    System.out.println("conditionalUsage= " + conditionalUsage);
    System.out.println("conditionalEmptyUsage= " + conditionalEmptyUsage);
    System.out.println("notSupportedUsage= " + notSupportedUsage);
    System.out.println("length= " + length);
    System.out.println("fieldLength= " + fieldLength);
    System.out.println("componentLength= " + componentLength);
    System.out.println("subComponentLength= " + subComponentLength);
    System.out.println("dataValue= " + dataValue);
    System.out.println("dataType= " + dataType);
    System.out.println("outOfRange= " + outOfRange);
    System.out.println("format= " + format);

  }

}
