/**
 * This class stores information about the dataElement
 */

package gov.nist.hl7maker.control;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


// MessageMaker
import gov.nist.hl7maker.util.Constants;

public class DataElement {
    
    private final String DATA_ELEMENTS = "." + Constants.FS + "data" + Constants.FS + "dataElements.xml";
    private Vector vDE;

    /**
     * Constructor
     */
    public DataElement() {
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc = builder.parse(new File(DATA_ELEMENTS));
	    vDE = new Vector();
	    browseXML(doc);
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}	
    }

    /**
     * Browse recursively the DOM
     * @param n The current node
     */
    private void browseXML(Node n) {
	if(n.getNodeName().equals("dataElement")) {
	    DE de = new DE();
	    // Segment
 	    Node node = n.getAttributes().getNamedItem("name");
	    if(node != null) {
		de.setSegment(node.getNodeValue());
	    }
	    // Description
 	    node = n.getAttributes().getNamedItem("description");
	    if(node != null) {
		de.setDescription(node.getNodeValue());
	    }
	    // Sequence Number
 	    node = n.getAttributes().getNamedItem("sequenceNum");
	    if(node != null) {
		de.setSequenceNum(node.getNodeValue());
	    }
	    vDE.add(de);
	}
 	// Apply method on children
 	NodeList nl = n.getChildNodes();
 	for(int i=0;i<nl.getLength();i++) {
 	    browseXML(nl.item(i));
	}
    }

    /**
     * Get the sequence number of a field
     * @param seg The name of the segment
     * @param name The name of the field
     */
    public int getFieldSequenceNum(String seg, String name) {
	//	System.out.println(seg + " " + name);
	DE tmp = new DE();
	tmp.setSegment(seg);
	tmp.setDescription(name);
	int idx = vDE.indexOf(tmp);
	//System.out.println(idx);
	if(idx != -1) return Integer.parseInt(((DE)vDE.get(idx)).getSequenceNum());
	return -1;
    }

    // inner class to represent a data element item
    public class DE {

	private String segment;
	private String description;
	private String sequenceNum;

	public String getSegment() {
	    return segment;
	}

	public String getDescription() {
	    return description;
	}

	public String getSequenceNum() {
	    return sequenceNum;
	}

	public void setSegment(String segment) {
	    this.segment = segment;
	}

	public void setDescription(String description) {
	    this.description = description;
	}

	public void setSequenceNum(String sequenceNum) {
	    this.sequenceNum = sequenceNum;
	}

	public boolean equals(Object o) {
	    if(o instanceof DE) {
		DE de = (DE)o;
		return segment.equals(de.getSegment()) && description.equals(de.getDescription());
	    }
	    return false;
	}
    }
}
