
package gov.nist.hl7maker.control;

import java.util.*;
import java.io.File;

import gov.nist.hl7maker.message.MessageOptions;
import gov.nist.hl7maker.view.config.TablePriority;

/*
 * This Properties class is used to map the options set by the user on
 * the user interface to setting required by the XSL transformation. A
 * Properties object is created that captures this mapping. 
 *
 * TODO: Some properties are not yet implemented and are currently hard
   coded (e.g., offset). Implement as necessary.
 */

public class MessageGenerationProperties {

    private Properties properties = null;
    
    public MessageGenerationProperties(Session session) {
	
	properties = new Properties();
	
	properties.setProperty("xmlProfile", session.getProfile()); 
	properties.setProperty("outputMsgDir", session.getMsgPath()); 
	/* The name of the metafile is hard-coded. */
	/* Someday this should be grabbed from Session. Session should save
	   this as well. Also, the user should be able to change this with 
	   the GUI. */
	File f = new File(session.getProfile());
	properties.setProperty("metaFile", "TD_" + f.getName()); 
	properties.setProperty("preTransform", "xslt/profilePreProcess.xsl"); 
	properties.setProperty("xsltTransform", "xslt/profile2msg.xsl"); 
	properties.setProperty("optionsTransform", "xslt/messageProperties.xsl");
	properties.setProperty("configTable", session.getConfig()); 
	properties.setProperty("hl7Tables", session.getHL7Tables()); 
	properties.setProperty("userTables", session.getUserTables()); 
	properties.setProperty("externalTables", session.getExternalTables()); 
	properties.setProperty("referenceTable", session.getRepositoryData());
	//properties.setProperty("referenceTable", session.getReferenceDB());
	properties.setProperty("dataElements", "data" + File.separator + "dataElements.xml"); 
	properties.setProperty("errorCodes", "data" + File.separator + "errorCodes.xml");
	Vector v = session.getLocalTable();
	if(v.size() != 0)
	    properties.setProperty("localTables", ((TablePriority)v.get(0)).getFilename());
	else
	properties.setProperty("localTables", "data/emptyLocal.xml");
    }
    
    /*
     * Return the properties file associated with the this Message Test Set.
     */
    public Properties getProperties() {
	return properties;
    }
    
    /**
     * Set the messageControlId property
     * @param value The value of the messageControlId
     */
    public void setMessageControlIdProperty(String value) {
	properties.setProperty("messageControlId", value);
    }
    
    /**
     * Set the optionsFile property
     * @param value The value of the optionsFile
     */
    public void setOptionsFileProperty(String value) {
	properties.setProperty("optionsFile", value);
	// Add the option properties
/*
	MessageOptions msgs = null;
	try {
	    msgs = new MessageOptions(properties.getProperty("optionsTransform"), value);
	    Properties options = msgs.getOptions();
	    for(Enumeration e = options.propertyNames();e.hasMoreElements();) {
		String name = (String)e.nextElement();
		properties.setProperty(name, options.getProperty(name));
		}
	}
	catch(Exception e) {
	    System.err.println("Exception: " + e.getMessage());
	    e.printStackTrace();
	}*/
    }
}
