/*
 * NIST/HL7 Message Maker
 * MessageMaker.java
 *
 * See the file "nist.disclaimer" in the top-level directory for information
 * on usage and redistribution of this file.
 *
 */

package gov.nist.hl7maker.control;

// JDK
import java.io.File;

// AWT
import java.awt.Dimension;
import java.awt.Toolkit;

// MessageMaker
import gov.nist.hl7maker.util.CheckResources;
import gov.nist.hl7maker.util.MMLog;
import gov.nist.hl7maker.view.GUIControl;
import gov.nist.hl7maker.view.BeginnersDialog;

/** 
 * Ths Class is the launch point for the Message Maker Tool. It takes care of
 * initialization and input arguments.
 */
public class MessageMaker {

  private GUIControl theGUIControl;

  /** 
   * Bring up the GUI for Message Maker. 
   */
  public MessageMaker() {
    super();
    //new MakerProperties();
    theGUIControl = new GUIControl();
    theGUIControl.show();

    if(!theGUIControl.getUser().getAskMeAgain()) {
	BeginnersDialog dlgBeginners = new BeginnersDialog(theGUIControl, theGUIControl.getSession(), theGUIControl.getUser());
	dlgBeginners.pack();
	// Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension windowSize = dlgBeginners.getSize();
	dlgBeginners.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				 Math.max(0,(screenSize.height-windowSize.height)/2));
	dlgBeginners.setVisible(true);
    }
  }

  /** 
   * Launch point forthe Message Maker application.
   *
   * @param args the command line arguments
   */
  public static void main(String args[]) {
      // Check for mandatory resources
      String check = CheckResources.checkStartup();
      if(!check.equals("")) {
	  System.err.println(check);
	  System.exit(-1);
      }
      // Check if the log directory exists
      File fLog = new File("log");
      if(!fLog.exists())
	  fLog.mkdir();
      MMLog.log("Message Maker v1.4 Started");
  
      // Create the application
      new MessageMaker();
  }

}
