
package gov.nist.hl7maker.control;

import java.io.*;
import java.util.*;

/*
 * TODO: Set the location of the properties file.
 * Normally, you can put property files on a relative or absolute path. But 
 * if you desire more location independence and flexibility, you can also put 
 * them on the CLASSPATH and get an InputStream like this:
 *
 *   this.getClass().getClassLoader().getResourceAsStream("my.properties");
 * 
 * I have an example of this under the programming/java/util directory. We may
 * need to do it this way.
 * 
 */

public class MessageTestSetProperties {

  private String propertiesFile;  
  private Properties properties = null;
  
  public MessageTestSetProperties() {
    this("default");
  }

  public MessageTestSetProperties(ConfigParameters cp) {
    propertiesFile = cp.getName();
    loadProperties();    
    properties.setProperty("SegmentCardinality", 
                           String.valueOf(cp.getSegmentCardinality()));
    //properties.list(System.out);
  }

  public MessageTestSetProperties(String testSetName) {
    
    propertiesFile = testSetName;
    loadProperties();    
    
    properties.setProperty("SegmentCardinality", "1");
    properties.setProperty("FieldCardinality", "1");
    properties.setProperty("volume", "50");
    //properties.list(System.out);
      
  }

  /*
   * Return the properties file associated with the this Message Test Set.
   */

  public Properties getProperties() {
    return properties;
  }

  /*
   * This is a pass-through method. It checks to see if we are correctly
   * setting a property for this property file.
   *
   * TODO: check the property that is be set/updated.
   */

  public void setProperty(String property, String value) {
    properties.setProperty(property, value);
  }

  public void storeProperties() {

    FileOutputStream out = null;

    try {
      out = new FileOutputStream(propertiesFile);
      properties.store(out, null);
    }
    catch (FileNotFoundException e) {
    }
    catch (IOException e) {
    }
    catch (Exception e) {
    }
  }

  public void loadProperties () {

    FileInputStream in = null;
    properties = new Properties();
    
    try {
      in = new FileInputStream(propertiesFile);
      properties.load(in);
    }
    catch (FileNotFoundException e) {
    }
    catch (IOException e) {
    }
    catch (Exception e) {
    }

    //listDetails();
    
  }

  /* 
   * This is a convenience method to print the details of the properties
   * file. It is very similar to the Java version.
   */

  public void listDetails() {

    /* START DEBUGGING CODE */
    System.out.println("Default property values:");
    properties.list(System.out);

    for (Enumeration e = properties.propertyNames(); e.hasMoreElements();) {
      String name = (String)e.nextElement();
      String prop = properties.getProperty(name);
      System.out.println("   " + name + " = " +prop);
    }
    /* END DEBUGGING CODE */
 }
  
}
