/**
 * This class represents the reference data
 */

package gov.nist.hl7maker.control;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.Observable;
import java.util.StringTokenizer;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;

// SAX
import org.xml.sax.SAXException;  

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// MessageMaker
//import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.view.config.ValueNode;

public class ReferenceData extends Observable {

    private Session session;
    private String name, segment, qualifier, qualifier2, datatype;
/*    private ProfileNode curPN;
    private ValueNode curVN;
    private Object[] curVVN;

    private Node refNode, itemNode;*/

    private final String REFERENCE_DATA = "." + File.separator + "data" + File.separator + "ReferenceData.xml";
    private Vector vRefDB;

    /**
     * Constructor
     */
    public ReferenceData() {
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc = builder.parse(new File(REFERENCE_DATA));
	    //.getCanonicalPath());
	    //Document doc = builder.parse(session.getProtocolPath(REFERENCE_DATA));
	    //	Document doc = builder.parse(new File(REFERENCE_DATA).getCanonicalPath());
	    // Browser the XML file
	    vRefDB = new Vector();
	    browseXML(doc);
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}	
    }
    
    /**
     * Browse recursively the DOM and collects the values
     * @param n The current node
     */
    private void browseXML(Node n) {
	// Reference Data
	if(n.getNodeName().equals("ReferenceData")) {
 	    Node attributes = n.getAttributes().getNamedItem("Name");
 	    if(attributes != null) {
		name = attributes.getNodeValue();
	    }
	    attributes = n.getAttributes().getNamedItem("Qualifier");
	    if(attributes != null) {
		qualifier = attributes.getNodeValue();
	    }
	    attributes = n.getAttributes().getNamedItem("Qualifier2");
	    if(attributes != null) {
		qualifier2 = attributes.getNodeValue();
	    }
	    attributes = n.getAttributes().getNamedItem("Segment");
	    if(attributes != null) {
		segment = attributes.getNodeValue();
	    }
	    attributes = n.getAttributes().getNamedItem("Datatype");
	    if(attributes != null) {
		datatype = attributes.getNodeValue();
	    }
	}
	// Item
	else if(n.getNodeName().equals("Item")) {
	    ValueNode curValue = new ValueNode();
	    curValue.setName(name);
	    curValue.setQualifier(qualifier);
	    curValue.setQualifier2(qualifier2);
	    curValue.setSegment(segment);
	    curValue.setDatatype(datatype);
	    vRefDB.add(curValue);
	    Node attributes = n.getAttributes().getNamedItem("Value");
	    if(attributes != null) {
		curValue.setValue(attributes.getNodeValue());
	    }
	    attributes = n.getAttributes().getNamedItem("Complexity");
	    if(attributes != null) {
		curValue.setComplexity(attributes.getNodeValue());
	    }
	    attributes = n.getAttributes().getNamedItem("Priority");
	    if(attributes != null) {
		curValue.setPriority(attributes.getNodeValue());
	    }
	    attributes = n.getAttributes().getNamedItem("Category");
	    if(attributes != null) {
		curValue.setCategory(attributes.getNodeValue());
	    }
	    attributes = n.getAttributes().getNamedItem("Description");
	    if(attributes != null) {
		curValue.setDescription(attributes.getNodeValue());
	    }
	}
 	// Apply method on children
 	NodeList nl = n.getChildNodes();
 	for(int i=0;i<nl.getLength();i++) {
 	    browseXML(nl.item(i));
	}
    }

    /**
     * Get the values for a name, segment and qualifiers
     * @param name The name
     * @param segment The segment
     * @param qualifier The qualifier
     * @param qualifier2 The 2nd qualifier
     * @return A vector with the values
     */
    public Vector getValues(String name, String segment, String qualifier, String qualifier2) {
	Vector v = new Vector();
	ValueNode tmp = new ValueNode();
	tmp.setName(name);
	tmp.setQualifier(qualifier);
	tmp.setQualifier2(qualifier2);
	tmp.setSegment(segment);

	for(int i=0;i<vRefDB.size();i++) {
	    ValueNode vn = (ValueNode)vRefDB.get(i);
	    if(tmp.equals(vn) && vn.getStatus() != ValueNode.REMOVED) {
		v.add(vRefDB.get(i));
	    }
	}
	return v;
    }

    /**
     * Get the values
     * @return A vector with the values
     */
/*    public Vector getValues() {
	return vRefDB;
	}*/
}
