/**
 * This class contains all the information about the application
 */

package gov.nist.hl7maker.control;

// JDK
import java.util.Properties;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;

//XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// MessageMaker
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.view.display.MetaData;

// HAPI
//import ca.uhn.hl7v2.model.Message;

public class User {

    private static Properties props;
    private static final String USER_DAT = "config" + Constants.FS + "user.dat";
    private Vector vMsg;
    private Vector vMD;

    /**
     * Constructor
     */
    public User() throws IOException {
	props = new Properties();
	props.load(new FileInputStream(new File(USER_DAT)));
	vMsg = new Vector();
	vMD = new Vector();
    }

    /**
     * Save the properties
     */
    public void saveProperties() throws IOException {
	props.store(new FileOutputStream(USER_DAT), null);
	gov.nist.hl7maker.util.MVCLog.info(USER_DAT + " saved");
    }

    /* Last opened directory */
    public String getOpenedDir() {
	return props.getProperty("user.openedDir");
    }

    public static String getStaticOpenedDir() {
	try {
	    props = new Properties();
	    props.load(new FileInputStream(new File(USER_DAT)));
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	return props.getProperty("user.openedDir");
    }

    public void setOpenedDir(String openedDir) {
	props.setProperty("user.openedDir", openedDir);
    }

    public static void setStaticOpenedDir(String openedDir) {
	try {
	    props.setProperty("user.openedDir", openedDir);
	    props.store(new FileOutputStream(USER_DAT), null);
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
    }

    public boolean getAskMeAgain() {
	if(props.getProperty("user.askMeAgain").equals("0")) {
	    return true;
	}
	return false;
    }

    public void setAskMeAgain(boolean askMeAgain) {
	if(askMeAgain) {
	    props.setProperty("user.askMeAgain", "0");
	}
	else {
	    props.setProperty("user.askMeAgain", "1");
	}
    }

    /* Item 1 in the mmk file history */
    public String getMmk1() {
	return props.getProperty("user.mmk.1");
    }

    public void setMmk1(String mmk) {
	if(mmk != null) {
	    if(props.getProperty("user.mmk.1") == null || (props.getProperty("user.mmk.1") != null && mmk.compareTo(props.getProperty("user.mmk.1")) != 0)){
		if(!mmk.equals("")) {
		    setMmk2(props.getProperty("user.mmk.1"));
		    props.setProperty("user.mmk.1", mmk);
		}
	    }
	}
    }

    /* Item 2 in the mmk file history */
    public String getMmk2() {
	return props.getProperty("user.mmk.2");
    }

    public void setMmk2(String mmk) {
	if(mmk != null) {
	    setMmk3(props.getProperty("user.mmk.2"));
	    props.setProperty("user.mmk.2", mmk);
	}
    }

    /* Item 3 in the mmk file history */
    public String getMmk3() {
	return props.getProperty("user.mmk.3");
    }

    public void setMmk3(String mmk) {
	if(mmk != null) {
	    setMmk4(props.getProperty("user.mmk.3"));
	    props.setProperty("user.mmk.3", mmk);
	}
    }

    /* Item 4 in the mmk file history */
    public String getMmk4() {
	return props.getProperty("user.mmk.4");
    }

    public void setMmk4(String mmk) {
	if(mmk != null) {
	    props.setProperty("user.mmk.4", mmk);
	}
    }

    /**
     * Clean the history
     */
    public void cleanMmkHistory() {
	Vector v = new Vector();
	for(int i=1;i<=4;i++) {
	    String s = props.getProperty("user.mmk." + i);
	    if(s != null) {
		// Check double-entry
		if(v.contains(s)) {
		    props.remove("user.mmk." + i);
		}
		else {
		    File f = new File(s);
		    if(!f.exists()) {
			props.remove("user.mmk." + i);
		    }
		    else {
			v.add(s);
		    }
		}
	    }
	}
	for(int i=v.size()-1;i>=0;i--) {
	    setMmk1((String)v.get(i));
	}
	for(int i=v.size()+1;i<=4;i++) {
	    props.remove("user.mmk." + i);
	}
    }

    /**
     * Add a metadata file
     * @param md The metadata file
     */
    public void addMD(String md) {
	if(!vMD.contains(md)) {
	    vMD.add(md);
	}
    }

    /**
     * Add a message
     * @param msg The filename of message
     * @param profile The profile of the message
     */
    public void addMsg(String msg, String profile) {
	MakerMessage mmMsg = new MakerMessage(msg, profile);
	if(!vMsg.contains(mmMsg)) {
	    vMsg.add(mmMsg);
	}
    }

    /**
     * Remove a message
     * @param msg The filename of message
     */
    public void removeMsg(String msg) {
	int idx = vMsg.indexOf(new MakerMessage(msg));
	if(idx != -1)
	    vMsg.remove(idx);
    }

    /**
     * List all messages
     */
    public void listMsg() {
	for(int i=0;i<vMsg.size();i++) {
	    MakerMessage mmMsg = (MakerMessage)vMsg.get(i);
	    gov.nist.hl7maker.util.MVCLog.debug("[User] " + mmMsg.getFilename());
	}
    }

    /**
     * Is there unsaved messages?
     * @return true if there are unsaved messages
     */
    public boolean isUnsavedMsg() {
	boolean unsaved = false;
	for(int i=0;i<vMsg.size() && !unsaved;i++) {
	    MakerMessage mm = (MakerMessage)vMsg.get(i);
	    if(mm.getStatus() == MakerMessage.GENERATED)
		unsaved = true;
	}
	return unsaved;
    }

    /**
     * Get the list of unsaved messages
     * @return A vector with the unsaved messages
     */
    public Vector getUnsavedMsg() {
	Vector vUnsaved = new Vector();
	for(int i=0;i<vMsg.size();i++) {
	    MakerMessage mm = (MakerMessage)vMsg.get(i);
	    if(mm.getStatus() == MakerMessage.GENERATED)
		vUnsaved.add(mm.getFilename());
	}
	return vUnsaved;
    }

    /**
     * Delete Unsaved Messages
     */
    public void deleteUnsavedMsg() {
	for(int i=0;i<vMsg.size();i++) {
	    MakerMessage mm = (MakerMessage)vMsg.get(i);
	    if(mm.getStatus() == MakerMessage.GENERATED) {
		File f = new File(mm.getFilename());
		f.delete();
	    }
	}
    }	

    /**
     * Delete Messages associated to a metadata file
     * @param mdFile The Metadata file
     */
    public void deleteMsg(String mdFile) {
	try {
	    MetaData md = new MetaData(mdFile);
	    Vector vM = md.getMessageList();
	    for(int i=0;i<vM.size();i++) {
		File f = new File(md.getMsgPath() + (String)vM.get(i));
		removeMsg(f.getPath());
		f.delete();
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	}
    }	

    /**
     * Remove all unneeded MetaData files
     */
    public void removeAllMD() {
	try {
	    MetaData md = null;
	    for(int i=0;i<vMD.size();i++) {
		md = new MetaData((String)vMD.get(i));
		if(md.getMessageList().size() == 0) {
		    File f = new File((String)vMD.get(i));
		    f.delete();
		}
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	}
    }

    /**
     * Get the status of a message
     * @param sMsg The filename of the message
     * @return The status
     */
    public int getStatus(String sMsg) {
	MakerMessage mmTmp = new MakerMessage(sMsg);
	int idx = vMsg.indexOf(mmTmp);
	if(idx != -1) {
	    return ((MakerMessage)vMsg.get(idx)).getStatus();
	}
	return -1;
    }

    /**
     * Set the status of a message
     * @param sMsg The filename of the message
     * @param status The status
     */
    public void setStatus(String sMsg, int status) {
	MakerMessage mmTmp = new MakerMessage(sMsg);
	int idx = vMsg.indexOf(mmTmp);
	if(idx != -1) {
	    mmTmp = (MakerMessage)vMsg.get(idx);
	    mmTmp.setStatus(status);
	}
	else {
	    System.err.println("This message doesn't exist");
	    System.exit(-1);
	}
    }

    /**
     * Get the Message object of a message
     * @param sMsg The filename of the message
     * @return The Message Object
     */
    public MakerMessage getMessage(String sMsg) {
	MakerMessage mmTmp = new MakerMessage(sMsg);
	int idx = vMsg.indexOf(mmTmp);
	if(idx == -1) {
	    System.err.println("This message doesn't exist");
	    System.exit(-1);
	}
	return (MakerMessage)vMsg.get(idx);
    }
}
