package gov.nist.hl7maker.message;

import gov.nist.hl7maker.util.Constants;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Map;
import java.util.Properties;

import gov.nist.hl7maker.util.MMLog;

 class MessageDescription {
   private static final String cName = "MessageDescription";
   
   private final static String SESSION_TAG = "SESSION";
   private final static String lineSeparator = Constants.LS;
   
   private String outputMsgDir = null;
   private String metaFileName = null;
   private PrintWriter metaDataFile = null;
   
   //  Segment Cardinality:
   boolean MIN_SC = false;
   boolean MAX_SC = false;
   boolean RANDOM_SC = false;
  //  Field Cardinality:
   boolean MIN_FC = false;
   boolean MAX_FC = false;
   boolean RANDOM_FC = false;
   boolean ELEMENT_CARDINALITY = false;
   boolean ELEMENT_REUSAGE = false;
  //  no longer an option, always true:
   final static boolean ALL_RU = true;
  //  no longer an option, always true:
   final static boolean NONE_XU = true;
  //  RE Usage
   boolean NONE_REU = false;
   boolean ALL_REU = false;
   boolean SOME_REU = false;

  // errors:
   boolean MIN_SC_ERR = false;
   boolean MAX_SC_ERR = false;
   boolean MIN_FC_ERR = false;
   boolean MAX_FC_ERR = false;
   boolean RUSAGE_ERR = false;
   boolean XUSAGE_ERR = false;
   boolean FIELD_LEN_ERR = false;
   boolean COMP_LEN_ERR = false;
   boolean SUBCOMP_LEN_ERR = false;
   boolean TABLEVALUE_ERR = false;
   
   private String sessionDescription = "";
   private String description = "";
   private String elementDescription = "";
   private String elementName = "";
   private String errorElement = "";
   
   //  empty unless changed from initial setting:
   private String segCardElement = "";
   private String fieldCardElement = "";
   private String reUsageElement = "";
   
   //private Properties sessionProperties = null;
   private GroupOptions sessionProperties = null;
   private ProfileProperties profileProperties = null;
   
   /**
   * 
   */
   public MessageDescription () {
   }
   
   /* effects saving of properties of initial msg group to metadata file */ 
   public void startOfSession (GroupOptions options,
       String outputDir,
       String metaFile,
       String mmVersion,
       String hl7Version,
       String sessionName,
       String orgName,
       String userComment,
       String xmlProfile,
       int totalMsgCount) {
     outputMsgDir = outputDir;
     metaFileName = metaFile;
     
     metaDataFile = getPrintWriter(outputMsgDir + metaFileName);
     markStartOfSession();

     sessionProperties = new GroupOptions(options);

     setSessionDescription();
     
     // meta data is recorded for each msg generated
     metaDataFile = getPrintWriter(outputMsgDir + metaFileName);
     markStartOfSession();
     saveSessionProperties(totalMsgCount,
         sessionProperties.getOption("validity"),
         outputMsgDir,
         metaFileName,
         mmVersion,
         hl7Version,
         sessionName,
         orgName,
         userComment,
         xmlProfile);    
   }

   //  save properties of this set of msgs to the metadata file 
   private void saveSessionProperties (int numOfMsgs, 
       String validity,
       String outputMsgDir,
       String metaFileName,
       String mmVersion,
       String hl7Version,
       String sessionName,
       String orgName,
       String userComment,
       String xmlProfile) {
     String date = (new Date()).toString();
     
     String fcard = sessionProperties.getOption("fieldCardinality");
     String scard = sessionProperties.getOption("segmentCardinality");
     String reUsage = sessionProperties.getOption("RE_Usage");
     String minSegCardError = sessionProperties.getOption("minSegCardError");
     String maxSegCardError = sessionProperties.getOption("maxSegCardError");
     String minFieldCardError = sessionProperties.getOption("minFieldCardError");
     String maxFieldCardError = sessionProperties.getOption("maxFieldCardError");
     String rusageError = sessionProperties.getOption("RusageError");
     String xusageError = sessionProperties.getOption("XusageError");
     String fieldLengthError = sessionProperties.getOption("fieldLengthError");
     String componentLengthError = sessionProperties.getOption("componentLengthError");
     String subComponentLengthError = 
         sessionProperties.getOption("subComponentLengthError");
       
     metaDataFile.println(
  "  <MMversion>" +mmVersion +"</MMversion>" +lineSeparator
 +"  <HL7version>" +hl7Version +"</HL7version>" +lineSeparator
 +"  <Session>" +sessionName +"</Session>" +lineSeparator
 +"  <Profile>" +xmlProfile +"</Profile>" +lineSeparator
 +"  <MessagePath>" +outputMsgDir +"</MessagePath>" +lineSeparator
 +"  <Date>" +date +"</Date>" +lineSeparator
 +"  <Organization>" +orgName +"</Organization>" +lineSeparator
 +"  <Description>" +sessionDescription +"</Description>" +lineSeparator
 +"  <UserComment>" +userComment +"</UserComment>" +lineSeparator
 +"  <MessageCountTotal>" +numOfMsgs +"</MessageCountTotal>" +lineSeparator
 +lineSeparator
 +"  <OPTIONS_Description>" +lineSeparator
 +"      Unless overridden, these settings apply to all messages." +lineSeparator
 +"  </OPTIONS_Description>" +lineSeparator
 +"  <MessageTypes>" +validity +"</MessageTypes>" +lineSeparator
 +"  <FieldCardinality>" +fcard +"</FieldCardinality>" +lineSeparator
 +"  <SegmentCardinality>" +scard +"</SegmentCardinality>" +lineSeparator
 +"  <REusage>" +reUsage +"</REusage>" +lineSeparator
 +"  <MessageTypes>" +validity +"</MessageTypes>" +lineSeparator
 +"  <MinSegCardError>" +minSegCardError +"</MinSegCardError>" +lineSeparator
 +"  <MaxSegCardError>" +maxSegCardError +"</MaxSegCardError>" +lineSeparator
 +"  <MinFieldCardError>" +minFieldCardError +"</MinFieldCardError>" 
        +lineSeparator
 +"  <MaxFieldCardError>" +maxFieldCardError +"</MaxFieldCardError>" 
        +lineSeparator
 +"  <RusageError>" +rusageError +"</RusageError>" +lineSeparator
 +"  <XusageError>" +xusageError +"</XusageError>" +lineSeparator
 +"  <FieldLengthError>" +fieldLengthError +"</FieldLengthError>" +lineSeparator
 +"  <ComponentLengthError>" +componentLengthError +"</ComponentLengthError>" 
        +lineSeparator
 +"  <SubComponentLengthError>" +subComponentLengthError 
        +"</SubComponentLengthError>" +lineSeparator);
   }
   
   /* writes message description to metadata file */
   public void saveMessageDescription (String name, 
                           int msgNumber, 
                           String valid, 
                           boolean er7) {
     String type = "";
     if ("true".equalsIgnoreCase(valid)) type = "VALID";
     else type = "INVALID";
     
     metaDataFile.println(    
         "<Message SequenceID=\"" +msgNumber +"\">" +lineSeparator
         +"  <Name>" +name +"</Name>" +lineSeparator
         +"  <Type>" +type +"</Type>" +lineSeparator
         +"  <Description>" +lineSeparator 
           +description
         +"  </Description>" +lineSeparator);
     
     if (!"".equals(segCardElement)) {
       metaDataFile.print("  " +segCardElement +lineSeparator);
     }
     if (!"".equals(fieldCardElement)) {
       metaDataFile.print("  " +fieldCardElement +lineSeparator);
     }
     if (!"".equals(reUsageElement)) {
       metaDataFile.print("  " +reUsageElement +lineSeparator +lineSeparator);
     }
         
     if (!"".equals(elementName)) {
       metaDataFile.println(
           "  <ELEMENT description=\"" +elementDescription +"\">" +lineSeparator
           +elementName +lineSeparator);
         if (!"".equals(errorElement)) {
           metaDataFile.println("   " +errorElement +lineSeparator);
         }
       metaDataFile.println("  </ELEMENT>");
     }
     metaDataFile.println("</Message>" +lineSeparator);
   }

   private void setSessionDescription () {
     
     if (sessionProperties == null) return;
     
     String newline = Constants.LS;

     sessionDescription = "These messages were automatically  ";
     sessionDescription += "generated by Message Maker. ";
     sessionDescription += newline;
     
     // it's a min msg if min card & no RE elements
     boolean min = "MIN".equalsIgnoreCase(
         sessionProperties.getOption("segmentCardinality"));
     min = 
       "MIN".equalsIgnoreCase(sessionProperties.getOption("fieldCardinality"))
       && "NONE".equalsIgnoreCase(sessionProperties.getOption("RE_Usage"));

     // it's a max msg if max card & all RE elements
     boolean max = "MAX".equalsIgnoreCase(
         sessionProperties.getOption("segmentCardinality"));
     max = 
       "MAX".equalsIgnoreCase(sessionProperties.getOption("fieldCardinality"))
       && "ALL".equalsIgnoreCase(sessionProperties.getOption("RE_Usage"));
     
     if (min) {
       sessionDescription += "The Base message (and any repetitions) ";
       sessionDescription += "is minimally populated, i.e., ";
       sessionDescription += "minimum cardinality for all Segments and ";
       sessionDescription += "Fields, no RE elements included.";
       sessionDescription += newline;
     }
     else if (max) {
       sessionDescription += "The Base message (and any repetitions) ";
       sessionDescription += "is maximally populated, i.e., ";
       sessionDescription += "maximum cardinality for all Segments and ";
       sessionDescription += "Fields, all RE elements included.";
       sessionDescription += newline;
     }
     else {
       sessionDescription += "The Base message (and any repetitions) ";
       sessionDescription += "is neither minimally nor maximally populated.";
       sessionDescription += newline;
     }
     
     String validity = sessionProperties.getOption("validity");
     if ("VALID".equals(validity)) {
       sessionDescription += "The message (and all repetitions) includes no errors.";
       sessionDescription += newline;
     }
     else if ("INVALID".equals(validity)) {
       sessionDescription += "The message (and any repetitions) may includes an error.";
       sessionDescription += newline;
     }
     else {
       sessionDescription += "The message (or some repetitions) may includes an error.";
       sessionDescription += newline;
     }
   }
   
   public String getSessionDescription () {
     return sessionDescription;
   }

   public void setMessageProperties (MessageOptions options, 
       Properties errLocations) {
     initProperties();
     
     segCardElement = "";
     fieldCardElement = "";
     reUsageElement = "";
     
     String segmentCardinality = options.getOption("segmentCardinality");
     String fieldCardinality = options.getOption("fieldCardinality");
     String RE_Usage = options.getOption("RE_Usage");
     //float C_Usage = Float.parseFloat(options.getProperty("C_Usage"));
     //float CE_Usage = Float.parseFloat(options.getProperty("CE_Usage"));

     String sessionSegCardinality = 
         sessionProperties.getOption("segmentCardinality");
     String sessionFieldCardinality   = 
         sessionProperties.getOption("fieldCardinality");
     String sessionRE_Usage = sessionProperties.getOption("RE_Usage");

     if ("MIN".equalsIgnoreCase(segmentCardinality)) {
       MIN_SC = true;
       // we want to include the element only if there's been a change
       if (!"MIN".equalsIgnoreCase(sessionSegCardinality)) {
         segCardElement = 
           "<SegmentCardinality>MIN</SegmentCardinality>";
       }
     }
     else if ("MAX".equalsIgnoreCase(segmentCardinality)) {
       MAX_SC = true;
       if (!"MAX".equalsIgnoreCase(sessionSegCardinality)) {
         segCardElement = 
           "<SegmentCardinality>MAX</SegmentCardinality>";
       }
     }
     else {
       RANDOM_SC = true;
       if ("MIN".equalsIgnoreCase(sessionSegCardinality) ||
           "MAX".equalsIgnoreCase(sessionSegCardinality)) {
         segCardElement = 
           "<SegmentCardinality>RANDOM</SegmentCardinality>";
       }
     }
     if ("MIN".equalsIgnoreCase(fieldCardinality)) {
       MIN_FC = true;
       if (!"MIN".equalsIgnoreCase(sessionFieldCardinality)) {
         fieldCardElement = 
           "<FieldCardinality>MIN</FieldCardinality>";
       }
     }
     else if ("MAX".equalsIgnoreCase(fieldCardinality)) {
       MAX_FC = true;
       if (!"MAX".equalsIgnoreCase(sessionFieldCardinality)) {
         fieldCardElement = 
           "<FieldCardinality>MAX</FieldCardinality>";
       }
     }
     else {
       RANDOM_FC = true;
       if ("MIN".equalsIgnoreCase(sessionFieldCardinality) ||
             "MAX".equalsIgnoreCase(sessionFieldCardinality)) {
         fieldCardElement = 
           "<FieldCardinality>RANDOM</FieldCardinality>";
       }
     }
     if ("NONE".equalsIgnoreCase(RE_Usage)) {
       NONE_REU = true;
       if (!"NONE".equalsIgnoreCase(sessionRE_Usage)) {
         reUsageElement = "<REusage>NONE</REusage>";
       }
     }
     else if ("ALL".equalsIgnoreCase(RE_Usage)) {
       ALL_REU = true;
       if (!"ALL".equalsIgnoreCase(sessionRE_Usage)) {
         reUsageElement = "<REusage>ALL</REusage>";
       }
     }
     else {
       SOME_REU = true;
       if ("NONE".equalsIgnoreCase(sessionRE_Usage) ||
             "ALL".equalsIgnoreCase(sessionRE_Usage)) {
         reUsageElement = "<REusage>SOME</REusage>";
       }
     }

     boolean valid = "true".equalsIgnoreCase(
       options.getOption("valid"));
     boolean minSegCardErr = "true".equalsIgnoreCase(
       options.getOption("minSegCardError"));
     boolean maxSegCardErr = "true".equalsIgnoreCase(
       options.getOption("maxSegCardError"));
     boolean minFieldCardErr = "true".equalsIgnoreCase(
       options.getOption("minFieldCardError"));
     boolean maxFieldCardErr = "true".equalsIgnoreCase(
       options.getOption("maxFieldCardError"));
     boolean RusageErr  = "true".equalsIgnoreCase(
       options.getOption("RusageError"));
     boolean XusageErr  = "true".equalsIgnoreCase(
       options.getOption("XusageError"));
     boolean fieldLengthErr = "true".equalsIgnoreCase(
       options.getOption("fieldLengthError"));
     boolean compLengthErr  = "true".equalsIgnoreCase(
       options.getOption("componentLengthError"));
     boolean subCompLengthErr = "true".equalsIgnoreCase(
       options.getOption("subComponentLengthError"));
     boolean tableValueErr = "true".equalsIgnoreCase(
     		 options.getOption("elementErrorCode"));

     errorElement = ""; 
       
     if (minSegCardErr) {
       MIN_SC_ERR = true;
       errorElement = "<MinSegCardError>true</MinSegCardError>"; 
     }
     if (maxSegCardErr) {
       MAX_SC_ERR = true;
       errorElement = "<MaxSegCardError>true</MaxSegCardError>"; 
     }
     if (minFieldCardErr) {
       MIN_FC_ERR = true;
       errorElement = "<MinFieldCardError>true</MinFieldCardError>";
     }
     if (maxFieldCardErr) {
       MAX_FC_ERR = true;
         errorElement = "<MaxFieldCardError>true</MaxFieldCardError>";
     }
     if (RusageErr) {
       RUSAGE_ERR = true;
       errorElement = "<RusageError>true</RusageError>";
     }
     if (XusageErr) {
       XUSAGE_ERR = true;
       errorElement = "<XusageError>true</XusageError>";
     }
     if (fieldLengthErr) {
       FIELD_LEN_ERR = true;
       errorElement = "<FieldLengthError>true</FieldLengthError>";
     }
     if (compLengthErr) {
       COMP_LEN_ERR = true;
       errorElement = "<ComponentLengthError>true</ComponentLengthError>";
     }
     if (subCompLengthErr) {
       SUBCOMP_LEN_ERR = true;
       errorElement = "<SubComponentLengthError>true</SubComponentLengthError>";
     }
     if (tableValueErr) {
    	 TABLEVALUE_ERR = true;
    	 errorElement = "<errorCode>true</errorCode>";
     }

     // even though it's not an error, only 1 of these or an error should be set
     String card, inc = "";
     if (!"".equals(card = options.getOption("cardinality"))) { 
       errorElement = "<cardinality>"+card+"</cardinality>"; 
       ELEMENT_CARDINALITY = true;
     }
     if (!"".equals(inc = options.getOption("REinclude"))) {
       errorElement = "<REInclude>"+inc+"</REInclude>"; 
       ELEMENT_REUSAGE = true;
     }
 
     setMessageDescription(options, errLocations);
  }

   private void setMessageDescription (MessageOptions options, 
         Properties errLocations) { 
     String cardErrorMinSeg = errLocations.getProperty("cardErrorMinSeg");
     String cardErrorMinField = errLocations.getProperty("cardErrorMinField"); 
     String cardErrorMaxSeg = errLocations.getProperty("cardErrorMaxSeg");
     String cardErrorMaxField = errLocations.getProperty("cardErrorMaxField"); 
     String rUErrorSegment = errLocations.getProperty("rUSegment");
     String rUErrorField = errLocations.getProperty("rUField");
     String rUErrorComponent = errLocations.getProperty("rUComponent");
     String rUErrorSubcomp = errLocations.getProperty("rUSubcomp");
     String xUErrorField = errLocations.getProperty("xUField");
     String xUErrorSegment = errLocations.getProperty("xUSegment");
     String xUErrorComponent = errLocations.getProperty("xUComponent");
     String xUErrorSubcomp = errLocations.getProperty("xUSubcomp");
     String fieldLengthError = errLocations.getProperty("fieldLengthError");
     String compLengthError = errLocations.getProperty("compLengthError");
     String subLengthError = errLocations.getProperty("subLengthError");
     
     String segmentName = options.getOption("segmentName");
     String fieldName = options.getOption("fieldName");
     String componentName = options.getOption("componentName");
     String subComponentName = options.getOption("subComponentName");
     boolean valid = "true".equalsIgnoreCase(options.getOption("valid"));

     /*
     Map segmentNames = profileProperties.getSegmentNames();
     Map fieldNames = profileProperties.getFieldNames();
     Map compNames = profileProperties.getComponentNames();
     Map subcompNames = profileProperties.getSubcomponentNames();
     */
     
     String newline = Constants.LS;
      
      elementName = getElementName(segmentName, 
             fieldName, 
             componentName, 
             subComponentName);
      
      description = "Unless noted otherwise below: " +newline;
      elementDescription = "";
      
      if (MIN_SC) {
        description += "all Segments were generated using the Segment's ";
        description += "minimum cardinality," +newline;
      }
      if (MAX_SC) {
        description += "all Segments were generated using the Segment's ";
        description += "maximum cardinality," +newline;
      }
      if (RANDOM_SC) {
        description += "all Segments were generated using a random, but valid, ";
        description += "value for the Segment's cardinality," +newline;
      }
      if (MIN_FC) {
        description += "all Fields were generated using the Field's ";
        description += "minimum cardinality," +newline;
      }
      if (MAX_FC) {
        description += "all Fields were generated using the Field's ";
        description += "maximum cardinality," +newline;
      }
      if (RANDOM_FC) {
        description += "all Fields were generated using a random, but valid, ";
        description += "value for the Field's cardinality," +newline;
      }
      if (MessageDescription.ALL_RU) {
        description += "all of the message's Required elements are included," +newline;
      }
      if (MessageDescription.NONE_XU) {
        description += "no message elements with Usage set to 'X' are included," +newline;
      }
      if (NONE_REU) {
        description += "and no message elements with Usage set to 'RE' are included." +newline;
      }
      if (ALL_REU) {
        description += "and all message elements with Usage set to 'RE' are included." +newline;
      }
      if (SOME_REU) {
        description += "and some message elements with Usage set to 'RE' are included. " +newline;
      }
      if (ELEMENT_CARDINALITY) {
        elementDescription += "Element cardinality set "; 
        elementDescription += "(reset, if not between MIN and MAX).";
      }
      if (ELEMENT_REUSAGE) {
        elementDescription += "RE element specified.";
      }
      description += newline;
      
      boolean invalid = false;
      
      if (valid) 
        description += "The message contains no errors." + newline;
      else {
        if (MIN_SC_ERR) {
          elementDescription += "Minimum Segment Cardinality Error.";
          if (!"0".equals(cardErrorMinSeg)) {
            //elementName = profileProperties.getSegmentName(cardErrorMinSeg);
          }
          invalid = true;
        }
        if (MAX_SC_ERR) {
          elementDescription += "Maximum Segment Cardinality Error.";
          if (!"0".equals(cardErrorMinSeg)) {
            //elementName = profileProperties.getSegmentName(cardErrorMaxSeg);
          }
          invalid = true;
        }
        if (MIN_FC_ERR) {
          elementDescription += "Minimum Field Cardinality Error.";
          if (!"0".equals(cardErrorMinSeg)) {
            //elementName = profileProperties.getFieldName(cardErrorMinField);
          }
          invalid = true;
        }
        if (MAX_FC_ERR) {
          elementDescription += "Maximum Field Cardinality Error.";
          if (!"0".equals(cardErrorMinSeg)) {
            //elementName = profileProperties.getFieldName(cardErrorMaxField);
          }
          invalid = true;
        }
        if (RUSAGE_ERR) {
          if (!"0".equals(rUErrorSegment)) {
            elementDescription += "Required Segment not present.";
            //elementName = profileProperties.getSegmentName(rUErrorSegment);
          }
          else if (!"0".equals(rUErrorField)) {
            elementDescription += "Required Field not present.";
            //elementName = profileProperties.getFieldName(rUErrorField);
          }
          else if (!"0".equals(rUErrorComponent)) {
            elementDescription += "Required Component not present.";
            //elementName = profileProperties.getComponentName(rUErrorComponent);
          }
          else if (!"0".equals(rUErrorSubcomp)) {
            elementDescription += "Required Subcomponent not present.";
            //elementName = profileProperties.getSubcomponentName(rUErrorSubcomp);
          }
          else if (!"".equals(elementName)) {
            elementDescription += "Required element not present.";
          }
          else {
            MMLog.error("Usage error set but no element identified!");
          }
          invalid = true;
        }
        if (XUSAGE_ERR) {
          if (!"0".equals(xUErrorSegment)) {
            elementDescription += "Unsupported Segment included.";
            //elementName = profileProperties.getSegmentName(xUErrorSegment);
          }
          else if (!"0".equals(xUErrorField)) {
            elementDescription += "Unsupported Field inlcuded.";
            //elementName = profileProperties.getFieldName(xUErrorField);
          }
          else if (!"0".equals(xUErrorComponent)) {
            elementDescription += "Unsupported Component included.";
            //elementName = profileProperties.getComponentName(xUErrorComponent);
          }
          else if (!"0".equals(xUErrorSubcomp)) {
            elementDescription += "Unsupported Subcomponent included.";
            //elementName = profileProperties.getSubcomponentName(xUErrorSubcomp);
          }
          else if (!"".equals(elementName)) {
            elementDescription += "Unsupported element included.";
          }
          else {
            MMLog.error("a Usage error set but no element identified!");
          }
          invalid = true;
        }
        if (FIELD_LEN_ERR) {
          elementDescription += "Field (may contain) a length error.";
          if (!"0".equals(fieldLengthError)) {
            //elementName = profileProperties.getFieldName(fieldLengthError);
          }
          invalid = true;
        }
        if (COMP_LEN_ERR) {
          elementDescription += "Component (may contain) a length error.";
          if (!"0".equals(compLengthError)) {
            //elementName = profileProperties.getComponentName(compLengthError);
          }
          invalid = true;
        }
        if (SUBCOMP_LEN_ERR) {
          elementDescription += "SubComponent (may contain) a length error.";
          if (!"0".equals(subLengthError)) {
            //elementName = profileProperties.getSubcomponentName(subLengthError);
          }
          invalid = true;
        }
        if (TABLEVALUE_ERR) {
         	elementDescription += "Element contains an invalid table code.";
         	invalid = true;
         }
        if (!invalid) {
          elementDescription += " Invalid option set, but message contains no errors!"
            +newline;
        }
      }
  }
   
  public String getMessageDescription () {
    return description;
  }
   
  public String getElementDescription () {
    return elementDescription;
  }
  
  public String getElementName () {
    return elementName;
  }
  
  /*
  public String getErrorType () {
    return errorType;
  }
*/
  public void endOfSession () {
    metaDataFile.println("</" +SESSION_TAG +">" +Constants.LS);
    metaDataFile.close();
  }
  
  public void closeMetaFile () {
    metaDataFile.close();
  }

  public void setMetaFile (String outputMsgDir, String metafile) {
    closeMetaFile();
    metaFileName = metafile;
    metaDataFile = getPrintWriter(outputMsgDir + metaFileName);
  }

  public void resetMetaFile () {
    if (metaDataFile != null) metaDataFile.close();
    if (metaFileName != null) new File(outputMsgDir + metaFileName).delete();
  }

  private String getElementName (String segName,
       String fieldName,
       String compName,
       String subCompName) {
   String name = "";
   
   String newline = Constants.LS;
   
   if (!"".equals(subCompName)) {
     name = "   <SEGMENT>" +segName +"</SEGMENT>" +newline
           +"   <FIELD>" +fieldName +"</FIELD>" +newline
           +"   <COMPONENT>" +compName +"</COMPONENT>"  +newline
           +"   <SUBCOMPONENT>" +subCompName +"</SUBCOMPONENT>";
   }
   else if (!"".equals(compName)) {
     name = "   <SEGMENT>" +segName +"</SEGMENT>" +newline
     +"   <FIELD>" +fieldName +"</FIELD>" +newline
     +"   <COMPONENT>" +compName +"</COMPONENT>";
   }
   else if (!"".equals(fieldName)) {
     name = "   <SEGMENT>" +segName +"</SEGMENT>" +newline
     +"   <FIELD>" +fieldName +"</FIELD>";
   }
   else if (!"".equals(segName)) {
     name = "   <SEGMENT>" +segName +"</SEGMENT>";
   }
   
   return name;
 }
  
  private PrintWriter getPrintWriter (String fileName) {
    String mName = "getPrintWriter";
    
    FileOutputStream fOutStream = null;
    PrintWriter pwriter = null;

    try {
      fOutStream = new FileOutputStream(fileName);
      pwriter = new PrintWriter(fOutStream, true);
    }
    catch (IOException e) {
      MMLog.error(cName, mName, "Exception: " +e);
    }
    catch (Exception e) {
      MMLog.error(cName, mName, "Exception: " +e);
    }    
    
    return pwriter;
  }
  
  private void markStartOfSession () {
    metaDataFile.println(    
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +Constants.LS);
    metaDataFile.println("<" +SESSION_TAG +">" +Constants.LS);
  }
  
  private void initProperties () {
    MIN_SC = false;
    MAX_SC = false;
    RANDOM_SC = false;
    MIN_FC = false;
    MAX_FC = false;
    RANDOM_FC = false;
    NONE_REU = false;
    ALL_REU = false;
    SOME_REU = false;
    MIN_SC_ERR = false;
    MAX_SC_ERR = false;
    MIN_FC_ERR = false;
    MAX_FC_ERR = false;
    RUSAGE_ERR = false;
    XUSAGE_ERR = false;
    FIELD_LEN_ERR = false;
    COMP_LEN_ERR = false;
    SUBCOMP_LEN_ERR = false;
    ELEMENT_CARDINALITY = false;
    ELEMENT_REUSAGE = false;
    TABLEVALUE_ERR = false;
  }
}
