package gov.nist.hl7maker.message;

import java.util.Enumeration;
import java.util.Properties;

import gov.nist.hl7maker.util.ErrorCodes;
import gov.nist.hl7maker.util.MMLog;

public class Options extends Properties
{
  // 'properties' must include all required message generation 
  //    options 
  Options (Properties properties) 
  throws IllegalArgumentException {
/*System.out.println("---Options-------");      
properties.list(System.out);
System.out.println("---END Options-------");*/
    for (Enumeration e = properties.propertyNames(); e.hasMoreElements(); ) {
      String key = (String)e.nextElement();
      setProperty(key, properties.getProperty(key));
    }    
    
    /* Variant only options, set to empty if not present */
    if (properties.getProperty("segmentName") == null)
      setProperty("segmentName", "");
    if(properties.getProperty("fieldName") == null)
      setProperty("fieldName", "");
    if(properties.getProperty("componentName") == null)
      setProperty("componentName", ""); 
    if(properties.getProperty("subComponentName") == null)
      setProperty("subComponentName", "");
    if(properties.getProperty("cardinality") == null)
      setProperty("cardinality", "");
    if(properties.getProperty("REinclude") == null)
      setProperty("REinclude", "");
    if(properties.getProperty("elementErrorCode") == null)
    	setProperty("elementErrorCode", "");

    /* required options */
    if (getProperty("messageControlId") == null ||
        getProperty("offset") == null ||
        getProperty("starValue") == null ||
        getProperty("segmentCardinality") == null ||
        getProperty("fieldCardinality") == null ||
        getProperty("RE_Usage") == null ||
        getProperty("lengthConstraints") == null ||
        getProperty("compositeLength") == null ||
        getProperty("minSegCardError") == null ||
        getProperty("maxSegCardError") == null ||
        getProperty("minFieldCardError") == null ||
        getProperty("maxFieldCardError") == null ||
        getProperty("RusageError") == null ||
        getProperty("XusageError") == null ||
        getProperty("fieldLengthError") == null ||
        getProperty("componentLengthError") == null ||
        getProperty("subComponentLengthError") == null ||
        getProperty("buildErrorCodes") == null ||
        getProperty("hl7ErrorCodes") == null ||
        getProperty("userErrorCodes") == null ||
        getProperty("localErrorCodes") == null ||
        getProperty("externalErrorCodes") == null) {
      MMLog.error("Options not set: required option missing!");
      throw(new IllegalArgumentException("required option missing!"));
    }
  }

  public void setOption (String name, String value)
  throws IllegalArgumentException {
    if (supportedOption(name)) {
    	setProperty(name, value);
    }
    else
      throw(new IllegalArgumentException("setOption. unsupported option: " +name));
  }

  public String getOption (String name)
  throws IllegalArgumentException {
    if (supportedOption(name)) return getProperty(name);
    else
      throw(new IllegalArgumentException("getOption. unsupported option: " +name));
  }
  
  private boolean supportedOption (String name) {
      if ("segmentName".equalsIgnoreCase(name)) return true;
      else if ("fieldName".equalsIgnoreCase(name)) return true;
      else if ("componentName".equalsIgnoreCase(name)) return true;
      else if ("subComponentName".equalsIgnoreCase(name)) return true;
      else if ("cardinality".equalsIgnoreCase(name)) return true;
      else if ("REinclude".equalsIgnoreCase(name)) return true;
      else if ("starValue".equalsIgnoreCase(name)) return true;
      else if ("messageControlId".equalsIgnoreCase(name)) return true;
      else if ("offset".equalsIgnoreCase(name)) return true;
      else if ("segmentCardinality".equalsIgnoreCase(name)) return true;
      else if ("fieldCardinality".equalsIgnoreCase(name)) return true;
      else if ("RE_Usage".equalsIgnoreCase(name)) return true;
      else if ("lengthConstraints".equalsIgnoreCase(name)) return true;
      else if ("compositeLength".equalsIgnoreCase(name)) return true;
      else if ("minSegCardError".equalsIgnoreCase(name)) return true;
      else if ("maxSegCardError".equalsIgnoreCase(name)) return true;
      else if ("minFieldCardError".equalsIgnoreCase(name)) return true;
      else if ("maxFieldCardError".equalsIgnoreCase(name)) return true;
      else if ("RusageError".equalsIgnoreCase(name)) return true;
      else if ("XusageError".equalsIgnoreCase(name)) return true;
      else if ("fieldLengthError".equalsIgnoreCase(name)) return true;
      else if ("componentLengthError".equalsIgnoreCase(name)) return true;
      else if ("subComponentLengthError".equalsIgnoreCase(name)) return true;
      else if ("elementErrorCode".equalsIgnoreCase(name)) return true;
      else if ("buildErrorCodes".equalsIgnoreCase(name)) return true;
      else if ("hl7ErrorCodes".equalsIgnoreCase(name)) return true;
      else if ("userErrorCodes".equalsIgnoreCase(name)) return true;
      else if ("localErrorCodes".equalsIgnoreCase(name)) return true;
      else if ("externalErrorCodes".equalsIgnoreCase(name)) return true;
      
      return false;
  }
}
