package gov.nist.hl7maker.message;

import java.io.*;
import java.util.*;
import java.io.StringWriter;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import gov.nist.hl7maker.util.MMLog;


public class XMLOptionsReader
{
  private static final String cName = "MessageProperties";
  
  private int messageCount = 0;
  private Properties options = null;
  private ArrayList variants = new ArrayList();
  
  /**
    The name of the XML options file along with the transform used to
    read it are required for creating new instances of this class
    */
  public XMLOptionsReader (String xslTransform, String msgOptions)
        throws TransformerFactoryConfigurationError, 
        TransformerException,
        IOException {
    String mName = "XMLOptionsReader";
    
    if (!new File(xslTransform).exists()) {
      MMLog.error(cName, mName, "transform: " +xslTransform +" does not exist");
      throw new IOException("File not found: " +xslTransform);
    }
    if (!new File(msgOptions).exists()) {
      MMLog.error(cName, mName, "profile: " +msgOptions +" does not exist");
      throw new IOException("File not found: " +msgOptions);
    }
    
/*    InputStream xsltInStream = null; 
    InputStream optionsInStream = null;
    
    if ((xsltInStream = 
        this.getClass().getResourceAsStream(xslTransform)) == null) {
      MMLog.error(cName, mName, "transform: " +xslTransform +" does not exist");
      throw new IOException("File not found: " +xslTransform);
    }
    if ((optionsInStream = 
        this.getClass().getResourceAsStream(msgOptions)) == null) {
      MMLog.error(cName, mName, "profile: " +msgOptions +" does not exist");
      throw new IOException("File not found: " +msgOptions);
    }
*/    
    //  create a StreamSource for both files
    StreamSource xsltStream = new StreamSource(new File(xslTransform));
    StreamSource propertiesStream = new StreamSource(new File(msgOptions));
    //StreamSource xsltStream = new StreamSource(xsltInStream);
    //StreamSource propertiesStream = new StreamSource(optionsInStream);

    TransformerFactory tfactory = TransformerFactory.newInstance();
    Transformer transformer = tfactory.newTransformer(xsltStream);
    StringWriter writer = new StringWriter();
    StreamResult result = new  StreamResult(writer);
    
    transformer.transform(propertiesStream, result);
    setOptions(writer);      
  }
  
  /*
   *    The output from applying the transformation looks like:
   * 
   *    propertyName value
   *        ...
   *    "%endOfProperties%"
   *    propertyName value
   *        ...
   *    "%endOfMessage%"
   *    propertyName value
   *        ...
   *    "%endOfMessage%"
   *        ...
   */
  private void setOptions (StringWriter xsltOutput) {
    String mName = "processXsltOutput";
    
    String output = xsltOutput.toString();
    
    Properties options = new Properties();
    StringReader sreader = new StringReader(output);
    BufferedReader reader = new BufferedReader(sreader);

    String line = "";
    boolean startOfMessages = false;
    boolean endOfProperties = false;
    Properties message = null;
    for ( ;; ) {
        try {
        	line = reader.readLine();
        }
        catch (IOException e) {
        	MMLog.error(cName, mName, "Exception: " +e);   
        }
        catch (Exception e) {
            MMLog.error(cName, mName, "Exception: " +e);   
        }
        
        if (line == null) break;
        // once the messages descriptions start:
        if (startOfMessages) {
          int msgCount = 1;   // default
          if (!"%EofMessage%".equals(line)) {
            int index = line.indexOf(' ');
            //String[] nvPair = line.split(" ");
            //System.out.println("name = " +nvPair[0]);
            //System.out.println("value = " +nvPair[1]);
            String name = line.substring(0, index);
            String value = line.substring(index+1);
            //System.out.println("name = " +name);
            //System.out.println("value = " +value);
            
            message.setProperty(name, value);
            
            if ("msgCount".equalsIgnoreCase(name)) 
              msgCount = Integer.parseInt(value);
          }
          else {
            messageCount += msgCount;
            variants.add(message);
            message = new Properties();
          }
        }

        
        // once we see end of prop, mark endofprop & startofmsgs 
        //  (variants)
        if ("%EofProperties%".equals(line)) {
          endOfProperties = true;
          startOfMessages = true;
          message = new Properties();   // first variant msg
        }
        
        // gather global properties till we see end of Properties
        if (!endOfProperties) {
          int index = line.indexOf(' ');
          //String[] nvPair = line.split(" ");
          //System.out.println("name = " +nvPair[0]);
          //System.out.println("value = " +nvPair[1]);
          String name = line.substring(0, index);
          String value = line.substring(index+1);
          //System.out.println("name = " +name);
          //System.out.println("value = " +value);
          
          options.setProperty(name, value);
          
          if ("msgCount".equalsIgnoreCase(name)) 
            messageCount = Integer.parseInt(value);
        }
    }
    
    this.options = new Properties(options);
  }
  
  public ArrayList getMessageVariants() {
    return variants;
  }
  
  public Properties getOptions () {
  	return options;
  }
  
  public int getMessageCount () {
    return messageCount;
  }
}
