package gov.nist.hl7maker.util;

/**
 * This class checks if mandatory resources exists or not
 */

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.view.config.TablePriority;

public class CheckResources {

    /**
     * Check the resources at startup
     * @return A string with the error message in case of missing files
     */
    public static String checkStartup() {
	String error = "";
	File f = null;
	String[] files = {"config/user.dat", "config/maker.dat", "data/ReferenceData.xml", "data/ConfigData.xml", "data/hl7tables.xml", "data/usertables.xml", "data/externaltables.xml", "data/HL7RegistrationSchema.xsd", "data/LooseHL7RegistrationSchema.xsd", "data/dataElements.xml", "data/DefaultValue.xml", "data/emptyLocal.xml", "data/BatchSegments.xml"};
	for(int i=0;i<files.length;i++) {
	    f = new File(files[i]);
	    if(!f.exists()) {
		try {
		    error += ("The file " + f.getCanonicalPath() + " does not exist\n");
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    System.exit(0);
		}
	    }
	}
	return error;
    }

    /**
     * Check the resources linked to a session
     * @param session The session
     * @return A string with the error message in case of missing files
     */
    public static String checkSession(Session session) {
	String error = "";
	File f = null;
	Vector vFiles = new Vector();
	vFiles.add(session.getProfile());
	vFiles.add(session.getRepositoryData());
	vFiles.add(session.getConfig());
	vFiles.add(session.getHL7Tables());
	vFiles.add(session.getUserTables());
	vFiles.add(session.getExternalTables());
	// Local Table
	Vector vLocalTable = session.getLocalTable();
	for(int i=0;i<vLocalTable.size();i++) {
	    TablePriority tp = (TablePriority)vLocalTable.get(i);
	    vFiles.add(tp.getFilename());
	}
	for(int i=0;i<vFiles.size();i++) {
	    f = new File((String)vFiles.get(i));
	    if(!f.exists()) {
		try {
		    error += ("The file " + f.getCanonicalPath() + " does not exist");
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    System.exit(0);
		}
	    }
	}
	return error;
    }
}

