/**
 * This class is for checking if a file is a MessageMaker file or not
 */

package gov.nist.hl7maker.util;

// JDK
import java.io.File;
import java.io.IOException;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

// SAX
import org.xml.sax.SAXException;  

public class MagicNumber {

    /**
     * Check if the file has the magic number
     * @param file The filename
     */
    public static boolean check(File file) {
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc = builder.parse(file);
	    
	    removeEmptyTextElement(doc);
	    // Magic Number
	    Element root = doc.getDocumentElement();
	    NodeList nl = root.getChildNodes();
	    for(int i=0;i<nl.getLength();i++) {
		Element e = (Element)nl.item(i);
		if(e.getNodeName().compareTo("MessageMaker") == 0) {
		    return true;
		}
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(Exception e) {
	    return false;
	}
	return false;
    }

    /**
     * Check if the file has the magic number
     *@param filename The filename
     */
    public static boolean check(String filename) {
	return check(new File(filename));
    }

    /**
     * This method removes all TEXT_NODE with empty text
     * @param node The current node
     */    
    private static void removeEmptyTextElement(Node node) {
	if(node.hasChildNodes()) {
	    NodeList nl = node.getChildNodes();
	    for(int i=0;i<nl.getLength();i++) {
		Node n = nl.item(i);
		if(n.getNodeValue() != null && n.getNodeValue().trim().equals(""))
		    node.removeChild(n);
	    }
	}
	// Execute the method on the children
	if(node.hasChildNodes()) {
	    NodeList nl = node.getChildNodes();
	    for(int i=0;i<nl.getLength();i++) {
		removeEmptyTextElement(nl.item(i));
	    }
	}
    }
}
