package gov.nist.hl7maker.util.tableconverter;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

/**
 * Supports the configuration of an <code>XSLTransformer</code> to convert
 * generic HL7 tables into Message Maker format.
 * 
 * @author Justin Manweiler
 */
public class TableConverter {

    // --- Private Fields ---

    private static final String xsltFilePath = "xslt/tableconverter/tableconversion.xsl";
    private static final String tableKey = "table";
    private static final String itemKey = "item";
    private static final String typeKey = "type";
    private static final String nameKey = "name";
    private static final String idKey = "id";
    private static final String codeKey = "code";
    private static final String valueKey = "value";

    private final XSLTransformer transformer = new XSLTransformer();

    // --- Constructor ---

    public TableConverter() throws IOException, FileNotFoundException,
            TransformerConfigurationException {
	File f = new File(new File(xsltFilePath).getCanonicalPath()); 
	transformer.setXSLT(new FileInputStream(f));
    }

    // --- Public Methods ---

    public void convert( final File inFile, final File outFile )
            throws FileNotFoundException, TransformerException {
        transformer.transform( inFile, outFile );
    }

    public void setTableMapping( final String mapping ) {
        transformer.setParam( tableKey, mapping );
    }

    public void setItemMapping( final String mapping ) {
        transformer.setParam( itemKey, mapping );
    }

    public void setTypeMapping( final String mapping ) {
        transformer.setParam( typeKey, mapping );
    }

    public void setNameMapping( final String mapping ) {
        transformer.setParam( nameKey, mapping );
    }

    public void setIDMapping( final String mapping ) {
        transformer.setParam( idKey, mapping );
    }

    public void setCodeMapping( final String mapping ) {
        transformer.setParam( codeKey, mapping );
    }

    public void setValueMapping( final String mapping ) {
        transformer.setParam( valueKey, mapping );
    }
}
