package gov.nist.hl7maker.util.tableconverter;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Performs XSL transformations based on an XSLT file and parameter values.
 * 
 * @author Justin Manweiler
 */
public class XSLTransformer {

    // --- Private Fields ---

    private static final TransformerFactory factory = TransformerFactory
            .newInstance();

    private Transformer transformer;

    // --- Public Methods ---

    public void setXSLT( final InputStream inStream )
            throws TransformerConfigurationException {
        final StreamSource sSource = new StreamSource( inStream );
        final Templates template = factory.newTemplates( sSource );

        try {
            transformer = template.newTransformer();
        }
        finally {
            try {
                inStream.close();
            }
            catch ( final IOException e ) {
            }
        }
    }

    public void setParam( final String param, final Object value ) {
        transformer.setParameter( param, value );
    }

    public void transform( final File inFile, final File outFile )
            throws FileNotFoundException, TransformerException {
        final FileInputStream sourceFIS = new FileInputStream( inFile );
        final Source source = new StreamSource( sourceFIS );

        final FileOutputStream resultFOS = new FileOutputStream( outFile );
        final Result result = new StreamResult( resultFOS );

        try {
            transformer.transform( source, result );
        }
        finally {
            try {
                sourceFIS.close();
            }
            catch ( final IOException e ) {
            }

            try {
                resultFOS.close();
            }
            catch ( final IOException e ) {
            }
        }
    }
}
