package gov.nist.hl7maker.verifier;

/**
 * Exception thrown on a verification failure (condition where verification
 * cannot continue or a fault has been detected). It may optionally include an
 * array of additional information describing the conditions of the error as
 * well as an array of information describing faults.
 * 
 * @author Justin Manweiler
 */
public class VerifierException extends Exception {

    // --- Private Fields ---

    private final String[] details;
    private final String[] faults;

    // --- Constructors ---

    public VerifierException( final String msg ) {
        this( msg, null, null );
    }

    public VerifierException( final String msg, final String[] faults ) {
        this( msg, null, faults );
    }

    public VerifierException( final String msg, final String[] details,
            final String[] faults ) {
        super( faults == null ? msg : getExtendedMessage( msg, faults.length ) );
        this.details = details;
        this.faults = faults;
    }

    // --- Public Methods ---

    public String[] getDetails() {
        return details;
    }

    public String[] getFaults() {
        return faults;
    }

    // --- Private Methods ---

    private static String getExtendedMessage( final String origMessage,
            final int numSubErrors ) {
        String extendedMsg = origMessage;

        extendedMsg += " (" + numSubErrors + " fault"
                + (numSubErrors == 1 ? "" : "s") + ")";

        return extendedMsg;
    }
}
