package gov.nist.hl7maker.verifier.hl7;

import gov.nist.hl7maker.verifier.VerifierException;
import gov.nist.hl7maker.verifier.util.BadXPathException;
import gov.nist.hl7maker.verifier.xmlverifier.XMLVerifier;

import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * Allows for the verification of HL7 table values.
 * 
 * @author Justin Manweiler
 */
public class TableLookup {

    // --- Private Fields ---

    private final ArrayList tvArrList = new ArrayList();

    // --- Public Methods ---

    public void addTable( final XMLVerifier tVerifier ) {
        tvArrList.add( tVerifier );
    }

    public boolean isInTable( final String tableID, final String value )
            throws VerifierException {
        final Iterator iter = tvArrList.iterator();

        while ( iter.hasNext() ) {
            final XMLVerifier tVerifier = (XMLVerifier)iter.next();

            if ( isInTable( tVerifier, tableID, value ) ) {
                return true;
            }
        }

        return false;
    }

    // --- Private Methods ---

    private static boolean isInTable( final XMLVerifier tVerifier,
            final String tableID, final String value ) throws VerifierException {
        final Document msgDoc = tVerifier.getDOM();
        final String xPath = "/*/Table[@ID='" + tableID + "']/Item[@Code='"
                + value + "']";

        try {
            final NodeList tMatchList = XPathAPI.selectNodeList( msgDoc, xPath );
            return tMatchList.getLength() > 0;
        }
        catch ( final TransformerException e ) {
            throw new BadXPathException( xPath, e.getMessage() );
        }
    }
}
