package gov.nist.hl7maker.verifier.message;

import gov.nist.hl7maker.verifier.util.ErrorSet;

/**
 * Stores and describes errors associated with expected or missing implanted
 * message errors and ignored specified settings.
 * 
 * @author Justin Manweiler
 */
public class ImplantedErrorSet extends ErrorSet {

    // --- Public Fields ---

    public static final int ME_RusageError = 10;
    public static final int ME_XusageError = 20;
    public static final int ME_MinSegCardError = 30;
    public static final int ME_MaxSegCardError = 40;
    public static final int ME_MinFieldCardError = 50;
    public static final int ME_MaxFieldCardError = 60;
    public static final int ME_FieldLengthError = 70;
    public static final int ME_ComponentLengthError = 80;
    public static final int ME_SubComponentLengthError = 90;
    public static final int ME_ValueNotInTable = 100;
    public static final int UE_RusageError = 11;
    public static final int UE_XusageError = 21;
    public static final int UE_MinSegCardError = 31;
    public static final int UE_MaxSegCardError = 41;
    public static final int UE_MinFieldCardError = 51;
    public static final int UE_MaxFieldCardError = 61;
    public static final int UE_FieldLengthError = 71;
    public static final int UE_ComponentLengthError = 81;
    public static final int UE_SubComponentLengthError = 91;
    public static final int UE_ValueNotInTable = 101;
    public static final int BS_REincludeTrue = 1000;
    public static final int BS_REincludeFalse = 1001;
    public static final int BS_Cardinality = 1002;

    // --- Public Methods ---

    public String getMessage( final int errType ) {
        if ( errType >= 1000 ) {
            return "Specified property not followed: "
                    + getSettingType( errType );
        }

        final boolean isMissing = errType % 2 == 0;

        return (isMissing ? "Missing error: " : "Unexpected error: ")
                + getType( errType );
    }

    // --- Private Methods ---

    private static String getType( final int type ) {
        final int baseType = (type / 10) * 10;

        switch ( baseType ) {
            case ME_RusageError:
                return "RusageError";
            case ME_XusageError:
                return "XusageError";
            case ME_MinSegCardError:
                return "MinSegCardError";
            case ME_MaxSegCardError:
                return "MaxSegCardError";
            case ME_MinFieldCardError:
                return "MinFieldCardError";
            case ME_MaxFieldCardError:
                return "MaxFieldCardError";
            case ME_FieldLengthError:
                return "FieldLengthError";
            case ME_ComponentLengthError:
                return "ComponentLengthError";
            case ME_SubComponentLengthError:
                return "SubComponentLengthError";
            case ME_ValueNotInTable:
                return "value not in table";
            default:
                return "UNKOWN TYPE";
        }
    }

    private static String getSettingType( final int type ) {
        switch ( type ) {
            case BS_REincludeTrue:
                return "REinclude TRUE";
            case BS_REincludeFalse:
                return "REinclude FALSE";
            case BS_Cardinality:
                return "Cardinality";
            default:
                return "UNKOWN TYPE";
        }
    }
}
