package gov.nist.hl7maker.verifier.message;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Stores the cardinality for all unique message elements in a message file.
 * "Uniqueness" is determined by XPath location.
 * 
 * @author Justin Manweiler
 */
public class MessageInspector {

    // --- Private Classes ---

    private final Document msgDOM;

    private Map pathMap = null;

    // --- Constructor ---

    public MessageInspector( final Document msgDoc ) {
        msgDOM = msgDoc;
    }

    // --- Public Methods ---

    public int getCardinality( final String msgXPath ) {
        final MessageElement me = (MessageElement)getPathMap().get( msgXPath );
        return me == null ? 0 : me.getCount();
    }
    
    public String[] getDataValues( final String msgXPath ) {
        final MessageElement me = (MessageElement)getPathMap().get( msgXPath );
        return me == null ? null : me.getDataValues();
    }

    // --- Private Methods ---

    private Map getPathMap() {
        if ( pathMap == null ) {
            pathMap = new HashMap();
            storeElements( msgDOM, "" );
        }

        return pathMap;
    }

    private void storeElements( final Node node, final String path ) {
        final NodeList childList = node.getChildNodes();
        final int childCnt = childList.getLength();

        for ( int i = 0; i < childCnt; i++ ) {
            final Node child = childList.item( i );
            final short childType = child.getNodeType();

            if ( childType == Node.ELEMENT_NODE ) {
                final String childPath = path + "/" + child.getNodeName();
                final MessageElement me = (MessageElement)pathMap
                        .get( childPath );

                if ( me == null ) {
                    pathMap.put( childPath, new MessageElement() );
                }
                else {
                    me.increment();
                }

                storeElements( child, childPath );
            }
            else if ( childType == Node.TEXT_NODE ) {
                final String dValue = child.getNodeValue().trim();

                if ( dValue.length() > 0 ) {
                    final MessageElement me = (MessageElement)pathMap
                            .get( path );

                    if ( me != null ) {
                        me.addDataValue( dValue );
                    }
                }
            }
        }
    }
}
