package gov.nist.hl7maker.verifier.message;

import gov.nist.hl7maker.verifier.util.ErrorSet;

/**
 * Stores errors associated with message-to-profile verification.
 * 
 * @author Justin Manweiler
 */
public class ProfileErrorSet extends ErrorSet {

    // --- Public Fields ---

    public static final int ET_RusageError = 1;
    public static final int ET_XusageError = 2;
    public static final int ET_MinSegCardError = 3;
    public static final int ET_MaxSegCardError = 4;
    public static final int ET_MinFieldCardError = 5;
    public static final int ET_MaxFieldCardError = 6;
    public static final int VT_REElementNotFound = 10;
    public static final int VT_REElementFound = 20;
    public static final int VT_SegCardNEMin = 30;
    public static final int VT_SegCardNEMax = 40;
    public static final int VT_FieldCardNEMin = 50;
    public static final int VT_FieldCardNEMax = 60;

    // --- Protected Methods ---

    protected String getMessage( final int errType ) {
        return (errType < 10 ? "HL7 error: " : "Metadata violation: ")
                + getType( errType );
    }

    // --- Private Methods ---

    private static String getType( final int errType ) {
        switch ( errType ) {
            case ET_RusageError:
                return "RusageError";
            case ET_XusageError:
                return "XusageError";
            case ET_MinSegCardError:
                return "MinSegCardError";
            case ET_MaxSegCardError:
                return "MaxSegCardError";
            case ET_MinFieldCardError:
                return "MinFieldCardError";
            case ET_MaxFieldCardError:
                return "MaxFieldCardError";
            case VT_REElementNotFound:
                return "RE element not found";
            case VT_REElementFound:
                return "RE element found";
            case VT_SegCardNEMin:
                return "segment cardinality not equal to minimum";
            case VT_SegCardNEMax:
                return "segment cardinaltiy not equal to maximum";
            case VT_FieldCardNEMin:
                return "field cardinality not equal to minimum";
            case VT_FieldCardNEMax:
                return "field cardinaltiy not equal to maximum";
            default:
                return "UNKOWN";
        }
    }
}
