package gov.nist.hl7maker.verifier.metadata;

import org.w3c.dom.Node;

/**
 * Parses, stores, and provides general metadata properties.
 * 
 * @author Justin Manweiler
 */
public class GeneralInfo extends SharedInfo {

    // --- Inner Classes ---

    public class Properties extends SharedInfo.Properties {
        public final boolean hasREAll = hasREALL();
        public final boolean hasRENone = hasRENone();
        public final boolean isSgmntCardMin = isSegmentCardinalityMin();
        public final boolean isSgmntCardMax = isSegmentCardinalityMax();
        public final boolean isFieldCardMin = isFieldCardinalityMin();
        public final boolean isFieldCardMax = isFieldCardinalityMax();
    }

    // --- Private Fields ---

    private static final String valueAll = "ALL";
    private static final String valueNone = "NONE";
    private static final String valueMin = "MIN";
    private static final String valueMax = "MAX";
    private static final String gp_MessageTypes = "MessageTypes";
    private static final String gp_FieldCardinality = "FieldCardinality";
    private static final String gp_SegmentCardinality = "SegmentCardinality";
    private static final String gp_REusage = "REusage";

    private static final String[] gProperties = {
            gp_MessageTypes, gp_FieldCardinality,
            gp_SegmentCardinality, gp_REusage
    };

    // --- Public Methods ---

    public Properties getProperties() {
        return new Properties();
    }

    public void parse( final Node parseRoot ) {
        super.parse( parseRoot );
        propertyParser.parse( parseRoot, gProperties );
    }

    public void parse( final Node parseRoot, final String baseXPath ) {
        super.parse( parseRoot, baseXPath );
        propertyParser.parse( parseRoot, baseXPath, gProperties );
    }

    // --- Private Methods ---
    
    private boolean hasREALL() {
        final String value = propertyParser.getValue( gp_REusage );
        return value != null && value.equals( valueAll );
    }

    private boolean hasRENone() {
        final String value = propertyParser.getValue( gp_REusage );
        return value != null && value.equals( valueNone );
    }

    private boolean isSegmentCardinalityMin() {
        final String value = propertyParser.getValue( gp_SegmentCardinality );
        return value != null && value.equals( valueMin );
    }

    private boolean isSegmentCardinalityMax() {
        final String value = propertyParser.getValue( gp_SegmentCardinality );
        return value != null && value.equals( valueMax );
    }

    private boolean isFieldCardinalityMin() {
        final String value = propertyParser.getValue( gp_FieldCardinality );
        return value != null && value.equals( valueMin );
    }

    private boolean isFieldCardinalityMax() {
        final String value = propertyParser.getValue( gp_FieldCardinality );
        return value != null && value.equals( valueMax );
    }
}
