package gov.nist.hl7maker.verifier.metadata;

import gov.nist.hl7maker.verifier.util.PropertyMap;

import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Extracts, stores, and provides information from a DOM sub-tree.
 * 
 * @author Justin Manweiler
 */
class PropertyParser {

    // --- Private Fields ---

    private final PropertyMap propertyMap = new PropertyMap();

    // --- Public Methods ---

    public boolean isTrue( final String option ) {
        final String value = propertyMap.getValue( option );
        return value != null && value.equals( "true" );
    }

    public String getValue( final String option ) {
        return propertyMap.getValue( option );
    }

    public void parse( final Node parseRoot, final String[] properties ) {
        parse( parseRoot, "", properties );
    }

    public void parse( final Node parseRoot, final String baseXPath,
            final String[] properties ) {
        for ( int i = 0; i < properties.length; i++ ) {
            try {
                final String propXPath = baseXPath + properties[i];
                final String newValue = getTagText( parseRoot, propXPath );
                propertyMap.setOption( properties[i], newValue );
            }
            catch ( final TransformerException e ) {
            }
        }
    }

    public void setDefaults( final PropertyParser defaults ) {
        propertyMap.setDefaults( defaults.propertyMap );
    }

    // --- Private Methods ---

    private static String getTagText( final Node parent,
            final String childTagName ) throws TransformerException {
        final Node node = XPathAPI.selectSingleNode( parent, childTagName );
        return node == null ? null : getNodeText( node );
    }

    private static String getNodeText( final Node node ) {
        final NodeList childNodes = node.getChildNodes();

        String result = "";

        final int numChildren = childNodes.getLength();
        for ( int j = 0; j < numChildren; j++ ) {
            if ( childNodes.item( j ).getNodeType() == Node.TEXT_NODE ) {
                result += childNodes.item( j ).getNodeValue();
            }
        }

        return result;
    }
}
