package gov.nist.hl7maker.verifier.util;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Infrastructure for storing a set of error message codes and providing an
 * associated error message.
 * 
 * @author Justin Manweiler
 */
public abstract class ErrorSet {

    // --- Private Fields ---

    private final Set errSet = new HashSet();

    // --- Public Methods ---

    public void addError( final int errType ) {
        errSet.add( new Integer( errType ) );
    }

    public String[] getErrMessages() {
        final String[] messages = new String[errSet.size()];
        final Iterator iter = errSet.iterator();

        for ( int i = 0; i < messages.length; i++ ) {
            final int typeCode = ((Integer)iter.next()).intValue();
            messages[i] = getMessage( typeCode );
        }

        return messages;
    }

    public boolean hasError() {
        return !errSet.isEmpty();
    }

    public boolean hasError( final int errType ) {
        return errSet.contains( new Integer( errType ) );
    }

    // --- Protected Methods ---

    protected abstract String getMessage( final int errType );
}
