package gov.nist.hl7maker.verifier.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.channels.FileChannel;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author Justin Manweiler
 */
public class FileUtils {

    // --- Constructor ---

    private FileUtils() {
    }

    // --- Public Methods ---

    public static void copyFile( final File source, final File dest )
            throws IOException {
        FileChannel sourceChannel = null;
        FileChannel destChannel = null;

        try {
            sourceChannel = new FileInputStream( source ).getChannel();
            destChannel = new FileOutputStream( dest ).getChannel();

            sourceChannel.transferTo( 0, sourceChannel.size(), destChannel );
        }
        finally {
            if ( sourceChannel != null ) {
                sourceChannel.close();
            }

            if ( destChannel != null ) {
                destChannel.close();
            }
        }
    }

    public static void copyFile( final InputStream source, final File dest )
            throws Exception {
        OutputStream os = null;

        try {
            os = new FileOutputStream( dest );

            final byte[] buf = new byte[1024];
            int i = 0;

            while ( (i = source.read( buf )) != -1 ) {
                os.write( buf, 0, i );
            }
        }
        finally {
            if ( source != null ) {
                try {
                    source.close();
                }
                catch ( final IOException e ) {
                }
            }

            if ( os != null ) {
                try {
                    os.close();
                }
                catch ( final IOException e ) {
                }
            }
        }
    }

    public static ZipOutputStream createZip( final File outFile )
            throws FileNotFoundException {
        final FileOutputStream fos = new FileOutputStream( outFile );
        final ZipOutputStream zipStream = new ZipOutputStream( fos );

        return zipStream;
    }

    public static void addToZip( final ZipOutputStream zipStream,
            final File toAdd, final ZipEntry addEntry ) throws IOException {
        final byte[] buf = new byte[1024];
        final FileInputStream fis = new FileInputStream( toAdd );

        zipStream.putNextEntry( addEntry );

        int len;

        while ( (len = fis.read( buf )) > 0 ) {
            zipStream.write( buf, 0, len );
        }

        fis.close();
        zipStream.closeEntry();
    }
}
