package gov.nist.hl7maker.verifier.util;

import java.util.HashMap;
import java.util.Map;

/**
 * Stores a hash of <code>String</code> property names to <code>String</code>
 * values. <code>PropertyMap</code> s can be linked together so that one
 * <code>PropertyMap</code> can provide default values to another.
 * 
 * @author Justin Manweiler
 */
public class PropertyMap {

    // --- Private Fields ---

    private final Map hashMap = new HashMap();
    private PropertyMap defaults = null;

    // --- Public Methods ---

    public String getValue( final String option ) {
        final Object val = hashMap.get( option );

        if ( val != null ) {
            return (String)val;
        }

        if ( defaults != null ) {
            return defaults.getValue( option );
        }

        return null;
    }

    public void setDefaults( final PropertyMap defaultOptions ) {
        defaults = defaultOptions;
    }

    public void setOption( final String option, final String value ) {
        if ( value != null ) {
            hashMap.put( option, value );
        }
    }
}
