package gov.nist.hl7maker.verifier.util;

/**
 * Stores the result of a test for efficient repeated retrieval (i.e. so that
 * the test itself does not have to be repeated).
 * 
 * @author Justin Manweiler
 */
public class TestResult {

    // --- Private Fields ---

    private boolean called = false;
    private boolean passed = true;
    private String failCause = null;

    // --- Public Methods ---

    public boolean isUntested() {
        return !called;
    }

    public boolean hasFailed() {
        return !passed;
    }

    public String getFailCause() {
        return failCause;
    }

    public void passed() {
        called = true;
        passed = true;
    }

    public void failure( final String cause ) {
        called = true;
        passed = false;
        failCause = cause;
    }

    public void reset() {
        called = false;
        passed = true;
    }
}