package gov.nist.hl7maker.verifier.xmlverifier;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Wrapper class for XML API validation and parsing methods.
 * 
 * @author Justin Manweiler
 */
public class XMLParser {

    // --- Private Fields ---

    private static final DefaultHandler defaultHandler;
    private static final DocumentBuilderFactory dbFactory;
    private static final SAXParserFactory saxFactory;
    private static final TransformerFactory tFactory;

    private static DocumentBuilder db = null;
    private static Transformer transformer = null;

    static {
        defaultHandler = new DefaultHandler();

        dbFactory = DocumentBuilderFactory.newInstance();

        saxFactory = SAXParserFactory.newInstance();
        saxFactory.setValidating( true );

        tFactory = TransformerFactory.newInstance();
    }

    // --- Constructor ---

    private XMLParser() {
    }

    // --- Public Methods ---

    public static boolean isValidXML( final File file ) {
        try {
            saxFactory.newSAXParser().parse( file, defaultHandler );
        }
        catch ( Exception e ) {
            return false;
        }

        return true;
    }

    public static Document parseXMLDOM( final File file )
            throws ParserConfigurationException, SAXException, IOException {
        return getDocumentBuilder().parse( file );
    }

    public static void printXML( final Document dom ) {
        try {
            final DOMSource source = new DOMSource( dom );
            final StreamResult result = new StreamResult( System.out );

            getTransformer().transform( source, result );
        }
        catch ( final Exception e ) {
        }
    }

    public static void writeXML( final Document dom, final File outFile ) {
        try {
            final DOMSource source = new DOMSource( dom );
            final StreamResult result = new StreamResult( outFile );

            getTransformer().transform( source, result );
        }
        catch ( final Exception e ) {
            e.printStackTrace();
        }
    }

    // --- Private Methods ---

    private static DocumentBuilder getDocumentBuilder()
            throws ParserConfigurationException {
        if ( db == null ) {
            db = dbFactory.newDocumentBuilder();
        }

        return db;
    }

    private static Transformer getTransformer()
            throws TransformerConfigurationException {
        if ( transformer == null ) {
            transformer = tFactory.newTransformer();
        }

        return transformer;
    }
}
