/**
 * This dialog preloads a profile for beginners
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.IOException;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.util.WaitDialog;

public class BeginnersDialog extends JDialog implements ActionListener {

    protected JCheckBox chkAskMeAgain;
    protected JButton btnYes;
    protected JButton btnNo;

    protected Session session;
    protected User user;

    protected final String DEMO_MMK = "testing" + File.separator + "Simple" + File.separator + "NIST_SampleADT.mmk";

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     * @param user A user
     */
    public BeginnersDialog(JFrame frame, Session ses, User user) {
	super(frame, "Load Sample Profile", true);
	session = ses;
	this.user = user;

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setAlignmentX(0.5f);
	try {
	    p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
	    JLabel lbl = new JLabel("Do you want to load an example profile to explore Message Maker?");
	    JLabel lbl2 = new JLabel("The profile is located in the directory " + new File(DEMO_MMK).getParentFile().getCanonicalPath());
	    p0.add(lbl);
	    p0.add(lbl2);
	}
	catch(IOException ioe) {
	    System.err.println("IOException: " + ioe.getMessage());
	}

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	chkAskMeAgain = new JCheckBox("Don't ask me again.", false);
	p1.add(chkAskMeAgain);

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	btnYes = new JButton("Yes");
        btnYes.setBackground(new Color(204, 204, 255));
	btnYes.addActionListener(this);
	p2.add(btnYes);
	p2.add(Box.createRigidArea(new Dimension(10, 0)));
	btnNo = new JButton("No");
        btnNo.setBackground(new Color(204, 204, 255));
	btnNo.addActionListener(this);
	p2.add(btnNo);

	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.PAGE_AXIS));
	p3.add(p0);
	p3.add(Box.createRigidArea(new Dimension(0,10)));
	p3.add(p1);
	p3.add(Box.createRigidArea(new Dimension(0,10)));
	p3.add(p2);
	p3.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p3);
    }

    /**
     * Handles clicks on JButton
     * @param evt The Event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnYes) {
	    JDialog dlgWait = new WaitDialog("Load Profile and Generate the Data Repository ...");
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgWait.getSize();
	    dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				Math.max(0,(screenSize.height-windowSize.height)/2));
	    
	    final JDialog fDlg = dlgWait;
	    Thread t = new Thread() {
		    public void run() {
			try {
			    session.loadProperties(DEMO_MMK);
			    fDlg.dispose();
			}
			catch(IOException ioe) {
			    System.err.println("IOException " + ioe.getMessage());
			    ioe.printStackTrace();
			}
		    }
		};
	    t.start();
	    
	    dlgWait.show();
	}
	user.setAskMeAgain(chkAskMeAgain.isSelected());
	setVisible(false);
    }
}
