/**
 * This class is a dialog to show the current data files used
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.io.File;
import java.io.IOException;

// AWT
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dimension;

// Swing
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JScrollPane;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;

public class ConfigureCapsuleDialog extends JDialog implements Observer {

    private JTextField txtVersion;
    private JTextField txtReferenceDB;
    private JTextField txtHL7Tables;
    private JTextField txtUserTables;
    private JTextField txtExternalTables;
    private JTextField txtConfigData;
    private JTextField txtProfile;
    private JTextField txtName;
    private JTextField txtMsgPath;
    private JTextField txtRepository;
    private JList lstLocalTable;
    private JScrollPane lstLocalTablePane;

    /**
     * Constructor
     * @param frame A JFrame
     */
    public ConfigureCapsuleDialog(JFrame frame) {
	super(frame, "Settings", true);

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// Version
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	p0.add(Box.createRigidArea(new Dimension(44, 0)));
	JLabel lblVersion = new JLabel("Version: ");
	p0.add(lblVersion);
	txtVersion = new JTextField();
	txtVersion.setBackground(Color.WHITE);
	txtVersion.setEditable(false);
	p0.add(txtVersion);

	// ReferenceDB
	/*	JPanel p1 = new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
		JLabel lblReferenceDB = new JLabel("Reference Data: ");
		p1.add(lblReferenceDB);
		txtReferenceDB = new JTextField();
		txtReferenceDB.setBackground(Color.WHITE);
		txtReferenceDB.setPreferredSize(new Dimension(500, 20));
		txtReferenceDB.setEditable(false);
		p1.add(txtReferenceDB);*/
	
	// Repository
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	p1.add(Box.createRigidArea(new Dimension(28, 0)));
	JLabel lblRepository = new JLabel("Repository: ");
	p1.add(lblRepository);
	txtRepository = new JTextField();
	txtRepository.setBackground(Color.WHITE);
	txtRepository.setPreferredSize(new Dimension(500, 20));
	txtRepository.setEditable(false);
	p1.add(txtRepository);

	// HL7 Tables
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	p2.add(Box.createRigidArea(new Dimension(26, 0)));
	JLabel lblHL7Tables = new JLabel("HL7 Tables: ");
	p2.add(lblHL7Tables);
	txtHL7Tables = new JTextField();
	txtHL7Tables.setBackground(Color.WHITE);
	txtHL7Tables.setEditable(false);
	p2.add(txtHL7Tables);

	// User Tables
	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	p3.add(Box.createRigidArea(new Dimension(21, 0)));
	JLabel lblUserTables = new JLabel("User Tables: ");
	p3.add(lblUserTables);
	txtUserTables = new JTextField();
	txtUserTables.setBackground(Color.WHITE);
	txtUserTables.setEditable(false);
	p3.add(txtUserTables);

	// External Tables
	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	p4.add(Box.createRigidArea(new Dimension(1, 0)));
	JLabel lblExternalTables = new JLabel("External Tables: ");
	p4.add(lblExternalTables);
	txtExternalTables = new JTextField();
	txtExternalTables.setBackground(Color.WHITE);
	txtExternalTables.setEditable(false);
	p4.add(txtExternalTables);

	// Config Data
/*	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.LINE_AXIS));
	p5.add(Box.createRigidArea(new Dimension(24, 0)));
	JLabel lblConfigData = new JLabel("Config Data: ");
	p5.add(lblConfigData);
	txtConfigData = new JTextField();
	txtConfigData.setBackground(Color.WHITE);
	txtConfigData.setEditable(false);
	p5.add(txtConfigData);*/

	// Profile
	JPanel p8 = new JPanel();
	p8.setLayout(new BoxLayout(p8, BoxLayout.LINE_AXIS));
	p8.add(Box.createRigidArea(new Dimension(52, 0)));
	JLabel lblProfile = new JLabel("Profile: ");
	p8.add(lblProfile);
	txtProfile = new JTextField();
	txtProfile.setBackground(Color.WHITE);
	txtProfile.setEditable(false);
	p8.add(txtProfile);

	// Session Name
	JPanel p9 = new JPanel();
	p9.setLayout(new BoxLayout(p9, BoxLayout.LINE_AXIS));
	p9.add(Box.createRigidArea(new Dimension(7, 0)));
	JLabel lblName = new JLabel("Session Name: ");
	p9.add(lblName);
	txtName = new JTextField();
	txtName.setBackground(Color.WHITE);
	txtName.setEditable(false);
	p9.add(txtName);

	// Message Path
	JPanel p10 = new JPanel();
	p10.setLayout(new BoxLayout(p10, BoxLayout.LINE_AXIS));
	p10.add(Box.createRigidArea(new Dimension(8, 0)));
	JLabel lblMsgPath = new JLabel("Message Path: ");
	p10.add(lblMsgPath);
	txtMsgPath = new JTextField();
	txtMsgPath.setBackground(Color.WHITE);
	txtMsgPath.setEditable(false);
	p10.add(txtMsgPath);

	// Local Tables
	JPanel p11 = new JPanel();
	p11.setLayout(new BoxLayout(p11, BoxLayout.LINE_AXIS));
	p11.add(Box.createRigidArea(new Dimension(18, 0)));
	JLabel lblLocalTables = new JLabel("Local Tables: ");
	p11.add(lblLocalTables);
	lstLocalTable = new JList();
	//lstLocalTablePane.setPreferredSize(new Dimension(350, 10));
	lstLocalTable.setEnabled(false);
	lstLocalTablePane = new JScrollPane(lstLocalTable);
	p11.add(lstLocalTablePane);

/*	txtLocalTables = new JTextField();
	txtLocalTables.setBackground(Color.WHITE);
	txtLocalTables.setEditable(false);
	p11.add(txtLocalTables);*/

        // Close Button
	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
	JButton btnClose = new JButton("Close");
        btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		}
	    });
	p6.add(btnClose);

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.PAGE_AXIS));
	p7.add(p0);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p8);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p9);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p10);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p1);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p2);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p3);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p4);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p11);
	/*p7.add(Box.createRigidArea(new Dimension(0,10)));
	  p7.add(p);*/
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p6);
	p7.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p7);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    try {
		if(arg instanceof MVCArg) {
		    if("repositoryData".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtRepository.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("HL7Tables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtHL7Tables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("userTables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtUserTables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("externalTables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtExternalTables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
/*		    else if("configData".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtConfigData.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
			}*/
		    else if("version".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtVersion.setText(((MVCArg)arg).getValue());
		    }
		    else if("profile".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtProfile.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("name".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtName.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("msgPath".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
			txtMsgPath.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		}
		else if(arg instanceof Vector) {
		    lstLocalTable.setListData((Vector)arg);
		}
		else if(arg instanceof Session) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureCapsuleDialog");
		    Session s = (Session)arg;
		    txtVersion.setText(s.getVersion());
		    txtHL7Tables.setText(new File(s.getHL7Tables()).getCanonicalPath());
		    txtUserTables.setText(new File(s.getUserTables()).getCanonicalPath());
		    txtExternalTables.setText(new File(s.getExternalTables()).getCanonicalPath());
		    lstLocalTable.setListData(s.getLocalTable());
		    //txtConfigData.setText(new File(s.getConfigData()).getCanonicalPath());
		    if(s.getProfile().compareTo("") != 0) {
			txtProfile.setText(new File(s.getProfile()).getCanonicalPath());
		    }
		    if(s.getName().compareTo("") != 0) {
			txtName.setText(new File(s.getName()).getCanonicalPath());
		    }
		    if(s.getMsgPath().compareTo("") != 0) {
			txtMsgPath.setText(new File(s.getMsgPath()).getCanonicalPath());
		    }
		    if(s.getRepositoryData().compareTo("") != 0) {
			txtRepository.setText(new File(s.getRepositoryData()).getCanonicalPath());
		    }
		}
	    }
	    catch(IOException ioe) {
		System.err.println("IOException: " + ioe.getMessage());
	    }
	}
    }
}
