/**
 * The dialog allows the user to configure the default value for missing data
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JComboBox;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.DefaultValue;

public class ConfigureDefaultValueDialog extends JDialog implements ActionListener, ListSelectionListener { //implements Observer, ActionListener, ListSelectionListener {

    private Session session;
    private DefaultValue dv;
    private DefaultValue dvOri;

    private JComboBox cbDatatype;
    private JScrollPane listScrollPane;
    private JList lstValue;
    private JButton btnNew;
    private JButton btnAdd;
    private JButton btnRemove;
    private JButton btnEdit;
    private JButton btnCancel;
    private JTextField txtValue;
    private JComboBox cbCategory;
    private JButton btnSave;
    private JButton btnClose;

    private Vector vDV;
    private int lastDatatypeSelectedIndex;

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     */
    public ConfigureDefaultValueDialog(JFrame frame, Session ses) {
	super(frame, "Set Default Value", true);
	session = ses;
	dv = session.getDefaultValue();
	dvOri = new DefaultValue();

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JLabel lblDatatype = new JLabel("Datatype: ");
	p0.add(lblDatatype);
	cbDatatype = new JComboBox();
	cbDatatype.addActionListener(this);
	p0.add(cbDatatype);

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));

	lstValue = new JList();
	lstValue.addListSelectionListener(this);
	listScrollPane = new JScrollPane(lstValue);
	listScrollPane.setPreferredSize(new Dimension(200,100));
	p1.add(listScrollPane);

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));

	btnNew = new JButton("New");
        btnNew.setBackground(new Color(204, 204, 255));
	btnNew.setEnabled(false);
	btnNew.addActionListener(this);
	p2.add(btnNew);

	p2.add(Box.createRigidArea(new Dimension(10,0)));

	btnAdd = new JButton("Add");
        btnAdd.setBackground(new Color(204, 204, 255));
	btnAdd.setEnabled(false);
	btnAdd.addActionListener(this);
	p2.add(btnAdd);

	p2.add(Box.createRigidArea(new Dimension(10,0)));

	btnRemove = new JButton("Remove");
        btnRemove.setBackground(new Color(204, 204, 255));
	btnRemove.setEnabled(false);
	btnRemove.addActionListener(this);
	p2.add(btnRemove);

	p2.add(Box.createRigidArea(new Dimension(10,0)));

	btnEdit = new JButton("Edit");
        btnEdit.setBackground(new Color(204, 204, 255));
	btnEdit.setEnabled(false);
	btnEdit.addActionListener(this);
	p2.add(btnEdit);

	p2.add(Box.createRigidArea(new Dimension(10,0)));

	btnCancel = new JButton("Cancel");
        btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.setEnabled(false);
	btnCancel.addActionListener(this);
	p2.add(btnCancel);

	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	txtValue = new JTextField("");
	txtValue.setEditable(false);
	p3.add(txtValue);
	p3.add(Box.createRigidArea(new Dimension(10, 0)));
	cbCategory = new JComboBox();
	cbCategory.addItem("DefaultValid");
	cbCategory.addItem("DefaultInvalid");
	cbCategory.setEnabled(false);
	p3.add(cbCategory);

	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	btnSave = new JButton("Save");
	btnSave.setBackground(new Color(204, 204, 255));
	btnSave.addActionListener(this);
	p4.add(btnSave);
	p4.add(Box.createRigidArea(new Dimension(10, 0)));
	btnClose = new JButton("Cancel");
	btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p4.add(btnClose);

	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.PAGE_AXIS));
	p5.add(p0);
	p5.add(Box.createRigidArea(new Dimension(0, 10)));
	p5.add(p1);
	p5.add(Box.createRigidArea(new Dimension(0, 10)));
	p5.add(p2);
	p5.add(Box.createRigidArea(new Dimension(0, 10)));
	p5.add(p3);
	p5.add(Box.createRigidArea(new Dimension(0, 10)));
	p5.add(p4);
	p5.add(Box.createRigidArea(new Dimension(0, 10)));
	p5.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p5);
    }

    /**
     * Init the dialog
     */
    public void init() {
	cbDatatype.removeAllItems();
	vDV = dv.getDefaultValue();
	for(int i=0;i<vDV.size();i++) {
	    cbDatatype.addItem(((DefaultValue.DV)vDV.get(i)).getDatatype());
	}
	cbDatatype.setSelectedIndex(lastDatatypeSelectedIndex);
	Vector vValue = dv.getDefaultValue((String)cbDatatype.getSelectedItem());
	lstValue.setListData(vValue);
	setButton(true, false, false, false, false);
	txtValue.setText("");
	txtValue.setEditable(false);
	cbCategory.setSelectedIndex(0);
	cbCategory.setEnabled(false);
    }

    /**
     * Handles the event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	String datatype = null;
	int idx = -1;
	DefaultValue.DV tmp = null;

	if(evt.getSource() == btnNew) {
	    setButton(false, true, false, false, true);
	    txtValue.setText("");
	    txtValue.setEditable(true);
	    cbCategory.setSelectedIndex(0);
	    cbCategory.setEnabled(true);
	    lstValue.clearSelection();
	}
	else if(evt.getSource() == btnAdd) {
	    datatype = (String)cbDatatype.getSelectedItem();
	    tmp = dv.new DV();
	    tmp.setDatatype(datatype);
	    idx = vDV.indexOf(tmp);
	    if(idx != -1)
		((DefaultValue.DV)vDV.get(idx)).addValue(txtValue.getText(), (String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnRemove) {
	    datatype = (String)cbDatatype.getSelectedItem();
	    tmp = dv.new DV();
	    tmp.setDatatype(datatype);
	    idx = vDV.indexOf(tmp);
	    if(idx != -1)
		((DefaultValue.DV)vDV.get(idx)).removeValue(txtValue.getText(), (String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnEdit) {
	    DefaultValue.DVItem dvItem = (DefaultValue.DVItem)lstValue.getSelectedValue();
	    dvItem.setValue(txtValue.getText());
	    dvItem.setCategory((String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnCancel) {
	    init();
	}
	else if(evt.getSource() == cbDatatype) {
	    if(cbDatatype.getSelectedIndex() != -1) {
		Vector vValue = dv.getDefaultValue((String)cbDatatype.getSelectedItem());
		lstValue.setListData(vValue);
		txtValue.setText("");
		txtValue.setEditable(false);
		cbCategory.setSelectedIndex(0);
		cbCategory.setEnabled(false);
	    }
	}
	else if(evt.getSource() == btnClose) {
	    dv = dvOri;
	    setVisible(false);
	}
	else if(evt.getSource() == btnSave) {
	    dv.save();
	    setVisible(false);
	}
    }

    /**
     * Handles a click on a JList
     * @param e The event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(lstValue.getSelectedIndex() != -1) {
	    DefaultValue.DVItem dvItem = (DefaultValue.DVItem)lstValue.getSelectedValue();
	    txtValue.setText(dvItem.getValue());
	    txtValue.setEditable(true);
	    cbCategory.setSelectedItem(dvItem.getCategory());
	    cbCategory.setEnabled(true);
	    setButton(true, false, true, true, true);
	}
        else {
	    txtValue.setEditable(false);
	    cbCategory.setEnabled(false);
	    setButton(true, false, false, false, false);
	}
    }

    /**
     * Set the enabled property for buttons
     * @param bNew
     * @param bAdd
     * @param bRemove
     * @param bEdit
     * @param bCancel
     */
    private void setButton(boolean bNew, boolean bAdd, boolean bRemove, boolean bEdit, boolean bCancel) {
	btnNew.setEnabled(bNew);
	btnAdd.setEnabled(bAdd);
	btnRemove.setEnabled(bRemove);
	btnEdit.setEnabled(bEdit);
	btnCancel.setEnabled(bCancel);
    }
}
    
