/**
 * This class is a dialog to select the current data files used
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.MagicNumber;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.tableconverter.TableConverter;
import gov.nist.hl7maker.view.config.RepositoryDataModel;
import gov.nist.hl7maker.view.config.TablePriority;

// XML
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

// XSL
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

// DOM
import org.w3c.dom.Document;

// SAX
import org.xml.sax.SAXException;

public class ConfigureLoadDataDialog extends JDialog implements Observer, ActionListener, ListSelectionListener {

    //private final String REFERENCEDB_MASTER = "." + Constants.FS + "data" + Constants.FS + "ReferenceData.xml";
    private final String HL7TABLES_MASTER = "." + Constants.FS + "data" + Constants.FS + "hl7tables.xml";
    private final String USERTABLES_MASTER = "." + Constants.FS + "data" + Constants.FS + "usertables.xml";
    private final String EXTERNALTABLES_MASTER =  "." + Constants.FS + "data" + Constants.FS + "externaltables.xml";
    //private final String CONFIGDATA_MASTER = "." + Constants.FS + "data" + Constants.FS + "ConfigData.xml";

    private JTextField txtRepository;
    private JTextField txtHL7Tables;
    private JTextField txtUserTables;
    private JTextField txtExternalTables;
    //private JTextField txtConfigData;

    private JButton btnRepository;//ferenceDB;
    private JButton btnHL7Tables;
    private JButton btnUserTables;
    private JButton btnExternalTables;
    //private JButton btnConfigData;

    //private JButton btnRestoreReferenceDB;
    private JButton btnMergeRepositories;
    private JButton btnRestoreHL7Tables;
    private JButton btnRestoreUserTables;
    private JButton btnRestoreExternalTables;
    //private JButton btnRestoreConfigData;

    private boolean statusRepository;
    private boolean statusHL7Tables;
    private boolean statusUserTables;
    private boolean statusExternalTables;
    //private boolean statusConfigData;

    private JButton btnOK;
    private JButton btnCancel;

    private Session session;
    private User user;
/*    private ReferenceDBModel refDBModel;
    private TableDataModel tableModel;
    private ConfigDataModel configModel;*/

    private JList lstLocalTable;
    private JScrollPane lstLocalTablePane;
    private JButton btnLoadLocal;
    private JButton btnRemoveLocal;
    private Vector vLocalTable;
    private Vector vOldLocalTable;
    private JButton btnPriority;
    private JButton btnImportMWB;
    private JButton btnImportAny;

    private TablePriorityDialog dlgPriority;
    private TableConverterMappingDialog dlgImport;

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     * @param user A user
     */
    public ConfigureLoadDataDialog(JFrame frame, Session ses, User user) {
	super(frame, "Set Data Files", true);
	session = ses;
	this.user = user;

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// Repository
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	p0.add(Box.createRigidArea(new Dimension(1,0)));
	JLabel lblRepository = new JLabel("Repository Data: ");
	p0.add(lblRepository);
	txtRepository = new JTextField();
	txtRepository.setPreferredSize(new Dimension(500, 20));
	txtRepository.setEditable(false);
	txtRepository.setBackground(Color.WHITE);
	p0.add(txtRepository);
	p0.add(Box.createRigidArea(new Dimension(5,0)));
	btnRepository = new JButton("Browse ...");
	btnRepository.setBackground(new Color(204, 204, 255));
	btnRepository.addActionListener(this);
	p0.add(btnRepository);
	p0.add(Box.createRigidArea(new Dimension(5,0)));
	btnMergeRepositories = new JButton("Merge ...");
	btnMergeRepositories.setBackground(new Color(204, 204, 255));
	btnMergeRepositories.addActionListener(this);
	p0.add(btnMergeRepositories);
/*	btnRestoreRepository = new JButton("Restore Default");
	btnRestoreRepository.setBackground(new Color(204, 204, 255));
	btnRestoreRepository.addActionListener(this);
	p0.add(btnRestoreRepository);*/

	// HL7 Tables
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	p1.add(Box.createRigidArea(new Dimension(26,0)));
	JLabel lblHL7Tables = new JLabel("HL7 Tables: ");
	p1.add(lblHL7Tables);
	txtHL7Tables = new JTextField();
	txtHL7Tables.setEditable(false);
	txtHL7Tables.setBackground(Color.WHITE);
	p1.add(txtHL7Tables);
	p1.add(Box.createRigidArea(new Dimension(5,0)));
	btnHL7Tables = new JButton("Browse ...");
	btnHL7Tables.setBackground(new Color(204, 204, 255));
	btnHL7Tables.addActionListener(this);
	p1.add(btnHL7Tables);
	p1.add(Box.createRigidArea(new Dimension(5,0)));
	btnRestoreHL7Tables = new JButton("Restore Default");
	btnRestoreHL7Tables.setBackground(new Color(204, 204, 255));
	btnRestoreHL7Tables.addActionListener(this);
	p1.add(btnRestoreHL7Tables);

	// User Tables
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	p2.add(Box.createRigidArea(new Dimension(21,0)));
	JLabel lblUserTables = new JLabel("User Tables: ");
	p2.add(lblUserTables);
	txtUserTables = new JTextField();
	txtUserTables.setEditable(false);
	txtUserTables.setBackground(Color.WHITE);
	p2.add(txtUserTables);
	p2.add(Box.createRigidArea(new Dimension(5,0)));
	btnUserTables = new JButton("Browse ...");
	btnUserTables.setBackground(new Color(204, 204, 255));
	btnUserTables.addActionListener(this);
	p2.add(btnUserTables);
	p2.add(Box.createRigidArea(new Dimension(5,0)));
	btnRestoreUserTables = new JButton("Restore Default");
	btnRestoreUserTables.setBackground(new Color(204, 204, 255));
	btnRestoreUserTables.addActionListener(this);
	p2.add(btnRestoreUserTables);

	// External Tables
	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	p3.add(Box.createRigidArea(new Dimension(1,0)));
	JLabel lblExternalTables = new JLabel("External Tables: ");
	p3.add(lblExternalTables);
	txtExternalTables = new JTextField();
	txtExternalTables.setEditable(false);
	txtExternalTables.setBackground(Color.WHITE);
	p3.add(txtExternalTables);
	p3.add(Box.createRigidArea(new Dimension(5,0)));
	btnExternalTables = new JButton("Browse ...");
	btnExternalTables.setBackground(new Color(204, 204, 255));
	btnExternalTables.addActionListener(this);
	p3.add(btnExternalTables);
	p3.add(Box.createRigidArea(new Dimension(5,0)));
	btnRestoreExternalTables = new JButton("Restore Default");
	btnRestoreExternalTables.setBackground(new Color(204, 204, 255));
	btnRestoreExternalTables.addActionListener(this);
	p3.add(btnRestoreExternalTables);

	// Config Data
/*	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	p4.add(Box.createRigidArea(new Dimension(24,0)));
	JLabel lblConfigData = new JLabel("Config Data: ");
	p4.add(lblConfigData);
	txtConfigData = new JTextField();
	txtConfigData.setEditable(true);
	p4.add(txtConfigData);
	p4.add(Box.createRigidArea(new Dimension(5,0)));
	btnConfigData = new JButton("Browse ...");
	btnConfigData.setBackground(new Color(204, 204, 255));
	btnConfigData.addActionListener(this);
	p4.add(btnConfigData);
	p4.add(Box.createRigidArea(new Dimension(5,0)));
	btnRestoreConfigData = new JButton("Restore Default");
	btnRestoreConfigData.setBackground(new Color(204, 204, 255));
	btnRestoreConfigData.addActionListener(this);
	p4.add(btnRestoreConfigData);*/

	// Local Table
	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.LINE_AXIS));

	JPanel p8 = new JPanel();
	p8.setLayout(new BoxLayout(p8, BoxLayout.PAGE_AXIS));
	JLabel lblLocalTable = new JLabel("Local Tables: ");
	p8.add(lblLocalTable);
	p8.add(Box.createRigidArea(new Dimension(0, 110)));

	JPanel p9 = new JPanel();
	p9.setLayout(new BoxLayout(p9, BoxLayout.PAGE_AXIS));
	lstLocalTable = new JList();
	lstLocalTable.addListSelectionListener(this);
	lstLocalTablePane = new JScrollPane(lstLocalTable);
	p9.add(lstLocalTablePane);

	JPanel p10 = new JPanel();
	p10.setLayout(new BoxLayout(p10, BoxLayout.PAGE_AXIS));
	btnLoadLocal = new JButton("Load ...");
	btnLoadLocal.setBackground(new Color(204, 204, 255));
	btnLoadLocal.addActionListener(this);
	p10.add(btnLoadLocal);
	p10.add(Box.createRigidArea(new Dimension(0, 10)));
	btnRemoveLocal = new JButton("Remove");
	btnRemoveLocal.setBackground(new Color(204, 204, 255));
	btnRemoveLocal.addActionListener(this);
	btnRemoveLocal.setEnabled(false);
	p10.add(btnRemoveLocal);
	p10.add(Box.createRigidArea(new Dimension(0, 10)));
	btnPriority = new JButton("Set priority ...");
	btnPriority.setBackground(new Color(204, 204, 255));
	btnPriority.addActionListener(this);
	//btnPriority.setEnabled(false);
	p10.add(btnPriority);
	p10.add(Box.createRigidArea(new Dimension(0, 10)));
	btnImportMWB = new JButton("Import MWB table ...");
	btnImportMWB.setBackground(new Color(204, 204, 255));
	btnImportMWB.addActionListener(this);
	p10.add(btnImportMWB);
	p10.add(Box.createRigidArea(new Dimension(0, 10)));
	btnImportAny = new JButton("Import table ...");
	btnImportAny.setBackground(new Color(204, 204, 255));
	btnImportAny.addActionListener(this);
	p10.add(btnImportAny);
	//p10.add(Box.createRigidArea(new Dimension(0, 75)));

	p7.add(Box.createRigidArea(new Dimension(17, 0)));
	p7.add(p8);
	p7.add(p9);
	p7.add(Box.createRigidArea(new Dimension(10, 0)));
	p7.add(p10);

	// OK Cancel
	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.LINE_AXIS));
	btnOK = new JButton("OK");
	btnOK.setBackground(new Color(204, 204, 255));
	btnOK.addActionListener(this);
	p5.add(btnOK);
	p5.add(Box.createRigidArea(new Dimension(5,0)));
	btnCancel = new JButton("Cancel");
	btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.addActionListener(this);
	p5.add(btnCancel);

	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.PAGE_AXIS));
	p6.add(p0);
	p6.add(Box.createRigidArea(new Dimension(0,10)));
	p6.add(p1);
	p6.add(Box.createRigidArea(new Dimension(0,10)));
	p6.add(p2);
	p6.add(Box.createRigidArea(new Dimension(0,10)));
	p6.add(p3);
	p6.add(Box.createRigidArea(new Dimension(0,10)));
/*	p6.add(p4);
	p6.add(Box.createRigidArea(new Dimension(0,10)));*/
	p6.add(p7);
	p6.add(Box.createRigidArea(new Dimension(0,10)));
	p6.add(p5);
	p6.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p6);

	dlgPriority = new TablePriorityDialog(frame, session);
	dlgImport = new TableConverterMappingDialog(frame, session, user);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof MVCArg) {
		try {
		    if("repositoryData".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
			//refDBModel.loadReferenceDB();
			//session.
			txtRepository.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("HL7Tables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
			//session.getTableDataModel().loadTableData("HL7", session.getHL7Tables());
			txtHL7Tables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("userTables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
			//session.getTableDataModel().loadTableData("User", session.getUserTables());
			txtUserTables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    else if("externalTables".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
			//session.getTableDataModel().loadTableData("User", session.getUserTables());
			//session.getTableDataModel().loadTableData("External", session.getExternalTables());
			txtExternalTables.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
		    }
		    /*else if("configData".compareTo(((MVCArg)arg).getName()) == 0) {
			gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
			configModel.loadConfigData();
			txtConfigData.setText(new File(((MVCArg)arg).getValue()).getCanonicalPath());
			}*/
		}
		catch(IOException ioe) {
		    System.err.println("IOException: " + ioe.getMessage());
		    ioe.printStackTrace();
		}
	    }
	    else if(arg instanceof Vector) {
		//		try {
		    //System.out.println("Update local tables list");
		    Vector vLocalTable = (Vector)arg;
/*		    for(int i=0;i<vLocalTable.size();i++) {
			TablePriority tp = (TablePriority)vLocalTable.get(i);
			session.getTableDataModel().loadLocalTableData(tp.getFilename());
			}*/
		    lstLocalTable.setListData(vLocalTable);
/*		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		    }*/
	    }
	    else if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureLoadDataDialog");
		try {
		    Session s = (Session)arg;
		    txtRepository.setText(new File(s.getRepositoryData()).getCanonicalPath());
		    txtHL7Tables.setText(new File(s.getHL7Tables()).getCanonicalPath());
		    txtUserTables.setText(new File(s.getUserTables()).getCanonicalPath());
		    txtExternalTables.setText(new File(s.getExternalTables()).getCanonicalPath());
		    //txtConfigData.setText(new File(s.getConfigData()).getCanonicalPath());
		    s.loadLocalTable();
		    vLocalTable = s.getLocalTable();
		    vOldLocalTable = new Vector();
		    for(int i=0;i<vLocalTable.size();i++) {
			vOldLocalTable.add(vLocalTable.get(i));
		    }
		    lstLocalTable.setListData(vLocalTable);
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}
	    }
	}
    }

    /**
     * Handles the event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnRepository) {
	    JFileChooser fc = new JFileChooser(new File(session.getRepositoryData()).getParent());
	    fc.setFileFilter(new XMLFileFilter());
	    int returnVal = fc.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		txtRepository.setText(fc.getSelectedFile().getPath());
		statusRepository = true;
	    }
	}
	else if(evt.getSource() == btnHL7Tables) {
	    JFileChooser fc = new JFileChooser(new File(session.getHL7Tables()).getParent());
	    fc.setFileFilter(new XMLFileFilter());
	    int returnVal = fc.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		txtHL7Tables.setText(fc.getSelectedFile().getPath());
		statusHL7Tables = true;
	    }
	}
	else if(evt.getSource() == btnUserTables) {
	    JFileChooser fc = new JFileChooser(new File(session.getUserTables()).getParent());
	    fc.setFileFilter(new XMLFileFilter());
	    int returnVal = fc.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		txtUserTables.setText(fc.getSelectedFile().getPath());
		statusUserTables = true;
	    }
	}
	else if(evt.getSource() == btnExternalTables) {
	    JFileChooser fc = new JFileChooser(new File(session.getExternalTables()).getParent());
	    fc.setFileFilter(new XMLFileFilter());
	    int returnVal = fc.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		txtExternalTables.setText(fc.getSelectedFile().getPath());
		statusExternalTables = true;
	    }
	}
	else if(evt.getSource() == btnMergeRepositories) {
	    try {
		JFileChooser fc = new JFileChooser(new File(session.getRepositoryData()).getParent());
		fc.setFileFilter(new XMLFileFilter());
		int returnVal = fc.showDialog(this, "Select a repository");
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    RepositoryDataModel rep = new RepositoryDataModel();
		    rep.loadRepository(fc.getSelectedFile().getPath());
		    session.getRepositoryDataModel().merge(rep);
		    //session.getRepositoryDataModel().saveToFile(new File(session.getRepositoryData()));
		}
	    }
	    catch(ParserConfigurationException pce) {
		System.err.println("ParserConfigurationException " + pce.getMessage());
		pce.printStackTrace();
	    }
	    catch(SAXException saxe) {
		System.err.println("SAXException " + saxe.getMessage());
		saxe.printStackTrace();
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	}
	/*else if(evt.getSource() == btnConfigData) {
	JFileChooser fc = new JFileChooser(new File(session.getConfigData()).getParent());
	    int returnVal = fc.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		txtConfigData.setText(fc.getSelectedFile().getPath());
		statusConfigData = true;
	    }
	    }*/
	/*else if(evt.getSource() == btnRestoreReferenceDB) {
	    try {
		txtReferenceDB.setText(new File(REFERENCEDB_MASTER).getCanonicalPath());
		statusReferenceDB = true;
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	    }*/
	else if(evt.getSource() == btnRestoreHL7Tables) {
	    try {
		txtHL7Tables.setText(new File(HL7TABLES_MASTER).getCanonicalPath());
		statusHL7Tables = true;
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	}
	else if(evt.getSource() == btnRestoreUserTables) {
	    try {
		txtUserTables.setText(new File(USERTABLES_MASTER).getCanonicalPath());
		statusUserTables = true;
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	}
	else if(evt.getSource() == btnRestoreExternalTables) {
	    try {
		txtExternalTables.setText(new File(EXTERNALTABLES_MASTER).getCanonicalPath());
		statusExternalTables = true;
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	}
	/*else if(evt.getSource() == btnRestoreConfigData) {
	    try {
		txtConfigData.setText(new File(CONFIGDATA_MASTER).getCanonicalPath());
		statusConfigData = true;
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	    }*/
	else if(evt.getSource() == btnOK) {
	    try {
		String errorMsg = "";
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		// Check the filename with the magic number
		if(statusRepository) {
		    if(MagicNumber.check(txtRepository.getText())) {
			session.setRepositoryData(txtRepository.getText());
			session.generateRepository(false);
			statusRepository = false;
		    }
		    else {
			errorMsg += "The file chosen for the repository is not valid";
		    }
		}
		if(statusHL7Tables) {
		    if(MagicNumber.check(txtHL7Tables.getText())) {
			session.setHL7Tables(txtHL7Tables.getText());
			session.getTableDataModel().loadTableData("HL7", session.getHL7Tables());
			statusHL7Tables = false;
		    }
		    else {
			if(!errorMsg.equals(""))
			    errorMsg += "\n";
			errorMsg += "The file chosen for the HL7 Tables is not valid";
		    }
		}
		if(statusUserTables) {
		    if(MagicNumber.check(txtUserTables.getText())) {
			session.setUserTables(txtUserTables.getText());
			session.getTableDataModel().loadTableData("User", session.getUserTables());
			statusUserTables = false;
		    }
		    else {
			if(!errorMsg.equals(""))
			    errorMsg += "\n";
			errorMsg += "The file chosen for the User Tables is not valid";
		    }
		}
		if(statusExternalTables) { 
		    if(MagicNumber.check(txtExternalTables.getText())) {
			session.setExternalTables(txtExternalTables.getText());
			session.getTableDataModel().loadTableData("External", session.getExternalTables());
			statusExternalTables = false;
		    }
		    else {
			if(!errorMsg.equals(""))
			    errorMsg += "\n";
			errorMsg += "The file chosen for the External Tables is not valid";
		    }
		}
		/*if(statusConfigData && MagicNumber.check(txtConfigData.getText())) {
		  session.setConfigData(txtConfigData.getText());
		  statusConfigData = false;
		  }*/
		// LocalTable
		session.saveLocalTable();
		for(int i=0;i<vLocalTable.size();i++) {
		    TablePriority tp = (TablePriority)vLocalTable.get(i);
		    session.getTableDataModel().loadLocalTableData(tp.getFilename());
		}
		//	    lstLocalTable.setListData(vLocalTable);
		
		if(!errorMsg.equals(""))
			JOptionPane.showMessageDialog(null, errorMsg, "Error loading files", JOptionPane.ERROR_MESSAGE);

		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		setVisible(false);
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}
	}
	else if(evt.getSource() == btnCancel) {
	    for(int i=0;i<vLocalTable.size();i++) {
		TablePriority tp = (TablePriority)vLocalTable.get(i);
		String type = "Local[" + new File(tp.getFilename()).getName() + "]";
		session.getTableDataModel().cleanType(type);
	    }
	    session.setLocalTable(vOldLocalTable, false);
	    setVisible(false);
	    //statusReferenceDB = false;
	}
	else if(evt.getSource() == btnLoadLocal) {
	    JFileChooser fc = new JFileChooser(user.getOpenedDir());
	    // Set the chooser to accept only XML files.
	    fc.addChoosableFileFilter(new XMLFileFilter());
	    fc.setAcceptAllFileFilterUsed(false);
	    int returnVal = fc.showOpenDialog(this);
				    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		File f = fc.getSelectedFile();
		if(!vLocalTable.contains(f.getPath())) {
		    TablePriority tp = new TablePriority(f.getPath(), 100);
		    if(!session.getTable().contains(tp)) {
			vLocalTable.add(new TablePriority(f.getPath(), 100));
		    }
		}
		//lstLocalTable.setListData(vLocalTable);
		session.setLocalTable(vLocalTable, true);
		user.setOpenedDir(f.getParent());
		/*if(vLocalTable.size() == 1)
		  btnLoadLocal.setEnabled(false);*/
	    }
	}
	else if(evt.getSource() == btnRemoveLocal) {
	    if(lstLocalTable.getSelectedIndex() != -1) {
		//		int idx = vLocalTable.indexOf(lstLocalTable.getSelectedValue());
		String type = "Local[" + new File(((TablePriority)lstLocalTable.getSelectedValue()).getFilename()).getName() + "]";
		session.getTableDataModel().cleanType(type);
		vLocalTable.remove((TablePriority)lstLocalTable.getSelectedValue());
		//lstLocalTable.setListData(vLocalTable);
		session.setLocalTable(vLocalTable, true);
		btnRemoveLocal.setEnabled(false);
		btnLoadLocal.setEnabled(true);
	    }
	}
	else if(evt.getSource() == btnPriority) {
	    //session.saveLocalTable();
	    dlgPriority.pack();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgPriority.getSize();
	    dlgPriority.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			       Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgPriority.init();
	    dlgPriority.setVisible(true);
	}
	else if(evt.getSource() == btnImportMWB) {
	    // Select the file to import
	    JFileChooser fc = new JFileChooser(user.getOpenedDir());
	    // Set the chooser to accept only XML files.
	    fc.addChoosableFileFilter(new XMLFileFilter());
	    fc.setAcceptAllFileFilterUsed(false);
	    int returnVal = fc.showDialog(this, "Select the local table file to import");
				    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		File fMWB = fc.getSelectedFile();
		user.setOpenedDir(fMWB.getParent());
		// Select the file to save
		fc = new JFileChooser(user.getOpenedDir());
		fc.setSelectedFile(new File("MM_" + fMWB.getName()));
		// Set the chooser to accept only XML files.
		fc.addChoosableFileFilter(new XMLFileFilter());
		fc.setAcceptAllFileFilterUsed(false);
		returnVal = fc.showSaveDialog(this);
				    
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fMM = fc.getSelectedFile();
		    if(!fMM.getPath().endsWith(".xml")) {
			fMM = new File(fMM.getPath() + ".xml");
		    }
		    user.setOpenedDir(fMM.getParent());

		    // Do the transformation
		    try {
			TableConverter tc = new TableConverter();
			
			tc.setTableMapping("hl7table");
			tc.setItemMapping("tableElement");
			tc.setTypeMapping("type");
			tc.setNameMapping("name");
			tc.setIDMapping("id");
			tc.setCodeMapping("code");
			tc.setValueMapping("description");			

			tc.convert(fMWB, fMM);
		    }
		    catch(FileNotFoundException fnfe) {
			System.err.println("FileNotFoundException " + fnfe.getMessage());
			fnfe.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(TransformerConfigurationException tce) {
			System.err.println("TransformerConfigurationException " + tce.getMessage());
			tce.printStackTrace();
		    }
		    catch(TransformerException te) {
			System.err.println("TransformerException " + te.getMessage());
			te.printStackTrace();
		    }	
		    // Add the file to the local table list
		    if(!vLocalTable.contains(fMM.getPath())) {
			TablePriority tp = new TablePriority(fMM.getPath(), 100);
			if(!session.getTable().contains(tp)) {
			    vLocalTable.add(new TablePriority(fMM.getPath(), 100));
			}
		    }
		    //lstLocalTable.setListData(vLocalTable);
		    session.setLocalTable(vLocalTable, true);
		}
	    }	
	}
	else if(evt.getSource() == btnImportAny) {
	    //session.saveLocalTable();
	    dlgImport.pack();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgImport.getSize();
	    dlgImport.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			       Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgImport.init();
	    dlgImport.prepareDisplay();
	}
    }

    /**
     * Handle the event for JList
     * @param e The Event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(lstLocalTable.getSelectedIndex() != -1) {
	    btnRemoveLocal.setEnabled(true);
	}
    }

    /**
     * Init the panel
     */
    public void init() {
	try {
	    txtRepository.setText(new File(session.getRepositoryData()).getCanonicalPath());
	    txtHL7Tables.setText(new File(session.getHL7Tables()).getCanonicalPath());
	    txtUserTables.setText(new File(session.getUserTables()).getCanonicalPath());
	    txtExternalTables.setText(new File(session.getExternalTables()).getCanonicalPath());
	    //txtConfigData.setText(new File(session.getConfigData()).getCanonicalPath());
	    session.loadLocalTable();
	    vLocalTable = session.getLocalTable();
	    vOldLocalTable = new Vector();
	    for(int i=0;i<vLocalTable.size();i++) {
		vOldLocalTable.add(vLocalTable.get(i));
	    }
	    lstLocalTable.setListData(vLocalTable);
	    btnLoadLocal.setEnabled(true);
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
    }
}
