/**
 * This class is a dialog to configure the MSH segment
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;
import java.util.EventObject;
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.AbstractCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeModel;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// DOM
import org.w3c.dom.Document;

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.XMLFileFilter;
//import gov.nist.hl7maker.view.config.ReferenceDBModel;
//import gov.nist.hl7maker.view.config.TableDataModel;
//import gov.nist.hl7maker.view.config.ConfigDataModel;
import gov.nist.hl7maker.view.config.ProfileNode;
//import gov.nist.hl7maker.view.config.ConfigNode;
import gov.nist.hl7maker.view.config.ValueNode;
import gov.nist.hl7maker.view.config.TableValueNode;

public class ConfigureMSHDialog extends JDialog implements Observer, ActionListener {

    private static JTable tableProfile;
    private TreeTableCellRenderer tree;
    private MSHTableModel model;
    //private ConfigDataModel configModel;
    private Session session;

    private JButton btnSave, btnSaveAs, btnReset, btnClose;

    //private final String REFERENCEDB_MASTER = "." + Constants.FS + "data" + Constants.FS + "ReferenceData.xml";
    //private final String CONFIGDATA_MASTER = "." + Constants.FS + "data" + Constants.FS + "ConfigData.xml";

    /**
     * Constructor
     * @param frame A JFrame
     */
    public ConfigureMSHDialog(JFrame frame) {
	super(frame, "MSH Segment Configuration", true);
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    closeDialog(evt);
		}
	    });

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// JTable
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	tableProfile = new JTable() {
		public boolean isCellEditable(int row, int col) {        
		    return true;
		}

		/* Workaround for BasicTableUI anomaly. Make sure the UI never tries to 
		 * paint the editor. The UI currently uses different techniques to 
		 * paint the renderers and editors and overriding setBounds() below 
		 * is not the right thing to do for an editor. Returning -1 for the 
		 * editing row in this case, ensures the editor is never painted. 
		 */
		public int getEditingRow() {
		    //		    return (model.getColumnClass(editingColumn) == MSHTableModel.class) ? -1 : editingRow;  
		    return -1;
		}

	    };
	model = new MSHTableModel();
	tableProfile.setModel(model);
	JScrollPane tableScrollPane = new JScrollPane(tableProfile);
	tableScrollPane.setPreferredSize(new Dimension(800,550));
	tableScrollPane.setOpaque(true);
	p0.add(tableScrollPane);
	
        // Save / Save As / Reset / Close Buttons
	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
	// Save
	btnSave = new JButton("Save");
        btnSave.setBackground(new Color(204, 204, 255));
	btnSave.addActionListener(this);
	p6.add(btnSave);
	p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Save As
	btnSaveAs = new JButton("Save As ...");
        btnSaveAs.setBackground(new Color(204, 204, 255));
	btnSaveAs.addActionListener(this);
	//p6.add(btnSaveAs);
	//p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Reset
	btnReset = new JButton("Undo Modification");
        btnReset.setBackground(new Color(204, 204, 255));
	btnReset.addActionListener(this);
	p6.add(btnReset);
	p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Close
	btnClose = new JButton("Close");
        btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p6.add(btnClose);

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.PAGE_AXIS));
	p7.add(p0);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p6);
	p7.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p7);
    }

    /**
     * Set the data model
     * @param refDBModel The reference data
     * @param tableModel The table data
     * @param configModel The config data
     */
/*    public void setDataModel(ReferenceDBModel refDBModel, TableDataModel tableModel, ConfigDataModel configModel) {
	model.setDataModel(refDBModel, tableModel, configModel);
	this.configModel = configModel;
	}*/

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureMSHDialog");
		session = (Session)arg;
		model.setSession(session);
		if(((Session)arg).getProfile().compareTo("") != 0) {
		    Document doc = null;
		    // TableModel
		    try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			doc = builder.parse(new File(((Session)arg).getProfile()));
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    model.init(doc);
		    // Create the tree. It will be used as a renderer and editor. 
		    tree = new TreeTableCellRenderer(model);
		    // Force the JTable and JTree to share their row selection models. 
		    tree.setSelectionModel(new DefaultTreeSelectionModel() { 
			    // Extend the implementation of the constructor, as if: 
			    /* public this() */ {
			tableProfile.setSelectionModel(listSelectionModel); 
			    } 
			}); 
		    // Make the tree and table row heights the same. 
		    tree.setRowHeight(tableProfile.getRowHeight()); // <= Permet d'afficher les sous arbres
		    
		    model.setTree(tree);
		    tableProfile.setDefaultRenderer(MSHTableModel.class, tree); 
		    tableProfile.setDefaultEditor(MSHTableModel.class, new TreeTableCellEditor());  
		    tableProfile.setShowGrid(false);
		    tableProfile.setIntercellSpacing(new Dimension(1, 0));

		    //tableProfile.setDefaultRenderer(Vector.class, new CurrentTableCellRenderer()); 
		    //tableProfile.setDefaultEditor(Vector.class, new CurrentTableCellEditor());  

		    tableProfile.setDefaultRenderer(Vector.class, new ValueTableCellRenderer());
		    tableProfile.setDefaultEditor(Vector.class, new ValueTableCellEditor());

		    //		    tableProfile.setDefaultRenderer(JButton.class, new SetTableCellRenderer());
		    //tableProfile.setDefaultEditor(JButton.class, new SetTableCellEditor());
		}
	    }
	    if(arg instanceof MVCArg) {
		if("profile".compareTo(((MVCArg)arg).getName()) == 0 && !((MVCArg)arg).getValue().equals("")) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureMSHDialog");
		    Document doc = null;
		    // TableModel
		    try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			doc = builder.parse(new File(((MVCArg)arg).getValue()));
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    model.init(doc);
		    // Create the tree. It will be used as a renderer and editor. 
		    tree = new TreeTableCellRenderer(model);
		    // Force the JTable and JTree to share their row selection models. 
		    tree.setSelectionModel(new DefaultTreeSelectionModel() { 
			    // Extend the implementation of the constructor, as if: 
			    /* public this() */ {
			tableProfile.setSelectionModel(listSelectionModel); 
			    } 
			}); 
		    // Make the tree and table row heights the same. 
		    tree.setRowHeight(tableProfile.getRowHeight()); // <= Permet d'afficher les sous arbres
		    
		    model.setTree(tree);
		    tableProfile.setDefaultRenderer(MSHTableModel.class, tree); 
		    tableProfile.setDefaultEditor(MSHTableModel.class, new TreeTableCellEditor());  
		    tableProfile.setShowGrid(false);
		    tableProfile.setIntercellSpacing(new Dimension(1, 0));

		    //tableProfile.setDefaultRenderer(Vector.class, new CurrentTableCellRenderer()); 
		    //tableProfile.setDefaultEditor(Vector.class, new CurrentTableCellEditor());  

		    tableProfile.setDefaultRenderer(Vector.class, new ValueTableCellRenderer());
		    tableProfile.setDefaultEditor(Vector.class, new ValueTableCellEditor());

		    //tableProfile.setDefaultRenderer(JButton.class, new SetTableCellRenderer());
		    //tableProfile.setDefaultEditor(JButton.class, new SetTableCellEditor());
		}
	    }
	}
    }

    /**
     * Handles clicks on JButton
     * @param evt The Event
     */
    public void actionPerformed(ActionEvent evt) {
	try {
	    //boolean update = false;
	    boolean needToSave = false;
	    if(evt.getSource() == btnClose) {
		Vector vUpdate = model.getUpdateElement();
		if(vUpdate.size() != 0) {
		    Object[] options = {"Yes", "No"};
		    JOptionPane optionPane = new JOptionPane("MSH Segment data was modified. Do you want to save?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Yes");
		    JDialog dialog = optionPane.createDialog(this, "Save the data");
		    dialog.show();
		    Object selectedValue = optionPane.getValue();
		    
		    if(selectedValue != null) {
			if(((String)selectedValue).compareTo(options[0]) == 0) {
			    // Save The data
			    needToSave = true;
			}
			else if(((String)selectedValue).compareTo(options[1]) == 0) {
			    setVisible(false);
			    session.setProfile(session.getProfile());
			}
		    }
		}
		else if(evt.getSource() == btnClose) {
		    setVisible(false);
		}
	    }
	    if(evt.getSource() == btnSave || evt.getSource() == btnSaveAs || evt.getSource() == btnClose) {
		Vector vUpdate = model.getUpdateElement();
		for(int i=0;i<vUpdate.size();i++) {
		    String newValue;
		    newValue = ((MSHTableModel.UpdateElement)vUpdate.get(i)).newValue;
		    if(newValue.compareTo("") != 0) {
			// Get the profile node at row row, and column column
			ProfileNode pn = ((MSHTableModel.UpdateElement)vUpdate.get(i)).pn;
			// Check if a config value exists
			Vector vConf = session.getRepositoryDataModel().getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			boolean isConf = false;
			for(int j=0;j<vConf.size();j++) {
			    ValueNode vn = (ValueNode)vConf.get(j);
			    if(vn.getCategory().equals("Config")) {
				j = vConf.size() + 1;
				isConf = true;
			    }
			}
			// Create a ValueNode from a ProfileNode
			if(!isConf) {
			    ValueNode vn = new ValueNode();
			    vn.setName(pn.getName());
			    vn.setQualifier(pn.getQualifier());
			    vn.setQualifier2(pn.getQualifier2());
			    vn.setSegment(pn.getSegment());
			    vn.setType("ConfigData");
			    vn.setUsage(pn.getUsage());
			    vn.setValue(newValue);
			    vn.setStatus(ValueNode.NEW);
			    vn.setCategory("Config");
			    session.getRepositoryDataModel().addValue(vn, false);
			}
			// Modify the existing ValueNode
			else {
			    for(int j=0;j<vConf.size();j++) {
				ValueNode vn = (ValueNode)vConf.get(j);
				if(vn.getCategory().equals("Config")) {
				    vn.modifyValue(newValue);
				    vn.setStatus(ValueNode.EDITED);
				    session.getRepositoryDataModel().modifyValue(vn, false);
				    j = vConf.size() + 1;
				}
			    }
			}		    
		    }
		}
		model.clearUpdate();
	    }
	    if(evt.getSource() == btnSave || (evt.getSource() == btnClose && needToSave)) {
/*		File masterFile = null, sessionFile = null;
		masterFile = new File(CONFIGDATA_MASTER);
		sessionFile = new File(session.getConfig());
		if(masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {                 
		    JFileChooser fc = new JFileChooser(sessionFile.getParent());
		    // Set the chooser to accept only XML files.
		    fc.addChoosableFileFilter(new XMLFileFilter());
		    fc.setAcceptAllFileFilterUsed(false);
		    fc.setSelectedFile(new File("LocalConfigData.xml"));
		    int returnVal = fc.showOpenDialog(this);
		    
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
			File f = fc.getSelectedFile();
			if(!f.getPath().endsWith(".xml")) {
			    f = new File(f.getPath() + ".xml");
			}
			configModel.saveConfigToFile(f);
		    }
		}
		else {
		    configModel.saveConfigToFile(new File(session.getConfig()));
		    }*/
		session.getRepositoryDataModel().saveConfigToFile(new File(session.getConfig()));
		//		if(needToSave) {
		    setVisible(false);
		    //		}
	    }		
/*	    else if(evt.getSource() == btnSaveAs) {
		JFileChooser fc = null;
		fc = new JFileChooser(new File(session.getConfig()).getParent());
		// Set the chooser to accept only XML files.
		fc.addChoosableFileFilter(new XMLFileFilter());
		fc.setAcceptAllFileFilterUsed(false);
		fc.setSelectedFile(new File("LocalConfigData.xml"));
		int returnVal = fc.showOpenDialog(this);
		
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File f = fc.getSelectedFile();
		    if(!f.getPath().endsWith(".xml")) {
			f = new File(f.getPath() + ".xml");
		    }
		    configModel.saveConfigToFile(f);
		}
		}*/
	    else if(evt.getSource() == btnReset) {
		session.setProfile(session.getProfile());
	    }	    
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
    }

    /**
     * Handle the event of closing the dialog
     * @param evt The Event
     */
    public void closeDialog(WindowEvent evt) {
	try {
	    //boolean update = false;
	    Vector vUpdate = model.getUpdateElement();
	    if(vUpdate.size() != 0) {
		JOptionPane optionPane = new JOptionPane("MSH Config data was modified. Do you want to save?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
		JDialog dialog = optionPane.createDialog(this, "Save the data");
		dialog.show();
		Object selectedValue = optionPane.getValue();
		
		if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		    // Save The data
/*		    File masterFile = null, sessionFile = null;
		    masterFile = new File(CONFIGDATA_MASTER);
		    sessionFile = new File(session.getConfig());
		    if(masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {
			JFileChooser fc = new JFileChooser(sessionFile.getParent());
			// Set the chooser to accept only XML files.
			fc.addChoosableFileFilter(new XMLFileFilter());
			fc.setAcceptAllFileFilterUsed(false);
			fc.setSelectedFile(new File("LocalConfigData.xml"));
			int returnVal = fc.showOpenDialog(this);
			
			if(returnVal == JFileChooser.APPROVE_OPTION) {
			    File f = fc.getSelectedFile();
			    if(!f.getPath().endsWith(".xml")) {
				f = new File(f.getPath() + ".xml");
			    }
			    configModel.saveConfigToFile(f);
			}
		    }
		    else {*/
		    for(int i=0;i<vUpdate.size();i++) {
			String newValue;
			newValue = ((MSHTableModel.UpdateElement)vUpdate.get(i)).newValue;
			if(newValue.compareTo("") != 0) {
			    // Get the profile node at row row, and column column
			    ProfileNode pn = ((MSHTableModel.UpdateElement)vUpdate.get(i)).pn;
			    // Check if a config value exists
			    Vector vConf = session.getRepositoryDataModel().getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			    boolean isConf = false;
			    for(int j=0;j<vConf.size();j++) {
				ValueNode vn = (ValueNode)vConf.get(j);
				if(vn.getCategory().equals("Config")) {
				    j = vConf.size() + 1;
				    isConf = true;
				}
			    }
			    // Create a ValueNode from a ProfileNode
			    if(!isConf) {
				ValueNode vn = new ValueNode();
				vn.setName(pn.getName());
				vn.setQualifier(pn.getQualifier());
				vn.setQualifier2(pn.getQualifier2());
				vn.setSegment(pn.getSegment());
				vn.setType("ConfigData");
				vn.setUsage(pn.getUsage());
				vn.setValue(newValue);
				vn.setStatus(ValueNode.NEW);
				vn.setCategory("Config");
				session.getRepositoryDataModel().addValue(vn, false);
			    }
			    // Modify the existing ValueNode
			    else {
				for(int j=0;j<vConf.size();j++) {
				    ValueNode vn = (ValueNode)vConf.get(j);
				    if(vn.getCategory().equals("Config")) {
					vn.modifyValue(newValue);
					vn.setStatus(ValueNode.EDITED);
					session.getRepositoryDataModel().modifyValue(vn, false);
					j = vConf.size() + 1;
				    }
				}
			    }		    
			}
		    }
		    model.clearUpdate();
		    
		    session.getRepositoryDataModel().saveConfigToFile(new File(session.getConfig()));
		    //}
		}
	    }
	    

	    //setVisible(false);
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
    }

    /**
     * Get the message control id
     * @return The message control id
     */
    public String getMessageControlId() {
	return ((ValueNode)session.getRepositoryDataModel().getValues("Message Control ID", "MSH", "", "").get(0)).getValue();
    }

    // 
    // The renderer used to display the tree nodes, a JTree.  
    //
	
    public class TreeTableCellRenderer extends JTree implements TableCellRenderer {

	protected int visibleRow;
   
	public TreeTableCellRenderer(TreeModel model) { 
	    super(model);
	    expandRow(1);
	}

	public void setBounds(int x, int y, int w, int h) {
	    super.setBounds(x, 0, w, ConfigureMSHDialog.tableProfile.getHeight());
	}

	public void paint(Graphics g) {
	    g.translate(0, -visibleRow * getRowHeight());
	    super.paint(g);
	}

	public Component getTableCellRendererComponent(JTable table,
						       Object value,
						       boolean isSelected,
						       boolean hasFocus,
						       int row, int column) {
	    if(isSelected)
		setBackground(table.getSelectionBackground());
	    else
		setBackground(table.getBackground());
	    
	    visibleRow = row;
	    return this;
	}
    }

    // 
    // The editor used to interact with tree nodes, a JTree.  
    //

    public class TreeTableCellEditor extends AbstractCellEditor implements TableCellEditor {
 	public Component getTableCellEditorComponent(JTable table, Object value,
	 					     boolean isSelected, int r, int c) {
	    return tree;
	}
	
	public Object getCellEditorValue() { return null; }
	public boolean isCellEditable(EventObject e) { return true; }
	public boolean shouldSelectCell(EventObject anEvent) { return false; }
	public boolean stopCellEditing() { return true; }
	public void cancelCellEditing() {}
    }

    //
    // Renderer for the column "Value"
    //

    public class ValueTableCellRenderer extends JComboBox implements TableCellRenderer {

	public Component getTableCellRendererComponent(JTable table,
						       Object value,
						       boolean isSelected,
						       boolean hasFocus,
						       int row, int column) {

	    removeAllItems();
	    if(value instanceof Vector) {
		Vector v = (Vector)value;
		if(v.size() > 0) {
		    // Get the type
		    String type = (String)v.get(0);
		    //System.out.println(type);
		    // HardCoded
		    if(type.equals("HC")) {
			// Return a JLabel
			JLabel lblHC = new JLabel(" " + (String)v.get(1)) {
				public void paint(Graphics g) {
				    Color bColor;
				    g.setColor(new Color(67, 205, 128));
				    // Draw a rectangle in the background of the cell
				    g.fillRect(0, 0, getWidth(), getHeight());
				    super.paint(g);
				}
			    };
			lblHC.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
			return lblHC;
		    }
		    // Dynamic
		    else if(type.equals("Dynamic")) {
			// Get the selected value
			String selectedValue = (String)v.get(1);
			// Values
			for(int i=2;i<v.size();i++) {
			    addItem(v.get(i));
			}
			if(selectedValue == null) {
			    insertItemAt("No Config Value Selected", 0);
			    setSelectedIndex(0);
			}
			else {
			    // Check if the selectedValue exists
			    boolean exists = false;
			    for(int i=0;i<getItemCount() && !exists;i++) {
				exists = selectedValue.equals(getItemAt(i));
			    }
			    if(!exists) {
				addItem(selectedValue);
			    }
			    setSelectedItem(selectedValue);
			}
			return this;
		    }
		    // Usage "R" or not "X"
		    else if(type.equals("R")) {
/*			System.out.println("This is this case " + v.get(1).getClass());
			System.exit(0);*/
			// Get the selected value
			//System.out.println("SelectedValue " + v.get(1));
			String selectedValue = (String)v.get(1);
			//System.out.println(selectedValue);
			// Values
			for(int i=2;i<v.size();i++) {
			    Object o = v.get(i);
			    //System.out.println(i + " " + o.getClass());
			    if(o instanceof TableValueNode) {
				addItem(((TableValueNode)o).getValue());
			    }
			    else if(o instanceof ValueNode) {
				addItem(((ValueNode)o).getValue());
			    }
			    else if(o instanceof String) {
				addItem(o);
			    }
			    else {
				System.err.println("A case has not been processed " + o.getClass());
				System.exit(0);
			    }
			}
			if(selectedValue == null) {
			    insertItemAt("No Config Value Selected", 0);
			    setSelectedIndex(0);
			}
			else {
			    // Check if the selectedValue exists
			    boolean exists = false;
			    for(int i=0;i<getItemCount() && !exists;i++) {
				exists = selectedValue.equals(getItemAt(i));
			    }
			    if(!exists) {
				addItem(selectedValue);
			    }
			    setSelectedItem(selectedValue);
			}
			setBackground(new Color(255,236,139));
			return this;
		    }
		    // Usage "X"
		    else if(type.equals("X")) {
			// Return a JLabel
			JLabel lblX = new JLabel(" " + (String)v.get(1)) {
				public void paint(Graphics g) {
				    Color bColor;
				    g.setColor(new Color(190, 190, 190));
				    // Draw a rectangle in the background of the cell
				    g.fillRect(0, 0, getWidth(), getHeight());
				    super.paint(g);
				}
			    };
			lblX.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
			return lblX;
		    }
		}
	    }
	    //   if(true) {
		JLabel lblOther = new JLabel(" ") {
			public void paint(Graphics g) {
			    Color bColor;
			    g.setColor(Color.WHITE);
			    // Draw a rectangle in the background of the cell
			    g.fillRect(0, 0, getWidth(), getHeight());
			    super.paint(g);
			}
		    };
		//lblOther.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		return lblOther;
		// }
	}
    }


    // 
    // Editor for the column "Value"
    //

    public class ValueTableCellEditor extends AbstractCellEditor implements TableCellEditor, ActionListener {
	private JComboBox cb;
	//private JTextField tf;
	private int row, column;
	private JTable table;

	public ValueTableCellEditor() {
	    cb = new JComboBox();
	    cb.setEditable(true);
	    cb.addActionListener(this);
	    //tf = new JTextField();
	    //tf.addActionListener(this);
	}

 	public Component getTableCellEditorComponent(JTable table, Object value,
	 					     boolean isSelected, int r, int c) {
	    cb.removeAllItems();
	    this.table = table;
	    row = r;
	    column = c;
	    if(value instanceof Vector) {
		Vector v = (Vector)value;
		if(v.size() > 0) {
		    // Get the type
		    String type = (String)v.get(0);
		    // HardCoded
		    if(type.equals("HC")) {
			// Return a JLabel
			JLabel lblHC = new JLabel(" " + (String)v.get(1)) {
				public void paint(Graphics g) {
				    Color bColor;
				    g.setColor(new Color(67, 205, 128));
				    // Draw a rectangle in the background of the cell
				    g.fillRect(0, 0, getWidth(), getHeight());
				    super.paint(g);
				}
			    };
			lblHC.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
			return lblHC;
		    }
		    // Dynamic
		    else if(type.equals("Dynamic")) {
			// Get the selected value
			String selectedValue = (String)v.get(1);
			// Values
			cb.addItem("");
			for(int i=2;i<v.size();i++) {
			    cb.addItem(v.get(i));
			}
			if(selectedValue == null) {
			    //cb.insertItemAt("No Config Value Selected", 0);
			    //			    cb.setSelectedIndex(0);
			}
			else {
			    //cb.setSelectedItem(selectedValue);
			    // Check if the selectedValue exists
			    boolean exists = false;
			    for(int i=0;i<cb.getItemCount() && !exists;i++) {
				exists = selectedValue.equals(cb.getItemAt(i));
			    }
			    if(!exists) {
				cb.addItem(selectedValue);
			    }
			}
			return cb;
		    }
		    // Usage "R" or not "X"
		    else if(type.equals("R")) {
			// Get the selected value
			String selectedValue = (String)v.get(1);
			// Values
			cb.addItem("");
			for(int i=2;i<v.size();i++) {
			    Object o = v.get(i);
			    if(o instanceof TableValueNode) {
				cb.addItem(((TableValueNode)o).getValue());
			    }
			    else if(o instanceof ValueNode) {
				cb.addItem(((ValueNode)o).getValue());
			    }
			    else if(o instanceof String) {
				cb.addItem(o);
			    }
			    else {
				System.err.println("A case has not been processed " + o.getClass());
				System.exit(0);
			    }
			}
			if(selectedValue == null) {
			    //cb.insertItemAt("No Config Value Selected", 0);
			    //cb.setSelectedIndex(0);
			}
			else {
			    //cb.setSelectedItem(selectedValue);
			    // Check if the selectedValue exists
			    boolean exists = false;
			    for(int i=0;i<cb.getItemCount() && !exists;i++) {
				exists = selectedValue.equals(cb.getItemAt(i));
			    }
			    if(!exists) {
				cb.addItem(selectedValue);
			    }
			}
			cb.setBackground(new Color(255,236,139));
			return cb;
		    }
		    // Usage "X"
		    else if(type.equals("X")) {
			// Return a JLabel
			JLabel lblX = new JLabel(" " + (String)v.get(1)) {
				public void paint(Graphics g) {
				    Color bColor;
				    g.setColor(new Color(190, 190, 190));
				    // Draw a rectangle in the background of the cell
				    g.fillRect(0, 0, getWidth(), getHeight());
				    super.paint(g);
				}
			    };
			lblX.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
			return lblX;
		    }
		}
	    }
	    //	    if(true) {
		JLabel lblOther = new JLabel(" ") {
			public void paint(Graphics g) {
			    Color bColor;
			    g.setColor(Color.WHITE);
			    // Draw a rectangle in the background of the cell
			    g.fillRect(0, 0, getWidth(), getHeight());
			    super.paint(g);
			}
		    };
		//lblOther.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		return lblOther;
		//	    }
	}

	public Object getCellEditorValue() { return null; }
	public boolean isCellEditable(EventObject e) { return true; }
	public boolean shouldSelectCell(EventObject anEvent) { return false; }

	public boolean stopCellEditing() {
	    fireEditingStopped();
	    return true;
	}
	
	public void cancelCellEditing() {
	    fireEditingCanceled();
	}

	public void addCellEditorListener(CellEditorListener l) {
	    listenerList.add(CellEditorListener.class, l);
	}
	
	public void removeCellEditorListener(CellEditorListener l) {
	    listenerList.remove(CellEditorListener.class, l);
	}

	/**
	 * Handles event
	 * @param evt The Event
	 */
	public void actionPerformed(ActionEvent evt) {
	    if(evt.getSource() == cb) {
		if(table != null && cb.getSelectedItem() != null) {
		    if(!((String)cb.getSelectedItem()).equals("")) {
			//System.out.println("Click or new value: " + cb.getSelectedItem() + " @ #" + cb.getSelectedIndex());
			table.setValueAt(new Integer(cb.getSelectedIndex()), row, column);
			table.setValueAt((String)cb.getSelectedItem(), row, column);
		    }

		    //model.fireTableStructureChanged();
		}
	    }
	    /*	    else if(evt.getSource() == tf) {
		if(table != null) {
		    table.setValueAt(new Integer(-1), row, column);
		    table.setValueAt(new String(tf.getText()), row, column);
		    //model.fireTableStructureChanged();
		}
		}*/
	}
    }
}
