/**
 * This class is a dialog to select the Settings
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.awt.Color;

// AWT
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JRadioButton;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.DefaultValue;
import gov.nist.hl7maker.util.MVCArg;

public class ConfigureSettingsDialog extends JDialog implements Observer, ActionListener, ListSelectionListener {

    //private JRadioButton[] rbs;

    private JComboBox cbVersion;
    private JCheckBox chkValidate;
    //    private JCheckBox chkHL7Compliant;
    private JRadioButton rNo;
    private JRadioButton rLoose;
    private JRadioButton rStrict;
    private JTextField txtStar;
    private JButton btnOK;
    private JButton btnClose;

    private JComboBox cbDatatype;
    private JScrollPane listScrollPane;
    private JList lstValue;
    private JButton btnNew;
    private JButton btnAdd;
    private JButton btnRemove;
    private JButton btnEdit;
    private JButton btnCancel;
    private JTextField txtValue;
    private JComboBox cbCategory;
/*    private JButton btnSave;
      private JButton btnClose;*/
    private Vector vDV;
    private int lastDatatypeSelectedIndex;
    private DefaultValue dv;
    private DefaultValue dvOri;

    private Session session;

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     */
    public ConfigureSettingsDialog(JFrame frame, Session ses) {
	super(frame, "Settings", true);
	setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	session = ses;

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	JPanel p = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));

	JPanel pLeft = new JPanel();
	pLeft.setLayout(new BoxLayout(pLeft, BoxLayout.PAGE_AXIS));

	// Version
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	p0.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "HL7 Version"));
	JLabel lbl0 = new JLabel(" Version: ");
	cbVersion = new JComboBox();
	//cbVersion.addItem("2.3");
	//cbVersion.addItem("2.3.1");
	cbVersion.addItem("2.4");
	//cbVersion.addItem("2.5");
	//cbVersion.addItem("2.6");
	p0.add(Box.createRigidArea(new Dimension(10, 0)));
	p0.add(lbl0);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));
	p0.add(cbVersion);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	/*
	  ButtonGroup group = new ButtonGroup();
	  String[] versions = {"v2.3", "v2.3.1", "v2.4", "v2.5", "v2.6"};
	  JPanel[] ps = new JPanel[versions.length];
	  rbs = new JRadioButton[versions.length];
	  for(int i=0;i<versions.length;i++) {
	  JRadioButton rb = new JRadioButton(versions[i]);
	  group.add(rb);
	  rbs[i] = rb;
	  if(i != 2) {
	  rbs[i].setEnabled(false);
	  }
	  
	  ps[i] = new JPanel();
	  ps[i].setLayout(new BoxLayout(ps[i], BoxLayout.LINE_AXIS));
	  if(i == 1) {
	  ps[i].add(Box.createRigidArea(new Dimension(10, 0)));
	  }
	  ps[i].add(rbs[i]);
	  p.add(ps[i]);
	  }
	*/

	// Validation
	JPanel pValidation = new JPanel();
	pValidation.setLayout(new BoxLayout(pValidation, BoxLayout.PAGE_AXIS));
	pValidation.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Validation"));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	p1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Validate Messages"));
	chkValidate = new JCheckBox("Validate", false);
	chkValidate.addActionListener(this);
	chkValidate.setEnabled(false);
	p1.add(chkValidate);
	p1.add(Box.createRigidArea(new Dimension(140, 0)));
	pValidation.add(p1);

	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.PAGE_AXIS));
	p5.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Validate Profile"));
	ButtonGroup group = new ButtonGroup();
	rNo = new JRadioButton("Do Not Validate");
	rLoose = new JRadioButton("Loose Validation");
	rStrict = new JRadioButton("Strict Validation");
	group.add(rNo);
	group.add(rLoose);
	group.add(rStrict);
	p5.add(rNo);
	p5.add(rLoose);
	p5.add(rStrict);
	JComponent b = (JComponent)Box.createRigidArea(new Dimension(210,1));
	b.setAlignmentX(0f);
	p5.add(b);
	pValidation.add(p5);

	// Value of the * in a profile
	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	p4.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "General"));
	JLabel lbl3 = new JLabel(" Set cardinality upperbound (*) to ");
	txtStar = new JTextField("");
	p4.add(lbl3);
	p4.add(Box.createRigidArea(new Dimension(3, 0)));
	p4.add(txtStar);

	// OK Cancel
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	btnOK = new JButton("OK");
	btnOK.setBackground(new Color(204, 204, 255));
	btnOK.addActionListener(this);
	p2.add(Box.createRigidArea(new Dimension(249,0)));
	p2.add(btnOK);
	p2.add(Box.createRigidArea(new Dimension(5,0)));
	btnClose = new JButton("Cancel");
	btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p2.add(btnClose);

	pLeft.add(p0);
	pLeft.add(pValidation);
	pLeft.add(p4);

	JPanel pRight = new JPanel();
	pRight.setLayout(new BoxLayout(pRight, BoxLayout.PAGE_AXIS));
	pRight.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Default Data Values"));

	JPanel p10 = new JPanel();
	p10.setLayout(new BoxLayout(p10, BoxLayout.LINE_AXIS));
	JLabel lblDatatype = new JLabel("Datatype: ");
	p10.add(lblDatatype);
	cbDatatype = new JComboBox();
	cbDatatype.addActionListener(this);
	p10.add(cbDatatype);

	JPanel p11 = new JPanel();
	p11.setLayout(new BoxLayout(p11, BoxLayout.LINE_AXIS));

	lstValue = new JList();
	lstValue.addListSelectionListener(this);
	listScrollPane = new JScrollPane(lstValue);
	listScrollPane.setPreferredSize(new Dimension(200,140));
	p11.add(listScrollPane);

	JPanel p12 = new JPanel();
	p12.setLayout(new BoxLayout(p12, BoxLayout.LINE_AXIS));

	btnNew = new JButton("New");
        btnNew.setBackground(new Color(204, 204, 255));
	btnNew.setEnabled(false);
	btnNew.addActionListener(this);
	p12.add(btnNew);

	p12.add(Box.createRigidArea(new Dimension(10,0)));

	btnAdd = new JButton("Add");
        btnAdd.setBackground(new Color(204, 204, 255));
	btnAdd.setEnabled(false);
	btnAdd.addActionListener(this);
	p12.add(btnAdd);

	p12.add(Box.createRigidArea(new Dimension(10,0)));

	btnRemove = new JButton("Remove");
        btnRemove.setBackground(new Color(204, 204, 255));
	btnRemove.setEnabled(false);
	btnRemove.addActionListener(this);
	p12.add(btnRemove);

	p12.add(Box.createRigidArea(new Dimension(10,0)));

	btnEdit = new JButton("Edit");
        btnEdit.setBackground(new Color(204, 204, 255));
	btnEdit.setEnabled(false);
	btnEdit.addActionListener(this);
	p12.add(btnEdit);

	p12.add(Box.createRigidArea(new Dimension(10,0)));

	btnCancel = new JButton("Cancel");
        btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.setEnabled(false);
	btnCancel.addActionListener(this);
	p12.add(btnCancel);

	JPanel p13 = new JPanel();
	p13.setLayout(new BoxLayout(p13, BoxLayout.LINE_AXIS));
	txtValue = new JTextField("");
	txtValue.setEditable(false);
	p13.add(txtValue);
	p13.add(Box.createRigidArea(new Dimension(10, 0)));
	cbCategory = new JComboBox();
	cbCategory.addItem("DefaultValid");
	//cbCategory.addItem("DefaultInvalid");
	cbCategory.setEnabled(false);
	p13.add(cbCategory);

	p10.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
	p11.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
	p12.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
	p13.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));

/*	JPanel p14 = new JPanel();
	p14.setLayout(new BoxLayout(p14, BoxLayout.LINE_AXIS));
	btnSave = new JButton("Save");
	btnSave.setBackground(new Color(204, 204, 255));
	btnSave.addActionListener(this);
	p14.add(btnSave);
	p14.add(Box.createRigidArea(new Dimension(10, 0)));
	btnClose = new JButton("Cancel");
	btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p14.add(btnClose);*/

	JPanel p15 = new JPanel();
	p15.setLayout(new BoxLayout(p15, BoxLayout.PAGE_AXIS));
	p15.add(p10);
	p15.add(Box.createRigidArea(new Dimension(0, 10)));
	p15.add(p11);
	p15.add(Box.createRigidArea(new Dimension(0, 10)));
	p15.add(p12);
	p15.add(Box.createRigidArea(new Dimension(0, 10)));
	p15.add(p13);
	p15.add(Box.createRigidArea(new Dimension(0, 10)));
/*	p15.add(p14);
	p15.add(Box.createRigidArea(new Dimension(0, 10)));*/

	pRight.add(p15);

	JPanel pOption = new JPanel();
	pOption.setLayout(new BoxLayout(pOption, BoxLayout.LINE_AXIS));
	pOption.add(pLeft);
	pOption.add(Box.createRigidArea(new Dimension(10, 0)));
	pOption.add(pRight);

	p0.setAlignmentX(0f);
	p1.setAlignmentX(0f);
	p2.setAlignmentX(0f);
	p4.setAlignmentX(0f);
	p5.setAlignmentX(0f);
	pOption.setAlignmentX(0f);

	p.add(pOption);
	p.add(Box.createRigidArea(new Dimension(0, 5)));
	p.add(p2);

	p.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p);
    }

    /**
     * Init the dialog
     */
    public void init() {
	String v = session.getVersion();
	cbVersion.setSelectedItem(v);
	boolean bV = session.getValidate();
	//chkValidate.setSelected(bV);
	String sVP = session.getValidateProfile();
	if(sVP.equals("donot")) rNo.setSelected(true);
	else if(sVP.equals("loose")) rLoose.setSelected(true);
	else if(sVP.equals("strict")) rStrict.setSelected(true);
	/*boolean bHL7C = session.getHL7CheckCompliance();
	  chkHL7Compliant.setSelected(bHL7C);*/
	initDV();
    }

    /**
     * Init the DefaultValue GUI
     */
    public void initDV() {
	dv = session.getDefaultValue();
	dvOri = new DefaultValue();
	cbDatatype.removeAllItems();
	vDV = dv.getDefaultValue();
	for(int i=0;i<vDV.size();i++) {
	    cbDatatype.addItem(((DefaultValue.DV)vDV.get(i)).getDatatype());
	}
	cbDatatype.setSelectedIndex(lastDatatypeSelectedIndex);
	Vector vValue = dv.getDefaultValue((String)cbDatatype.getSelectedItem());
	lstValue.setListData(vValue);
	setButton(true, false, false, false, false);
	txtValue.setText("");
	txtValue.setEditable(false);
	cbCategory.setSelectedIndex(0);
	cbCategory.setEnabled(false);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureSettingsDialog");
		String v = ((Session)arg).getVersion();
		cbVersion.setSelectedItem(v);
/*		for(int i=0;i<rbs.length;i++) {
		    if(rbs[i].getText().compareTo(v) == 0) {
			rbs[i].setSelected(true);
		    }
		    }*/
		boolean bV = ((Session)arg).getValidate();
		//chkValidate.setSelected(bV);
		String sVP = ((Session)arg).getValidateProfile();
		if(sVP.equals("donot")) rNo.setSelected(true);
		else if(sVP.equals("loose")) rLoose.setSelected(true);
		else if(sVP.equals("strict")) rStrict.setSelected(true);
		/*boolean bHL7C = ((Session)arg).getHL7CheckCompliance();
		  chkHL7Compliant.setSelected(bHL7C);*/
		txtStar.setText(session.getStar());
	    }
	    else if(arg instanceof MVCArg) {
		if("validate".equals(((MVCArg)arg).getName())) {
		    //chkValidate.setSelected(session.getValidate());
		}
	    }
	}
    }

    /**
     * Handles the event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	String datatype = null;
	int idx = -1;
	DefaultValue.DV tmp = null;

	if(evt.getSource() == btnOK) {
/*	    for(int i=0;i<rbs.length;i++) {
		if(rbs[i].isSelected()) {
		    session.setVersion(rbs[i].getText());
		}
		}*/
	    boolean starError = false;
	    try {
		Integer.parseInt(txtStar.getText());
	    }
	    catch(NumberFormatException nfe) {
		starError = true;
	    }
	    if(starError) {
		JOptionPane.showMessageDialog(null, "The value of the field * must be an integer", "Star Error", JOptionPane.ERROR_MESSAGE);
	    }
	    else {
		session.setVersion((String)cbVersion.getSelectedItem());
		//session.setValidate(chkValidate.isSelected());
		if(rNo.isSelected()) session.setValidateProfile("donot");
		else if(rLoose.isSelected()) session.setValidateProfile("loose");
		else if(rStrict.isSelected()) session.setValidateProfile("strict");
		//session.setHL7CheckCompliance(chkHL7Compliant.isSelected());
		session.setStar(txtStar.getText());	    
		setVisible(false);
	    }
	    dv.save();
	}
	else if(evt.getSource() == btnClose) {
	    session.setDefaultValue(dvOri);
	    setVisible(false);
	}
	else if(evt.getSource() == chkValidate && chkValidate.isSelected()) {
	    if(rNo.isSelected()) rLoose.setSelected(true);
	}
	if(evt.getSource() == btnNew) {
	    setButton(false, true, false, false, true);
	    txtValue.setText("");
	    txtValue.setEditable(true);
	    cbCategory.setSelectedIndex(0);
	    cbCategory.setEnabled(true);
	    lstValue.clearSelection();
	}
	else if(evt.getSource() == btnAdd) {
	    datatype = (String)cbDatatype.getSelectedItem();
	    tmp = dv.new DV();
	    tmp.setDatatype(datatype);
	    idx = vDV.indexOf(tmp);
	    if(idx != -1)
		((DefaultValue.DV)vDV.get(idx)).addValue(txtValue.getText(), (String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnRemove) {
	    datatype = (String)cbDatatype.getSelectedItem();
	    tmp = dv.new DV();
	    tmp.setDatatype(datatype);
	    idx = vDV.indexOf(tmp);
	    if(idx != -1)
		((DefaultValue.DV)vDV.get(idx)).removeValue(txtValue.getText(), (String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnEdit) {
	    DefaultValue.DVItem dvItem = (DefaultValue.DVItem)lstValue.getSelectedValue();
	    dvItem.setValue(txtValue.getText());
	    dvItem.setCategory((String)cbCategory.getSelectedItem());
	    lastDatatypeSelectedIndex = cbDatatype.getSelectedIndex();
	    init();
	}
	else if(evt.getSource() == btnCancel) {
	    initDV();
	}
	else if(evt.getSource() == cbDatatype) {
	    if(cbDatatype.getSelectedIndex() != -1) {
		Vector vValue = dv.getDefaultValue((String)cbDatatype.getSelectedItem());
		lstValue.setListData(vValue);
		txtValue.setText("");
		txtValue.setEditable(false);
		cbCategory.setSelectedIndex(0);
		cbCategory.setEnabled(false);
	    }
	}
    }

    /**
     * Handles a click on a JList
     * @param e The event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(lstValue.getSelectedIndex() != -1) {
	    DefaultValue.DVItem dvItem = (DefaultValue.DVItem)lstValue.getSelectedValue();
	    txtValue.setText(dvItem.getValue());
	    txtValue.setEditable(true);
	    cbCategory.setSelectedItem(dvItem.getCategory());
	    cbCategory.setEnabled(true);
	    setButton(true, false, true, true, true);
	}
        else {
	    txtValue.setEditable(false);
	    cbCategory.setEnabled(false);
	    setButton(true, false, false, false, false);
	}
    }

    /**
     * Set the enabled property for buttons
     * @param bNew
     * @param bAdd
     * @param bRemove
     * @param bEdit
     * @param bCancel
     */
    private void setButton(boolean bNew, boolean bAdd, boolean bRemove, boolean bEdit, boolean bCancel) {
	btnNew.setEnabled(bNew);
	btnAdd.setEnabled(bAdd);
	btnRemove.setEnabled(bRemove);
	btnEdit.setEnabled(bEdit);
	btnCancel.setEnabled(bCancel);
    }
}
