/*
 * FolderControl.java
 *
 * Created on March 16, 2004, 10:22 PM
 */

package gov.nist.hl7maker.view;

// Swing
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

// Message Maker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.control.ConfigParameters;
import gov.nist.hl7maker.view.init.MessageSetupPanel;
import gov.nist.hl7maker.view.display.DisplayMessageView;
import gov.nist.hl7maker.view.config.DataConfigurationView;
import gov.nist.hl7maker.view.management.MessageManagementView;

/**
 *
 * @author  Rob Snelick
 */
public class FolderControl {
    
    private JTabbedPane mainTabbedPanel;
    private MessageSetupPanel messageConfigFolder;
    private DataConfigurationView dataConfigFolder;
    private DisplayMessageView  messageViewFolder;
    private MessageManagementView messageManagementFolder;

    /** 
     * Creates new form FolderControl 
     * @param ses A Session object
     * @param user A User object
     */
    public FolderControl(Session ses, User user) {
        initComponents(ses, user);
    }
    
    /**
     * Init the components
     * @param ses A Session object
     * @param user A User object
     */
    private void initComponents(Session ses, User user) {
	mainTabbedPanel = new javax.swing.JTabbedPane();
	mainTabbedPanel.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    JTabbedPane tp = (JTabbedPane) e.getSource();  
		    String s = tp.getTitleAt(tp.getSelectedIndex());
		}
	    });
	
	/*
	 * Message Config Folder Initialization
	 */
	messageConfigFolder = new MessageSetupPanel(ses, user);
	
	/*
	 * Data Config Folder Initialization
	 */
	dataConfigFolder = new DataConfigurationView(ses);
	
	/*
	 * Message View Folder Initialization
	 */
	messageViewFolder = new DisplayMessageView(ses, user);
	
	/*
	 * Message Management Folder Initialization
	 */
	messageManagementFolder = new MessageManagementView();
	messageManagementFolder.setSession(ses);
	
	/*
	 * Add the Folders to the root Tabbed Panel
	 *
	 */ 
	mainTabbedPanel.addTab("Initialization", messageConfigFolder);
	mainTabbedPanel.addTab("Data Configuration", dataConfigFolder);
	mainTabbedPanel.addTab("Message View", messageViewFolder);
	mainTabbedPanel.addTab("Message Management", messageManagementFolder);
    }

    /**
     * Return the JTabbedPane
     * @return The JTabbedPane
     */    
    public JTabbedPane getFolderMainPanel() {
	return mainTabbedPanel;
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	dataConfigFolder.setSession(ses);
    }
}
