/*
 * GUIControl.java
 *
 * 
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.IOException;

// AWT
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// Message Maker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.util.Constants;
//import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.view.config.TableDataModel;

// Swing
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JFrame;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

/**
 *
 * @author  Rob Snelick
 */
public class GUIControl extends JFrame { 
    
    private MainMenu theMainMenu;
    private FolderControl theFolders;
    private Session session;
    private User user;
    
    /**
     * Creates new form GUIControl 
     */
    public GUIControl() {
	initComponents();
    }
    
    /**
     * Init the components
     */
    private void initComponents() {
	setTitle("Message Maker 1.4");
	setIconImage(java.awt.Toolkit.getDefaultToolkit().getImage("images" + File.separator + "mm.gif"));
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    exitForm(evt);
		}
	    });

	/*
	 * Create a user object
	 */
	try {
	    user = new User();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}

	/*
	 * Create a session
	 */
	session = new Session();

	/*
	 * Create and add the Menu Bar to the main window.
	 */
	theMainMenu = new MainMenu(this, session, user);
	setJMenuBar(theMainMenu.getJMenuBar());
	
	/*
	 * Create and add the Folders to the main window.
	 */
	theFolders  = new FolderControl(session, user);
	
	/*
	 * Load the default session
	 */
	try {
	    session.loadProperties("config" + Constants.FS + "maker.dat");
	    theMainMenu.disableMenu();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}

	getContentPane().add(theFolders.getFolderMainPanel());
	pack();
	// Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension windowSize = getSize();
	setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			   Math.max(0,(screenSize.height-windowSize.height)/2));
    }

    /** 
     * Exit the Application 
     */
    private void exitForm(WindowEvent evt) {
	// Check for unsaved items; add dialog
	if(session.isModified()) {
	    JOptionPane optionPane = new JOptionPane("Do you want to save the current session?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	    JDialog dialog = optionPane.createDialog(this, "Save session");
	    dialog.show();
	    Object selectedValue = optionPane.getValue();
	    if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		try {
		    session.saveProperties(session.getName());
		    user.setMmk1(session.getName());
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		}
	    }
	}
	// Ask for saving unsaved generated messages
	if(user.isUnsavedMsg()) {
	    SaveMsgDialog dlgSaveMsg = new SaveMsgDialog(null);
	    dlgSaveMsg.setLstMsg(user.getUnsavedMsg());
	    dlgSaveMsg.setUser(user);
	    dlgSaveMsg.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgSaveMsg.getSize();
	    dlgSaveMsg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				   Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgSaveMsg.setVisible(true);
	    // Delete unsaved messages
	    user.deleteUnsavedMsg();
	}	
	// Remove All unneeded metadata file
	user.removeAllMD();
	// Save the user object
	try {
	    user.saveProperties();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	System.exit(0);
    }

    /**
     * Clean recursively the reports directory
     * @param The current file
     */
/*    private void clean(File node) {
	if(node.isDirectory()) {
	    File[] c = node.listFiles();
	    for(int i=0;i<c.length;i++)
		clean(c[i]);
	    node.delete();
	}
	// It's a file
	else {
	    node.delete();
	}
	}*/

    /**
     * Get the session object
     * @return The session
     */
    public Session getSession() {
	return session;
    }

    /**
     * Get the user object
     * @return The User
     */
    public User getUser() {
	return user;
    }
}
