/**
 * This class is a model for the MSH Configuration JTable
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Vector;

// Swing
import javax.swing.JTree;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.table.AbstractTableModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeModelListener;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.Profile;
import gov.nist.hl7maker.view.config.ProfileNode;
/*import gov.nist.hl7maker.view.config.ReferenceDBModel;
  import gov.nist.hl7maker.view.config.TableDataModel;*/
import gov.nist.hl7maker.view.config.TableValueNode;
//import gov.nist.hl7maker.view.config.ConfigNode;
import gov.nist.hl7maker.view.config.ValueNode;

public class MSHTableModel extends AbstractTableModel implements TreeModel {

    private DefaultTreeModel treeModel;

    private Class[] classes = {MSHTableModel.class, Vector.class};
    private String[] names = {"Element", "Value"};

    //private Document doc;
    private JTree tree;
    private Session session;

    /*private ReferenceDBModel refDBModel;
    private TableDataModel tableModel;
    private ConfigDataModel configModel;*/

    private Vector vUpdate;
    private String messageType, versionID, triggerEvent, messageStructure;

    /**
     * Init the model
     * @param doc A Document object
     */
    public void init(Document doc) {
	//	this.doc = doc;
	removeEmptyTextElement(doc);
	//	createTreeModel(doc);
	vUpdate = new Vector();
	// messageType
	Element root = doc.getDocumentElement();
	NodeList nl = root.getChildNodes();
	for(int i=0;i<nl.getLength();i++) {
	    Element e = (Element)nl.item(i);
	    if(e.getNodeName().compareTo("HL7v2xStaticDef") == 0) {
		messageType = e.getAttribute("MsgType");
		triggerEvent = e.getAttribute("EventType");
		messageStructure = e.getAttribute("MsgStructID");
		i = nl.getLength();
	    }	    
	}
	// versionID
	versionID = doc.getDocumentElement().getAttribute("HL7Version");
	Profile p = session.getRAMProfile();
	treeModel = new DefaultTreeModel(p.getMSHTree(session.getProfile()));
    }

    /**
     * Set the JTree
     * @param tree The JTree
     */
    public void setTree(JTree tree) {
	this.tree = tree;
    }

    /**
     * Set the session
     * @param session The session
     */
    public void setSession(Session session) {
	this.session = session;
    }

    // Table Model

    /**
     * Return the number of rows
     * @return Number of rows
     */
    public int getRowCount() {
	if(tree == null) return 0;
	return tree.getRowCount();
    }

    /**
     * Return the number of column
     * @return Number of columns
     */
    public int getColumnCount() {
	return classes.length;
    }

    /**
     * Get the value of the object at row i and column j
     * @param row Row position
     * @param column Column position
     */
    public Object getValueAt(int row, int column) {
	TreePath treePath = tree.getPathForRow(row);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    updateRowColumnProfileNode(pn, row);
	    Vector v = new Vector();
	    if(pn.isLeaf()) {
		// HardCoded Values
		if(pn.getName().equals("message type")) {
		    // Type
		    v.add("HC");
		    // Value
		    v.add(messageType);
		}
		else if(pn.getName().equals("version ID")) {
		    // Type
		    v.add("HC");
		    // Value
		    v.add(versionID);
		}
		else if(pn.getName().equals("message structure")) {
		    // Type
		    v.add("HC");
		    // Value
		    v.add(messageStructure);
		}
		else if(pn.getName().equals("trigger event")) {
		    // Type
		    v.add("HC");
		    // Value
		    v.add(triggerEvent);
		}
		// Dynamic Values
		else if(pn.getName().equals("Date/Time")) {
		    // Type
		    v.add("Dynamic");
		    // Selected Value
		    // User Data
		    if(getIndexCell(row) != -5 && getStringValue(row).compareTo("") != 0) {
			v.add(getStringValue(row));
		    }
		    // ConfigData
		    else if(session.getRepositoryDataModel().isRepository(pn)) {
			Vector v1 = session.getRepositoryDataModel().getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			for(int i=0;i<v1.size();i++) {
			    ValueNode vn = (ValueNode)v1.get(i);
			    if(vn.getCategory().equals("Config")) {
				i = v1.size()+1;
				v.add(vn.getValue());
			    }
			}
		    }
/*		    else if(configModel.isConfig(pn)) {
			Vector v1 = configModel.getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			if(v1.size() > 1) {
			    System.err.println("No more than one config values if allowed");
			    System.exit(0);
			}
			if(v1.size() == 1) {
			    v.add(((ConfigNode)v1.get(0)).getValue());
			}
			}*/
		    else {
			v.add(null);
		    }
		    // Other Values
		    String[] dates = {"YYYY", "YYYYMM", "YYYYMMSS"};
		    for(int i=0;i<dates.length;i++) {
			v.add(dates[i]);
		    }		    
		}
		// Usage "R" or not "X"
		else if(!pn.getUsage().equals("X")) {
		    // Type
		    v.add("R");
		    // Selected Value
		    // User Data
		    if(getIndexCell(row) != -5 && getStringValue(row).compareTo("") != 0) {
			v.add(getStringValue(row));
		    }
		    // ConfigData
		    else if(session.getRepositoryDataModel().isRepository(pn)) {
			Vector v1 = session.getRepositoryDataModel().getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			for(int i=0;i<v1.size();i++) {
			    ValueNode vn = (ValueNode)v1.get(i);
			    if(vn.getCategory().equals("Config")) {
				i = v1.size()+1;
				v.add(vn.getValue());
			    }
			}
		    }
		    if(v.size() == 1) {
			v.add(null);
		    }
		    /*else if(configModel.isConfig(pn)) {
			Vector v1 = configModel.getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			if(v1.size() > 1) {
			    System.err.println("No more than one config values if allowed");
			    System.exit(0);
			}
			if(v1.size() == 1) {
			    v.add(((ConfigNode)v1.get(0)).getValue());
			}
			}*/
		    // Other Values
		    // TableData
		    if(session.getTableDataModel().isTable(pn)) {
			Vector v1 = session.getTableDataModel().getValues(pn.getTable());
			for(int i=0;i<v1.size();i++) {
			    ((TableValueNode)v1.get(i)).setLstType((short)-1);
			}
			v.addAll(v1);
		    }
		    // OR RefDB 
		    else if(session.getRepositoryDataModel().isRepository(pn)) {
			v.addAll(session.getRepositoryDataModel().getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2()));
		    }
		    // AND ConfigData
/*		    if(configModel.isConfig(pn)) {
			Vector v1 = configModel.getValues(pn.getName(), pn.getSegment(), pn.getQualifier(), pn.getQualifier2());
			if(v1.size() > 1) {
			    System.err.println("No more than one config values if allowed");
			    System.exit(0);
			}
			if(v1.size() == 1) {
			    // Check if the value exists
			    boolean exists = v.indexOf(((ConfigNode)v1.get(0)).getValue()) != -1;
			    if(!exists) {
				v.add(((ConfigNode)v1.get(0)).getValue());
			    }
			}
			}*/
		}
		// Usage "X"
		else if(pn.getUsage().equals("X")) {
		    // Type
		    v.add("X");
		    // Selected Value
		    v.add("Not Supported by Profile");
		}
	    }
	    return v;
	}
	return null;
    }

    /**
     * Returns Object.class regardless of columnIndex
     * @param columnIndex The column index
     * @return The class
     */
    public Class getColumnClass(int columnIndex) {
	return classes[columnIndex];
    }

    /**
     * Returns a name for the column
     * @param column The column
     * @return The name
     */
    public String getColumnName(int column) {
	return names[column];
    }          

    /**
     * Handles the modification of the model
     * @param aValue The new value
     * @param rowIndex The row
     * @param columnIndex The column
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	if(aValue != null) {
	    TreePath treePath = tree.getPathForRow(rowIndex);
	    if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
		ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
		int idx = vUpdate.indexOf(new UpdateElement(pn));
		// if aValue is a Integer remove the item
		if(idx != -1 && aValue instanceof Integer) {
		    vUpdate.remove(idx);
		    idx = -1;
		}
		if(idx == -1) {
		    vUpdate.add(new UpdateElement(pn, "", ((Integer)aValue).intValue(), rowIndex));
		}
		else {
		    ((UpdateElement)vUpdate.get(idx)).newValue = (String)aValue;
		}
	    }
	}
    }

    // TreeModel interface implementation

    public Object getRoot() {
	return treeModel.getRoot();
    }

    public int getChildCount(Object parent) {
 	return treeModel.getChildCount(parent);
    }

    public Object getChild(Object parent, int index) {
	return treeModel.getChild(parent, index);
    }

    public int getIndexOfChild(Object parent, Object child) {
	return treeModel.getIndexOfChild(parent, child);
    }

    /**
     * Return true if the node is a leaf
     * @param node The Node
     * @return is a leaf?
     */
    public boolean isLeaf(Object node) {
	return treeModel.isLeaf(node);
    }

    public void addTreeModelListener(TreeModelListener listener) {
    }

    public void removeTreeModelListener(TreeModelListener listener) {
    }
    
    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    /**
     * This method removes all TEXT_NODE with empty text
     * @param node The current node
     */    
    private void removeEmptyTextElement(Node node) {
	if(node.hasChildNodes()) {
	    NodeList nl = node.getChildNodes();
	    for(int i=0;i<nl.getLength();i++) {
		Node n = nl.item(i);
		if(n.getNodeValue() != null && n.getNodeValue().trim().equals(""))
		    node.removeChild(n);
	    }
	}
	// Execute the method on the children
	if(node.hasChildNodes()) {
	    NodeList nl = node.getChildNodes();
	    for(int i=0;i<nl.getLength();i++) {
		removeEmptyTextElement(nl.item(i));
	    }
	}
    }

    /**
     * Get the selected index value for a row and column
     * @param row The row
     * @return Get the index
     */
    public int getIndexCell(int row) {
	TreePath treePath = tree.getPathForRow(row);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    int idx = vUpdate.indexOf(new UpdateElement(pn));
	    if(idx != -1) {
		return ((UpdateElement)vUpdate.get(idx)).cbIndex;
	    }
	}

/*
	UpdateElement ue = null;
	for(int i=0;i<vUpdate.size();i++) {
	    ue = (UpdateElement)vUpdate.get(i);
	    if(ue.row == row) {
		return ue.cbIndex;
	    }
	    }*/
	return -5;
    }

    /**
     * Get the string value for a row and column
     * @param row The row
     * @return Get the string value
     */
    public String getStringValue(int row) {
	TreePath treePath = tree.getPathForRow(row);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    int idx = vUpdate.indexOf(new UpdateElement(pn));
	    if(idx != -1) {
		return ((UpdateElement)vUpdate.get(idx)).newValue;
	    }
	}
	return "";
    }

    /**
     * Update the row and column for a profileNode
     * @param pn The ProfileNode
     * @param row The row
     */
    public void updateRowColumnProfileNode(ProfileNode pn, int row) {
	int idx = vUpdate.indexOf(new UpdateElement(pn));
	if(idx != -1) {
	    ((UpdateElement)vUpdate.get(idx)).row = row;
	}
    }

    /**
     * Get the ProfileNode for a row
     * @param row The row
     * @return The profile node
     */
    public ProfileNode getProfileNode(int row) {
	for(int i=0;i<vUpdate.size();i++) {
	    UpdateElement ue = (UpdateElement)vUpdate.get(i);
	    if(ue.row == row) {
		return ue.pn;
	    }
	}
	return null;
    }

    /**
     * Remove the item from the vUpdate vector after being saved.
     * @param row The row
     */
    public void removeUpdatedItem(int row) {
	for(int i=0;i<vUpdate.size();i++) {
	    UpdateElement ue = (UpdateElement)vUpdate.get(i);
	    if(ue.row == row) {
		vUpdate.remove(ue);
	    }
	}
    }

    /**
     * Get the vector of updated element
     * @return The Vector
     */
    public Vector getUpdateElement() {
	return vUpdate;
    }

    /**
     * Clear the update vector
     */
    public void clearUpdate() {
	vUpdate.clear();
    }

    // This inner class handles the update of the model
    public class UpdateElement {

	public ProfileNode pn; // The profile node
	public String newValue; // The new value
	private int cbIndex; // The index of the selected item in a comboBox -1 if unselected
	public int row; // The row of the cell modified

	/**
	 * Constructor
	 * @param pn The profile node
	 */
	public UpdateElement(ProfileNode pn) {
	    this.pn = pn;
	}
	
	/**
	 * Constructor
	 * @param pn The profile node
	 * @param newValue The new value
	 * @param idx The index
	 * @param r The row of the cell modified
	 */
	public UpdateElement(ProfileNode pn, String newValue, int idx, int r) {
	    this.pn = pn;
	    this.newValue = newValue;
	    cbIndex = idx;
	    row = r;
	}

	public boolean equals(Object o) {
	    if(o instanceof UpdateElement) {
		if(pn.equals(((UpdateElement)o).pn)) {
		    return true;
		}
	    }
	    return false;
	}

	public String toString() {
	    return pn + ": " + newValue + " @row " + row;
	}
    }
}
