/*
 * MainMenu.java
 * Provides the main menu and handles actions for user selections.
 * 
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;

// Swing
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JSeparator;
import javax.swing.JButton;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;
import javax.swing.BorderFactory;
import javax.swing.Timer;

// Message Maker
import gov.nist.hl7maker.util.MMKFileFilter;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.MVCArg;
//import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.control.MessageGenerationProperties;
import gov.nist.hl7maker.control.MakerProperties;
import gov.nist.hl7maker.control.MakerMessage;
import gov.nist.hl7maker.message.MessageGenerator;
import gov.nist.hl7maker.util.MMLog;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.CheckResources;
import gov.nist.hl7maker.util.WaitDialog;
import gov.nist.hl7maker.view.display.XMLDirectory;
import gov.nist.hl7maker.view.display.MetaData;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// HAPI
/*import ca.uhn.hl7v2.parser.DefaultXMLParser;
import ca.uhn.hl7v2.parser.PipeParser;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.HL7Exception;*/

/**
/*
 * @author  Rob Snelick
 */
public class MainMenu implements Observer, ActionListener {

    private JMenuBar jMenuBar1;

    private JMenu fileMenu;
    private JMenuItem newItem;
    private JMenuItem openItem;
    private JMenuItem saveAsItem;
    private JMenuItem saveItem;
    private JMenuItem exitItem;
    private JSeparator fileMmkSeparator;
    private JMenuItem fileMmk1;
    private JMenuItem fileMmk2;
    private JMenuItem fileMmk3;
    private JMenuItem fileMmk4;

    //private JMenu editMenu;

    private JMenu configureMenu;
    //private JMenuItem loadProfileMenuItem;
    //private JMenuItem setDefaultMenuItem;
    private JMenuItem loadDataFilesMenuItem;
    private JMenuItem settingsMenuItem;
    private JMenuItem mshMenuItem;
    private JMenuItem tableViewerMenuItem;
    //private JMenuItem defaultValueMenuItem;

    private JMenu messagesMenu;
    private JMenuItem generateMessagesMenuItem;
    private JMenuItem saveMessagesMenuItem;
    //    private JMenuItem loadMessagesMenuItem;
    private JMenuItem deleteMessagesMenuItem;
    private JMenuItem mountMessagesMenuItem;
    private JMenuItem buildSetMessagesMenuItem;

    private JMenu importMenu;

    private JMenu toolsMenu;
    private JMenuItem capsuleMenuItem;
    private JMenuItem logMenuItem;
    private JMenuItem profileMenuItem;

    private JMenu helpMenu;
    private JMenuItem ContentsItem;
    private JSeparator jSeparator1;
    private JMenuItem AboutItem;
    private JMenuItem ContributorsItem;

    private ConfigureCapsuleDialog dlgCapsule;
    private ConfigureLoadDataDialog dlgLoadData;
    private ConfigureSettingsDialog dlgSettings;
    private ConfigureMSHDialog dlgMSH;
    private TableViewer dlgTableViewer;
    private LogDialog dlgLog;
    private ProfileDialog dlgProfile;
    private MonitorGenerationDialog dlgGenerate;
    private ConfigureDefaultValueDialog dlgDefaultValue;
    private AboutDialog dlgAbout;
    private ContributorsDialog dlgContributors;

    private File profile;
    private MessageGenerator generator;

    private JProgressBar progressBar;
    private Timer timer;
    private JButton generateButton;
    private JTextArea taskOutput;
    private JPanel generatePanel; 
    private JDialog generateDlg;
    private final static int THREE_SECONDS = 3000;

    private GUIControl theMainGuiWindow;
    private Session session; // session contains the current list of user settings
    private User user;



    /** 
     * Creates new form GUIControl 
     * @param controlWindow A GUIControl object
     * @param ses A Session object
     * @param user A User object
     */
    public MainMenu(GUIControl controlWindow, Session ses, User user) {
	theMainGuiWindow = controlWindow;
	session = ses;
	this.user = user;
	
	/* Initialize the user interface components */
	initComponents();
	
	// Add a separator and most recent mmk file opened
	addRecentMMK();

	// MVC
	session.addObserver(this);
	session.addObserver(dlgCapsule);
	session.addObserver(dlgMSH);
	session.addObserver(dlgLoadData);
	session.addObserver(dlgSettings);
	session.addObserver(dlgTableViewer);
    }
    
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
	jMenuBar1 = new JMenuBar();
	fileMenu = new JMenu();
	newItem = new JMenuItem();
	openItem = new JMenuItem();
	saveItem = new JMenuItem();
	saveAsItem = new JMenuItem();
	exitItem = new JMenuItem();
	//editMenu = new JMenu();
	configureMenu = new JMenu();
	//loadProfileMenuItem = new JMenuItem();
	//setDefaultMenuItem = new JMenuItem();
	capsuleMenuItem = new JMenuItem();
	settingsMenuItem = new JMenuItem();
	loadDataFilesMenuItem = new JMenuItem();
	mshMenuItem = new JMenuItem();
	tableViewerMenuItem = new JMenuItem();
	//defaultValueMenuItem = new JMenuItem();
	//importMenu = new JMenu();
	toolsMenu = new JMenu();
	logMenuItem = new JMenuItem();
	profileMenuItem = new JMenuItem();
	messagesMenu = new JMenu();
	generateMessagesMenuItem = new JMenuItem();
	saveMessagesMenuItem = new JMenuItem();
	//loadMessagesMenuItem = new JMenuItem();
	deleteMessagesMenuItem = new JMenuItem();
	mountMessagesMenuItem = new JMenuItem();
	buildSetMessagesMenuItem = new JMenuItem();
	helpMenu = new JMenu();
	ContentsItem = new JMenuItem();
	jSeparator1 = new JSeparator();
	AboutItem = new JMenuItem();
	ContributorsItem = new JMenuItem();
	
	fileMenu.setMnemonic('F');
	fileMenu.setText("File");
	fileMenu.setToolTipText("File Menu");
	newItem.setMnemonic('N');
	newItem.setText("New");
	newItem.setToolTipText("New");
	newItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    fileActionPerformed(evt);
		}
	    });
	
	fileMenu.add(newItem);
	
	openItem.setMnemonic('O');
	openItem.setText("Open");
	openItem.setToolTipText("Open");
	openItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    fileActionPerformed(evt);
		}
	    });
	
	fileMenu.add(openItem);
	
	saveItem.setMnemonic('S');
	saveItem.setText("Save");
	saveItem.setToolTipText("Save");
	saveItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    fileActionPerformed(evt);
		}
	    });
	
	fileMenu.add(saveItem);
	
	saveAsItem.setMnemonic('A');
	saveAsItem.setText("SaveAs");
	saveAsItem.setToolTipText("SaveAs");
	saveAsItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    fileActionPerformed(evt);
		}
	    });
	
	fileMenu.add(saveAsItem);
	
	exitItem.setMnemonic('X');
	exitItem.setText("Exit");
	exitItem.setToolTipText("Exit");
	exitItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    fileActionPerformed(evt);
		}
	    });
	
	fileMenu.add(exitItem);
	
	jMenuBar1.add(fileMenu);
	
/*	editMenu.setMnemonic('E');
	editMenu.setText("Edit");
	editMenu.setToolTipText("Edit Menu");
	jMenuBar1.add(editMenu);*/
	
	configureMenu.setMnemonic('C');
	configureMenu.setText("Configure");
	configureMenu.setToolTipText("Configure");

/*
		    mshMenuItem.setEnabled(false);
		    tableViewerMenuItem.setEnabled(false);
*/
/*	loadProfileMenuItem.setMnemonic('L');
	loadProfileMenuItem.setText("Load Profile");
	loadProfileMenuItem.setToolTipText("Load Profile");
	loadProfileMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });
	
	configureMenu.add(loadProfileMenuItem);
	
	setDefaultMenuItem.setMnemonic('S');
	setDefaultMenuItem.setText("Set Default Fields");
	setDefaultMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });
	
	    configureMenu.add(setDefaultMenuItem);*/

	settingsMenuItem.setMnemonic('O');
	settingsMenuItem.setText("Options");
	settingsMenuItem.setToolTipText("Options");
	settingsMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });
	
	configureMenu.add(settingsMenuItem);

	loadDataFilesMenuItem.setMnemonic('D');
	loadDataFilesMenuItem.setText("Data Files");
	loadDataFilesMenuItem.setToolTipText("Data Files");
	loadDataFilesMenuItem.setEnabled(false);
	loadDataFilesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });
	
	configureMenu.add(loadDataFilesMenuItem);

	mshMenuItem.setMnemonic('M');
	mshMenuItem.setText("MSH Segment");
	mshMenuItem.setToolTipText("MSH Segment");
	mshMenuItem.setEnabled(false);
	mshMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });

	configureMenu.add(mshMenuItem);

	tableViewerMenuItem.setMnemonic('T');
	tableViewerMenuItem.setText("Tables");
	tableViewerMenuItem.setToolTipText("Tables");
	tableViewerMenuItem.setEnabled(false);
	tableViewerMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });

	configureMenu.add(tableViewerMenuItem);

/*	defaultValueMenuItem.setMnemonic('V');
	defaultValueMenuItem.setText("Default Value");
	defaultValueMenuItem.setToolTipText("Default Value");
	//defaultValueMenuItem.setEnabled(false);
	defaultValueMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });

	    configureMenu.add(defaultValueMenuItem);*/

	jMenuBar1.add(configureMenu);
	
	/* Messages Menu */
	messagesMenu.setMnemonic('M');
	messagesMenu.setText("Messages");
	messagesMenu.setToolTipText("Messages Menu");
	
	/* Generate Message Menu Item */
	generateMessagesMenuItem.setMnemonic('G');
	generateMessagesMenuItem.setText("Generate...");
	generateMessagesMenuItem.setToolTipText("Generate Messages");
	generateMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    messagesActionPerformed(evt);
		}
	    });
	messagesMenu.add(generateMessagesMenuItem);
	
	/* Save Message Menu Item */
	saveMessagesMenuItem.setMnemonic('S');
	saveMessagesMenuItem.setText("Save");
	saveMessagesMenuItem.setToolTipText("Save Messages");
	saveMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    messagesActionPerformed(evt);
		}
	    });
	messagesMenu.add(saveMessagesMenuItem);
	
	/* Load Message Menu Item */
	/*	loadMessagesMenuItem.setMnemonic('L');
		loadMessagesMenuItem.setText("Load");
		loadMessagesMenuItem.setToolTipText("Load Messages");
		loadMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		messagesActionPerformed(evt);
		}
		});
		messagesMenu.add(loadMessagesMenuItem);
		
		messagesMenu.add(new JSeparator());*/
	
	/* Delete Message Menu Item */
	deleteMessagesMenuItem.setMnemonic('D');
	deleteMessagesMenuItem.setText("Delete");
	deleteMessagesMenuItem.setToolTipText("Delete Messages");
	deleteMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    messagesActionPerformed(evt);
		}
	    });
	messagesMenu.add(deleteMessagesMenuItem);
	
	/* Mount Directory Message Menu Item */
	mountMessagesMenuItem.setMnemonic('M');
	mountMessagesMenuItem.setText("Mount Directory");
	mountMessagesMenuItem.setToolTipText("Mount Directory");
	mountMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    messagesActionPerformed(evt);
		}
	    });
	messagesMenu.add(mountMessagesMenuItem);
	
	/* Build Set Message Menu Item */
	buildSetMessagesMenuItem.setMnemonic('B');
	buildSetMessagesMenuItem.setText("Build Set");
	buildSetMessagesMenuItem.setToolTipText("Build Set of Messages");
	buildSetMessagesMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    messagesActionPerformed(evt);
		    }
		});
	//messagesMenu.add(buildSetMessagesMenuItem);
	buildSetMessagesMenuItem.setEnabled(false);

	jMenuBar1.add(messagesMenu);

	/* Import  Menu */
	/*importMenu.setMnemonic('I');
	importMenu.setText("Import");
	importMenu.setToolTipText("Import Menu");
	importMenu.setEnabled(false);
	jMenuBar1.add(importMenu);*/

	/* Tools  Menu */
	toolsMenu.setMnemonic('T');
	toolsMenu.setText("Tools");
	toolsMenu.setToolTipText("Tools Menu");
	//toolsMenu.setEnabled(false);

	capsuleMenuItem.setMnemonic('S');
	capsuleMenuItem.setText("View Settings ...");
	capsuleMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    configureActionPerformed(evt);
		}
	    });
	
	toolsMenu.add(capsuleMenuItem);

	logMenuItem.setText("View log ...");
	logMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    toolsActionPerformed(evt);
		}
	    });
	toolsMenu.add(logMenuItem);

	profileMenuItem.setText("View profile ...");
	profileMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    toolsActionPerformed(evt);
		}
	    });
	profileMenuItem.setEnabled(false);
	toolsMenu.add(profileMenuItem);

	jMenuBar1.add(toolsMenu);

	/* Help  Menu */
	helpMenu.setMnemonic('H');
	helpMenu.setText("Help");
	helpMenu.setToolTipText("Help Menu");

	ContentsItem.setMnemonic('C');
	ContentsItem.setText("Contents");
	ContentsItem.setToolTipText("Contents");
	//	helpMenu.add(ContentsItem);

	//	helpMenu.add(jSeparator1);

	AboutItem.setMnemonic('A');
	AboutItem.setText("About");
	AboutItem.setToolTipText("About");
	AboutItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    helpActionPerformed(evt);
		}
	    });
	helpMenu.add(AboutItem);

	ContributorsItem.setMnemonic('C');
	ContributorsItem.setText("Contributors");
	ContributorsItem.setToolTipText("Contributors");
	ContributorsItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    helpActionPerformed(evt);
		}
	    });
	helpMenu.add(ContributorsItem);

	jMenuBar1.add(helpMenu);

	dlgCapsule = new ConfigureCapsuleDialog(theMainGuiWindow);
	dlgLoadData = new ConfigureLoadDataDialog(theMainGuiWindow, session, user);
	dlgSettings = new ConfigureSettingsDialog(theMainGuiWindow, session);
	dlgMSH = new ConfigureMSHDialog(theMainGuiWindow);
	dlgTableViewer = new TableViewer(theMainGuiWindow);
	dlgTableViewer.setSession(session);
	//dlgDefaultValue = new ConfigureDefaultValueDialog(theMainGuiWindow, session);
	dlgLog = new LogDialog();
	dlgProfile = new ProfileDialog();
	dlgGenerate = new MonitorGenerationDialog(theMainGuiWindow, session, user);
	dlgAbout = new AboutDialog(theMainGuiWindow);
	dlgContributors = new ContributorsDialog(theMainGuiWindow);
    }
    
    /**
     * Get the JMenuBar
     * @return The JMenuBar
     */
    public JMenuBar getJMenuBar() {
	return jMenuBar1;
    }

    /**
     * Handles action for File items
     * @param evt The event
     */
    private void fileActionPerformed(ActionEvent evt) {
        // Create a file chooser
	JFileChooser fc = new JFileChooser(user.getOpenedDir());
	// Set the chooser to accept only MMK files.
	fc.addChoosableFileFilter(new MMKFileFilter());
	fc.setAcceptAllFileFilterUsed(false);
	
	// New
        if(evt.getSource() == newItem) {
	    try {
		session.loadProperties("config" + Constants.FS + "maker.dat");
		disableMenu();

		//		theMainGuiWindow.loadSession(session);
		//session.setModified(false);
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
	    }
	}
	
	// Open
	else if(evt.getSource() == openItem) {
	    int returnVal = fc.showOpenDialog(theMainGuiWindow);

	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		JDialog dlgWait = new WaitDialog("Load Profile and Generate the Data Repository ...");
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dlgWait.getSize();
		dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				    Math.max(0,(screenSize.height-windowSize.height)/2));
		
		final JDialog fDlg = dlgWait;
		final JFileChooser fFc = fc;
		Thread t = new Thread() {
			public void run() {
			    try {
				// Read the file and create a Session object
				session.loadProperties(fFc.getSelectedFile());
				fDlg.dispose();
			    }
			    catch(IOException ioe) {
				System.err.println("IOException " + ioe.getMessage());
			    }
			}
		    };
		t.start();
		
		dlgWait.show();
		
		user.setOpenedDir(fc.getSelectedFile().getParent());
		user.setMmk1(fc.getSelectedFile().getPath());
		addRecentMMK();
	    } 
	}
	
	// Save
	else if(evt.getSource() == saveItem) {
	    try {
		if(session.getName() == null || session.getName().compareTo("") == 0) {
		    int returnVal = fc.showSaveDialog(theMainGuiWindow);
		    
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
			session.setName(fc.getSelectedFile().getPath());
			session.saveProperties(fc.getSelectedFile());
			user.setOpenedDir(fc.getSelectedFile().getParent());
			user.setMmk1(fc.getSelectedFile().getPath());
			addRecentMMK();
		    }
		}
		else {
		    session.saveProperties(session.getName());
		    user.setOpenedDir(new File(session.getName()).getParent());
		    user.setMmk1(session.getName());
		    addRecentMMK();
		}
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
	    }
	}
	
	// Save As
	else if(evt.getSource() == saveAsItem) {
	    int returnVal = fc.showDialog(theMainGuiWindow, "Save As");

	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		try {
		    // Check this is a mmk file
		    File f = fc.getSelectedFile();
		    if(!f.getPath().endsWith(".mmk")) {
			f = new File(fc.getSelectedFile().getPath() + ".mmk");
		    }
		    session.setName(f.getPath());
		    session.saveProperties(f);
		    session.loadProperties(f);
		    user.setOpenedDir(f.getParent());
		    user.setMmk1(f.getPath());
		    addRecentMMK();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		}
	    } 
	}
          
	else if(evt.getSource() == exitItem) {
	    // Check for unsaved items; add dialog
	    if(session.isModified()) {
		JOptionPane optionPane = new JOptionPane("Do you want to save the current session?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
		JDialog dialog = optionPane.createDialog(theMainGuiWindow, "Save session");
		dialog.show();
		Object selectedValue = optionPane.getValue();
		if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		    try {
			session.saveProperties(session.getName());
			user.setMmk1(session.getName());
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
		    }
		}
	    }

	    // Ask for saving unsaved generated messages
	    if(user.isUnsavedMsg()) {
		SaveMsgDialog dlgSaveMsg = new SaveMsgDialog(null);
		dlgSaveMsg.setLstMsg(user.getUnsavedMsg());
		dlgSaveMsg.setUser(user);
		dlgSaveMsg.pack();
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dlgSaveMsg.getSize();
		dlgSaveMsg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				       Math.max(0,(screenSize.height-windowSize.height)/2));
		dlgSaveMsg.setVisible(true);
		// Delete unsaved messages
		user.deleteUnsavedMsg();
	    }
	    // Remove All unneeded metadata file
	    user.removeAllMD();

	    // Save the user object
	    try {
		user.saveProperties();
	    }
	    catch(IOException ioe) {
		System.err.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
/*	    CustomHAPI cHAPI = new CustomHAPI();
	    cHAPI.clean();*/
	    //clean(new File("reports"));

	    System.exit(0);
	}
    }

    /**
     * Handles action for Configure items
     * @param evt The event
     */
    private void configureActionPerformed(ActionEvent evt) {
	if(evt.getSource() == capsuleMenuItem) {
	    dlgCapsule.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgCapsule.getSize();
	    dlgCapsule.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgCapsule.setVisible(true);
	}
	else if(evt.getSource() == settingsMenuItem) {
	    dlgSettings.init();
	    dlgSettings.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgSettings.getSize();
	    dlgSettings.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgSettings.setVisible(true);
	}
	else if(evt.getSource() == loadDataFilesMenuItem) {
	    dlgLoadData.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgLoadData.getSize();
	    dlgLoadData.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				    Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgLoadData.init();
	    dlgLoadData.setVisible(true);
	}
	else if(evt.getSource() == mshMenuItem) {
	    session.setProfile(session.getProfile());
	    dlgMSH.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgMSH.getSize();
	    dlgMSH.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				    Math.max(0,(screenSize.height-windowSize.height)/2));
	    //dlgMSH.init();
	    dlgMSH.setVisible(true);
	}
	else if(evt.getSource() == tableViewerMenuItem) {
	    // Load the Table Data
	    /*session.setHL7Tables(session.getHL7Tables());
	    session.setUserTables(session.getUserTables());
	    session.setExternalTables(session.getExternalTables());
	    session.setLocalTable(session.getLocalTable(), false);*/
	    //session.loadTables();
	    dlgTableViewer.pack();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgTableViewer.getSize();
	    dlgTableViewer.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			       Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgTableViewer.init();
	    dlgTableViewer.setVisible(true);
	}
/*	else if(evt.getSource() == defaultValueMenuItem) {
	    dlgDefaultValue.init();
	    dlgDefaultValue.pack();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgDefaultValue.getSize();
	    dlgDefaultValue.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			       Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgDefaultValue.init();
	    dlgDefaultValue.setVisible(true);

	    }*/
    }
    
    /**
     * Handles action for Message items
     * @param evt The event
     */
    private void messagesActionPerformed(ActionEvent evt) {
	if (evt.getSource() == generateMessagesMenuItem) {
	    // Check if there are previous generated messages
	    //try {
	    boolean show = true;
	    String mdFile = session.getMsgPath() + Constants.FS + "TD_" + new File(session.getProfile()).getName();
	    File mdF = new File(mdFile);
	    if(mdF.exists()) {
		Object[] options = {"Overwrite previous messages", "Change Message Path", "Cancel"};
		JOptionPane optionPane = new JOptionPane("Messages from a previous generation will be overwritten. Do you want to overwrite these message or change the Message Path directory?\nAlternatively, you can select Cancel and use the file system utilities to move or rename the messages.",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Cancel");
		JDialog dialog = optionPane.createDialog(theMainGuiWindow, "New Generation Process");
		dialog.show();
		Object selectedValue = optionPane.getValue();
		
		if(selectedValue != null) {
		    if(((String)selectedValue).compareTo(options[0]) == 0) {
			// Overwrite the files
			user.deleteMsg(mdFile);
			user.removeAllMD();
		    }
		    else if(((String)selectedValue).compareTo(options[1]) == 0) {
			// Change the message path
			JFileChooser fc = new JFileChooser(new File(session.getMsgPath()).getParent());
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int returnVal = fc.showOpenDialog(theMainGuiWindow);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
			    if (fc.getSelectedFile().isDirectory()) {
				session.setMsgPath(fc.getSelectedFile().getPath());
				/*			    MessageGenerationProperties prop = new MessageGenerationProperties(session);
				// Set the options
				prop.setOptionsFileProperty(session.generateMGOptions());
				prop.setMessageControlIdProperty(dlgMSH.getMessageControlId());
				generator.setOptions(prop.getProperties());
				// Set the output dir
				generator.setOutputDir(session.getMsgPath());
				// Set the meta file
				generator.setMetaFile("MD_" + new File(session.getProfile()).getName());*/
			    }
			}
		    }
		    else if(((String)selectedValue).compareTo(options[2]) == 0) {
			dialog.dispose();
			show = false;
		    }
		}
	    }
	    // Create the messages directory if it doesn't exist
	    File f = new File(session.getMsgPath());
	    if(!f.exists()) {
		f.mkdir();
	    }

	    // Check that all the resources are available before the generation
	    String check = CheckResources.checkSession(session);
	    if(!check.equals("")) {
		JOptionPane.showMessageDialog(null, "Some resources are missing:\n" + check, "Resources errors", JOptionPane.ERROR_MESSAGE);
		show = false;
	    }
	    if(show) {
		JDialog dlgWait = new WaitDialog("Adding default values to the repository for missing data elements ...");
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dlgWait.getSize();
		dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				    Math.max(0,(screenSize.height-windowSize.height)/2));
		
		final JDialog fDlg = dlgWait;
		//final String fUserMmk = userMmk;
		Thread t = new Thread() {
			public void run() {
			    //try {
				dlgGenerate.init();
				fDlg.dispose();
				dlgGenerate.pack();
				// Center the window
				Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
				Dimension windowSize = dlgGenerate.getSize();
				dlgGenerate.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
							Math.max(0,(screenSize.height-windowSize.height)/2));
				dlgGenerate.setVisible(true);
				/*}
			    catch(IOException ioe) {
				System.err.println("IOException " + ioe.getMessage());
				ioe.printStackTrace();
				}*/
			}
		    };
		t.start();
		
		dlgWait.show();
	    }
	    /*	    if(generateDlg != null && show) {
		    generateDlg.show();
		    }*/
	    /*
	    if (generateDlg == null) {
		generateDlg = new JDialog(theMainGuiWindow, "Generate Messages", true);

		//Create the demo's UI.
		generatePanel = new JPanel();//new BoxLayout(generatePanel, BoxLayout.PAGE_AXIS));

		JPanel p0 = new JPanel();
		//p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));

		generateButton = new JButton("Start");
		generateButton.setActionCommand("start");
		generateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
			    generateActionPerformed(evt);
			}
		    });
		
		progressBar = new JProgressBar(0, session.getVolume());
		progressBar.setValue(0);
		progressBar.setStringPainted(true);

		taskOutput = new JTextArea(5, 20);
		taskOutput.setMargin(new Insets(5,5,5,5));
		taskOutput.setEditable(false);
		taskOutput.setCursor(null); 

		JPanel panel = new JPanel();
		panel.add(generateButton);
		panel.add(progressBar);

		generatePanel.add(panel, BorderLayout.PAGE_START);
		generatePanel.add(new JScrollPane(taskOutput), BorderLayout.CENTER);
		generatePanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
*/
		/* 
		 * Create a timer. Report progress periodically. Upon termination or
		 * sucessful completion, reset the generator settings.
		 */

/*		timer = new Timer(THREE_SECONDS, new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
			    progressBar.setValue(generator.getProgressValue());
			    String s = generator.getStatusMessage();
			    if (s != null) {
				taskOutput.append(s + "\n");
				taskOutput.setCaretPosition(taskOutput.getDocument().getLength());
			    }
			    if (generator.isDone()) {
				Toolkit.getDefaultToolkit().beep();
				timer.stop();
				generateButton.setText("Close");
				generatePanel.setCursor(null); //turn off the wait cursor
				progressBar.setValue(0);
				generator.reset();
			    }
			}
			});*/
/*
		generateDlg.setContentPane(generatePanel);
		generateDlg.pack();
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = generateDlg.getSize();
		generateDlg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
		Math.max(0,(screenSize.height-windowSize.height)/2));*/
	
	}
	else if(evt.getSource() == saveMessagesMenuItem) {
	    session.fireEvent(new MVCArg("saveMsg",""));
	}
	else if(evt.getSource() == deleteMessagesMenuItem) {
	    session.fireEvent(new MVCArg("deleteMsg",""));
	}
	else if(evt.getSource() == mountMessagesMenuItem) {
	    JFileChooser fc = new JFileChooser(user.getOpenedDir());
	    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    int returnVal = fc.showOpenDialog(theMainGuiWindow);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		session.fireEvent(new MVCArg("mountDir", fc.getSelectedFile().getAbsolutePath()));
		user.setOpenedDir(fc.getSelectedFile().getPath());
	    }
	}
    }

    /**
     * This handles the Tools menu action
     * @param evt The event
     */
    public void toolsActionPerformed(ActionEvent evt) {
	if(evt.getSource() == logMenuItem) {
	    dlgLog.init();
	    dlgLog.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgLog.getSize();
	    dlgLog.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				   Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgLog.setVisible(true);
	}
	else if(evt.getSource() == profileMenuItem) {
	    try {
		dlgProfile.displayProfile(session.getProfile());
		dlgProfile.pack();
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dlgProfile.getSize();
		dlgProfile.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				       Math.max(0,(screenSize.height-windowSize.height)/2));
		dlgProfile.setVisible(true);
	    }
	    catch(ParserConfigurationException pce) {
		System.err.println("ParserConfigurationException " + pce.getMessage());
		pce.printStackTrace();
	    }
	    catch(SAXException saxe) {
		System.out.println("SAXException " + saxe.getMessage());
		saxe.printStackTrace();
	    }
	    catch(IOException ioe) {
		System.out.println("IOException " + ioe.getMessage());
		ioe.printStackTrace();
	    }
	}
    }

    /**
     * This handles the Help menu action
     * @param evt The event
     */
    public void helpActionPerformed(ActionEvent evt) {
	if(evt.getSource() == AboutItem) {
            dlgAbout.pack();
            // Center the window
    	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	    Dimension windowSize = dlgAbout.getSize();
    	    dlgAbout.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				 Math.max(0,(screenSize.height-windowSize.height)/2));
    	    dlgAbout.setVisible(true);
	}
	else if(evt.getSource() == ContributorsItem) {
            dlgContributors.pack();
            // Center the window
    	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	    Dimension windowSize = dlgContributors.getSize();
    	    dlgContributors.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				 Math.max(0,(screenSize.height-windowSize.height)/2));
    	    dlgContributors.setVisible(true);
	}
    }

    /**
     * This handles the Generate action.
     *
     * 1) the current list of session parameters is retrieved.
     * 2) a properties object is created from the session parameters
     * 3) the default message generation "options" is set 
     *    (i.e., the optionFile properties)
     * 4) initalize the progress bar
     * 5) create the Message Generation object (if not done already)
     * 6) set the Message Generation options 
     * 7) spawn the message generation thread
     * @param evt The event
     */
    private void generateActionPerformed(ActionEvent evt) {
	MMLog.log("Generate Button Pressed");

	if (generateButton.getText() == "Start") {
	    generateButton.setText("Stop");

	    // Create the messages directory if it doesn't exist
	    File f = new File(session.getMsgPath());
	    if(!f.exists()) {
		f.mkdir();
	    }

	    // Check that all the resources are available before the generation
	    String check = CheckResources.checkSession(session);
	    System.out.println(check);
	    System.exit(0);
	    if(!check.equals("")) {
		JOptionPane.showMessageDialog(null, "Resources errors", "Some resources are missing:\n" + check, JOptionPane.ERROR_MESSAGE);
	    }
	    else {
		/* Create a properties object from the current user settings */
		MessageGenerationProperties prop = new MessageGenerationProperties(session);
		// Set the options
		prop.setOptionsFileProperty(session.generateMGOptions());
		//prop.setMessageControlIdProperty(dlgMSH.getMessageControlId());
		
		/* Set the progress indicator to the correct volume */
		progressBar.setMaximum(session.getVolume());
		
		/* Obtain an instance and initialize the Message Generator with default settings. */
		if (generator == null) {
		    generator = new MessageGenerator(prop.getProperties());
		}
		else {
		    // Set the output dir
		    generator.setOutputDir(session.getMsgPath());
		    // Set the meta file
		    generator.setMetaFile("TD_" + new File(session.getProfile()).getName());
		}
		
		generator.setOptions(prop.getProperties());
		
		/* Generate the messages and monitor and display the progress */
		GenerateMessagesThread gmt = new GenerateMessagesThread();
		gmt.start();
		timer.start();       
	    }
	} else if (generateButton.getText() == "Stop") {
	    generator.stop();
	    generateButton.setText("Close");
	    taskOutput.append(generator.getStatusMessage());
	} else if (generateButton.getText() == "Close") {
	    generateDlg.dispose();
	    generateButton.setText("Start");
	    progressBar.setValue(0);
	    taskOutput.setText("");
	}
    }

    /*
     * Launches the the message generation process while providing the input
     * to a JProgressBar. This is a separate thread. The progress value is
     * obtained from the Message Generation object (generator).
     *
     */
    class GenerateMessagesThread extends Thread {
	boolean stop = false;

	public GenerateMessagesThread() {
	}

	/* 
	 * This is called from another object, the MessageGeneration object
	 * in this case. It gets called after a message is created. The 
	 * progress of the message creation process is captured and updated
	 * with the setValue() method.
	 */

	public void run() {
	    Runnable runner = new Runnable() {
		    public void run() {
			MMLog.log("MainMenu:" + "GenerateMessagesThread:" +
				  "run# " + generator.getProgressValue());
			progressBar.setValue(generator.getProgressValue());
		    }
		};

	    /* Generate the messages */
	    generator.generateMessages(session.getValidate(), runner);
	    session.fireEvent(new MVCArg("mountDir", session.getMsgPath()));
	    // Update the list of messages in the user object
	    Vector vMsg = generator.getMessageFileNames();
	    user.addMD(session.getMsgPath() + Constants.FS + "TD_" + new File(session.getProfile()).getName());
	    for(int i=0;i<vMsg.size();i++) {
		String sMsg = (String)vMsg.get(i);
		if(!sMsg.startsWith("TD")) {
		    user.addMsg(sMsg, session.getProfile());
		    user.setStatus(sMsg, MakerMessage.GENERATED);
		}
	    }	
	}
    }

    /**
     * This method adds a separator and all mmk files opened recently to the File Menu
     */
    private void addRecentMMK() {
	user.cleanMmkHistory();
	boolean bMMK = user.getMmk1() != null;

	if(bMMK) {
	    if(fileMmkSeparator == null) {
		fileMmkSeparator = new JSeparator();
	    }
	    fileMenu.add(fileMmkSeparator);
	    // MMK 1
	    if(user.getMmk1() != null) {
		if(fileMmk1 == null) {
		    fileMmk1 = new JMenuItem();
		    fileMmk1.addActionListener(this);
		    fileMmk1.setMnemonic('1');
		}
		fileMmk1.setText("1 " + user.getMmk1());
		fileMenu.add(fileMmk1);
	    }
	    // MMK 2
	    if(user.getMmk2() != null) {
		if(fileMmk2 == null) {
		    fileMmk2 = new JMenuItem();
		    fileMmk2.addActionListener(this);
		    fileMmk2.setMnemonic('2');		
		}
		fileMmk2.setText("2 " + user.getMmk2());
		fileMenu.add(fileMmk2);
	    }
	    // MMK 3
	    if(user.getMmk3() != null) {
		if(fileMmk3 == null) {
		    fileMmk3 = new JMenuItem();
		    fileMmk3.addActionListener(this);
		    fileMmk3.setMnemonic('3');		
		}
		fileMmk3.setText("3 " + user.getMmk3());
		fileMenu.add(fileMmk3);
	    }
	    // MMK 4
	    if(user.getMmk4() != null) {
		if(fileMmk4 == null) {
		    fileMmk4 = new JMenuItem();
		    fileMmk4.addActionListener(this);
		    fileMmk4.setMnemonic('4');
		}
		fileMmk4.setText("4 " + user.getMmk4());
		fileMenu.add(fileMmk4);
	    }
	}
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in MainMenu");
		mshMenuItem.setEnabled(true);
		loadDataFilesMenuItem.setEnabled(true);
		tableViewerMenuItem.setEnabled(true);
		generateMessagesMenuItem.setEnabled(true);
		profileMenuItem.setEnabled(true);
	    }
	    else if(arg instanceof MVCArg) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in MainMenu");
		if("profile".compareTo(((MVCArg)arg).getName()) == 0) {
		    if(!((MVCArg)arg).getValue().equals("")) {
			mshMenuItem.setEnabled(true);
			loadDataFilesMenuItem.setEnabled(true);
			tableViewerMenuItem.setEnabled(true);
			generateMessagesMenuItem.setEnabled(true);
			profileMenuItem.setEnabled(true);
		    }
		    else {
			loadDataFilesMenuItem.setEnabled(false);
			mshMenuItem.setEnabled(false);
			tableViewerMenuItem.setEnabled(false);
			generateMessagesMenuItem.setEnabled(false);
			profileMenuItem.setEnabled(false);
		    }
		}
		else if("saveMenu".equals(((MVCArg)arg).getName())) {
		    saveMessagesMenuItem.setEnabled((Boolean.valueOf(((MVCArg)arg).getValue())).booleanValue());
		}
		else if("deleteMenu".equals(((MVCArg)arg).getName())) {
		    deleteMessagesMenuItem.setEnabled((Boolean.valueOf(((MVCArg)arg).getValue())).booleanValue());
		}
		else if("volumeMenu".equals(((MVCArg)arg).getName()) || "volume".equals(((MVCArg)arg).getName())) {
		    if(Integer.parseInt(((MVCArg)arg).getValue()) == 0)
			generateMessagesMenuItem.setEnabled(false);
		    else
			generateMessagesMenuItem.setEnabled(true);
		}
	    }
	}
    }

    /**
     * Disable items of the Configure menu
     */
    public void disableMenu() {
	mshMenuItem.setEnabled(false);
	loadDataFilesMenuItem.setEnabled(false);
	tableViewerMenuItem.setEnabled(false);
	generateMessagesMenuItem.setEnabled(false);
	profileMenuItem.setEnabled(false);
	saveMessagesMenuItem.setEnabled(false);
	deleteMessagesMenuItem.setEnabled(false);
    }

    /**
     * Handles action for click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	//boolean 
	String userMmk = null;
	if(e.getSource() == fileMmk1) {
	    userMmk = user.getMmk1();
	}
	else if(e.getSource() == fileMmk2) {
	    userMmk = user.getMmk2();
	}
	else if(e.getSource() == fileMmk3) {
	    userMmk = user.getMmk3();
	}
	else if(e.getSource() == fileMmk4) {
	    userMmk = user.getMmk4();
	}
	if(userMmk != null) {
	    JDialog dlgWait = new WaitDialog("Load Profile and Generate the Data Repository ...");
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgWait.getSize();
	    dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				Math.max(0,(screenSize.height-windowSize.height)/2));
	    
	    final JDialog fDlg = dlgWait;
	    final String fUserMmk = userMmk;
	    Thread t = new Thread() {
		    public void run() {
			try {
			    session.loadProperties(fUserMmk);
			    user.setMmk1(fUserMmk);
			    addRecentMMK();
			    fDlg.dispose();
			}
			catch(IOException ioe) {
			    System.err.println("IOException " + ioe.getMessage());
			    ioe.printStackTrace();
			}
		    }
		};
	    t.start();
	    
	    dlgWait.show();
	}
    }

    /**
     * Clean recursively the reports directory
     * @param The current file
     */
/*    private void clean(File node) {
	if(node.isDirectory()) {
	    File[] c = node.listFiles();
	    for(int i=0;i<c.length;i++)
		clean(c[i]);
	    node.delete();
	}
	// It's a file
	else {
	    node.delete();
	}
	}*/
}
