/**
 * This class is a dialog to save unsaved messages
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

// AWT
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;

// XML
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

// SAX
import org.xml.sax.SAXException;

// MessageMaker
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.control.MakerMessage;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.ER7FileFilter;
import gov.nist.hl7maker.view.display.XMLtoER7;
/*import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.util.hapi.GenerationDialog;
import gov.nist.hl7maker.util.hapi.SwingWorker;*/

// HAPI
/*import ca.uhn.hl7v2.HL7Exception;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.parser.XMLParser;
import ca.uhn.hl7v2.parser.DefaultXMLParser;
import ca.uhn.hl7v2.parser.PipeParser;*/

public class SaveMsgDialog extends JDialog implements ActionListener {

    protected JCheckBox chkXML;
    protected JCheckBox chkER7;
    protected JCheckBox chkXMLInOne;
    protected JCheckBox chkER7InOne;
    protected JButton btnSave;
    protected JButton btnClose;
    protected JScrollPane lstMsgScroll;
    protected JList lstMsg;
    protected JLabel lbl;
    protected JPanel p1bis, p3;

    protected User user;
    protected Vector vMsg;

    private Vector vProfile;
    protected boolean allInOne;

    /**
     * Constructor
     * @param frame A JFrame
     */
    public SaveMsgDialog(JFrame frame) {
	super(frame, "Save Messages", true);

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
	// Label
	lbl = new JLabel("Save messages");
	lbl.setAlignmentX(Component.CENTER_ALIGNMENT);
	p0.add(lbl);
	// List
	lstMsg = new JList();
	lstMsg.setEnabled(false);
	lstMsg.setAlignmentX(Component.CENTER_ALIGNMENT);
	lstMsgScroll = new JScrollPane(lstMsg);
	lstMsgScroll.setPreferredSize(new Dimension(500, 250));
	p0.add(lstMsgScroll);

	// Checkbox XML or ER7
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	chkXML = new JCheckBox("Save XML messages in seperate files", true);
	chkER7 = new JCheckBox("Save ER7 messages in seperate files");
	p1.add(chkXML);
	p1.add(Box.createRigidArea(new Dimension(10, 0)));
	p1.add(chkER7);

	// Checkbox XML or ER7 inOne
	p1bis = new JPanel();
	p1bis.setLayout(new BoxLayout(p1bis, BoxLayout.LINE_AXIS));
	chkXMLInOne = new JCheckBox("Save XML messages in one file (msgALL.xml)");
	chkER7InOne = new JCheckBox("Save ER7 messages in one file (msgALL.er7)");
	p1bis.add(chkXMLInOne);
	p1bis.add(Box.createRigidArea(new Dimension(10, 0)));
	p1bis.add(chkER7InOne);

	// Command Panel Save or Close
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	btnSave = new JButton("Save");
        btnSave.setBackground(new Color(204, 204, 255));
	btnSave.addActionListener(this);
	p2.add(btnSave);
	p2.add(Box.createRigidArea(new Dimension(10, 0)));
	btnClose = new JButton("Close");
        btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		}
	    });
	p2.add(btnClose);

	p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.PAGE_AXIS));
	p3.add(p0);
	p3.add(Box.createRigidArea(new Dimension(0, 10)));
	p3.add(p1);
	p3.add(Box.createRigidArea(new Dimension(0, 10)));
	p3.add(p2);
	p3.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p3);

	vProfile = new Vector();
	allInOne = false;
    }

    /**
     * Set the User object
     * @param user The User
     */
    public void setUser(User user) {
	this.user = user;
    }

    /**
     * Set the list of message
     *@param vMsg The list of messages
     */
    public void setLstMsg(Vector vMsg) {
	lstMsg.setListData(vMsg);
	this.vMsg = vMsg;

	if(vMsg.size() == 1) lbl.setText("Save Message");
	if(vMsg.size() > 1 && !allInOne) {
	    p3.add(p1bis, 4);
	    p3.add(Box.createRigidArea(new Dimension(0, 10)), 5);
	    allInOne = true;
	}
	else if(vMsg.size() < 2 && allInOne) {
	    p3.remove(3);
	    p3.remove(p1bis);
	    allInOne = false;
	}
    }

    /**
     * Handles a click on a button
     * @param e The Event
     */
    public void actionPerformed(ActionEvent e) {
	    if(e.getSource() == btnSave) {
		// XML File (They are already saved, we just have to change the status to saved
		if(chkXML.isSelected()) {
		    for(int i=0;i<vMsg.size();i++) {
			user.setStatus((String)vMsg.get(i), MakerMessage.SAVED);
		    }
		}
		if(chkXMLInOne.isSelected()) {
		    try {
			File f = new File(new File((String)vMsg.get(0)).getParent() + Constants.FS + "msgALL.xml");
			BufferedWriter bw = new BufferedWriter(new FileWriter(f));
			for(int i=0;i<vMsg.size();i++) {
			    BufferedReader br = new BufferedReader(new FileReader((String)vMsg.get(i)));
			    String line = null;
			    while((line = br.readLine()) != null) {
				bw.write(line);
			    }
			    br.close();
			    bw.write("\n\n");
			}
			bw.close();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		}
		// ER7 File
		if(chkER7.isSelected()) {
		    try {
			for(int i=0;i<vMsg.size();i++) {
			    String sMsg = (String)vMsg.get(i);
			    // Get the MakerMessage Object
			    MakerMessage mm = user.getMessage(sMsg);
			    String filename = mm.getFilename();
			    String er7FN = sMsg.substring(0, sMsg.length()-4).concat(".er7");
			    String er7 = XMLtoER7.transform(filename);//getDocumentFromModel());
			    // Save the file
			    BufferedWriter bw = new BufferedWriter(new FileWriter(er7FN));
			    bw.write(er7);
			    bw.close();
			}
		    }
		    catch(IOException ioe) {
			System.err.println("IOException: " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		}
		if(chkER7InOne.isSelected()) {
		    try {
			File f = new File(new File((String)vMsg.get(0)).getParent() + Constants.FS + "msgALL.er7");
			BufferedWriter bwER7 = new BufferedWriter(new FileWriter(f));
			for(int i=0;i<vMsg.size();i++) {
			    String sMsg = (String)vMsg.get(i);
			    // Get the MakerMessage Object
			    MakerMessage mm = user.getMessage(sMsg);
			    String filename = mm.getFilename();
			    String er7FN = sMsg.substring(0, sMsg.length()-4).concat(".er7");
			    String er7 = XMLtoER7.transform(filename);//getDocumentFromModel());
			    // Save the file
			    bwER7.write(er7);
			    bwER7.write("\n");
			}
			bwER7.close();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException: " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		}
		/*		if(chkER7.isSelected() || chkER7InOne.isSelected()) {
		    final GenerationDialog dlgGeneration = new GenerationDialog(null);
		    
		    boolean needGeneration = false;
		    for(int i=0;i<vMsg.size();i++) {
			// Get the MakerMessage Object
			MakerMessage mm = user.getMessage((String)vMsg.get(i));
			
			// Check if the message has a Z-Segment or group segment
			String profile = mm.getProfile();
			needGeneration = CustomHAPI.needCustom(profile);
			if(needGeneration) i = vMsg.size() + 1;
		    }

		    SwingWorker worker = new SwingWorker() {
			    public Object construct() {
				try {
				    BufferedWriter bwER7 = null;
				    if(chkER7InOne.isSelected()) {
					File f = new File(new File((String)vMsg.get(0)).getParent() + Constants.FS + "msgALL.er7");
					bwER7 = new BufferedWriter(new FileWriter(f));
				    }
				    for(int i=0;i<vMsg.size();i++) {
					final String sMsg = (String)vMsg.get(i);
					// Get the MakerMessage Object
					MakerMessage mm = user.getMessage(sMsg);
					
					// Check if the message has a Z-Segment or group segment
					String profile = mm.getProfile();
					final boolean needCustom = CustomHAPI.needCustom(profile);
					final boolean needGeneration = needCustom && !vProfile.contains(profile);
					if(needGeneration) vProfile.add(profile);
					
					final String fProfile = mm.getProfile();
					final String fFilename = mm.getFilename();
					//try {
					if(needGeneration) {
					    CustomHAPI cHAPI = new CustomHAPI();
					    cHAPI.addObserver(dlgGeneration);
					    //cHAPI.generate(xmlDir.getProfileFromMessage(fFileName));
					    cHAPI.generate(fProfile);
					}
					// Get a Document object from the XML file
					DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
					DocumentBuilder builder = factory.newDocumentBuilder();
					Document doc = builder.parse(new File(fFilename));
					
					String prefix = "";
					if(needCustom) {
					    prefix = new File(fProfile).getName();
					    prefix = prefix.substring(0, prefix.length()-4).replaceAll("_","XX").replaceAll(" ","XX");
					    doc = CustomHAPI.modifyDocument(doc, prefix);
					}
					
					// Get a Message Object from the Document
					XMLParser xmlParser = new DefaultXMLParser();
					Message msg = xmlParser.parseDocument(doc, "2.4");
					//doc = xmlParser.encodeDocument(msg);
					
					//instantiate a PipeParser, which handles the "traditional encoding" 
					PipeParser pipeParser = new PipeParser();
					
					if(chkER7.isSelected()) {
					    // Get the target filename
					    String er7FN = sMsg.substring(0, sMsg.length()-4).concat(".er7");
					    String er7 = CustomHAPI.cleanER7(formatER7(pipeParser.encode(msg)), prefix);
					    // Save the file
					    BufferedWriter bw = new BufferedWriter(new FileWriter(er7FN));
					    bw.write(er7);
					    bw.close();
					}
					if(chkER7InOne.isSelected()) {
					    String er7 = CustomHAPI.cleanER7(formatER7(pipeParser.encode(msg)), prefix);
					    // Save the file
					    bwER7.write(er7);
					    bwER7.write("\n");
					}
				    }
				    if(chkER7InOne.isSelected())
					bwER7.close();
				}
				catch(ParserConfigurationException pce) {
				    System.err.println("ParserConfigurationException " + pce.getMessage());
				    pce.printStackTrace();
				}
				catch(SAXException saxe) {
				    System.err.println("SAXException " + saxe.getMessage());
				    saxe.printStackTrace();
				}
				catch(TransformerConfigurationException tce) {
				    System.err.println("TransformerConfigurationException " + tce.getMessage());
				    tce.printStackTrace();
				}
				catch(TransformerException te) {
				    System.err.println("TransformerException " + te.getMessage());
				    te.printStackTrace();
				}
				catch(IOException ioe) {
				    System.err.println("IOException: " + ioe.getMessage());
				    ioe.printStackTrace();
				}
				catch(HL7Exception hl7e) {
				    System.err.println("HL7Exception: " + hl7e.getMessage());
				    hl7e.printStackTrace();
				}
				setVisible(false);
				return null;
			    }
			};
		    worker.start();

		    if(needGeneration) {
			dlgGeneration.pack();
			// Center the window
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension windowSize = dlgGeneration.getSize();
			dlgGeneration.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
						  Math.max(0,(screenSize.height-windowSize.height)/2));
			dlgGeneration.setVisible(true);
		    }
		    }*/
		/*if(!chkER7.isSelected() && !chkER7InOne.isSelected()) {
		    setVisible(false);
		    }*/
		setVisible(false);
	    }
    }

    /**
     * Format the ER7, it means add CR after each segment
     * @param er7 The ER7 unformatted
     * @return The ER7 formatted
     */
    public static String formatER7(String er7) {
	StringBuffer buffy = new StringBuffer();
	for(int i=0;i<er7.length();i++) {
	    if(er7.charAt(i) == 0x0D)
		buffy.append("\n");
	    else
		buffy.append(er7.charAt(i));
	}
	return new String(buffy);
    }
}
