/**
 * This class is a dialog to do the mapping between any format and our local table format
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;

// XSL
//import javax.xml.transform.TransformerFactory;
//import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.view.config.TablePriority;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.tableconverter.TableConverter;

public class TableConverterMappingDialog extends JDialog implements ActionListener {

    private Session session;
    private User user;

    private JTextField txtEltTable;
    //private JComboBox cbType;
    private JTextField txtType;
    //private JComboBox cbName;
    private JTextField txtName;
    //private JComboBox cbId;
    private JTextField txtId;
    private JTextField txtEltItem;
    //private JComboBox cbCode;
    private JTextField txtCode;
    //private JComboBox cbValue;
    private JTextField txtValue;

    private JButton btnImport;
    private JButton btnCancel;

    private File fAny;

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     * @param user A user
     */
    public TableConverterMappingDialog(JFrame frame, Session ses, User user) {
	super(frame, "Local Table Converter Mapping", true);
	session = ses;
	this.user = user;

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// Label Title
	JPanel p = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.LINE_AXIS));
	JLabel lblTitle = new JLabel("Enter mappings from local table for the following elements and attributes");
	p.add(lblTitle);

	// Element to idenfity a table
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	p0.add(Box.createRigidArea(new Dimension(49, 0)));
	JLabel lblEltTable = new JLabel("Table Container: ");
	p0.add(lblEltTable);
	txtEltTable = new JTextField("");
	txtEltTable.setPreferredSize(new Dimension(100, 25));
	p0.add(txtEltTable);

	// Type of the table
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	p1.add(Box.createRigidArea(new Dimension(72, 0)));
	JLabel lblType = new JLabel("Table Type: ");
	p1.add(lblType);
	/*cbType = new JComboBox();
	cbType.addItem("Attribute");
	cbType.addItem("Element");
	p1.add(cbType);*/
	p1.add(Box.createRigidArea(new Dimension(5, 0)));
	txtType = new JTextField("");
	txtType.setPreferredSize(new Dimension(0, 25));
	p1.add(txtType);

	// Name of the table
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	p2.add(Box.createRigidArea(new Dimension(66, 0)));
	JLabel lblName = new JLabel("Table Name: ");
	p2.add(lblName);
	/*cbName = new JComboBox();
	cbName.addItem("Attribute");
	cbName.addItem("Element");
	p2.add(cbName);*/
	p2.add(Box.createRigidArea(new Dimension(5, 0)));
	txtName = new JTextField("");
	txtName.setPreferredSize(new Dimension(0, 25));
	p2.add(txtName);

	// Id of the table
	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	p3.add(Box.createRigidArea(new Dimension(89, 0)));
	JLabel lblId = new JLabel("Table Id: ");
	p3.add(lblId);
	/*cbId = new JComboBox();
	cbId.addItem("Attribute");
	cbId.addItem("Element");
	p3.add(cbId);*/
	p3.add(Box.createRigidArea(new Dimension(5, 0)));
	txtId = new JTextField("");
	txtId.setPreferredSize(new Dimension(0, 25));
	p3.add(txtId);

	// Element to identify an item of a table
	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	JLabel lblEltItem = new JLabel("Table Element Container: ");
	p4.add(lblEltItem);
	txtEltItem = new JTextField("");
	txtEltItem.setPreferredSize(new Dimension(150, 25));
	p4.add(txtEltItem);

	// Code of the item
	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.LINE_AXIS));
	p5.add(Box.createRigidArea(new Dimension(104, 0)));
	JLabel lblCode = new JLabel("Code: ");
	p5.add(lblCode);
	/*cbCode = new JComboBox();
	cbCode.addItem("Attribute");
	cbCode.addItem("Element");
	p5.add(cbCode);*/
	p5.add(Box.createRigidArea(new Dimension(5, 0)));
	txtCode = new JTextField("");
	txtCode.setPreferredSize(new Dimension(0, 25));
	p5.add(txtCode);

	// Value of the item
	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
	p6.add(Box.createRigidArea(new Dimension(101, 0)));
	JLabel lblValue = new JLabel("Value: ");
	p6.add(lblValue);
	/*cbValue = new JComboBox();
	cbValue.addItem("Attribute");
	cbValue.addItem("Element");
	p6.add(cbValue);*/
	p6.add(Box.createRigidArea(new Dimension(5, 0)));
	txtValue = new JTextField("");
	txtValue.setPreferredSize(new Dimension(0, 25));
	p6.add(txtValue);

	// Import / Cancel
	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.LINE_AXIS));
	btnImport = new JButton("Import ...");
	btnImport.setBackground(new Color(204, 204, 255));
	btnImport.addActionListener(this);
	p7.add(btnImport);
	p7.add(Box.createRigidArea(new Dimension(5, 0)));
	btnCancel = new JButton("Cancel");
	btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.addActionListener(this);
	p7.add(btnCancel);

	// The final panel
	JPanel p8 = new JPanel();
	p8.setLayout(new BoxLayout(p8, BoxLayout.PAGE_AXIS));
	p8.add(p);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p0);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p1);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p2);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p3);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p4);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p5);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p6);
	p8.add(Box.createRigidArea(new Dimension(0, 10)));
	p8.add(p7);
	p8.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

	getContentPane().add(p8);
    }

    /**
     * Init the panel
     */
    public void init() {
	txtEltTable.setText("");
	//cbType.setSelectedIndex(0);
	txtType.setText("");
	//cbName.setSelectedIndex(0);
	txtName.setText("");
	//cbId.setSelectedIndex(0);
	txtId.setText("");
	txtEltItem.setText("");
	//cbCode.setSelectedIndex(0);
	txtCode.setText("");
	//cbValue.setSelectedIndex(0);
	txtValue.setText("");
    }

    /**
     * Ask for the file to be imported before the dialog is visible
     */
    public void prepareDisplay() {
	fAny = null;
	// Select the file to import
	JFileChooser fc = new JFileChooser(user.getOpenedDir());
	// Set the chooser to accept only XML files.
	fc.addChoosableFileFilter(new XMLFileFilter());
	fc.setAcceptAllFileFilterUsed(false);
	int returnVal = fc.showDialog(this, "Select the local table file to import");

	if(returnVal == JFileChooser.APPROVE_OPTION) {
	    fAny = fc.getSelectedFile();
	    user.setOpenedDir(fAny.getParent());
	    setVisible(true);
	}
    }

    /**
     * Handle the events
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnImport) {
	    if(fAny != null) {
		// Select the file to save
		JFileChooser fc = new JFileChooser(user.getOpenedDir());
		fc.setSelectedFile(new File("MM_" + fAny.getName()));
		// Set the chooser to accept only XML files.
		fc.addChoosableFileFilter(new XMLFileFilter());
		fc.setAcceptAllFileFilterUsed(false);
		int returnVal = fc.showSaveDialog(this);
		
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fMM = fc.getSelectedFile();
		    if(!fMM.getPath().endsWith(".xml")) {
			fMM = new File(fMM.getPath() + ".xml");
		    }
		    user.setOpenedDir(fMM.getParent());
		    
		    // Do the transformation
		    try {
			TableConverter tc = new TableConverter();
			
			tc.setTableMapping(txtEltTable.getText());
			tc.setItemMapping(txtEltItem.getText());
			/*if(cbType.getSelectedIndex() == 0) tc.setTypeMapping("@" + txtType.getText());
			else tc.setTypeMapping(txtType.getText());
			if(cbName.getSelectedIndex() == 0) tc.setNameMapping("@" + txtName.getText());
			else tc.setNameMapping(txtName.getText());
			if(cbId.getSelectedIndex() == 0) tc.setIDMapping("@" + txtId.getText());
			else tc.setIDMapping(txtId.getText());
			if(cbCode.getSelectedIndex() == 0) tc.setCodeMapping("@" + txtCode.getText());
			else tc.setCodeMapping(txtCode.getText());
			if(cbValue.getSelectedIndex() == 0) tc.setValueMapping("@" + txtValue.getText());
			else tc.setValueMapping(txtValue.getText());*/

			tc.setTypeMapping(txtType.getText());
			tc.setNameMapping(txtName.getText());
			tc.setIDMapping(txtId.getText());
			tc.setCodeMapping(txtCode.getText());
			tc.setValueMapping(txtValue.getText());
			
			tc.convert(fAny, fMM);
		    }
		    catch(FileNotFoundException fnfe) {
			System.err.println("FileNotFoundException " + fnfe.getMessage());
			fnfe.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(TransformerConfigurationException tce) {
			System.err.println("TransformerConfigurationException " + tce.getMessage());
			tce.printStackTrace();
		    }
		    catch(TransformerException te) {
			System.err.println("TransformerException " + te.getMessage());
			te.printStackTrace();
		    }	
		    // Add the file to the local table list
		    Vector vLocalTable = session.getLocalTable();
		    if(!vLocalTable.contains(fMM.getPath())) {
			TablePriority tp = new TablePriority(fMM.getPath(), 100);
			if(!session.getTable().contains(tp)) {
			    vLocalTable.add(new TablePriority(fMM.getPath(), 100));
			}
		    }		    
		    session.setLocalTable(vLocalTable, true);
		    setVisible(false);
		}
	    }
	}
	else if(evt.getSource() == btnCancel) {
	    setVisible(false);
	}
    }
}
	
