/**
 * This class is dialog to set priorities for Tables
 */

package gov.nist.hl7maker.view;

// JDK
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.view.config.TablePriority;

public class TablePriorityDialog extends JDialog implements ActionListener, ListSelectionListener {

    private Session session;

    private JList lstTable;
    private JScrollPane lstTablePane;
    private JButton btnUp;
    private JButton btnDown;
    private JButton btnSet;
    private JButton btnCancel;

    /**
     * Constructor
     * @param frame A JFrame
     * @param ses A session
     */
    public TablePriorityDialog(JFrame frame, Session ses) {
	super(frame, "Set tables priority", true);
	session = ses;

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// A Label
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JLabel l = new JLabel("Priority top (higher) to bottom (lower)");
	p0.add(l);

	// List of tables
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	lstTable = new JList();
	lstTable.addListSelectionListener(this);
	lstTablePane = new JScrollPane(lstTable);
	p1.add(lstTablePane);

	// Button to set priority
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.PAGE_AXIS));
	btnUp = new JButton("Up");
	btnUp.setBackground(new Color(204, 204, 255));
	btnUp.addActionListener(this);
	btnUp.setEnabled(false);
	p2.add(btnUp);
	btnDown = new JButton("Down");
	btnDown.setBackground(new Color(204, 204, 255));
	btnDown.addActionListener(this);
	btnDown.setEnabled(false);
	p2.add(Box.createRigidArea(new Dimension(0, 10)));
	p2.add(btnDown);
	//p2.add(Box.createRigidArea(new Dimension(0, 0)));
	p1.add(Box.createRigidArea(new Dimension(10, 0)));
	p1.add(p2);

	// Set and Cancel button
	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	btnSet = new JButton("Set priorities");
	btnSet.setBackground(new Color(204, 204, 255));
	btnSet.addActionListener(this);
	p3.add(btnSet);
	btnCancel = new JButton("Cancel");
	btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.addActionListener(this);
	p3.add(Box.createRigidArea(new Dimension(5, 0)));
	p3.add(btnCancel);

	// The whole Panel
	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.PAGE_AXIS));
	p4.add(p0);
	p4.add(Box.createRigidArea(new Dimension(0, 10)));
	p4.add(p1);
	p4.add(Box.createRigidArea(new Dimension(0, 10)));
	p4.add(p3);
	p4.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p4);

	init();
    }

    /**
     * Init the dialog
     */
    public void init() {
	Vector v = session.getTable();
	if(v != null) {
	    lstTable.setListData(v);
	}
/*	btnUp.setEnabled(false);
	btnDown.setEnabled(false); */
    }

    /**
     * Handle event on button click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	btnUp.setEnabled(false);
	btnDown.setEnabled(false);
	// Get the ListModel
	ListModel model = lstTable.getModel();
	Vector vPriority = new Vector();
	for(int i=0;i<model.getSize();i++) {
	    vPriority.add(model.getElementAt(i));
	}
	if(e.getSource() == btnCancel) {
	    setVisible(false);
	}
	else if(e.getSource() == btnSet) {
	    for(int i=0;i<vPriority.size();i++) {
		((TablePriority)vPriority.get(i)).setPriority(i);
	    }
	    session.setPriority(vPriority);
	    setVisible(false);
	}
	else if(e.getSource() == btnUp) {
	    int idx = lstTable.getSelectedIndex();
	    vPriority.insertElementAt(vPriority.get(idx), idx-1);
	    vPriority.remove(idx+1);
	    lstTable.setListData(vPriority);
	    lstTable.setSelectedIndex(idx-1);
	}
	else if(e.getSource() == btnDown) {
	    int idx = lstTable.getSelectedIndex();
	    vPriority.insertElementAt(vPriority.get(idx), idx+2);
	    vPriority.remove(idx);
	    lstTable.setListData(vPriority);
	    lstTable.setSelectedIndex(idx+1);
	}
    }

    /**
     * Handle the event for JList
     * @param e The Event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(lstTable.getSelectedIndex() != -1) {
	    int idx = lstTable.getSelectedIndex();
	    btnUp.setEnabled(true);
	    btnDown.setEnabled(true);
	    if(idx == 0) {
		btnUp.setEnabled(false);
	    }
	    else if(idx == lstTable.getModel().getSize()-1) {
		btnDown.setEnabled(false);
	    }
	}
    }
}
