/**
 * Class used to display and manage the table
 */

package gov.nist.hl7maker.view;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.JFileChooser;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException; 

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// Message Maker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.view.config.TableDataModel;
import gov.nist.hl7maker.view.config.TableNode;
import gov.nist.hl7maker.view.config.TableValueNode;
import gov.nist.hl7maker.view.config.TableValuePanel;
import gov.nist.hl7maker.view.config.TableValueCommandPanel;
import gov.nist.hl7maker.view.config.TableValueSavePanel;
import gov.nist.hl7maker.view.config.TablePriority;

public class TableViewer extends JDialog implements Observer, ActionListener {

    private JComboBox cbTableNumber;
    private JComboBox cbTableName;
    //    private JTextField txtTableType;
    private JComboBox cbTableType;

    private JButton btnNewTable;
    //    private JButton btnCreateTable;
    //    private JButton btnEditTable;
    //    private JButton btnUpdateTable;
    private JButton btnRemoveTable;
    private JButton btnCancelTable;
    private JButton btnApply;

    private TableValuePanel tvp;
    private JList lstCode2;
    private JList lstItem;

    private TableValueCommandPanel tvcp;
    private TableValueSavePanel tvsp;
/*    private JScrollPane lstItemScrollPane;
    private JScrollPane lstCode2ScrollPane;*/

    private JButton btnClose;

    private Vector vTableInfo;
    private int currentIdx;
    private String action;

    private Session session;

    /**
     * Constructor
     * @param frame A JFrame
     */
    public TableViewer(JFrame frame) {
	super(frame, "Table Viewer", true);
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    closeDialog(evt);
		}
	    });

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	// Table Number
	JLabel lblTableNumber = new JLabel("Id: ");
	p0.add(lblTableNumber);
	cbTableNumber = new JComboBox();
        cbTableNumber.setBackground(new Color(204, 204, 255));
	p0.add(cbTableNumber);
	p0.add(Box.createRigidArea(new Dimension(15, 0)));
	// Table Type
	JLabel lblTableType = new JLabel("Type: ");
	p0.add(lblTableType);
/*	txtTableType = new JTextField();
	txtTableType.setEditable(false);
	txtTableType.setPreferredSize(new Dimension(75, 0));
	p0.add(txtTableType);*/
	cbTableType = new JComboBox();
        cbTableType.setBackground(new Color(204, 204, 255));
	cbTableType.setEnabled(false);
	p0.add(cbTableType);
	p0.add(Box.createRigidArea(new Dimension(15, 0)));
	// Table Name
	JLabel lblTableName = new JLabel("Name: ");
	p0.add(lblTableName);
	cbTableName = new JComboBox();
        cbTableName.setBackground(new Color(204, 204, 255));
	p0.add(cbTableName);
	p0.add(Box.createRigidArea(new Dimension(15, 0)));
	// Apply Button
	btnApply = new JButton("Edit Name");
	btnApply.setBackground(new Color(204, 204, 255));
	btnApply.addActionListener(this);
	p0.add(btnApply);
	p0.add(Box.createRigidArea(new Dimension(15, 0)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	// New
	btnNewTable = new JButton("New Table");
	btnNewTable.setBackground(new Color(204, 204, 255));
	btnNewTable.addActionListener(this);
	p1.add(btnNewTable);
	p1.add(Box.createRigidArea(new Dimension(15, 0)));
/*	// Create
	btnCreateTable = new JButton("Create Table");
	btnCreateTable.setBackground(new Color(204, 204, 255));
	btnCreateTable.addActionListener(this);
	p1.add(btnCreateTable);
	p1.add(Box.createRigidArea(new Dimension(15, 0)));
	// Edit
	btnEditTable = new JButton("Edit Table");
	btnEditTable.setBackground(new Color(204, 204, 255));
	btnEditTable.addActionListener(this);
	p1.add(btnEditTable);
	p1.add(Box.createRigidArea(new Dimension(15, 0)));
	// Update
	btnUpdateTable = new JButton("Update Table");
	btnUpdateTable.setBackground(new Color(204, 204, 255));
	btnUpdateTable.addActionListener(this);
	p1.add(btnUpdateTable);
	p1.add(Box.createRigidArea(new Dimension(15, 0)));*/
	// Remove
	btnRemoveTable = new JButton("Remove Table");
	btnRemoveTable.setBackground(new Color(204, 204, 255));
	btnRemoveTable.addActionListener(this);
	p1.add(btnRemoveTable);
	p1.add(Box.createRigidArea(new Dimension(15, 0)));
	// Cancel
	btnCancelTable = new JButton("Cancel");
	btnCancelTable.setBackground(new Color(204, 204, 255));
	btnCancelTable.addActionListener(this);
	p1.add(btnCancelTable);

	JPanel p2 = new JPanel();
	tvp = new TableValuePanel() {
	    };
	p2.add(tvp);		

	lstCode2 = tvp.getCode();
	lstItem = tvp.getItem();

	JPanel p3 = new JPanel();
	tvcp = new TableValueCommandPanel() {

		/**
		 * Handles action for click
		 * @param e The event
		 */
		public void actionPerformed(ActionEvent e) {
		    TableValueNode tvn;
		    if(e.getSource() == btnAdd) {
			tvn = new TableValueNode();
			//tvn.setType(tableModel.getTableType());
			tvn.setId((String)cbTableNumber.getSelectedItem());
			tvn.setName((String)cbTableName.getSelectedItem());
			//			tvn.setType(txtTableType.getText());
			tvn.setType((String)cbTableType.getSelectedItem());
			tvn.setStatus(TableValueNode.NEW);
			tvn.setCode(txtCode.getText());
			tvn.setItem(txtItem.getText());
			session.getTableDataModel().addValueTableViewer(tvn);
		    }
		    else if(e.getSource() == btnRemove) {
			Object[] tvnS = session.getTableDataModel().getCurrentTableValueNodes();
			if(tvnS.length == 1) {
			    tvn = session.getTableDataModel().getCurrentTableValueNode();
			    tvn.setStatus(TableValueNode.REMOVED);
			    session.getTableDataModel().modifyValueTableViewer(tvn);
			}
			else {
			    for(int i=0;i<tvnS.length;i++) {
				tvn = (TableValueNode)tvnS[i];
				tvn.setStatus(TableValueNode.REMOVED);
				session.getTableDataModel().modifyValueTableViewer(tvn);
			    }
			}
		    }
		    else if(e.getSource() == btnEdit) {
			tvn = session.getTableDataModel().getCurrentTableValueNode();
			if(tvn.getStatus() != TableValueNode.NEW) {
			    tvn.setStatus(TableValueNode.EDITED);
			    tvn.modifyCode(txtCode.getText());
			    tvn.modifyItem(txtItem.getText());
			}
			else {
			    tvn.setCode(txtCode.getText());
			    tvn.setItem(txtItem.getText());
			}
			session.getTableDataModel().modifyValueTableViewer(tvn);
		    }
		    else if(e.getSource() == btnNew) {
			session.getTableDataModel().newValue();
		    }
		    else if(e.getSource() == btnCancel) {
			if(session.getTableDataModel().isAllowedToModify()) {
			    btnNew.setEnabled(true);
			}
			else {
			    btnNew.setEnabled(false);
			}
			btnAdd.setEnabled(false); 
			btnRemove.setEnabled(false);
			btnEdit.setEnabled(false);
			btnCancel.setEnabled(false);
			txtCode.setEditable(false);
			txtCode.setText("");
			txtItem.setEditable(false);
			txtItem.setText("");
		    }
		}
	    };
	p3.add(tvcp);

	JPanel p4 = new JPanel();
	tvsp  = new TableValueSavePanel() {

		/**
		 * Handles action for click
		 * @param e The Event
		 */
		public void actionPerformed(ActionEvent e) {
		    try {
			if(e.getSource() == btnSave) {
			    File masterFile = null, sessionFile = null;
			    // HL7
			    if(session.getTableDataModel().needToSave("HL7User")) {
				masterFile = new File(HL7TABLES_MASTER);
				sessionFile = new File(session.getHL7Tables());
				if(masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {                 
				    JFileChooser fc = new JFileChooser(sessionFile.getParent());
				    // Set the chooser to accept only XML files.
				    fc.addChoosableFileFilter(new XMLFileFilter());
				    fc.setAcceptAllFileFilterUsed(false);
				    fc.setSelectedFile(new File("LocalHL7Tables.xml"));
				    int returnVal = fc.showOpenDialog(this);
				    
				    if(returnVal == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if(!f.getPath().endsWith(".xml")) {
					    f = new File(f.getPath() + ".xml");
					}
					session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), f);
				    }
				}
				else {
				    session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), new File(session.getHL7Tables()));
				}
			    }
			    // User
			    if(session.getTableDataModel().needToSave("User")) {
				masterFile = new File(USERTABLES_MASTER);
				sessionFile = new File(session.getUserTables());
				if(masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {                 
				    JFileChooser fc = new JFileChooser(sessionFile.getParent());
				    // Set the chooser to accept only XML files.
				    fc.addChoosableFileFilter(new XMLFileFilter());
				    fc.setAcceptAllFileFilterUsed(false);
				    fc.setSelectedFile(new File("LocalUserTables.xml"));
				    
				    int returnVal = fc.showOpenDialog(this);
				    
				    if(returnVal == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if(!f.getPath().endsWith(".xml")) {
					    f = new File(f.getPath() + ".xml");
					}
					session.getTableDataModel().saveTableFile("User", session.getUserTables(), f);
				    }
				}
				else {
				    session.getTableDataModel().saveTableFile("User", session.getUserTables(), new File(session.getUserTables()));
				}
			    }
			    // External
			    if(session.getTableDataModel().needToSave("External")) {
				masterFile = new File(EXTERNALTABLES_MASTER);
				sessionFile = new File(session.getExternalTables());
				if(masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {                 
				    JFileChooser fc = new JFileChooser(sessionFile.getParent());
				    // Set the chooser to accept only XML files.
				    fc.addChoosableFileFilter(new XMLFileFilter());
				    fc.setAcceptAllFileFilterUsed(false);
				    fc.setSelectedFile(new File("LocalExternalTables.xml"));
				    int returnVal = fc.showOpenDialog(this);
				    
				    if(returnVal == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if(!f.getPath().endsWith(".xml")) {
					    f = new File(f.getPath() + ".xml");
					}
					session.getTableDataModel().saveTableFile("External", session.getExternalTables(), f);
				    } 	
				}
				else {
				    session.getTableDataModel().saveTableFile("External", session.getExternalTables(), new File(session.getExternalTables()));
				}
			    }
			    // Local Table
			    Vector vLocalTable = session.getLocalTable();
			    for(int i=0;i<vLocalTable.size();i++) {
				TablePriority tp = (TablePriority)vLocalTable.get(i);
				String type = "Local[" + new File(tp.getFilename()).getName() + "]";
				if(session.getTableDataModel().needToSave(type))
				    session.getTableDataModel().saveTableFile(type, tp.getFilename(), new File(tp.getFilename()));
			    }			    
			    init();
			}
			if(e.getSource() == btnSaveAs) {
			    JFileChooser fc = null;
			    int returnVal = -1;
			    // HL7
			    if(session.getTableDataModel().needToSave("HL7User")) {
				fc = new JFileChooser(new File(session.getHL7Tables()).getParent());
				// Set the chooser to accept only XML files.
				fc.addChoosableFileFilter(new XMLFileFilter());
				fc.setAcceptAllFileFilterUsed(false);
				fc.setSelectedFile(new File("LocalHL7Tables.xml"));
				
				returnVal = fc.showOpenDialog(this);
				
				if(returnVal == JFileChooser.APPROVE_OPTION) {
				    File f = fc.getSelectedFile();
				    if(!f.getPath().endsWith(".xml")) {
					f = new File(f.getPath() + ".xml");
				    }
				    session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), f);
				}
			    }

			    // User
			    if(session.getTableDataModel().needToSave("User")) {
				fc = new JFileChooser(new File(session.getUserTables()).getParent());
				// Set the chooser to accept only XML files.
				fc.addChoosableFileFilter(new XMLFileFilter());
				fc.setAcceptAllFileFilterUsed(false);
				fc.setSelectedFile(new File("LocalUserTables.xml"));
				
				returnVal = fc.showOpenDialog(this);
				
				if(returnVal == JFileChooser.APPROVE_OPTION) {
				    File f = fc.getSelectedFile();
				    if(!f.getPath().endsWith(".xml")) {
					f = new File(f.getPath() + ".xml");
				    }
				    session.getTableDataModel().saveTableFile("User", session.getUserTables(), f);
				}
			    }

			    // External
			    if(session.getTableDataModel().needToSave("External")) {
				fc = new JFileChooser(new File(session.getExternalTables()).getParent());
				// Set the chooser to accept only XML files.
				fc.addChoosableFileFilter(new XMLFileFilter());
				fc.setAcceptAllFileFilterUsed(false);
				fc.setSelectedFile(new File("LocalExternalTables.xml"));
				
				returnVal = fc.showOpenDialog(this);
				
				if(returnVal == JFileChooser.APPROVE_OPTION) {
				    File f = fc.getSelectedFile();
				    if(!f.getPath().endsWith(".xml")) {
					f = new File(f.getPath() + ".xml");
				    }
				    session.getTableDataModel().saveTableFile("External", session.getExternalTables(), f);
				}
			    }
			    // Local Table
			    Vector vLocalTable = session.getLocalTable();
			    for(int i=0;i<vLocalTable.size();i++) {
				TablePriority tp = (TablePriority)vLocalTable.get(i);
				String type = "Local[" + new File(tp.getFilename()).getName() + "]";
				//if(session.getTableDataModel().needToSave(type)) {
				    fc = new JFileChooser(new File(tp.getFilename()).getParent());
				    // Set the chooser to accept only XML files.
				    fc.addChoosableFileFilter(new XMLFileFilter());
				    fc.setAcceptAllFileFilterUsed(false);
				    fc.setSelectedFile(new File("new" + new File(tp.getFilename()).getName()));
				    
				    returnVal = fc.showOpenDialog(this);
				    
				    if(returnVal == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if(!f.getPath().endsWith(".xml")) {
					    f = new File(f.getPath() + ".xml");
					}
					//session.getTableDataModel().cleanType(type);
					session.getTableDataModel().saveTableFile(type, tp.getFilename(), f);
				    }
				    //}			    
			    }
			    init();
			}
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(TransformerConfigurationException tce) {
			System.err.println("TransformerConfigurationException " + tce.getMessage());
			tce.printStackTrace();
		    }
		    catch(TransformerException te) {
			System.err.println("TransformerException " + te.getMessage());
			te.printStackTrace();
		    }	
		}		
	    };
	p4.add(tvsp);

        // Close Button
	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.LINE_AXIS));
	btnClose = new JButton("Close");
        btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p5.add(btnClose);

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.PAGE_AXIS));
	p7.add(p0);
	p7.add(Box.createRigidArea(new Dimension(0, 20)));
	p7.add(p1);
	p7.add(Box.createRigidArea(new Dimension(0, 20)));
	p7.add(p2);
	p7.add(Box.createRigidArea(new Dimension(0, 20)));
	p7.add(p3);
	p7.add(Box.createRigidArea(new Dimension(0, 20)));
	p7.add(p4);
	p7.add(Box.createRigidArea(new Dimension(0, 20)));
	p7.add(p5);
	p7.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

	getContentPane().add(p7);
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	tvp.setSession(session);
	tvcp.setSession(session);
	tvsp.setSession(session);
	// MVC
	session.getTableDataModel().addObserver(this);
	session.getTableDataModel().addObserver(tvp);
	session.getTableDataModel().addObserver(tvcp);
	session.getTableDataModel().addObserver(tvsp);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in TableViewer");
		init();
		tvsp.setSession((Session)arg);
	    }
	    else if(arg instanceof MVCArg) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in TableViewer");
		init();
	    }
	    else if(arg instanceof Vector) {
		cbTableType.removeAllItems();
		cbTableType.addItem("HL7");
		cbTableType.addItem("HL7User");
		cbTableType.addItem("External");
		cbTableType.addItem("User");

		Vector vLocalTable = (Vector)arg;
		for(int i=0;i<vLocalTable.size();i++) {
		    TablePriority tp = (TablePriority)vLocalTable.get(i);
		    cbTableType.addItem("Local[" + new File(tp.getFilename()).getName() + "]");
		}
		init();
	    }
	}
	else if(o instanceof TableDataModel) {
	    if(arg instanceof String) {
		if("addTable".compareTo(arg) == 0 || "modifyTable".compareTo(arg) == 0 || "removeTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableViewer");
		    init();
		}
		else if("newTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableViewer");
		    // Table Information
		    cbTableNumber.setEditable(true);
		    cbTableNumber.setSelectedItem("");
		    cbTableName.setEditable(true);
		    cbTableName.setSelectedItem("");
		    //		    txtTableType.setText("User");
		    cbTableType.setEnabled(true);
		    cbTableType.setSelectedItem("User");
		    // Apply Button
		    btnApply.setText("Apply");
		    // Table Command Button
		    btnNewTable.setEnabled(false);
/*		    btnCreateTable.setEnabled(true);
		    btnEditTable.setEnabled(false);
		    btnUpdateTable.setEnabled(false);*/
		    btnRemoveTable.setEnabled(false);
		    btnCancelTable.setEnabled(true);
		}
		else if("editTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableViewer");
		    // Table Information
		    //	    cbTableNumber.setEditable(true);
		    cbTableName.setEditable(true);
		    // Apply Button
		    btnApply.setText("Apply");
		    // Table Command Button
		    btnNewTable.setEnabled(false);
/*		    btnCreateTable.setEnabled(false);
		    btnEditTable.setEnabled(false);
		    btnUpdateTable.setEnabled(true);*/
		    btnRemoveTable.setEnabled(false);
		    btnCancelTable.setEnabled(true);
		}
	    }
	}
    }

    /**
     * Handle event for comboBox
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnClose) {
	    if(session.getTableDataModel().needToSave()) {
		Object[] options = {"Yes", "No", "Cancel"};
		JOptionPane optionPane = new JOptionPane("Table data was modified. Do you want to save?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Yes");
		JDialog dialog = optionPane.createDialog(this, "Save the data");
		dialog.show();
		Object selectedValue = optionPane.getValue();
		
		if(selectedValue != null) {
		    if(((String)selectedValue).compareTo(options[0]) == 0) {
			// Save The data
			tvsp.save();
			setVisible(false);
		    }
		    else if(((String)selectedValue).compareTo(options[1]) == 0) {
			setVisible(false);
			session.getTableDataModel().cancelModification();
		    }
		    else if(((String)selectedValue).compareTo(options[2]) == 0) {
			dialog.dispose();
		    }
		}
	    }
	    else {
		setVisible(false);
	    }
	}
	else if(evt.getSource() == cbTableNumber) {
	    if(cbTableNumber.getSelectedIndex() != -1) {
		cbTableNumber.setEditable(false);
		cbTableName.setEditable(false);
		cbTableName.removeActionListener(this);
		cbTableName.setSelectedIndex(cbTableNumber.getSelectedIndex());
		cbTableName.addActionListener(this);
		TableNode tn = (TableNode)vTableInfo.get(cbTableNumber.getSelectedIndex());
		//txtTableType.setText(tn.getType());
		cbTableType.setSelectedItem(tn.getType());
		
		// Table Command Button
		btnNewTable.setEnabled(true);
		/*		btnCreateTable.setEnabled(false);
				btnEditTable.setEnabled(true);
				btnUpdateTable.setEnabled(false);*/
		btnRemoveTable.setEnabled(true);
		btnCancelTable.setEnabled(false);
		
		session.getTableDataModel().updateLists(new MVCArg("updateLists", (String)cbTableNumber.getSelectedItem()));
	    }
	}
	else if(evt.getSource() == cbTableName) {
	    if(cbTableName.getSelectedIndex() != -1) {
		cbTableNumber.setEditable(false);
		cbTableName.setEditable(false);
		cbTableNumber.removeActionListener(this);
		cbTableNumber.setSelectedIndex(cbTableName.getSelectedIndex());
		cbTableNumber.addActionListener(this);
		TableNode tn = (TableNode)vTableInfo.get(cbTableName.getSelectedIndex());
		//txtTableType.setText(tn.getType());
		cbTableType.setSelectedItem(tn.getType());

		// Table Command Button
		btnNewTable.setEnabled(true);
/*		btnCreateTable.setEnabled(false);
		btnEditTable.setEnabled(true);
		btnUpdateTable.setEnabled(false);*/
		btnRemoveTable.setEnabled(true);
		btnCancelTable.setEnabled(false);

		session.getTableDataModel().updateLists(new MVCArg("updateLists", (String)cbTableNumber.getSelectedItem()));
	    }
	}
	else if(evt.getSource() == btnNewTable) {
	    currentIdx = cbTableNumber.getSelectedIndex();
	    action = "new";
	    session.getTableDataModel().newTable();
	}
	else if(evt.getSource() == btnApply) {
	    if(action.compareTo("new") == 0) {
		try { 
		    // Action / Check if the number is unique and add to the model
		    if("".compareTo((String)cbTableName.getSelectedItem()) == 0) {
			JOptionPane.showMessageDialog(this,
						      "The name must have a value", 
						      "Table Name Error",
						      JOptionPane.ERROR_MESSAGE);
		    }
		    else if(session.getTableDataModel().isUnique((String)cbTableNumber.getSelectedItem())) {
			//		    tableModel.addTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), txtTableType.getText());
			if("HL7".compareTo((String)cbTableType.getSelectedItem()) == 0) {
			    cbTableType.setSelectedItem("HL7User");
			}
			session.getTableDataModel().addTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), (String)cbTableType.getSelectedItem());
			cbTableNumber.setSelectedIndex(cbTableNumber.getItemCount()-1);
		    }
		    else {
			JOptionPane.showMessageDialog(this,
						      "The table number already exists, it must be unique",
						      "Table Number Error",
						      JOptionPane.ERROR_MESSAGE);
			cbTableNumber.setSelectedIndex(0);
			cbTableName.setSelectedIndex(0);
		    }
		    cbTableType.setEnabled(false);
		}
		catch(NumberFormatException nfe) {
		    JOptionPane.showMessageDialog(this,
						  "The input for the table number is incorrect, it must be an integer",
						  "Table Number Error",
						  JOptionPane.ERROR_MESSAGE);
		}
	    }
	    else if(action.compareTo("") == 0) {
		currentIdx = cbTableNumber.getSelectedIndex();
		action = "edit";
		session.getTableDataModel().editTable();
	    }
	    else if(action.compareTo("edit") == 0) {
		int idx = cbTableNumber.getSelectedIndex();
		// Action / Modify the model
		//tableModel.modifyTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), txtTableType.getText());
		if("".compareTo((String)cbTableName.getSelectedItem()) == 0) {
		    JOptionPane.showMessageDialog(this,
						  "The name must have a value", 
						  "Table Name Error",
						  JOptionPane.ERROR_MESSAGE);
		}
		else {
		    session.getTableDataModel().modifyTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), (String)cbTableType.getSelectedItem());
		    cbTableNumber.setSelectedIndex(idx);
		}
	    }
	}
/*	else if(evt.getSource() == btnEditTable) {
	    currentIdx = cbTableNumber.getSelectedIndex();
	    tableModel.editTable();
	    }
	else if(evt.getSource() == btnUpdateTable) {
	    int idx = cbTableNumber.getSelectedIndex();
	    // Action / Modify the model
	    //tableModel.modifyTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), txtTableType.getText());
	    if("".compareTo((String)cbTableName.getSelectedItem()) == 0) {
		JOptionPane.showMessageDialog(this,
					      "The name must have a value", 
					      "Table Name Error",
					      JOptionPane.ERROR_MESSAGE);
	    }
	    else {
		tableModel.modifyTable((String)cbTableNumber.getSelectedItem(), (String)cbTableName.getSelectedItem(), (String)cbTableType.getSelectedItem());
		cbTableNumber.setSelectedIndex(idx);
	    }
	    }*/
	else if(evt.getSource() == btnRemoveTable) {
	    // Action / Remove the table from the model
	    JOptionPane optionPane = new JOptionPane("Do you want to remove the table " + (String)cbTableNumber.getSelectedItem() + " - " + (String)cbTableName.getSelectedItem(),JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	    JDialog dialog = optionPane.createDialog(this, "Remove Table");
	    dialog.show();
	    Object selectedValue = optionPane.getValue();
	    if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		session.getTableDataModel().removeTable((String)cbTableNumber.getSelectedItem());
	    }
	}
	else if(evt.getSource() == btnCancelTable) {
	    cbTableNumber.setEditable(false);
	    cbTableName.setEditable(false);
	    cbTableType.setEnabled(false);
	    //tableModel.cancel();
	    // Table Command Button
	    // Action / Modify the model
	    init();
	    cbTableNumber.setSelectedIndex(currentIdx);
	    btnCancelTable.setEnabled(false);
	}
    }

    /**
     * Init the GUI
     */
    public void init() {
	if(session.getTableDataModel() != null) {
	    vTableInfo = session.getTableDataModel().getTablesInformation();
	    // Table Information
	    cbTableNumber.removeAllItems();
	    cbTableName.removeAllItems();
	    cbTableNumber.removeActionListener(this);
	    cbTableName.removeActionListener(this);
	    for(int i=0;i<vTableInfo.size();i++) {
		TableNode tn = (TableNode)vTableInfo.get(i);
		//System.out.println(tn.getId());
		// Table Number
		cbTableNumber.addItem(tn.getId());
		// Table Name
		cbTableName.addItem(tn.getName());
		// Table Type
		//txtTableType.setText(tn.getType());
		cbTableType.setSelectedItem(tn.getType());
	    }
	    // Set to the first item
	    TableNode tn = (TableNode)vTableInfo.get(0);
	    cbTableNumber.setSelectedIndex(0);
	    cbTableName.setSelectedIndex(0);
	    cbTableType.setSelectedItem(tn.getType());

	    cbTableNumber.addActionListener(this);
	    cbTableName.addActionListener(this);

	    // Apply Button
	    btnApply.setText("Edit Name");
	    action = "";
	    // Table Command Button
	    btnNewTable.setEnabled(true);
/*	    btnCreateTable.setEnabled(false);
	    btnEditTable.setEnabled(true);
	    btnUpdateTable.setEnabled(false);*/
	    btnRemoveTable.setEnabled(true);
	    btnCancelTable.setEnabled(false);

	    session.getTableDataModel().initTableViewer(new MVCArg("initTable", (String)cbTableNumber.getSelectedItem()));
	}
    }

    /**
     * Handle the event of closing the dialog
     * @param evt The Event
     */
    public void closeDialog(WindowEvent evt) {
	if(session.getTableDataModel().needToSave()) {
	    JOptionPane optionPane = new JOptionPane("Table data was modified. Do you want to save?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	    JDialog dialog = optionPane.createDialog(this, "Save the data");
	    dialog.show();
	    Object selectedValue = optionPane.getValue();
	    
	    if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		// Save The data
		tvsp.save();
	    }
	}
	setVisible(false);
    }
}
