/**
 * This class represents a component in the profile
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Vector;

public class ComponentNode extends ProfileNode {

    private Vector vSubComponent;
    private String msgLocPrefix;

    /**
     * Constructor
     */
    public ComponentNode() {
	vSubComponent = new Vector();
    }

    /** 
     * Constructor, create a new ComponentNode from a ComponentNode, kind of clone
     * @param cn A ComponentNode
     */
    public ComponentNode(ComponentNode cn) {
	this();
	name = cn.getName();
	qualifier = cn.getQualifier();
	segment = cn.getSegment();
	datatype = cn.getDatatype();
	usage = cn.getUsage();
	cardMin = cn.getCardMin();
	cardMax = cn.getCardMax();
	length = cn.getLength();
	itemNo = cn.getItemNo();
	sequenceNum = cn.getSequenceNum();
	parent = cn.getParent();
	batchValue = cn.getBatchValue();
    }

    /**
     * Get the subcomponents of the component
     * @return The subcomponents
     */
    public Vector getSubComponents() {
	return vSubComponent;
    }

    /**
     * Add a subcomponent
     * @param subcomponent The subcomponent
     */
    public void addSubComponent(SubComponentNode subcomponent) {
	if(subcomponent != null) // && !vSubComponent.contains(subcomponent))
	    vSubComponent.add(subcomponent);
    }

    /**
     * Return true if it is a leaf
     * @return is it a leaf?
     */
    public boolean isLeaf() {
	return vSubComponent.size() == 0;
    }

    public String getValue() {
	return null;
    }

    public String getMsgLoc() {
	int idx = sequenceNum.lastIndexOf(".");
	//	return msgLocPrefix  + "." + sequenceNum.substring(idx+1);
	return segment + "." + sequenceNum.substring(0, idx) + " " + parent.getDatatype() + "." + sequenceNum.substring(idx+1);
    }

    public void setMsgLocPrefix(String msgLocPrefix) {
	this.msgLocPrefix = msgLocPrefix;
    }
}
