/**
 * This class is a panel to configure the data
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Observer;
import java.util.Observable;
import java.io.IOException;

// AWT
import java.awt.Dimension;
import java.awt.CardLayout;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;
import javax.swing.JCheckBox;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JDialog;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.WaitDialog;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

public class DataConfigurationView extends JPanel implements Observer, ActionListener {

    private JSplitPane sp;

    private ProfileTree pt;
    private JScrollPane treeScrollPane;
    private JCheckBox chkMissing;

    private DataPanel dp;
    private JScrollPane dataScrollPane;

    private TablePanel tp;
    private JScrollPane tableScrollPane;

    private JPanel dataTablePanel;

    private Session session;
    private TableDataModel tableModel;

    /**
     * Constructor
     * @param ses The session
     */
    public DataConfigurationView(Session ses) {
	// ProfileTree
	pt = new ProfileTree();
	treeScrollPane = new JScrollPane(pt);
	treeScrollPane.setPreferredSize(new Dimension(250,500));

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
	p0.add(treeScrollPane);
	chkMissing = new JCheckBox("Show elements with missing values");
	chkMissing.setAlignmentX(0.5f);
	chkMissing.addActionListener(this);
	chkMissing.setEnabled(false);
	p0.add(chkMissing);

	// DataPanel
	dp = new DataPanel();
	dataScrollPane = new JScrollPane(dp);
	
	// TablePanel
	tp = new TablePanel();
	tableScrollPane = new JScrollPane(tp);

	setSession(ses);
	
	// CardLayout
	dataTablePanel = new JPanel(new CardLayout());
	dataTablePanel.add(dataScrollPane, "Value DB");
	dataTablePanel.add(tableScrollPane, "Value Table");

	//sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treeScrollPane, dataTablePanel);
	sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, p0, dataTablePanel);
	sp.setDividerLocation(250);

	this.add(sp);
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	dp.setSession(ses);
	tp.setSession(ses);
	// MVC
	session.addObserver(this);
	session.getRepositoryDataModel().addObserver(this);
	session.getTableDataModel().addObserver(this);
    }

    /**
     * Handles action for click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() == chkMissing) {
	    if(chkMissing.isSelected()) {
		JDialog dlgWait = new WaitDialog("Looking for elements with missing values");
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dlgWait.getSize();
		dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				    Math.max(0,(screenSize.height-windowSize.height)/2));
		
		final JDialog fDlg = dlgWait;
		Thread t = new Thread() {
			public void run() {
			    pt.showMissing(chkMissing.isSelected());
			    fDlg.dispose();
			}
		    };
		t.start();
		
		dlgWait.show();
	    }
	    else {
		pt.showMissing(chkMissing.isSelected());
	    }
	}
    }
    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof MVCArg) {
		if(((MVCArg)arg).getName().compareTo("profile") == 0) {
		    CardLayout cl = (CardLayout)(dataTablePanel.getLayout());
		    cl.show(dataTablePanel, "Value DB");
 		    pt.setTreeModel((Session)o);
		    if(!((MVCArg)arg).getValue().equals("")) {
			chkMissing.setEnabled(true);
			chkMissing.setSelected(false);
		    }
		    else {
			chkMissing.setEnabled(false);
			chkMissing.setSelected(false);
		    }
		}
	    }
	    else if(arg instanceof Session) {
		CardLayout cl = (CardLayout)(dataTablePanel.getLayout());
		cl.show(dataTablePanel, "Value DB");
		pt.setTreeModel((Session)o);
		if(!((Session)o).getProfile().equals("")) {
		    chkMissing.setEnabled(true);
		    chkMissing.setSelected(false);
		}
		else {
		    chkMissing.setEnabled(false);
		    chkMissing.setSelected(false);
		}
	    }
	}
	else if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ProfileNode) {
		CardLayout cl = (CardLayout)(dataTablePanel.getLayout());
		cl.show(dataTablePanel, "Value DB");
	    }
	}
	else if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in DataConfigurationPanel");
		CardLayout cl = (CardLayout)(dataTablePanel.getLayout());
		cl.show(dataTablePanel, "Value Table");
	    }
	    else if(arg instanceof String) {
		if("init".compareTo(arg) == 0) {
		    CardLayout cl = (CardLayout)(dataTablePanel.getLayout());
		    cl.show(dataTablePanel, "Value DB");
		}
	    }
	}
    }
}
