/**
 * This class is a popup for a list
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// AWT
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;

// Swing
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

// MessageMaker
import gov.nist.hl7maker.control.Session;

public class ListPopUp extends JPopupMenu implements Observer, ActionListener {

    private JMenuItem paste;
    private Vector vColumn1;
    private Vector vColumn2;
    private String type;

    private ValuePropertiesPanel vpp;
    private Session session;

    /**
     * Constructor
     */
    public ListPopUp() {
	paste = new JMenuItem("Paste from the clipboard");
	paste.addActionListener(this);
	add(paste);
	vColumn1 = new Vector();
	vColumn2 = new Vector();
	type = "";
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getRepositoryDataModel().addObserver(this);
	session.getTableDataModel().addObserver(this);
    }

    /**
     * Set the ValuePropertiesPanel
     * @param vpp The ValuePropertiesPanel
     */
    public void setValuePropertiesPanel(ValuePropertiesPanel vpp) {
	this.vpp = vpp;
    }

    /**
     * Handle a click on the JPopupMenu
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	if(type.compareTo("RepositoryModel") == 0) {
	    for(int i=0;i<vColumn2.size();i++) {
		ValueNode vn = new ValueNode();
		vpp.updateProperties(vn);
		vn.setStatus(ValueNode.NEW);
		vn.setValue((String)vColumn2.get(i));
		session.getRepositoryDataModel().addValue(vn);
	    }
	}
	else if(type.compareTo("HL7User") == 0 || type.compareTo("User") == 0 || type.compareTo("External") == 0) {
	    for(int i=0;i<vColumn1.size();i++) {
		TableValueNode tvn = new TableValueNode();
		tvn.setType(session.getTableDataModel().getTableType());
		tvn.setStatus(TableValueNode.NEW);
		tvn.setCode((String)vColumn1.get(i));
		tvn.setItem((String)vColumn2.get(i));
		session.getTableDataModel().addValue(tvn);
	    }
	}
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from ReferenceDBModel in ListPopUp");
		type = "RepositoryModel";
	    }
	}
	else if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in ListPopUp");
		type = session.getTableDataModel().getTableType();
	    }
	}
    }

    /**
     * Check the content of clipboard
     * @return Yes if the clipboard content is valid
     */
    public boolean checkClipboard() {
	boolean valid = true;
	Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
	Transferable clipData = clipboard.getContents(clipboard);
	String s = null;
	try {
	    s = (String)(clipData.getTransferData(DataFlavor.stringFlavor));
	    // Process the data in clipboard
	    vColumn1.clear();
	    vColumn2.clear();
	    
	    int idxStart = 0, idxEnd = 0;
	    for(int i=0;i<s.length();i++) {
		if(s.charAt(i) == '\t') {
		    idxEnd = i;
		    if(idxEnd > idxStart)
			vColumn1.add(s.substring(idxStart, idxEnd).trim());
		    idxStart = i+1;
		}
		else if(s.charAt(i) == '\n' || i == s.length()-1) {
		    idxEnd = i;
		    if(i == s.length()-1)
			idxEnd = s.length();
		    if(idxEnd > idxStart)
			vColumn2.add(s.substring(idxStart, idxEnd).trim());
		    idxStart = i+1;
		}
	    }
	    if(type.compareTo("RefDBModel") == 0 || type.compareTo("ConfigModel") == 0) {
		if(vColumn2.size() == 0) {
		    valid = false;
		}
	    }
	    else if(type.compareTo("HL7User") == 0 || type.compareTo("User") == 0 || type.compareTo("External") == 0) {
		if(vColumn1.size() == 0 || vColumn2.size() == 0 || vColumn1.size() != vColumn2.size()) {
		    valid = false;
		}
	    }
	    else if(type.compareTo("HL7") == 0) {
		valid = false;
	    }
	}
	catch (UnsupportedFlavorException ee) {
	    valid = false;
	}
	catch(IOException ioe){;}
	return valid;
    }

    /**
     * Show the JPopupMenu
     * @param invoker the component in whose space the popup menu is to appear
     * @param x the x coordinate in invoker's coordinate space at which the popup menu is to be displayed
     * @param y the y coordinate in invoker's coordinate space at which the popup menu is to be displayed
     */
    public void show(Component invoker, int x, int y) {
	if(checkClipboard()) {
	    paste.setEnabled(true);
	}
	else {
	    paste.setEnabled(false);
	}
	super.show(invoker, x, y);
    }
}
