/**
 * This class represents a node of the profile
 */

package gov.nist.hl7maker.view.config;

public abstract class ProfileNode {

    protected String name;
    protected String qualifier;
    protected String qualifier2;
    protected String segment;
    protected String table;
    protected String datatype;
    protected String type;
    protected String usage;
    protected String cardMin;
    protected String cardMax;
    protected String length;
    protected String itemNo;
    protected String sequenceNum;

    protected ProfileNode parent;

    protected String value;
    protected String batchValue;

    /**
     * Constructor
     */
    public ProfileNode() {
    }

    // Get Methods
    public String getName() {
	return name;
    }

    public String getQualifier() {
	return qualifier;
    }

    public String getQualifier2() {
	return qualifier2;
    }

    public String getSegment() {
	return segment;
    }

    public String getTable() {
	return table;
    }

    public String getDatatype() {
	return datatype;
    }

    public String getType() {
	return type;
    }

    public String getUsage() {
	return usage;
    }

    public String getCardMin() {
	return cardMin;
    }

    public String getCardMax() {
	return cardMax;
    }

    public String getLength() {
	return length;
    }

    public String getItemNo() {
	return itemNo;
    }

    public String getSequenceNum() {
	return sequenceNum;
    }

    public String getBatchValue() {
	return batchValue;
    }

    public ProfileNode getParent() {
	return parent;
    }

    // Set Methods
    public void setName(String name) {
	this.name = name;
    }

    public void setQualifier(String qualifier) {
	this.qualifier = qualifier;
    }

    public void setQualifier2(String qualifier2) {
	this.qualifier2 = qualifier2;
    }

    public void setSegment(String segment) {
	this.segment = segment;
    }

    public void setTable(String table) {
	this.table = table;
    }

    public void setDatatype(String datatype) {
	this.datatype = datatype;
    }

    public void setType(String type) {
	this.type = type;
    }

    public void setUsage(String usage) {
	this.usage = usage;
    }

    public void setCardMin(String cardMin) {
	this.cardMin = cardMin;
    }

    public void setCardMax(String cardMax) {
	this.cardMax = cardMax;
    }

    public void setLength(String length) {
	this.length = length;
    }

    public void setItemNo(String itemNo) {
	this.itemNo = itemNo;
    }

    public void setSequenceNum(String sequenceNum) {
	this.sequenceNum = sequenceNum;
    }

    public void setBatchValue(String batchValue) {
	this.batchValue = batchValue;
    }

    public void setParent(ProfileNode parent) {
	this.parent = parent;
    }

    // Methods
    public String toString() {
	return name;
    }

    /**
     * Return true if the two objects are equal
     * @param o The other object
     * @return return true if equal
     */
    public boolean equals(Object o) {
	ProfileNode pn = (ProfileNode)o;
	boolean equal = true;

	if(name.compareTo(pn.getName()) != 0)
	    equal = false;
	if(qualifier.compareTo(pn.getQualifier()) != 0)
	    equal = false;
	if(qualifier2.compareTo(pn.getQualifier2()) != 0)
	    equal = false;
	if(segment.compareTo(pn.getSegment()) != 0)
	    equal = false;
	if(!getParentPath().equals(pn.getParentPath()))
	    equal = false;
	return equal;
    }

    /**
     * Special Equals
     * @param o The other object
     * @return return true if equal
     */
    public boolean specialEquals(Object o) {
	ProfileNode pn = (ProfileNode)o;
	boolean equal = true;

	if(name.compareTo(pn.getName()) != 0)
	    equal = false;
	if(qualifier.compareTo(pn.getQualifier()) != 0)
	    equal = false;
	if(qualifier2.compareTo(pn.getQualifier2()) != 0)
	    equal = false;
	if(segment.compareTo(pn.getSegment()) != 0)
	    equal = false;
	return equal;
    }
    
    /**
     * Get the parent path
     * @return The parent path (xxx/xxx/xxx)
     */
    public String getParentPath() {
	StringBuffer sb = new StringBuffer("");
	ProfileNode parentRec = this;
	while(parentRec != null) {
	    if(parentRec instanceof SegmentGroupNode)
		sb.insert(0, (parentRec.getName() + "/"));
	    parentRec = parentRec.getParent();
	}
	if(sb.length() > 0)
	    return sb.substring(0, sb.length()-1);
	return "";
    }

    /**
     * Has an X usage
     * @return true or false
     */
    public boolean hasXUsage() {
	ProfileNode parentRec = this.getParent();
	while(parentRec != null) {
	    if(parentRec.getUsage().equals("X"))
		return true;
	    parentRec = parentRec.getParent();
	}
	return false;
    }

    /**
     * Has an RE usage
     * @return true or false
     */
    public boolean hasREUsage() {
	ProfileNode parentRec = this.getParent();
	while(parentRec != null) {
	    if(parentRec.getUsage().equals("RE"))
		return true;
	    parentRec = parentRec.getParent();
	}
	return false;
    }

    // Abstract methods
    public abstract String getValue();
    public abstract boolean isLeaf();
    public abstract String getMsgLoc();
}
