/**
 * This class represents all the action one can do with a table value
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.Box;
import javax.swing.BoxLayout;

// Message Maker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;

public class TableValueCommandPanel extends JPanel implements Observer, ActionListener {

    protected JButton btnNew;
    protected JButton btnAdd;
    protected JButton btnRemove;
    protected JButton btnEdit;
    protected JButton btnCancel;
    protected JTextField txtCode;
    protected JTextField txtItem;

    private Session session;

    public TableValueCommandPanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.PAGE_AXIS);
	setLayout(layout);
	
	JPanel btnPane = new JPanel();
	btnPane.setLayout(new BoxLayout(btnPane, BoxLayout.LINE_AXIS));

	btnNew = new JButton("New");
        btnNew.setBackground(new Color(204, 204, 255));
	btnNew.setEnabled(false);
	btnNew.addActionListener(this);
	btnPane.add(btnNew);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnAdd = new JButton("Add");
        btnAdd.setBackground(new Color(204, 204, 255));
	btnAdd.setEnabled(false);
	btnAdd.addActionListener(this);
	btnPane.add(btnAdd);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnRemove = new JButton("Remove");
        btnRemove.setBackground(new Color(204, 204, 255));
	btnRemove.setEnabled(false);
	btnRemove.addActionListener(this);
	btnPane.add(btnRemove);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnEdit = new JButton("Edit");
        btnEdit.setBackground(new Color(204, 204, 255));
	btnEdit.setEnabled(false);
	btnEdit.addActionListener(this);
	btnPane.add(btnEdit);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnCancel = new JButton("Cancel");
        btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.setEnabled(false);
	btnCancel.addActionListener(this);
	btnPane.add(btnCancel);

	add(btnPane);
	add(Box.createRigidArea(new Dimension(0, 10)));

	JPanel txtPane = new JPanel();
	txtPane.setLayout(new BoxLayout(txtPane, BoxLayout.LINE_AXIS));

	txtPane.add(Box.createRigidArea(new Dimension(75, 0)));

	txtCode = new JTextField("");
	txtCode.setPreferredSize(new Dimension(200, 20));
	txtCode.setEditable(false);
	txtCode.addActionListener(this);
	txtPane.add(txtCode);

	txtPane.add(Box.createRigidArea(new Dimension(25, 0)));
 
	txtItem = new JTextField("");
	txtItem.setPreferredSize(new Dimension(200, 20));
	txtItem.setEditable(false);
	txtItem.addActionListener(this);
	txtPane.add(txtItem);

	txtPane.add(Box.createRigidArea(new Dimension(75, 0)));

	txtPane.setMaximumSize(new Dimension(500, 25));

	add(txtPane);
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getTableDataModel().addObserver(this);
    }

    /**
     * Handles action for click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	TableValueNode tvn;
	if(e.getSource() == btnAdd) {
	    tvn = new TableValueNode();
	    if(session.getTableDataModel().getTableType().compareTo("") == 0)
		tvn.setType("User");
	    else
		tvn.setType(session.getTableDataModel().getTableType());
	    tvn.setStatus(TableValueNode.NEW);
	    tvn.setCode(txtCode.getText());
	    tvn.setItem(txtItem.getText());
	    tvn.setId(session.getTableDataModel().getCurrentProfileNode().getTable());
	    tvn.setName(session.getTableDataModel().getTableName(session.getTableDataModel().getCurrentProfileNode().getTable()));
	    session.getTableDataModel().addValue(tvn);
	}
	else if(e.getSource() == btnRemove) {
	    Object[] tvnS = session.getTableDataModel().getCurrentTableValueNodes();
	    if(tvnS.length == 1) {
		tvn = session.getTableDataModel().getCurrentTableValueNode();
		tvn.setStatus(TableValueNode.REMOVED);
		session.getTableDataModel().modifyValue(tvn);
	    }
	    else {
		for(int i=0;i<tvnS.length;i++) {
		    tvn = (TableValueNode)tvnS[i];
		    tvn.setStatus(TableValueNode.REMOVED);
		    session.getTableDataModel().modifyValue(tvn);
		}
	    }
	}
	else if(e.getSource() == btnEdit) {
	    tvn = session.getTableDataModel().getCurrentTableValueNode();
	    if(tvn.getStatus() != TableValueNode.NEW) {
		tvn.setStatus(TableValueNode.EDITED);
		tvn.modifyCode(txtCode.getText());
		tvn.modifyItem(txtItem.getText());
	    }
	    else {
		tvn.setCode(txtCode.getText());
		tvn.setItem(txtItem.getText());
	    }
	    session.getTableDataModel().modifyValue(tvn);
	}
	else if(e.getSource() == btnNew) {
	    session.getTableDataModel().newValue();
	}
	else if(e.getSource() == btnCancel) {
	    if(session.getTableDataModel().isAllowedToModify()) {
		btnNew.setEnabled(true);
	    }
	    else {
		btnNew.setEnabled(false);
	    }
	    btnAdd.setEnabled(false); 
	    btnRemove.setEnabled(false);
	    btnEdit.setEnabled(false);
	    btnCancel.setEnabled(false);
	    txtCode.setEditable(false);
	    txtCode.setText("");
	    txtItem.setEditable(false);
	    txtItem.setText("");
	}
	else if(e.getSource() == txtCode || e.getSource() == txtItem) {
	    // if click on ENTER, it adds the value, it does not work if you want to edit or remove a value
	    if(!btnRemove.isEnabled()) {
		btnAdd.doClick();
		btnAdd.grabFocus();
	    }
	}
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		if(session.getTableDataModel().isAllowedToModify()) {
		    btnNew.setEnabled(true);
		}
		else {
		    btnNew.setEnabled(false);
		}
		btnAdd.setEnabled(false); 
		btnRemove.setEnabled(false);
		btnEdit.setEnabled(false);
		btnCancel.setEnabled(false);
		txtCode.setEditable(false);
		txtCode.setText("");
		txtItem.setEditable(false);
		txtItem.setText("");
	    }
	    else if(arg instanceof String) {
		if("new".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		    txtCode.setText("");
		    txtCode.setEditable(true);
		    txtItem.setText("");
		    txtItem.setEditable(true);
		    btnAdd.setEnabled(true);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(true);
		}
		else if("init".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		    btnNew.setEnabled(false);
		    btnAdd.setEnabled(false);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(false);
		    txtCode.setEditable(false);
		    txtCode.setText("");
		    txtItem.setEditable(false);
		    txtItem.setText("");
		}
		else if("newTable".compareTo(arg) == 0 || "editTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		    btnNew.setEnabled(false);
		    btnAdd.setEnabled(false);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(false);
		    txtCode.setEditable(false);
		    txtCode.setText("");
		    txtItem.setEditable(false);
		    txtItem.setText("");
		}
	    }
	    else if(arg instanceof TableValueNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		btnNew.setEnabled(true);
		btnAdd.setEnabled(false);
		btnRemove.setEnabled(true);
		btnEdit.setEnabled(true);
		btnCancel.setEnabled(true);
		txtCode.setEditable(true);
		txtCode.setText(((TableValueNode)arg).getCode());
		txtItem.setEditable(true);
		txtItem.setText(((TableValueNode)arg).getItem());
	    }
	    else if(arg instanceof MVCArg) {
		if("initTable".compareTo(((MVCArg)arg).getName()) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		    btnNew.setEnabled(true);
		    btnAdd.setEnabled(false);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    txtCode.setEditable(false);
		    txtCode.setText("");
		    txtItem.setEditable(false);
		    txtItem.setText("");
		}
		else if("updateLists".compareTo(((MVCArg)arg).getName()) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueCommandPanel");
		    if(session.getTableDataModel().isAllowedToModify(((MVCArg)arg).getValue())) {
			btnNew.setEnabled(true);
		    }
		    else {
			btnNew.setEnabled(false);
		    }
		    btnAdd.setEnabled(false); 
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(false);
		    txtCode.setEditable(false);
		    txtCode.setText("");
		    txtItem.setEditable(false);
		    txtItem.setText("");
		}
	    }
	}
    }
}
