/**
 * This class is a panel to make a data link
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.ListModel;

// Message Maker
import gov.nist.hl7maker.control.Session;

public class TableValueDataLinkPanel extends JPanel implements Observer, ActionListener {

    private JComboBox cbSuggested;
    private JButton btnCopy;
    private JList lstValue;
    private JScrollPane listScrollPane;

    private Session session;
    private Vector vSimilar;

    /**
     * Constructor
     */
    public TableValueDataLinkPanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.PAGE_AXIS);
	setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JLabel l = new JLabel("Suggested Data Link: ");
	l.setAlignmentX(Component.LEFT_ALIGNMENT);
	p0.add(Box.createRigidArea(new Dimension(55, 0)));
	p0.add(l);
	p0.add(Box.createRigidArea(new Dimension(5, 0)));
	cbSuggested = new JComboBox();
	cbSuggested.setAlignmentX(Component.LEFT_ALIGNMENT);
	cbSuggested.setEnabled(false);
	cbSuggested.addActionListener(this);
	cbSuggested.setPreferredSize(new Dimension(200, 25));
	cbSuggested.setMaximumSize(new Dimension(200, 25));
	p0.add(cbSuggested);
	p0.add(Box.createRigidArea(new Dimension(15, 0)));
	btnCopy = new JButton("Copy Values");
	btnCopy.setAlignmentX(Component.LEFT_ALIGNMENT);
	btnCopy.setBackground(new Color(204, 204, 255));
	btnCopy.setEnabled(false);
	btnCopy.addActionListener(this);
	p0.add(btnCopy);
	p0.add(Box.createRigidArea(new Dimension(60, 0)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	lstValue = new JList();
	lstValue.setEnabled(false);
	listScrollPane = new JScrollPane(lstValue);
	listScrollPane.setPreferredSize(new Dimension(100, 80));
	//	listScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
	p1.add(Box.createRigidArea(new Dimension(30, 0)));
	p1.add(listScrollPane);
	p1.add(Box.createRigidArea(new Dimension(30, 0)));

	add(p0);
	add(Box.createRigidArea(new Dimension(0, 10)));
	add(p1);
	add(Box.createRigidArea(new Dimension(0, 0)));
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueDataLinkPanel");
		cbSuggested.removeAllItems();
		// Look for the closest data link
		ProfileNode pn = (ProfileNode)arg;
		vSimilar = session.getTableDataModel().getSimilarProfileNode(pn);
		if(vSimilar.size() > 0 && pn.getSegment().startsWith("Z")) {
		    // Add elements to the comboBox
		    for(int i=0;i<vSimilar.size();i++) {
			if(vSimilar.get(i) instanceof String)
			    cbSuggested.addItem(vSimilar.get(i));
		    }
		    cbSuggested.setEnabled(true);
		    btnCopy.setEnabled(true);
		}
		else {
		    cbSuggested.setEnabled(false);
		    btnCopy.setEnabled(false);
		    lstValue.setListData(new Vector());
		}
	    }
	}
    }

    /**
     * Handles action for click
     * @param e The event
     */    
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() == cbSuggested) {
	    if(cbSuggested.getSelectedItem() != null) {
		boolean flag = false;
		Vector vTmp = new Vector();
		for(int i=0;i<vSimilar.size();i++) {
		    if(vSimilar.get(i) instanceof String && cbSuggested.getSelectedItem().equals((String)vSimilar.get(i))) {
			flag = true;
		    }
		    else if(flag && vSimilar.get(i) instanceof String) {
			i = vSimilar.size()+1;
		    }
		    else if(flag && vSimilar.get(i) instanceof TableValueNode) {
			vTmp.add(vSimilar.get(i));
		    }
		}
		lstValue.setListData(vTmp);
	    }
	}
	else if(e.getSource() == btnCopy) {
	    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    ListModel model = lstValue.getModel();
	    for(int i=0;i<model.getSize();i++) {
		TableValueNode tvn = (TableValueNode)model.getElementAt(i);
		TableValueNode newTVN = new TableValueNode();
		newTVN.setId(session.getTableDataModel().getCurrentProfileNode().getTable());
		newTVN.setType("User");
		newTVN.setName(session.getTableDataModel().getCurrentProfileNode().getName());
		newTVN.setCode(tvn.getOriginalCode());
		newTVN.setItem(tvn.getOriginalItem());
		newTVN.modifyCode(tvn.getCode());
		newTVN.modifyItem(tvn.getItem());
		newTVN.setStatus(TableValueNode.NEW);
		//newTVN.setLstType(TableValueNode.CODE);
		//System.out.println("NEW ValueNode: " + newTVN.getName() + " " + newTVN.getValue());
		session.getTableDataModel().addValue(newTVN);
	    }
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getTableDataModel().addObserver(this);
    }
}
