/**
 * This class represents a table value in hl7 tables
 */

package gov.nist.hl7maker.view.config;

public class TableValueNode extends TableNode {

    private String code;
    private String item;
    private short status;
    private String newCode;
    private String newItem;
    private short lstType;

    public final static short ORIGINAL = 0;
    public final static short REMOVED = 1;
    public final static short NEW = 2;
    public final static short EDITED = 3;
    public final static short CODE = 4;
    public final static short ITEM = 5;

    /**
     * Constructor
     */
    public TableValueNode() {
	status = TableValueNode.ORIGINAL;
	lstType = TableValueNode.CODE;
    }

    // Get Methods
    public String getCode() {
	String res = "";
	if(status == TableValueNode.ORIGINAL || status == TableValueNode.NEW) {
	    res = code;
	}
	else if(status == TableValueNode.EDITED) {
	    if(newCode == null)
		res = code;
	    else
		res = newCode;
	}
	return res;
    }

    public String getItem() {
	String res = "";
	if(status == TableValueNode.ORIGINAL || status == TableValueNode.NEW) {
	    res = item;
	}
	else if(status == TableValueNode.EDITED) {
	    if(newItem == null)
		res = item;
	    else
		res = newItem;
	}
	return res;
    }

    public short getStatus() {
	return status;
    }

    public String getOriginalCode() {
	return code;
    }

    public String getOriginalItem() {
	return item;
    }

    // Set Methods
    public void setCode(String code) {
	this.code = code;
    }

    public void setItem(String item) {
	this.item = item;
    }

    public void setStatus(short status) {
	this.status = status;
    }

    public void setLstType(short lstType) {
	this.lstType = lstType;
    }

    /**
     * Modify the code from the DataConfigurationPanel
     * @param code The edited code
     */
    public void modifyCode(String code) {
	newCode = code;
    }

    /**
     * Modify the item from the DataCondfigurationPanel
     * @param item The edited item
     */
    public void modifyItem(String item) {
	newItem = item;
    }

    public String toString() {
	if(lstType == TableValueNode.CODE) {
	    return getCode();
	}
	else if(lstType == TableValueNode.ITEM) {
	    return getItem();
	}
	return getValue();
    }

    /**
     * Return the value like a string composed by [code]item
     * @return A String
     */
    public String getValue() {
	return getItem();
    }

    /**
     * Return true if the two objects are equal
     * @param o The other object
     * @return return true if equal
     */
    public boolean equals(Object o) {
	TableValueNode tvn = (TableValueNode)o;
	boolean equal = true;

	if(!getId().equals(tvn.getId()))
	    equal = false;
	if(!getCode().equals(tvn.getCode()) || !getItem().equals(tvn.getItem()))
	    equal = false;
	return equal;
    }
}
