/**
 * This class represents all the values as a couple of JList
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// Message Maker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;

public class TableValuePanel extends JPanel implements Observer, ListSelectionListener, AdjustmentListener, MouseListener {

    protected JLabel lblInfo;
    protected JList lstCode;
    protected JList lstItem;
    protected JScrollPane lstItemScrollPane;
    protected JScrollPane lstCodeScrollPane;
    protected ListPopUp lstPopUp;

    private Session session;

    public TableValuePanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.PAGE_AXIS);
	setLayout(layout);

	lblInfo = new JLabel("");
	lblInfo.setAlignmentX(Component.CENTER_ALIGNMENT);
	add(lblInfo);
	add(Box.createRigidArea(new Dimension(0, 10)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.PAGE_AXIS));
	JLabel lbl1 = new JLabel("Code");
	lbl1.setAlignmentX(Component.CENTER_ALIGNMENT);
	p1.add(lbl1);

	lstCode = new JList();
	lstCode.setEnabled(false);
	lstCode.addListSelectionListener(this);
	lstCode.addMouseListener(this);
	lstCodeScrollPane = new JScrollPane(lstCode);
	lstCodeScrollPane.setPreferredSize(new Dimension(200,305));
	lstCodeScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
	lstCodeScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	lstCodeScrollPane.getVerticalScrollBar().addAdjustmentListener(this);
	p1.add(lstCodeScrollPane);

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.PAGE_AXIS));
	JLabel lbl2 = new JLabel("Value");
	lbl2.setAlignmentX(Component.CENTER_ALIGNMENT);
	p2.add(lbl2);

	lstItem = new JList();
	lstItem.setEnabled(false);
	lstItem.addListSelectionListener(this);
	lstItem.addMouseListener(this);
	lstItemScrollPane = new JScrollPane(lstItem);
	lstItemScrollPane.setPreferredSize(new Dimension(200,305));
	lstItemScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
	lstItemScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	lstItemScrollPane.getVerticalScrollBar().addAdjustmentListener(this);
	p2.add(lstItemScrollPane);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	p0.add(Box.createRigidArea(new Dimension(10, 0)));
	p0.add(p1);
	p0.add(Box.createRigidArea(new Dimension(30, 0)));
	p0.add(p2);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	add(p0);

	lstPopUp = new ListPopUp();	    
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getTableDataModel().addObserver(this);
    }

    private boolean flag;
    /**
     * Set a flag to make the difference between a TableValuePanel from TableViewer (false) and from TablePanel (true)
     * because there was a conflict with the MVC
     * @param flag The flag
     */
    public void setFlag(boolean flag) {
	this.flag = flag;
    }

    /**
     * Handles a click on a JList
     * @param e The event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(e.getSource() == lstCode) {
	    lstItem.removeListSelectionListener(this);
	    lstItem.setSelectedIndices(lstCode.getSelectedIndices());
	    lstItem.addListSelectionListener(this);
	}
	else if(e.getSource() == lstItem) {
	    lstCode.removeListSelectionListener(this);
	    lstCode.setSelectedIndices(lstItem.getSelectedIndices());
	    lstCode.addListSelectionListener(this);
	}
	TableValueNode tvn = (TableValueNode)lstCode.getSelectedValue();
	if(tvn != null && session.getTableDataModel() != null) {
	    session.getTableDataModel().setTableValueNode(tvn);
	    session.getTableDataModel().setTableValueNodes(lstCode.getSelectedValues());
	}
    }

    /**
     * Handles a right click
     * @param e The event
     */
    public void mouseClicked(MouseEvent e) {
	if(e.getButton() == 3) {	    
	    lstPopUp.show((JList)e.getSource(), e.getX(), e.getY());
	}
    }
    public void mouseEntered(MouseEvent e) {;}
    public void mouseExited(MouseEvent e) {;}
    public void mousePressed(MouseEvent e) {;}
    public void mouseReleased(MouseEvent e) {;}

    /**
     * Handles the modification of JScrollBar value
     * @param e The event
     */
    public void adjustmentValueChanged(AdjustmentEvent e) {
	if(e.getAdjustable() == lstItemScrollPane.getVerticalScrollBar()) {
	    lstItemScrollPane.getVerticalScrollBar().removeAdjustmentListener(this);
	    lstCodeScrollPane.getVerticalScrollBar().setValue(e.getValue());
	    lstItemScrollPane.getVerticalScrollBar().addAdjustmentListener(this);
	}
	if(e.getAdjustable() == lstCodeScrollPane.getVerticalScrollBar()) {
	    lstCodeScrollPane.getVerticalScrollBar().removeAdjustmentListener(this);
	    lstItemScrollPane.getVerticalScrollBar().setValue(e.getValue());
	    lstCodeScrollPane.getVerticalScrollBar().addAdjustmentListener(this);
	}
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		ProfileNode pNode = (ProfileNode)arg;
		Vector vValue = session.getTableDataModel().getValues(pNode.getTable());
		if(vValue != null && vValue.size() > 0) {
		    TableValueNode tvnInfo = (TableValueNode)vValue.get(0);
		    lblInfo.setText("Table Number: " + tvnInfo.getId() + " - Table Name: " + tvnInfo.getName() + " - Version: " + tvnInfo.getVersion());
		    Vector v = new Vector();
		    for(int i=0;i<vValue.size();i++) {
			TableValueNode tvn = (TableValueNode)vValue.get(i);
			tvn.setLstType(TableValueNode.CODE);
			TableValueNode tvnNew = new TableValueNode();
			tvnNew.setId(tvn.getId());
			tvnNew.setType(tvn.getType());
			tvnNew.setCode(tvn.getOriginalCode());
			tvnNew.setItem(tvn.getOriginalItem());
			tvnNew.modifyCode(tvn.getCode());
			tvnNew.modifyItem(tvn.getItem());
			tvnNew.setLstType(TableValueNode.ITEM);
			tvnNew.setStatus(tvn.getStatus());
			v.add(tvnNew);
		    }
		    lstCode.setListData(vValue);
		    lstItem.setListData(v);
		    if(session.getTableDataModel().isAllowedToModify()) {
			lstCode.setEnabled(true);
			lstItem.setEnabled(true);
		    }
		    else {
			lstCode.setEnabled(false);
			lstItem.setEnabled(false);
		    }
		}
		else {
		    if(session.getTableDataModel().getTableType(pNode.getTable()).compareTo("") != 0) {
			lblInfo.setText("Table Number: " + pNode.getTable() + " - Table Name: " + session.getTableDataModel().getTableName(pNode.getTable()));
		    }
		    else {
			lblInfo.setText("Table Number: N/A - Table Name: N/A - Version: N/A");
		    }
		    Object[] ob = {null};
		    lstCode.setListData(ob);
		    lstItem.setListData(ob);
		}
	    }
	    else if(arg instanceof String) {
		if("new".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValuePanel");
		    lstCode.clearSelection();
		    lstItem.clearSelection();
		}
		else if("init".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValuePanel");
		    lblInfo.setText("Table Number: N/A - Table Name: N/A - Version: N/A");
		    Object[] ob = {null};
		    lstCode.setListData(ob);
		    lstCode.setEnabled(false);
		    lstItem.setListData(ob);
		    lstItem.setEnabled(false);
		}
		else if("newTable".compareTo(arg) == 0 || "editTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValuePanel");
		    lstCode.setEnabled(false);
		    lstItem.setEnabled(false);
		}
	    }
	    else if(arg instanceof MVCArg) {
		if(("initTable".compareTo(((MVCArg)arg).getName()) == 0 || "updateLists".compareTo(((MVCArg)arg).getName()) == 0) && !flag) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValuePanel");
		    lblInfo.setText("");
		    Vector vValue = session.getTableDataModel().getValues((String)((MVCArg)arg).getValue());//cbTableNumber.getSelectedItem());
		    if(vValue != null && vValue.size() > 0) {
			Vector v = new Vector();
			for(int i=0;i<vValue.size();i++) {
			    TableValueNode tvn = (TableValueNode)vValue.get(i);
			    TableValueNode tvnNew = new TableValueNode();
			    tvnNew.setId(tvn.getId());
			    tvnNew.setType(tvn.getType());
			    tvnNew.setCode(tvn.getOriginalCode());
			    tvnNew.setItem(tvn.getOriginalItem());
			    tvnNew.modifyCode(tvn.getCode());
			    tvnNew.modifyItem(tvn.getItem());
			    tvnNew.setLstType(TableValueNode.ITEM);
			    tvnNew.setStatus(tvn.getStatus());
			    v.add(tvnNew);
			}
			lstCode.setListData(vValue);
			lstItem.setListData(v);
			if(session.getTableDataModel().isAllowedToModify((String)((MVCArg)arg).getValue())) {
			    lstCode.setEnabled(true);
			    lstItem.setEnabled(true);
			}
			else {
			    lstCode.setEnabled(false);
			    lstItem.setEnabled(false);
			}
		    }
		    else {
			Object[] ob = {null};
			lstCode.setListData(ob);
			lstItem.setListData(ob);
		    }
		}
	    }
	    else if(arg instanceof TableValueNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValuePanel");
	    }
	}
    }

    /**
     * Get the JList Code component
     * @return The JList
     */
    public JList getCode() {
	return lstCode;
    }
    
    /**
     * Get the JList Item component
     * @return The JList
     */
    public JList getItem() {
	return lstItem;
    }
}
