/**
 * This class is a panel to save the modification done on the database
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.Box;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;

// SAX
import org.xml.sax.Attributes;
import org.xml.sax.SAXException; 

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.MVCArg;

public class TableValueSavePanel extends JPanel implements Observer, ActionListener {

    protected JButton btnSave;
    protected JButton btnSaveAs;

    //    private Node tableNode, itemNode;

    protected Session session;
    private String type;

    protected final String HL7TABLES_MASTER = "." + Constants.FS + "data" + Constants.FS + "hl7tables.xml";
    protected final String USERTABLES_MASTER = "." + Constants.FS + "data" + Constants.FS + "usertables.xml";
    protected final String EXTERNALTABLES_MASTER =  "." + Constants.FS + "data" + Constants.FS + "externaltables.xml";

    /**
     * Constructor
     */
    public TableValueSavePanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.LINE_AXIS);
	setLayout(layout);
 
	btnSave = new JButton("Save");
        btnSave.setBackground(new Color(204, 204, 255));
	btnSave.setEnabled(false);
	btnSave.addActionListener(this);

	add(btnSave);

	btnSaveAs = new JButton("Save As ...");
	btnSaveAs.setBackground(new Color(204, 204, 255));
	btnSaveAs.setEnabled(false);
	btnSaveAs.addActionListener(this);

	add(Box.createRigidArea(new Dimension(15,0)));
	add(btnSaveAs);
    }

    /**
     * Handles action for click
     * @param e The Event
     */
    public void actionPerformed(ActionEvent e) {
	try {
	    System.out.println("Type actionPerformed: " + type);
	    if(e.getSource() == btnSave) {
		File masterFile = null, sessionFile = null;
		if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
		    masterFile = new File(HL7TABLES_MASTER);
		    sessionFile = new File(session.getHL7Tables());
		}
		else if(type.compareTo("User") == 0) {
		    masterFile = new File(USERTABLES_MASTER);
		    sessionFile = new File(session.getUserTables());
		}
		else if(type.compareTo("External") == 0) {
		    masterFile = new File(EXTERNALTABLES_MASTER);
		    sessionFile = new File(session.getExternalTables());
		}
		if(masterFile != null && masterFile.getCanonicalPath().compareTo(sessionFile.getCanonicalPath()) == 0) {                 
		    JFileChooser fc = new JFileChooser(sessionFile.getParent());
		    // Set the chooser to accept only XML files.
		    fc.addChoosableFileFilter(new XMLFileFilter());
		    fc.setAcceptAllFileFilterUsed(false);
		    if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			fc.setSelectedFile(new File("LocalHL7Tables.xml"));
		    }
		    else if(type.compareTo("User") == 0) {
			fc.setSelectedFile(new File("LocalUserTables.xml"));
		    }
		    else if(type.compareTo("External") == 0) {
			fc.setSelectedFile(new File("LocalExternalTables.xml"));
		    }
		    int returnVal = fc.showOpenDialog(this);
	    
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
			File f = fc.getSelectedFile();
			if(!f.getPath().endsWith(".xml")) {
			    f = new File(f.getPath() + ".xml");
			}
			if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			    session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), f);
			}
			else if(type.compareTo("User") == 0) {
			    session.getTableDataModel().saveTableFile("User", session.getUserTables(), f);
			}
			else if(type.compareTo("External") == 0) {
			    session.getTableDataModel().saveTableFile("External", session.getExternalTables(), f);
			}
		    }		
		}
		else {
		    if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), new File(session.getHL7Tables()));
		    }
		    else if(type.compareTo("User") == 0) {
			session.getTableDataModel().saveTableFile("User", session.getUserTables(), new File(session.getUserTables()));
		    }
		    else if(type.compareTo("External") == 0) {
			session.getTableDataModel().saveTableFile("External", session.getExternalTables(), new File(session.getExternalTables()));
		    }
		    else if(type.startsWith("Local")) {
			// Local Table
			Vector vLocalTable = session.getLocalTable();
			for(int i=0;i<vLocalTable.size();i++) {
			    TablePriority tp = (TablePriority)vLocalTable.get(i);
			    String type = "Local[" + new File(tp.getFilename()).getName() + "]";
			    if(session.getTableDataModel().needToSave(type))
				session.getTableDataModel().saveTableFile(type, tp.getFilename(), new File(tp.getFilename()));
			}	
		    }
		}
		session.getTableDataModel().setProfileNode(session.getTableDataModel().getCurrentProfileNode());
	    }
	    if(e.getSource() == btnSaveAs) {
		JFileChooser fc = null;
		if(type.startsWith("Local")) {
		    Vector vLocalTable = session.getLocalTable();
		    for(int i=0;i<vLocalTable.size();i++) {
			TablePriority tp = (TablePriority)vLocalTable.get(i);
			String type = "Local[" + new File(tp.getFilename()).getName() + "]";
			//if(tableModel.needToSave(type)) {
			    fc = new JFileChooser(new File(tp.getFilename()).getParent());
			    // Set the chooser to accept only XML files.
			    fc.addChoosableFileFilter(new XMLFileFilter());
			    fc.setAcceptAllFileFilterUsed(false);
			    fc.setSelectedFile(new File("new" + new File(tp.getFilename()).getName()));
			    
			    int returnVal = fc.showOpenDialog(this);
			    
			    if(returnVal == JFileChooser.APPROVE_OPTION) {
				File f = fc.getSelectedFile();
				if(!f.getPath().endsWith(".xml")) {
				    f = new File(f.getPath() + ".xml");
				}
				//tableModel.cleanType(type);
				session.getTableDataModel().saveTableFile(type, tp.getFilename(), f);
			    }
			    //}
		    }
		}
		else {
		    if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			fc = new JFileChooser(new File(session.getHL7Tables()).getParent());
		    }
		    else if(type.compareTo("User") == 0) {
			fc = new JFileChooser(new File(session.getUserTables()).getParent());
		    }
		    else if(type.compareTo("External") == 0) {
			fc = new JFileChooser(new File(session.getExternalTables()).getParent());
		    }
		    // Set the chooser to accept only XML files.
		    fc.addChoosableFileFilter(new XMLFileFilter());
		    fc.setAcceptAllFileFilterUsed(false);
		    if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			fc.setSelectedFile(new File("LocalHL7Tables.xml"));
		    }
		    else if(type.compareTo("User") == 0) {
			fc.setSelectedFile(new File("LocalUserTables.xml"));
		    }
		    else if(type.compareTo("External") == 0) {
			fc.setSelectedFile(new File("LocalExternalTables.xml"));
		    }
		    int returnVal = fc.showOpenDialog(this);
		    
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
			File f = fc.getSelectedFile();
			if(!f.getPath().endsWith(".xml")) {
			    f = new File(f.getPath() + ".xml");
			}
			if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
			    session.getTableDataModel().saveTableFile("HL7User", session.getHL7Tables(), f);
			}
			else if(type.compareTo("User") == 0) {
			    session.getTableDataModel().saveTableFile("User", session.getUserTables(), f);
			}
			else if(type.compareTo("External") == 0) {
			    session.getTableDataModel().saveTableFile("External", session.getExternalTables(), f);
			}
		    }
		}
		session.getTableDataModel().setProfileNode(session.getTableDataModel().getCurrentProfileNode());
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}	
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getTableDataModel().addObserver(this);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof TableDataModel) {
	    if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueSavePanel");
		type = session.getTableDataModel().getTableType();
		if(type.equals(""))
		    type = "User";
		btnSave.setEnabled(true);
		if(type.compareTo("HL7") == 0 || type.compareTo("HL7User") == 0) {
		    btnSave.setText("Save the HL7 Table");
		}
		else if(type.compareTo("User") == 0) {
		    btnSave.setText("Save the User Table");
		}
		else if(type.compareTo("External") == 0) {
		    btnSave.setText("Save the External Table");
		}
		else if(type.startsWith("Local")) {
		    btnSave.setText("Save the Local Table");
		}
		btnSaveAs.setEnabled(true);
	    }
	    else if(arg instanceof String) {
		if("init".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueSavePanel");
		    btnSave.setEnabled(false);
		    btnSaveAs.setEnabled(false);
		}
		else if("newTable".compareTo(arg) == 0 || "editTable".compareTo(arg) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueSavePanel");
		    btnSave.setEnabled(false);
		    btnSaveAs.setEnabled(false);
		}
	    }
	    else if(arg instanceof MVCArg) {
		if("initTable".compareTo(((MVCArg)arg).getName()) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from TableDataModel in TableValueSavePanel");
		    btnSave.setEnabled(true);
		    btnSaveAs.setEnabled(true);
		    btnSave.setText("Save");
		    btnSaveAs.setText("Save As ...");
		}
	    }
	}
    }

    /**
     * Simulate a click to save the data
     */    
    public void save() {
	btnSave.doClick();
    }
}

