/**
 * This class represents all the action one can do with a value
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.Box;
import javax.swing.BoxLayout;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;

public class ValueCommandPanel extends JPanel implements Observer, ActionListener {

    private JButton btnNew;
    private JButton btnAdd;
    private JButton btnRemove;
    private JButton btnEdit;
    private JButton btnCancel;
    private JTextField txtValue;

    private ValuePropertiesPanel vpp;

    private Session session;

    /**
     * Constructor
     */
    public ValueCommandPanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.PAGE_AXIS);
	setLayout(layout);
	
	JPanel btnPane = new JPanel();
	btnPane.setLayout(new BoxLayout(btnPane, BoxLayout.LINE_AXIS));

	btnNew = new JButton("New");
        btnNew.setBackground(new Color(204, 204, 255));
	btnNew.setEnabled(false);
	btnNew.addActionListener(this);
	btnPane.add(btnNew);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnAdd = new JButton("Add");
        btnAdd.setBackground(new Color(204, 204, 255));
	btnAdd.setEnabled(false);
	btnAdd.addActionListener(this);
	btnPane.add(btnAdd);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnRemove = new JButton("Remove");
        btnRemove.setBackground(new Color(204, 204, 255));
	btnRemove.setEnabled(false);
	btnRemove.addActionListener(this);
	btnPane.add(btnRemove);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnEdit = new JButton("Edit");
        btnEdit.setBackground(new Color(204, 204, 255));
	btnEdit.setEnabled(false);
	btnEdit.addActionListener(this);
	btnPane.add(btnEdit);

	btnPane.add(Box.createRigidArea(new Dimension(10,0)));

	btnCancel = new JButton("Cancel");
        btnCancel.setBackground(new Color(204, 204, 255));
	btnCancel.setEnabled(false);
	btnCancel.addActionListener(this);
	btnPane.add(btnCancel);

	add(btnPane);
	add(Box.createRigidArea(new Dimension(0, 10)));
	
	txtValue = new JTextField("");
	txtValue.setEditable(false);
	txtValue.addActionListener(this);
	add(txtValue);
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	ses.getRepositoryDataModel().addObserver(this);
    }

    /**
     * Set the ValuePropertiesPanel
     * @param vpp The ValuePropertiesPanel
     */
    public void setValuePropertiesPanel(ValuePropertiesPanel vpp) {
	this.vpp = vpp;
    }

    /**
     * Handles action for click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	ValueNode vn;
	if(e.getSource() == btnAdd) {
	    vn = new ValueNode();
	    vpp.updateProperties(vn);
	    vn.setStatus(ValueNode.NEW);
	    vn.setValue(txtValue.getText());
	    vn.setDatatype(session.getRepositoryDataModel().getCurrentProfileNode().getDatatype());
	    session.getRepositoryDataModel().addValue(vn);
	}
	else if(e.getSource() == btnRemove) {
	    Object[] vnS = session.getRepositoryDataModel().getCurrentValueNodes();
	    if(vnS.length == 1) {
		vn = session.getRepositoryDataModel().getCurrentValueNode();
		vn.setStatus(ValueNode.REMOVED);
		session.getRepositoryDataModel().modifyValue(vn);
	    }
	    else {
		for(int i=0;i<vnS.length;i++) {
		    vn = (ValueNode)vnS[i];
		    vn.setStatus(ValueNode.REMOVED);
		    session.getRepositoryDataModel().modifyValue(vn);
		}
	    }
	    btnCancel.setEnabled(false);
	}
	else if(e.getSource() == btnEdit) {
	    vn = session.getRepositoryDataModel().getCurrentValueNode();
	    if(vn.getStatus() != ValueNode.NEW) {
		vn.setStatus(ValueNode.EDITED);
		vn.modifyValue(txtValue.getText());
	    }
	    else {
		vn.setValue(txtValue.getText());
	    }
	    vpp.updateProperties(vn);
	    session.getRepositoryDataModel().modifyValue(vn);
	    btnCancel.setEnabled(false);
	}
	else if(e.getSource() == btnCancel) {
	    btnNew.setEnabled(true);
	    btnAdd.setEnabled(false);
	    btnRemove.setEnabled(false);
	    btnEdit.setEnabled(false);
	    btnCancel.setEnabled(false);
	    txtValue.setEditable(false);
	    txtValue.setText("");
	}
	else if(e.getSource() == btnNew) {
	    session.getRepositoryDataModel().newValue();
	}
	else if(e.getSource() == txtValue) {
	    // if click on ENTER, it adds the value, it does not work if you want to edit or remove a value
	    if(!btnRemove.isEnabled()) {
		btnAdd.doClick();	    
		btnAdd.grabFocus();
	    }
	}
    }


    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ValueNode) {
		if(((ValueNode)arg).getCategory().equals("Config"))
		    btnNew.setEnabled(false);
		else
		    btnNew.setEnabled(true);
		btnAdd.setEnabled(false);
		btnRemove.setEnabled(true);
		btnEdit.setEnabled(true);
		btnCancel.setEnabled(false);
		txtValue.setEditable(true);
		txtValue.setText(((ValueNode)arg).getValue());
	    }
	    else if(arg instanceof String) {
		if("new".compareTo(arg) == 0) {
		    txtValue.setText("");
		    txtValue.setEditable(true);
		    //		    btnNew.setEnabled(false);
		    btnAdd.setEnabled(true);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(true);
		}
		else if("init".compareTo(arg) == 0) {
		    btnNew.setEnabled(false);
		    btnAdd.setEnabled(false);
		    btnRemove.setEnabled(false);
		    btnEdit.setEnabled(false);
		    btnCancel.setEnabled(false);
		    txtValue.setEditable(false);
		    txtValue.setText("");
		}
	    }
	    else if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from RepositoryDataModel in ValueCommandPanel");
		if(session.getRepositoryDataModel().getCurrentValueNode() != null && session.getRepositoryDataModel().getCurrentValueNode().getCategory().equals("Config"))
		    btnNew.setEnabled(false);
		else
		    btnNew.setEnabled(true);
		btnAdd.setEnabled(false);
		btnRemove.setEnabled(false);
		btnEdit.setEnabled(false);
		btnCancel.setEnabled(false);
		txtValue.setEditable(false);
		txtValue.setText("");
	    }
	    else if(arg instanceof MVCArg) {
		if("config".compareTo(((MVCArg)arg).getName()) == 0) {
		    btnNew.setEnabled(false);
		}
	    }
	}
    }
}
