/**
 * This class is a panel to make a data link
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.ListModel;

// MessageMaker
import gov.nist.hl7maker.control.Session;

public class ValueDataLinkPanel extends JPanel implements Observer, ActionListener {

    private JComboBox cbSuggested;
    private JButton btnCopy;
    private JList lstValue;
    private JScrollPane listScrollPane;

    private Session session;
    private Vector vSimilar;

    /**
     * Constructor
     */
    public ValueDataLinkPanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.LINE_AXIS);
	setLayout(layout);

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
	JLabel l = new JLabel("Suggested Data Link: ");
	l.setAlignmentX(Component.LEFT_ALIGNMENT);
	p0.add(l);
	p0.add(Box.createRigidArea(new Dimension(0, 5)));
	cbSuggested = new JComboBox();
	cbSuggested.setAlignmentX(Component.LEFT_ALIGNMENT);
	cbSuggested.setEnabled(false);
	cbSuggested.addActionListener(this);
	cbSuggested.setPreferredSize(new Dimension(150, 25));
	p0.add(cbSuggested);
	p0.add(Box.createRigidArea(new Dimension(0, 15)));
	btnCopy = new JButton("Copy Values");
	btnCopy.setAlignmentX(Component.LEFT_ALIGNMENT);
	btnCopy.setBackground(new Color(204, 204, 255));
	btnCopy.setEnabled(false);
	btnCopy.addActionListener(this);
	p0.add(btnCopy);
	p0.add(Box.createRigidArea(new Dimension(0, 60)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.PAGE_AXIS));
	lstValue = new JList();
	lstValue.setEnabled(false);
	listScrollPane = new JScrollPane(lstValue);
	listScrollPane.setPreferredSize(new Dimension(100, 10));
	//	listScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
	p1.add(listScrollPane);

	add(p0);
	add(Box.createRigidArea(new Dimension(15, 0)));
	add(p1);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ValueNode) {
	    }
	    else if(arg instanceof ProfileNode) {
		gov.nist.hl7maker.util.MVCLog.info("Event from ReferenceDBModel in ValueDataLinkPanel");
		cbSuggested.removeAllItems();
		// Look for the closest data link
		ProfileNode pn = (ProfileNode)arg;
		vSimilar = session.getRepositoryDataModel().getSimilarProfileNode(pn);
		if(vSimilar.size() > 0 && pn.getSegment().startsWith("Z")) {
		    // Add elements to the comboBox
		    for(int i=0;i<vSimilar.size();i++) {
			if(vSimilar.get(i) instanceof String)
			    cbSuggested.addItem(vSimilar.get(i));
		    }
		    cbSuggested.setEnabled(true);
		    btnCopy.setEnabled(true);
		}
		else {
		    cbSuggested.setEnabled(false);
		    btnCopy.setEnabled(false);
		    lstValue.setListData(new Vector());
		}
	    }
	}
    }

    /**
     * Handles action for click
     * @param e The event
     */    
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() == cbSuggested) {
	    //System.out.println("changement " + cbSuggested.getSelectedItem());
	    if(cbSuggested.getSelectedItem() != null) {
		boolean flag = false;
		Vector vTmp = new Vector();
		for(int i=0;i<vSimilar.size();i++) {
		    if(vSimilar.get(i) instanceof String && cbSuggested.getSelectedItem().equals((String)vSimilar.get(i))) {
			flag = true;
		    }
		    else if(flag && vSimilar.get(i) instanceof String) {
			i = vSimilar.size()+1;
		    }
		    else if(flag && vSimilar.get(i) instanceof ValueNode) {
			vTmp.add(vSimilar.get(i));
		    }
		}
		lstValue.setListData(vTmp);		   
	    }
	}
	else if(e.getSource() == btnCopy) {
	    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    ListModel model = lstValue.getModel();
	    for(int i=0;i<model.getSize();i++) {
		ValueNode vn = (ValueNode)model.getElementAt(i);
		ValueNode newVN = new ValueNode();
		newVN.setSegment(session.getRepositoryDataModel().getCurrentProfileNode().getSegment());
		newVN.setQualifier(session.getRepositoryDataModel().getCurrentProfileNode().getQualifier());
		newVN.setQualifier2(session.getRepositoryDataModel().getCurrentProfileNode().getQualifier2());
		newVN.setName(session.getRepositoryDataModel().getCurrentProfileNode().getName());
		newVN.setValue(vn.getValue());
		newVN.setStatus(ValueNode.NEW);
		//newVN.setComplexity(vn.getComplexity());
		//newVN.setPriority(vn.getPriority());
		newVN.setCategory(vn.getCategory());
		newVN.setDescription(vn.getDescription());
		//System.out.println("NEW ValueNode: " + newVN.getName() + " " + newVN.getValue());
		session.getRepositoryDataModel().addValue(newVN);
	    }
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getRepositoryDataModel().addObserver(this);
    }
}
