/**
 * This class represents a value in the reference database
 */

package gov.nist.hl7maker.view.config;

public class ValueNode extends ProfileNode {

    private String value;
    private String complexity;
    private String priority;
    private String category;
    private String description;
    private short status;
    private String newValue;

    public final static short ORIGINAL = 0;
    public final static short REMOVED = 1;
    public final static short NEW = 2;
    public final static short EDITED = 3;

    /**
     * Constructor
     */
    public ValueNode() {
	status = ValueNode.ORIGINAL;
    }

    // Get Methods
    public String getValue() {
	String res = "";
	if(status == ValueNode.ORIGINAL || status == ValueNode.NEW) {
	    res = value;
	}
	else if(status == ValueNode.EDITED) {
	    res = newValue;
	}
	return res;
    }

    public String getComplexity() {
	return complexity;
    }

    public String getPriority() {
	return priority;
    }

    public String getCategory() {
	return category;
    }

    public String getDescription() {
	return description;
    }

    public short getStatus() {
	return status;
    }

    public String getOriginalValue() {
	return value;
    }

    // Set Methods
    public void setValue(String value) {
	this.value = value;
    }

    public void setComplexity(String complexity) {
	this.complexity = complexity;
    }

    public void setPriority(String priority) {
	this.priority = priority;
    }

    public void setCategory(String category) {
	this.category = category;
    }

    public void setDescription(String description) {
	this.description = description;
    }

    public void setStatus(short status) {
	this.status = status;
    }

    /**
     * Return true if the two objects are equal
     * @param o The other object
     * @return return true if equal
     */
    public boolean equals(Object o) {
	ValueNode vn = (ValueNode)o;
	boolean equal = true;
	
	if(name.compareTo(vn.getName()) != 0)
	    equal = false;
	if(qualifier.compareTo(vn.getQualifier()) != 0)
	    equal = false;
	if(qualifier2.compareTo(vn.getQualifier2()) != 0)
	    equal = false;
	if(segment.compareTo(vn.getSegment()) != 0)
	    equal = false;
	return equal;
    }

    public boolean fullEquals(Object o) {
	ValueNode vn = (ValueNode)o;
	boolean equal = equals(o);

	if(!category.equals(vn.getCategory()))
	    equal = false;
	if(!getValue().equals(vn.getValue()))
	    equal = false;
	return equal;
    }

    /**
     * Modify the value from the DataConfigurationPanel
     * @param value The edited value
     */
    public void modifyValue(String value) {
	newValue = value;
    }

    public String toString() {
	return getValue();
    }

    /**
     * Return true if it is a leaf
     * @return is it a leaf?
     */
    public boolean isLeaf() {
	return true;
    }

    public String getMsgLoc() {
	return "";
    }
}
