/**
 * This class represents all the values as a JList
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// MessageMaker
import gov.nist.hl7maker.control.Session;

//import java.awt.event.InputEvent;
//import javax.swing.JPopupMenu;

public class ValuePanel extends JPanel implements Observer, ListSelectionListener, MouseListener {

    private JList lstValue;
    private JScrollPane listScrollPane;
    private ListPopUp lstPopUp;

    private Session session;

    /**
     * Constructor
     */
    public ValuePanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.PAGE_AXIS);
	setLayout(layout);

	JLabel lbl = new JLabel("Values");
	lbl.setAlignmentX(Component.CENTER_ALIGNMENT);
	add(lbl);

	lstValue = new JList();
	lstValue.setEnabled(false);
	lstValue.addListSelectionListener(this);
	lstValue.addMouseListener(this);
	listScrollPane = new JScrollPane(lstValue);
	listScrollPane.setPreferredSize(new Dimension(200,10));
	listScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
	add(listScrollPane);

	lstPopUp = new ListPopUp();	    
    }
    
    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	ses.getRepositoryDataModel().addObserver(this);
	lstPopUp.setSession(ses);
    }

    /**
     * Set the ValuePropertiesPanel
     * @param vpp The ValuePropertiesPanel
     */
    public void setValuePropertiesPanel(ValuePropertiesPanel vpp) {
	lstPopUp.setValuePropertiesPanel(vpp);
    }

    /**
     * Handles a click on a JList
     * @param e The event
     */
    public void valueChanged(ListSelectionEvent e) {
	if(lstValue.getSelectedIndex() != -1) {
	    ValueNode vn = (ValueNode)lstValue.getSelectedValue();
	    session.getRepositoryDataModel().setValueNode(vn);
	    session.getRepositoryDataModel().setValueNodes(lstValue.getSelectedValues());
	}
    }

    /**
     * Handles a right click
     * @param e The event
     */
    public void mouseClicked(MouseEvent e) {
	if(e.getButton() == 3) {	    
	    lstPopUp.show((JList)e.getSource(), e.getX(), e.getY());
	}
    }
    public void mouseEntered(MouseEvent e) {;}
    public void mouseExited(MouseEvent e) {;}
    public void mousePressed(MouseEvent e) {;}
    public void mouseReleased(MouseEvent e) {;}

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ValueNode) {
	    }
	    else if(arg instanceof ProfileNode) {
		ProfileNode pNode = (ProfileNode)arg;
		lstValue.setListData(((RepositoryDataModel)o).getValues(pNode.getName(), pNode.getSegment(), pNode.getQualifier(), pNode.getQualifier2()));
		lstValue.setEnabled(true);
	    }
	    else if(arg instanceof String) {
		if("new".compareTo(arg) == 0) {
		    lstValue.clearSelection();
		}
		else if("init".compareTo(arg) == 0) {
		    Object[] ob = {null};
		    lstValue.setListData(ob);
		    lstValue.setEnabled(false);
		}
	    }
	}
    }
}
