/**
 * This class is a panel to show the properties of a value
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;

// Swing
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.SpringLayout;
import javax.swing.BorderFactory;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;

public class ValuePropertiesPanel extends JPanel implements Observer {

    private JTextField txtName;
    private JTextField txtSegment;
    private JTextField txtField;
    private JTextField txtComponent;
    private JTextField txtSequenceNum;
    private JTextField txtDatatype;
    //private JComboBox cbComplexity;
    //private JComboBox cbPriority;
    private JComboBox cbCategory;
    private JTextField txtDescription;

    /**
     * Constructor
     */
    public ValuePropertiesPanel() {
	setLayout(new SpringLayout());
	JLabel l;

	// Name
	l = new JLabel("Name: ", JLabel.TRAILING);
	add(l);
	txtName = new JTextField();
	txtName.setEditable(false);
	l.setLabelFor(txtName);
	add(txtName);
	// Segment
	l = new JLabel("Segment: ", JLabel.TRAILING);
	add(l);
	txtSegment = new JTextField();
	txtSegment.setEditable(false);
	l.setLabelFor(txtSegment);
	add(txtSegment);
	// Field
	l = new JLabel("Field: ", JLabel.TRAILING);
	add(l);
	txtField = new JTextField();
	txtField.setEditable(false);
	l.setLabelFor(txtField);
	add(txtField);
	// Component
	l = new JLabel("Component: ", JLabel.TRAILING);
	add(l);
	txtComponent = new JTextField();
	txtComponent.setEditable(false);
	l.setLabelFor(txtComponent);
	add(txtComponent);
	// Sequence Number
	l = new JLabel("# Sequence: ", JLabel.TRAILING);
	add(l);
	txtSequenceNum = new JTextField();
	txtSequenceNum.setEditable(false);
	l.setLabelFor(txtSequenceNum);
	add(txtSequenceNum);
	// Datatype
	l = new JLabel("Datatype: ", JLabel.TRAILING);
	add(l);
	txtDatatype = new JTextField();
	txtDatatype.setEditable(false);
	l.setLabelFor(txtDatatype);
	add(txtDatatype);
	// Complexity
/*	String[] complexity = {""};
	l = new JLabel("Complexity: ", JLabel.TRAILING);
	add(l);
	cbComplexity = new JComboBox(complexity);
        cbComplexity.setBackground(new Color(204, 204, 255));
	cbComplexity.removeAllItems();
	cbComplexity.addItem("Typical");
	cbComplexity.addItem("Moderate");
	cbComplexity.addItem("High");
	cbComplexity.setEnabled(false);
	l.setLabelFor(cbComplexity);
	add(cbComplexity);*/
	// Priority
/*	String[] priority = {""};
	l = new JLabel("Priority: ", JLabel.TRAILING);
	add(l);
	cbPriority = new JComboBox(priority);
        cbPriority.setBackground(new Color(204, 204, 255));
	cbPriority.removeAllItems();
	cbPriority.addItem("Low");
	cbPriority.addItem("Normal");
	cbPriority.addItem("High");
	cbPriority.addItem("Very High");
	cbPriority.addItem("Mandatory");
	cbPriority.setEnabled(false);
	cbPriority.setSelectedIndex(1);
	l.setLabelFor(cbPriority);
	add(cbPriority);*/
	// Category
	String[] category = {""};
	l = new JLabel("Category: ", JLabel.TRAILING);
	add(l);
	cbCategory = new JComboBox(category);
        cbCategory.setBackground(new Color(204, 204, 255));
	cbCategory.removeAllItems();
	cbCategory.addItem("Valid: Typical");
	cbCategory.addItem("Valid: DataType");
	cbCategory.addItem("Invalid: Format");
	cbCategory.addItem("Invalid: Datatype");
	cbCategory.addItem("Invalid: Out-of-Range");
	cbCategory.addItem("Valid: Boundary");
	cbCategory.addItem("Valid: Length");
	cbCategory.addItem("Invalid: Length");
	cbCategory.addItem("Valid: Coded Entry");
	cbCategory.addItem("Invalid: Coded Entry");
	cbCategory.addItem("Valid: Value Set");
	cbCategory.addItem("Invalid: Value Set");
	cbCategory.addItem("Example");
	cbCategory.addItem("Constant");
	cbCategory.addItem("Config");
	cbCategory.setEnabled(false);
	l.setLabelFor(cbCategory);
	add(cbCategory);
	// Description
	l = new JLabel("Description: ", JLabel.TRAILING);
	add(l);
	txtDescription = new JTextField();
	txtDescription.setEditable(false);
	l.setLabelFor(txtDescription);
	add(txtDescription);

	SpringUtilities.makeCompactGrid(this, 8, 2, 6, 6, 6, 6);
	setBorder(BorderFactory.createLineBorder(Color.BLACK));
    }

    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	// MVC
	ses.getRepositoryDataModel().addObserver(this);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ValueNode) {
		ValueNode vn = (ValueNode)arg;
		// Name
		txtName.setText(vn.getName());
		// Segment
		txtSegment.setText(vn.getSegment());
		// Field
		if(vn.getQualifier().equals(""))
		    txtField.setText(vn.getName());
		else if(vn.getQualifier2().equals(""))
		    txtField.setText(vn.getQualifier());
		else
		    txtField.setText(vn.getQualifier2());
		// Component
		if(!vn.getQualifier2().equals(""))
		    txtComponent.setText(vn.getQualifier());
		else 
		    txtComponent.setText("");
		// Sequence Number
		//txtSequenceNum.setText(String.valueOf(vn.getSequenceNum()));
		// Datatype
		txtDatatype.setText(vn.getDatatype());
		// Complexity
/*		for(int i=0;i<cbComplexity.getItemCount();i++) {
		    if(vn.getComplexity().compareToIgnoreCase((String)cbComplexity.getItemAt(i)) == 0) {
			cbComplexity.setSelectedIndex(i);
		    }
		    }*/
		// Priority
/*		for(int i=0;i<cbPriority.getItemCount();i++) {
		    if(vn.getPriority().compareToIgnoreCase((String)cbPriority.getItemAt(i)) == 0) {
			cbPriority.setSelectedIndex(i);
		    }
		    }*/
		// Category
		for(int i=0;i<cbCategory.getItemCount();i++) {
		    if(vn.getCategory().compareToIgnoreCase((String)cbCategory.getItemAt(i)) == 0) {
			cbCategory.setSelectedIndex(i);
		    }
		}
		// Description
		txtDescription.setText(vn.getDescription());
		//cbComplexity.setEnabled(true);
		//cbPriority.setEnabled(true);
		cbCategory.setEnabled(true);
		txtDescription.setEditable(true);
	    }
	    else if(arg instanceof String) {
		if("new".compareTo(arg) == 0) {
		    /*cbComplexity.setEnabled(true);
		    cbComplexity.setSelectedIndex(0);
		    cbPriority.setEnabled(true);
		    cbPriority.setSelectedIndex(1);*/
		    cbCategory.setEnabled(true);
		    cbCategory.setSelectedIndex(0);
		    txtDescription.setText("");
		    txtDescription.setEditable(true);
		}
		else if("init".compareTo(arg) == 0) {
		    txtName.setText("");
		    txtSegment.setText("");
		    txtField.setText("");
		    txtComponent.setText("");
		    txtSequenceNum.setText("");
		    txtDatatype.setText("");
		    /*cbComplexity.setEnabled(false);
		    cbComplexity.setSelectedIndex(0);
		    cbPriority.setEnabled(false);
		    cbPriority.setSelectedIndex(1);*/
		    cbCategory.setEnabled(false);
		    cbCategory.setSelectedIndex(0);
		    txtDescription.setText("");
		    txtDescription.setEditable(false);
		}
	    }
	    else if(arg instanceof ProfileNode) {
		ProfileNode pn = (ProfileNode)arg;
		// Name
		txtName.setText(pn.getName());
		// Segment
		txtSegment.setText(pn.getSegment());
		// Field
		if(pn.getQualifier().equals(""))
		    txtField.setText(pn.getName());
		else if(pn.getQualifier2().equals(""))
		    txtField.setText(pn.getQualifier());
		else
		    txtField.setText(pn.getQualifier2());
		// Component
		if(!pn.getQualifier2().equals(""))
		    txtComponent.setText(pn.getQualifier());
		else 
		    txtComponent.setText("");
		// Sequence Number
		txtSequenceNum.setText(String.valueOf(pn.getSequenceNum()));
		// Datatype
		txtDatatype.setText(pn.getDatatype());
		/*cbComplexity.setEnabled(false);
		cbComplexity.setSelectedIndex(0);
		cbPriority.setEnabled(false);
		cbPriority.setSelectedIndex(1);*/
		cbCategory.setEnabled(false);
		cbCategory.setSelectedIndex(0);
		txtDescription.setText("");
		txtDescription.setEditable(false);
	    }
	    else if(arg instanceof MVCArg) {
		if("init".equals(((MVCArg)arg).getName())) {
		    ProfileNode pn = (ProfileNode)((MVCArg)arg).getOValue();
		// Name
		txtName.setText(pn.getName());
		// Segment
		txtSegment.setText(pn.getSegment());
		// Field
		if(pn.getQualifier().equals(""))
		    txtField.setText(pn.getName());
		else if(pn.getQualifier2().equals(""))
		    txtField.setText(pn.getQualifier());
		else
		    txtField.setText(pn.getQualifier2());
		// Component
		if(!pn.getQualifier2().equals(""))
		    txtComponent.setText(pn.getQualifier());
		else 
		    txtComponent.setText("");
		// Sequence Number
		txtSequenceNum.setText(String.valueOf(pn.getSequenceNum()));
		// Datatype
		txtDatatype.setText(pn.getDatatype());
		}
	    }
	}
    }

    /**
     * Update the properties for an existing ValueNode
     * @param vn The ValueNode
     */
    public void updateProperties(ValueNode vn) {
	vn.setName(txtName.getText());
	vn.setSegment(txtSegment.getText());
	// A field
	if(txtName.getText().equals(txtField.getText())) {
	    vn.setQualifier("");
	    vn.setQualifier2("");
	}
	// A component
	else if(txtComponent.getText().equals("")) {
	    vn.setQualifier(txtField.getText());
	    vn.setQualifier2("");
	}
	// A subcomponent
	else {
	    vn.setQualifier(txtComponent.getText());
	    vn.setQualifier2(txtField.getText());
	}
	vn.setDatatype(txtDatatype.getText());
	//vn.setComplexity((String)cbComplexity.getSelectedItem());
	//vn.setPriority((String)cbPriority.getSelectedItem());
	vn.setCategory((String)cbCategory.getSelectedItem());
	vn.setDescription(txtDescription.getText());
    }
}
