/**
 * This class is a panel to save the modification done on the database
 */

package gov.nist.hl7maker.view.config;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.Box;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException; 

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.Constants;

public class ValueSavePanel extends JPanel implements Observer, ActionListener {

    private JButton btnSave;
    private JButton btnSaveAs;

    private Session session;

    /**
     * Constructor
     */
    public ValueSavePanel() {
	BoxLayout layout = new BoxLayout(this, BoxLayout.LINE_AXIS);
	setLayout(layout);

	btnSave = new JButton("Save");
        btnSave.setBackground(new Color(204, 204, 255));
	btnSave.setEnabled(false);
	btnSave.addActionListener(this);
	
	add(btnSave);

	btnSaveAs = new JButton("Save As ...");
	btnSaveAs.setBackground(new Color(204, 204, 255));
	btnSaveAs.setEnabled(false);
	btnSaveAs.addActionListener(this);

	add(Box.createRigidArea(new Dimension(15,0)));
	add(btnSaveAs);
    }

    /**
     * Handles action for click
     * @param e The event
     */    
    public void actionPerformed(ActionEvent e) {
	try {
	    if(e.getSource() == btnSave) {
		session.getRepositoryDataModel().saveConfigToFile(new File(session.getConfig()));
		session.getRepositoryDataModel().saveToFile(new File(session.getRepositoryData()));
		session.getRepositoryDataModel().setProfileNode(session.getRepositoryDataModel().getCurrentProfileNode());
	    }
	    else if(e.getSource() == btnSaveAs) {
		JFileChooser fc = null;
		fc = new JFileChooser(new File(session.getRepositoryData()).getParent());
		// Set the chooser to accept only XML files.
		fc.addChoosableFileFilter(new XMLFileFilter());
		fc.setAcceptAllFileFilterUsed(false);
		fc.setSelectedFile(new File("LocalRepository.xml"));
		int returnVal = fc.showOpenDialog(this);

		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File f = fc.getSelectedFile();
		    if(!f.getPath().endsWith(".xml")) {
			f = new File(f.getPath() + ".xml");
		    }
		    session.getRepositoryDataModel().saveConfigToFile(new File(session.getConfig()));
		    session.getRepositoryDataModel().saveToFile(f);
		    session.getRepositoryDataModel().setProfileNode(session.getRepositoryDataModel().getCurrentProfileNode());
		    session.setRepositoryData(f.getPath());
		}
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
    }
    
    /**
     * Set the session
     * @param ses The session
     */
    public void setSession(Session ses) {
	session = ses;
	// MVC
	session.getRepositoryDataModel().addObserver(this);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof RepositoryDataModel) {
	    if(arg instanceof ProfileNode) {
		btnSave.setEnabled(true);
		btnSave.setText("Save the repository");
		btnSaveAs.setEnabled(true);
	    }
	    else if(arg instanceof String) {
		if("init".compareTo(arg) == 0) {
		    btnSave.setEnabled(false);
		    btnSaveAs.setEnabled(false);
		}
	    }
	}
    }
}

