/**
 * This class is the panel to view a message in 3 forms (XML, ER7 and Enhanced)
 */

package gov.nist.hl7maker.view.display;

// JDK
import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

// AWT
import java.awt.Dimension;
import java.awt.Component;
import java.awt.CardLayout;
import java.awt.Color;

// Swing
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JTextArea;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.util.MVCArg;

public class DisplayMessageView extends JPanel implements Observer {

    private JScrollPane xmlScrollPane;
    private JScrollPane er7ScrollPane;
    private JScrollPane enhancedScrollPane;
    private JScrollPane validationScrollPane;
    private JTabbedPane tabbedPane;
    private JPanel msg_mdPanel;

    private MessageTree mt;
    private JScrollPane treeScrollPane;

    private PanelDirProfileSelector pdps;

    private JSplitPane sp;

    private Session session;

    /**
     * Constructor
     */
    public DisplayMessageView(Session ses, User user) {
	session = ses;
	// MVC
	session.addObserver(this);

	// MetaDataPanel
	JPanel metadataPanel = new MetaDataPanel();
	metadataPanel.setPreferredSize(new Dimension(370, 600));

	// XMLView
        XMLView xmlView = new XMLView();
        xmlScrollPane = new JScrollPane(xmlView);
                                                                                                    
        // ER7View
        ER7View er7View = new ER7View();
        er7ScrollPane = new JScrollPane(er7View);
                                                                                                    
        // EnhancedView
        EnhancedView enhancedView = new EnhancedView();
	enhancedView.setOtherView(xmlView, er7View);
        enhancedScrollPane = new JScrollPane(enhancedView);
	session.addObserver(enhancedView.getSaveButton());

	// ValidationView
	ValidationView validationView = new ValidationView();
	validationScrollPane = new JScrollPane(validationView);
                                                                                                    
        // TabbedPane
        tabbedPane = new JTabbedPane();
        tabbedPane.setTabPlacement(SwingConstants.BOTTOM);
        tabbedPane.addTab("XML", xmlScrollPane);
        tabbedPane.addTab("ER7", er7ScrollPane);
        tabbedPane.addTab("Enhanced", enhancedScrollPane);
	//tabbedPane.addTab("Validation", validationScrollPane);
	//	tabbedPane.addTab("MetaData", new MetaDataPanel());
	tabbedPane.setPreferredSize(new Dimension(370, 500));
	//tabbedPane.setMaximumSize(new Dimension(550, 600));

	// 
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
	p0.add(tabbedPane);
	p0.add(Box.createRigidArea(new Dimension(0, 10)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	p1.add(Box.createRigidArea(new Dimension(10, 0)));
	JTextArea txtMsgInfo = new JTextArea(" ", 5, 1);
	txtMsgInfo.setFont(txtMsgInfo.getFont().deriveFont(1).deriveFont(16f));
	txtMsgInfo.setLineWrap(true);
	txtMsgInfo.setWrapStyleWord(true);
	txtMsgInfo.setEnabled(false);
	txtMsgInfo.setBackground(Color.WHITE);
	//	txtMsgInfo.setPreferredSize(new Dimension(370, 50));
	p1.add(new JScrollPane(txtMsgInfo));
	p1.add(Box.createRigidArea(new Dimension(10, 0)));

	/*	JLabel lblMsgInfo = new JTextArea(" ");
	lblMsgInfo.setPreferredSize(new Dimension(370, 20));
	p0.add(lblMsgInfo);*/
	p0.add(p1);
	p0.add(Box.createRigidArea(new Dimension(0, 10)));

	// CardLayout
	msg_mdPanel = new JPanel(new CardLayout());
	//	msg_mdPanel.add(tabbedPane, "Message");
	msg_mdPanel.add(p0, "Message");
	msg_mdPanel.add(metadataPanel, "MetaData");
	msg_mdPanel.add(new JPanel(), "Blank");

	// MessageTree
	mt = new MessageTree();
	mt.setUser(user);
	mt.setSession(session);
	//mt.setTabbedPane(tabbedPane);
	mt.setCardLayout(msg_mdPanel);
	//mt.updateMessageList(new XMLDirectory(session.getMsgPath()), false);
	treeScrollPane = new JScrollPane(mt);
	treeScrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
	treeScrollPane.setPreferredSize(new Dimension(475, 700));
	//treeScrollPane.setMaximumSize(new Dimension(350, 600));

	// Panel to choose the directory and to sort by profile the list of messages
	pdps = new PanelDirProfileSelector();
	pdps.setMessageTree(mt);
	//pdps.setXMLDirectory(new XMLDirectory(session.getMsgPath()));

/*	JPanel panelMessageChooser = new JPanel();
	panelMessageChooser.setLayout(new BoxLayout(panelMessageChooser, BoxLayout.PAGE_AXIS));
	panelMessageChooser.add(treeScrollPane);
	panelMessageChooser.add(pdps);
	panelMessageChooser.setPreferredSize(new Dimension(300, 550));*/

	// SplitPane 
	//	sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panelMessageChooser, msg_mdPanel);
	sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treeScrollPane, msg_mdPanel);
	sp.setDividerLocation(310);

	this.add(sp);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof MVCArg) {
		if(((MVCArg)arg).getName().compareTo("msgPath") == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in DisplayMessageView");
		    mt.updateMessageList(new XMLDirectory(((MVCArg)arg).getValue()));
		    //pdps.setXMLDirectory(new XMLDirectory(((MVCArg)arg).getValue()));
		}
		else if(((MVCArg)arg).getName().compareTo("mountDir") == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in DisplayMessageView");
		    mt.updateMessageList(new XMLDirectory(((MVCArg)arg).getValue()));
		}
		else if(((MVCArg)arg).getName().compareTo("saveMsg") == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in DisplayMessageView");
		    mt.saveMsg();
		}
		else if(((MVCArg)arg).getName().compareTo("deleteMsg") == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in DisplayMessageView");
		    mt.deleteMsg();
		}
	    }
	    else if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in DisplayMessageView");
		mt.updateMessageList(new XMLDirectory(((Session)arg).getMsgPath()));
		//pdps.setXMLDirectory(new XMLDirectory(((Session)arg).getMsgPath()));
	    }
	}
    }
}
