/**
 * This class is a Panel for the Enhanced View of a message
 */

package gov.nist.hl7maker.view.display;

// AWT
import java.awt.Dimension;
import java.awt.Component;

// Swing
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;

// DOM
import org.w3c.dom.Document;

public class EnhancedView extends JPanel {

    private XMLTreeView xmlTreeView;
    private JScrollPane xmlTreeScrollPane;
    private SaveButton saveButton;
    private boolean modified;

    /**
     * Constructor
     */
    public EnhancedView() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	xmlTreeView = new XMLTreeView();
	xmlTreeScrollPane = new JScrollPane(xmlTreeView);
	xmlTreeScrollPane.setPreferredSize(new Dimension(340,425));
	add(xmlTreeScrollPane);

	saveButton = new SaveButton();
	saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	add(saveButton);
    }

    /**
     * Set the Other View (XML and ER7)
     * @param xmlView The XMLView
     * @param er7View The ER7View
     */
    public void setOtherView(XMLView xmlView, ER7View er7View) {
	xmlTreeView.setOtherView(xmlView, er7View, this);
    }

    /**
     * Display the message from a DOM
     * @param doc The message as a DOM 
     * @param prefix The profile for this DOM
     * @param msgPath The path where the message is originaly saved
     */
    public void displayMessage(Document doc, String profile, String msgPath) {
	XMLTreeModel xmlTreeModel = new XMLTreeModel(doc);
	xmlTreeModel.addTreeModelListener(xmlTreeView);
	xmlTreeView.setModel(xmlTreeModel);
	saveButton.setTreeModel(xmlTreeModel);
	saveButton.setProfile(profile);
	saveButton.setMsgPath(msgPath);
    }

    /**
     * Reset the display
     */
    public void reset() {
	xmlTreeView.setModel(null);
    }

    /**
     * Get the SaveButton Component
     @return The component
    */
    public SaveButton getSaveButton() {
	return saveButton;
    }

    /** 
     * Set the flag if the message has been modified in EnhancedView
     * @param modified
     */
    public void setHasBeenModified(boolean modified) {
	this.modified = modified;
    }

    /**
     * Return true if the message has been modified in EnhancedView
     * @return true or false
     */
    public boolean hasBeenModified() {
	return modified;
    }

    /**
     * Save the message
     */
    public void saveMsg() {
	saveButton.actionPerformed(null);
    }
}
