/**
 * This class lists all xml file to represent them as a JTree
 */

package gov.nist.hl7maker.view.display;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Vector;

// AWT
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.JTabbedPane;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JTextArea;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

// Message Maker
import gov.nist.hl7maker.view.SaveMsgDialog;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.MMLog;
/*import gov.nist.hl7maker.util.hapi.GenerationDialog;
import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.util.hapi.SwingWorker;*/
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.MakerMessage;
import gov.nist.hl7maker.control.Profile;

// HAPI
//import ca.uhn.hl7v2.HL7Exception;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

// SAX
import org.xml.sax.SAXException;  

public class MessageTree extends JTree implements TreeSelectionListener, MouseListener {

    private Vector vDir;
    private XMLDirectory xmlDir;
    private JTabbedPane tabbedPane;
    private DefaultTreeModel model;
    private JPanel clPanel;
    private TreePopUp treePopUp;
    private User user;
    private Session session;
    private DefaultMutableTreeNode oldNode;

    private Vector vProfile;
    private Profile ramProfile;
    private String lastProfile;

    /**
     * Constructor
     */
    public MessageTree() {
	addTreeSelectionListener(this);
	addMouseListener(this);
	vDir = new Vector();
	treePopUp = new TreePopUp(this);
	vProfile = new Vector();
    }

    /**
     * Set the JTabbedPane
     * @param tabbedPane The JTabbedPane
     */
    public void setTabbedPane(JTabbedPane tabbedPane) {
	this.tabbedPane = tabbedPane;
    }
    
    /**
     * Set the CardLayout Panel to make the switch
     * @param clPanel The CardLayout
     */
    public void setCardLayout(JPanel clPanel) {
	this.clPanel = clPanel;
    }

    /**
     * Set the User object
     * @param user The User
     */
    public void setUser(User user) {
	this.user = user;
    }

    /**
     * Set the Session object
     * @param session The Session
     */
    public void setSession(Session session) {
	this.session = session;
    }

    /**
     * Mount a directory
     * @param dir The directory
     */
    public void mountDir(XMLDirectory dir) {
	try {
	    if(!vDir.contains(dir)) {
		vDir.add(dir);
	    }
	    model = new DefaultTreeModel(getTree());
	    setModel(model);
	    treePopUp.setCurrentXMLDirectory(null);
	    CardLayout cl = (CardLayout)(clPanel.getLayout());
	    cl.show(clPanel, "Message");
	    JPanel pTmp = (JPanel)clPanel.getComponent(0);
	    //		JTabbedPane tabbedPane = (JTabbedPane)clPanel.getComponent(0);
	    JTabbedPane tabbedPane = (JTabbedPane)pTmp.getComponent(0);
	    // Update the different views
	    XMLView xmlView = (XMLView)((JScrollPane)tabbedPane.getComponentAt(0)).getViewport().getView();
	    ER7View er7View = (ER7View)((JScrollPane)tabbedPane.getComponentAt(1)).getViewport().getView();
	    EnhancedView enhancedView = (EnhancedView)((JScrollPane)tabbedPane.getComponentAt(2)).getViewport().getView();
	    //ValidationView validationView = (ValidationView)((JScrollPane)tabbedPane.getComponentAt(3)).getViewport().getView();
	    xmlView.reset();
	    er7View.reset();
	    enhancedView.reset();
	    //validationView.reset();
	    JTextArea txtMsg = (JTextArea)((JScrollPane)((JPanel)pTmp.getComponent(2)).getComponent(1)).getViewport().getComponent(0);
	    txtMsg.setText(" ");
	    // Update the list of messages in the user object
	    Vector vMsg = dir.getMessages();
	    for(int i=0;i<vMsg.size();i++) {
		String sMsg = (String)vMsg.get(i);
		if(!sMsg.startsWith("TD_")) {
		    String sFile = dir.getDirectory() + Constants.FS + vMsg.get(i);
		    user.addMsg(sFile, dir.getProfileFromMessage(sFile));
		}
		else {
		    user.addMD(sMsg.substring(3));
		}
	    }	
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	}
	catch(SAXException saxe) {
	    MMLog.log("SAXException " + saxe.getMessage());
	    MMLog.log("The metadata file is not a correct XML file");
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	}
    }

    /** 
     * UnMount a directory
     * @param dir The directory
     */
    public void unMountDir(XMLDirectory dir) {
	try {
	    // Check if the directory contains generated but not saved messages
	    Vector vMsg = dir.getMessages();
	    Vector vGen = new Vector();
	    for(int i=0;i<vMsg.size();i++) {
		String sMsg = (String)vMsg.get(i);
		if(!sMsg.startsWith("TD")) {
		    if(user.getStatus(dir.getDirectory() + Constants.FS + sMsg) == 1) {
			vGen.add(dir.getDirectory() + Constants.FS + vMsg.get(i));
		    }
		}
	    }
	    // Ask if the user wants to save the messages
	    if(vGen.size() > 0) {
		JOptionPane optionPane = new JOptionPane("Do you want to save the generated messages?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
		JDialog dialog = optionPane.createDialog(this, "Save generated messages");
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dialog.getSize();
		dialog.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				       Math.max(0,(screenSize.height-windowSize.height)/2));
		dialog.show();
		Object selectedValue = optionPane.getValue();
		if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		    // If so then save them
		    SaveMsgDialog dlgSaveMsg = new SaveMsgDialog(null);
		    dlgSaveMsg.setLstMsg(vGen);
		    dlgSaveMsg.setUser(user);
		    dlgSaveMsg.pack();
		    // Center the window
		    //Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		    windowSize = dlgSaveMsg.getSize();
		    dlgSaveMsg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
					   Math.max(0,(screenSize.height-windowSize.height)/2));
		    dlgSaveMsg.setVisible(true);
		}
		else {
		    for(int i=0;i<vGen.size();i++) {
			File f = new File((String)vGen.get(i));
			f.delete();
		    }
		}
	    }

	    vDir.remove(dir);
	    model = new DefaultTreeModel(getTree());
	    setModel(model);
	    treePopUp.setCurrentXMLDirectory(null);
	    CardLayout cl = (CardLayout)(clPanel.getLayout());
	    cl.show(clPanel, "Message");
	    JPanel pTmp = (JPanel)clPanel.getComponent(0);
	    //		JTabbedPane tabbedPane = (JTabbedPane)clPanel.getComponent(0);
	    JTabbedPane tabbedPane = (JTabbedPane)pTmp.getComponent(0);
	    // Update the different views
	    XMLView xmlView = (XMLView)((JScrollPane)tabbedPane.getComponentAt(0)).getViewport().getView();
	    ER7View er7View = (ER7View)((JScrollPane)tabbedPane.getComponentAt(1)).getViewport().getView();
	    EnhancedView enhancedView = (EnhancedView)((JScrollPane)tabbedPane.getComponentAt(2)).getViewport().getView();
	    //ValidationView validationView = (ValidationView)((JScrollPane)tabbedPane.getComponentAt(3)).getViewport().getView();
	    xmlView.reset();
	    er7View.reset();
	    enhancedView.reset();
	    //validationView.reset();
	    JTextArea txtMsg = (JTextArea)((JScrollPane)((JPanel)pTmp.getComponent(2)).getComponent(1)).getViewport().getComponent(0);
	    txtMsg.setText(" ");
	    // Update the list of messages in the user object
	    for(int i=0;i<vMsg.size();i++) {
		String sMsg = (String)vMsg.get(i);
		if(!sMsg.startsWith("TD")) {
		    user.removeMsg(dir.getDirectory() + Constants.FS + vMsg.get(i));
		}
	    }
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
    }

    /**
     * Update the message list
     * @param xmlDir The directory where the messages are stored
     */
    public void updateMessageList(XMLDirectory xmlDir) {
	mountDir(xmlDir);
    }

    /**
     * Get a tree representation of the directories
     * @return The tree representation
     */
    private DefaultMutableTreeNode getTree() throws ParserConfigurationException, SAXException, IOException {
	DefaultMutableTreeNode top = new DefaultMutableTreeNode("Messages");
	DefaultMutableTreeNode dir = null;
	if(vDir.size() > 0) {
	    for(int i=0;i<vDir.size();i++) {
		XMLDirectory xmlDir = (XMLDirectory)vDir.get(i);
		if(xmlDir.getTree() != null) {
		    top.add(xmlDir.getTree());
		}
	    }
	}
	return top;
    }

    /**
     * Save one or several messages
     */
    public void saveMsg() {
	Vector vMsg = getSDMessageList();

	for(int i=0;i<vMsg.size();i++) {
	    String elt = (String)vMsg.get(i);
	    if(user.getStatus(elt) == -1) {
		vMsg.remove(elt);
		i--;
	    }
	}
	if(vMsg.size() > 0) {
	    SaveMsgDialog dlgSaveMsg = new SaveMsgDialog(null);
	    dlgSaveMsg.setLstMsg(vMsg);
	    dlgSaveMsg.setUser(user);
	    dlgSaveMsg.pack();
	    // Center the window
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension windowSize = dlgSaveMsg.getSize();
	    dlgSaveMsg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				   Math.max(0,(screenSize.height-windowSize.height)/2));
	    dlgSaveMsg.setVisible(true);
	}
    }

    /**
     * Delete one or several messages
     */
    public void deleteMsg() {
	JOptionPane optionPane = new JOptionPane("Are you sure you want to delete?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
	JDialog dialog = optionPane.createDialog(this, "Confirm File Delete");
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension windowSize = dialog.getSize();
	dialog.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			   Math.max(0,(screenSize.height-windowSize.height)/2));
	dialog.show();
	Object selectedValue = optionPane.getValue();
	if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
	    Vector vMsg = getSDMessageList();
	    for(int i=0;i<vMsg.size();i++) {
		String elt = (String)vMsg.get(i);
		if(elt.startsWith("TD")) {
		    elt = elt.substring(2);
		}
		else {
		    File f = new File(elt);
		    if(f.exists()) {
			String filename = f.getPath();
			user.removeMsg(filename);
			f.delete();
			oldNode = null;
		    }
		}
	    }
	    user.removeAllMD();
	}
	try {
	    DefaultMutableTreeNode curNode = (DefaultMutableTreeNode)getLastSelectedPathComponent();
	    model = new DefaultTreeModel(getTree());
	    setModel(model);
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
    }

    /**
     * List messages affected by the save or deletion
     *@return List a messages
     */
    private Vector getSDMessageList() {
	try {
	    Vector v = new Vector();
	    DefaultMutableTreeNode curNode = (DefaultMutableTreeNode)getLastSelectedPathComponent();
	    if(curNode != null) {
		String val = (String)curNode.getUserObject();
		int idxSlash = val.indexOf(Constants.FS);
		int idxDotXML = val.indexOf(".xml");
		// Messages Node (Root)
		if(val.compareTo("Messages") == 0) {
		    // Do nothing
		}
		// Directory
		else if(idxSlash != -1 && idxDotXML == -1) {
		    // List all files (MetaData and files associated)
		    XMLDirectory xmlDir = new XMLDirectory((String)curNode.getUserObject());
		    Vector vTmp = xmlDir.getMessages();
		    for(int i=0;i<vTmp.size();i++) {
			String sMsg = (String)vTmp.get(i);
			if(sMsg.startsWith("TD_")) {
			    v.add(sMsg.substring(3));
			}
			else {
			    v.add(curNode.getUserObject() + Constants.FS + vTmp.get(i));
			}
		    }
		    //unMountDir(xmlDir);
		}
		// Message Node
		else if (idxSlash == -1 && idxDotXML != -1) {
		    // Return only this message
		    v.add((String)((DefaultMutableTreeNode)curNode.getParent().getParent()).getUserObject() + Constants.FS + curNode.getUserObject());
		}
		// MetaData Node
		else if(idxSlash != -1 && idxDotXML != -1) {
		    // Return metadata and files associated
		    MetaData md = new MetaData((String)curNode.getUserObject());
		    v.add((String)curNode.getUserObject());
		    Vector vTmp = md.getMessageList();
		    for(int i=0;i<vTmp.size();i++) {
			v.add((String)((DefaultMutableTreeNode)curNode.getParent()).getUserObject() + Constants.FS + vTmp.get(i));
		    }
		}
	    }
	    return v;
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	return null;
    }

    /**
    * Update views when the user selects a file
    * @param e The Event of selecting a file
    */
    public void valueChanged(TreeSelectionEvent e) {
	// Check if a message has been modified in enhanced view
	if(oldNode != null) {
	    CardLayout cl = (CardLayout)(clPanel.getLayout());
	    JPanel pTmp = (JPanel)clPanel.getComponent(0);
	    JTabbedPane tabbedPane = (JTabbedPane)pTmp.getComponent(0);
	    EnhancedView enhancedView = (EnhancedView)((JScrollPane)tabbedPane.getComponentAt(2)).getViewport().getView();
	    if(enhancedView.hasBeenModified()) {
		JOptionPane optionPane = new JOptionPane("Do you want to save the modification made on the message?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION);
		JDialog dialog = optionPane.createDialog(this, "Save Message");
		// Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = dialog.getSize();
		dialog.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
				   Math.max(0,(screenSize.height-windowSize.height)/2));
		dialog.show();
		Object selectedValue = optionPane.getValue();
		if(selectedValue != null && ((Integer)selectedValue).intValue() == 0) {
		    enhancedView.saveMsg();
		}
		enhancedView.setHasBeenModified(false);
		oldNode = null;
	    }
	}
	DefaultMutableTreeNode curNode = (DefaultMutableTreeNode)getLastSelectedPathComponent();
	if(curNode == null) {
	    CardLayout cl = (CardLayout)(clPanel.getLayout());
	    cl.show(clPanel, "Blank");
	    treePopUp.setCurrentXMLDirectory(null);
	    session.fireEvent(new MVCArg("saveMenu", "false"));
	    session.fireEvent(new MVCArg("deleteMenu", "false"));
	}
	else if(curNode != null) {
	    String val = (String)curNode.getUserObject();
	    int idxSlash = val.indexOf(Constants.FS);
	    int idxDotXML = val.indexOf(".xml");
	    // Messages Node (Root)
	    if(val.compareTo("Messages") == 0) {
		CardLayout cl = (CardLayout)(clPanel.getLayout());
		cl.show(clPanel, "Blank");
		treePopUp.setCurrentXMLDirectory(null);
		session.fireEvent(new MVCArg("saveMenu", "false"));
		session.fireEvent(new MVCArg("deleteMenu", "false"));
	    }
	    else if(idxSlash != -1 && idxDotXML == -1) {
		CardLayout cl = (CardLayout)(clPanel.getLayout());
		cl.show(clPanel, "Blank");
		treePopUp.setCurrentXMLDirectory(new XMLDirectory((String)curNode.getUserObject()));
		session.fireEvent(new MVCArg("saveMenu", "false"));
		session.fireEvent(new MVCArg("deleteMenu", "false"));
	    }
	    // Message Node
	    else if (idxSlash == -1 && idxDotXML != -1) {
		oldNode = curNode;
		treePopUp.setCurrentXMLDirectory(null);
		CardLayout cl = (CardLayout)(clPanel.getLayout());
		cl.show(clPanel, "Message");
		JPanel pTmp = (JPanel)clPanel.getComponent(0);
		//		JTabbedPane tabbedPane = (JTabbedPane)clPanel.getComponent(0);
		JTabbedPane tabbedPane = (JTabbedPane)pTmp.getComponent(0);
		// Update the different views
		XMLView xmlView = (XMLView)((JScrollPane)tabbedPane.getComponentAt(0)).getViewport().getView();
		ER7View er7View = (ER7View)((JScrollPane)tabbedPane.getComponentAt(1)).getViewport().getView();
		EnhancedView enhancedView = (EnhancedView)((JScrollPane)tabbedPane.getComponentAt(2)).getViewport().getView();
		//ValidationView validationView = (ValidationView)((JScrollPane)tabbedPane.getComponentAt(3)).getViewport().getView();

		try {
		    // Get the XMLDirectory
		    xmlDir = (XMLDirectory)vDir.get(vDir.indexOf(new XMLDirectory((String)((DefaultMutableTreeNode)curNode.getParent().getParent()).getUserObject())));
		    String fileName = xmlDir.getDirectory() + Constants.FS + (String)curNode.getUserObject();
		    if(new File(fileName).isFile()) {
			xmlView.displayMessage(fileName);
			er7View.displayMessage(fileName);
			enhancedView.displayMessage(xmlDir.getMessageDocument(fileName), xmlDir.getProfileFromMessage(fileName), xmlDir.getDirectory());
		    }
		    // Get metadata about the message
		    String profile = xmlDir.getProfileFromMessage(fileName);
		    MetaData md = new MetaData((String)((DefaultMutableTreeNode)curNode.getParent()).getUserObject());
		    JTextArea txtMsg = (JTextArea)((JScrollPane)((JPanel)pTmp.getComponent(2)).getComponent(1)).getViewport().getComponent(0);
		    txtMsg.setText(md.getMsgType((String)curNode.getUserObject())); // + ": " + md.getMsgDescription((String)curNode.getUserObject()));
		    if(md.getLocation((String)curNode.getUserObject()) != null && !md.getLocation((String)curNode.getUserObject()).equals("")) {
			if(lastProfile == null || !lastProfile.equals(profile)) {
			    lastProfile = profile;
			    ramProfile = new Profile();
			    ramProfile.loadProfile(profile);
			}
			MetaData.MDMsg mdMsg = md.getMDMsg((String)curNode.getUserObject());
			txtMsg.setText(txtMsg.getText() + ": " + md.getVarDescription((String)curNode.getUserObject()));
			txtMsg.setText(txtMsg.getText() + "\n" + "Specific test location: " + md.getLocation((String)curNode.getUserObject()));
			txtMsg.setText(txtMsg.getText() + "Sequence number in the message: " + ramProfile.getMsgLocation(mdMsg.segmentGroup, mdMsg.segment, mdMsg.field, mdMsg.component, mdMsg.subcomponent));
		    }
		    txtMsg.moveCaretPosition(0);
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException: " + ioe.getMessage());
		    ioe.printStackTrace();
		}
		/*		    catch(TransformerConfigurationException tce) {
					System.err.println("TransformerConfigurationException " + tce.getMessage());
					tce.printStackTrace();
				    }
				    catch(TransformerException te) {
					System.err.println("TransformerException " + te.getMessage());
					te.printStackTrace();
					}*/

		/*try {
		    // Get the XMLDirectory
		    xmlDir = (XMLDirectory)vDir.get(vDir.indexOf(new XMLDirectory((String)((DefaultMutableTreeNode)curNode.getParent().getParent()).getUserObject())));
		    String fileName = xmlDir.getDirectory() + Constants.FS + (String)curNode.getUserObject();
		    if(new File(fileName).isFile()) {

			// Check if the message has a Z-Segment or group segment
			String profile = xmlDir.getProfileFromMessage(fileName);
			final boolean needCustom = CustomHAPI.needCustom(profile);
			final boolean needGeneration = needCustom && !vProfile.contains(profile);
			if(needGeneration) vProfile.add(profile);

			final GenerationDialog dlgGeneration = new GenerationDialog(null);
			final String fFileName = fileName;
			SwingWorker worker = new SwingWorker() {
				public Object construct() {
				    try {
					if(needGeneration) {
					    CustomHAPI cHAPI = new CustomHAPI();
					    cHAPI.addObserver(dlgGeneration);
					    cHAPI.generate(xmlDir.getProfileFromMessage(fFileName));
					}
					//er7View.displayMessage(xmlDir.getProfileFromMessage(fFileName), xmlDir.getMessageDocument(fFileName), "2.4", needCustom);
					xmlView.displayMessage(fFileName);
					er7View.displayMessage(fFileName);
					enhancedView.displayMessage(xmlDir.getMessageDocument(fFileName), xmlDir.getProfileFromMessage(fFileName));
					// validationView.displayReport(xxx);
					validationView.validate(new File(fFileName), xmlDir.getMessageDocument(xmlDir.getProfileFromMessage(fFileName)), xmlDir.getProfileFromMessage(fFileName));
				    }
				    catch(ParserConfigurationException pce) {
					System.err.println("ParserConfigurationException " + pce.getMessage());
					pce.printStackTrace();
				    }
				    catch(SAXException saxe) {
					System.err.println("SAXException " + saxe.getMessage());
					saxe.printStackTrace();
				    }
				    catch(TransformerConfigurationException tce) {
					System.err.println("TransformerConfigurationException " + tce.getMessage());
					tce.printStackTrace();
				    }
				    catch(TransformerException te) {
					System.err.println("TransformerException " + te.getMessage());
					te.printStackTrace();
				    }
				    catch(IOException ioe) {
					System.err.println("IOException: " + ioe.getMessage());
					ioe.printStackTrace();
				    }
				    catch(HL7Exception hl7e) {
					System.err.println("HL7Exception: " + hl7e.getMessage());
					hl7e.printStackTrace();
					}
				    return null;
				}
			    };
			worker.start();
			
			if(needGeneration) {
			    dlgGeneration.setProfile(xmlDir.getProfileFromMessage(fileName));
			    dlgGeneration.pack();
			    // Center the window
			    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			    Dimension windowSize = dlgGeneration.getSize();
			    dlgGeneration.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
						      Math.max(0,(screenSize.height-windowSize.height)/2));
			    dlgGeneration.setVisible(true);
			}
			 
			// Get metadata about the message
			MetaData md = new MetaData((String)((DefaultMutableTreeNode)curNode.getParent()).getUserObject());
			JTextArea txtMsg = (JTextArea)((JScrollPane)((JPanel)pTmp.getComponent(2)).getComponent(1)).getViewport().getComponent(0);
			txtMsg.setText(md.getMsgType((String)curNode.getUserObject())); // + ": " + md.getMsgDescription((String)curNode.getUserObject()));
			if(md.getLocation((String)curNode.getUserObject()) != null && !md.getLocation((String)curNode.getUserObject()).equals("")) {
			    if(lastProfile == null || !lastProfile.equals(profile)) {
				lastProfile = profile;
				ramProfile = new Profile();
				ramProfile.loadProfile(profile);
			    }
			    MetaData.MDMsg mdMsg = md.getMDMsg((String)curNode.getUserObject());
			    txtMsg.setText(txtMsg.getText() + ": " + md.getVarDescription((String)curNode.getUserObject()));
			    txtMsg.setText(txtMsg.getText() + "\n" + "Specific test location: " + md.getLocation((String)curNode.getUserObject()));
			    txtMsg.setText(txtMsg.getText() + "\n" + "Sequence number in the message: " + ramProfile.getMsgLocation(mdMsg.segmentGroup, mdMsg.segment, mdMsg.field, mdMsg.component, mdMsg.subcomponent));
			}
			txtMsg.moveCaretPosition(0);
		    }
		}
		catch(HL7Exception hl7e) {
		    System.err.println("HL7Exception " + hl7e.getMessage());
		    hl7e.printStackTrace();
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}*/
		session.fireEvent(new MVCArg("saveMenu", "true"));
		session.fireEvent(new MVCArg("deleteMenu", "true"));
	    }
	    // MetaData Node
	    else if(idxSlash != -1 && idxDotXML != -1) {
		try {
		    treePopUp.setCurrentXMLDirectory(null);
		    CardLayout cl = (CardLayout)(clPanel.getLayout());
		    cl.show(clPanel, "MetaData");
		    // Read the metadata
		    MetaData md = new MetaData(val);
		    // Show the metadata
		    MetaDataPanel mdPanel = (MetaDataPanel)clPanel.getComponent(1);
		    mdPanel.displayMetaData(md);
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}
		session.fireEvent(new MVCArg("saveMenu", "true"));
		session.fireEvent(new MVCArg("deleteMenu", "true"));
	    }
	}
    }

    /**
     * Handles a right click
     * @param e The event
     */
    public void mouseClicked(MouseEvent e) {
	if(e.getButton() == 3) {	
	    treePopUp.show((JTree)e.getSource(), e.getX(), e.getY());
	}
    }
    public void mouseEntered(MouseEvent e) {;}
    public void mouseExited(MouseEvent e) {;}
    public void mousePressed(MouseEvent e) {;}
    public void mouseReleased(MouseEvent e) {;}
}
