/**
 * This class is wrapping a Node object to use with the TreeModel
 */

package gov.nist.hl7maker.view.display;

// DOM
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MyNode {

    private Node domNode;

    /**
     * Constructor
     * @param node a Node
     */
    public MyNode(Node node) {
	domNode = node;
    }

    public String toString() {
	StringBuffer buffy = new StringBuffer();

	if(domNode.getNodeType() == Node.ELEMENT_NODE) {
	    buffy.append(domNode.getNodeName());
	    if(domNode.hasChildNodes()) {
		NodeList nl = domNode.getChildNodes();
		if(nl.getLength() == 1 && nl.item(0).getNodeType() == Node.TEXT_NODE) {
		    buffy.append(" ").append(nl.item(0).getNodeValue().trim());
		}
	    }	    
	}
	else {
	    System.err.println("This type of node is not processed: " + domNode.getNodeType());
	}
	return new String(buffy);	
    }

    /**
     * Return true if the node is a leaf
     * In this node, we consider that a leaf node is a node with one child
     * which is a TEXT_NODE
     * @return is a leaf?
     */
    public boolean isLeaf() {
	if(domNode.getNodeType() == Node.ELEMENT_NODE) {
	    if(domNode.hasChildNodes()) {
		NodeList nl = domNode.getChildNodes();
		if(nl.getLength() == 1 && nl.item(0).getNodeType() == Node.TEXT_NODE) {
		    return true;
		}
	    }
	}
	return false;
    }

    /**
     * Return the first text node value if it is a leaf
     * @return The value of the node
     */
    public String getFirstNodeValue() {
	if(isLeaf()) {
	    return domNode.getChildNodes().item(0).getNodeValue().trim();
	}
	return null;
    }

    /**
     * Get the Node object
     * @return The Node object
     */
    public Node getNode() {
	return domNode;
    }
}
