/**
 * This class represents a JButton to save the modifications made on a message
 */

package gov.nist.hl7maker.view.display;

// JDK
import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;

// Swing
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.tree.TreeModel;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Element;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// SAX
import org.xml.sax.SAXException;

// HAPI
/*import ca.uhn.hl7v2.parser.XMLParser;
import ca.uhn.hl7v2.parser.DefaultXMLParser;
import ca.uhn.hl7v2.parser.PipeParser;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.HL7Exception;*/

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.ER7FileFilter;
import gov.nist.hl7maker.view.SaveMsgDialog;
/*import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.util.hapi.GenerationDialog;
import gov.nist.hl7maker.util.hapi.SwingWorker;*/

public class SaveButton extends JButton implements ActionListener, Observer {

    private TreeModel treeModel;
    private Session session;
    private String profile;
    private String msgPath;

    /**
     * Constructor
     */
    public SaveButton() {
	super("Save modification ...");
        setBackground(new Color(204, 204, 255));
	this.setEnabled(false);
	addActionListener(this);
    }

    /**
     * Set the TreeModel
     * @param treeModel The TreeModel
     */
    public void setTreeModel(TreeModel treeModel) {
	this.treeModel = treeModel;
	this.setEnabled(true);
    }

    /** 
     * Set the profile for CustomHAPI
     * @param profile The profile
     */
    public void setProfile(String profile) {
	this.profile = profile;
    }

    /**
     * Set the messagePath
     * @param msgPath The path
     */
    public void setMsgPath(String msgPath) {
	this.msgPath = msgPath;
    }

    /**
     * Handles action on click
     * @param e The event
     */
    public void actionPerformed(ActionEvent e) {
	SaveMsgDialog dlgSaveMsg = new SaveMsgDialog(null) {
 		// Redefine the click handler
 		public void actionPerformed(ActionEvent evt) {
 		    try {
 			if(evt.getSource() == btnSave) {
 			    // XML File
 			    if(chkXML.isSelected()) {
	 			JFileChooser fc = new JFileChooser(msgPath);
	 			// Set the chooser to accept only XML files.
	  			fc.addChoosableFileFilter(new XMLFileFilter());
	 			fc.setAcceptAllFileFilterUsed(false);
	 			int answer = fc.showSaveDialog(this);
	 			if(answer == JFileChooser.APPROVE_OPTION) {
		 		    // Check this is a xml file
		  		    File f = fc.getSelectedFile();
		 		    if(!f.getPath().endsWith(".xml")) {
		 			f = new File(fc.getSelectedFile().getPath() + ".xml");
			 	    }
 				    saveModel(f);
 				}
 			    }
	  		}
			// ER7 File
			if(chkER7.isSelected()) {
			    File f = null;
			    JFileChooser fc = new JFileChooser(msgPath);
			    // Set the chooser to accept only ER7 files.
			    fc.addChoosableFileFilter(new ER7FileFilter());
			    fc.setAcceptAllFileFilterUsed(false);
			    int answer = fc.showSaveDialog(this);
			    if(answer == JFileChooser.APPROVE_OPTION) {
				// Check this is a er7 file
			        f = fc.getSelectedFile();
				if(!f.getPath().endsWith(".er7")) {
				    f = new File(fc.getSelectedFile().getPath() + ".er7");
				}
				try {
				    String er7 = XMLtoER7.transform(getDocumentFromModel());
				    // Save the file
				    BufferedWriter bw = new BufferedWriter(new FileWriter(f));
				    bw.write(er7);
				    bw.close();
				}
				catch(IOException ioe) {
				    System.err.println("IOException: " + ioe.getMessage());
				    ioe.printStackTrace();
				}
			    }

/*
			    final File fF = f;
			    
			    SwingWorker worker = new SwingWorker() {
				    public Object construct() {
					try {
					    Document doc = getDocumentFromModel();
					    final boolean needCustom = CustomHAPI.needCustom(profile);

					    final String fProfile = profile;
					    
					    String prefix = "";
					    if(needCustom) {
						prefix = new File(fProfile).getName();
						prefix = prefix.substring(0, prefix.length()-4).replaceAll("_","XX").replaceAll(" ","XX");
						doc = CustomHAPI.modifyDocument(doc, prefix);
					    }
					    
					    // Get a Message Object from the Document
					    XMLParser xmlParser = new DefaultXMLParser();
					    Message msg = xmlParser.parseDocument(doc, "2.4");
					    //doc = xmlParser.encodeDocument(msg);
					    
					    // Get the target filename
					    //					    String er7FN = sMsg.substring(0, sMsg.length()-4).concat(".er7");
					    //instantiate a PipeParser, which handles the "traditional encoding" 
					    PipeParser pipeParser = new PipeParser();
					    String er7 = CustomHAPI.cleanER7(formatER7(pipeParser.encode(msg)), prefix);
					    // Save the file
					    BufferedWriter bw = new BufferedWriter(new FileWriter(fF));
					    bw.write(er7);
					    bw.close();
					}				    
					catch(ParserConfigurationException pce) {
					    System.err.println("ParserConfigurationException " + pce.getMessage());
					    pce.printStackTrace();
					}
					catch(TransformerConfigurationException tce) {
					    System.err.println("TransformerConfigurationException " + tce.getMessage());
					    tce.printStackTrace();
					}
					catch(TransformerException te) {
					    System.err.println("TransformerException " + te.getMessage());
					    te.printStackTrace();
					}
					catch(IOException ioe) {
					    System.err.println("IOException: " + ioe.getMessage());
					    ioe.printStackTrace();
					}
					catch(HL7Exception hl7e) {
					    System.err.println("HL7Exception: " + hl7e.getMessage());
					    hl7e.printStackTrace();
					}
					setVisible(false);
					return null;
				    }
				};
			    worker.start();
			    }*/
			}
			setVisible(false);
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
		    }
		    catch(TransformerException te) {
			System.err.println("TransformerException " + te.getMessage());
		    }
		}
	    };
	Vector vMsg = new Vector();
	vMsg.add("The Current Edited Message");
	dlgSaveMsg.setLstMsg(vMsg);
	dlgSaveMsg.pack();
	// Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension windowSize = dlgSaveMsg.getSize();
	dlgSaveMsg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
			       Math.max(0,(screenSize.height-windowSize.height)/2));
	dlgSaveMsg.setVisible(true);
    }

    /**
     * Get the DOM object of the message from the current TreeModel
     * @return The DOM representation of the message
     */
    private Document getDocumentFromModel() throws TransformerConfigurationException, TransformerException, ParserConfigurationException {
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	DocumentBuilder builder = factory.newDocumentBuilder();
	Document doc = builder.newDocument();

	Element root = (Element)((MyNode)treeModel.getRoot()).getNode();
	root = (Element)doc.importNode(root, true);
	doc.appendChild(root);

	return doc;
    }

    /**
     * Save the message in a file
     * @param file The destination file
     */
    public void saveModel(File file) throws TransformerConfigurationException, TransformerException, ParserConfigurationException {
	Document doc = getDocumentFromModel();	
	// Write the DOM document to the file
	Transformer transformer = TransformerFactory.newInstance().newTransformer();
	DOMSource source = new DOMSource(doc);
	StreamResult result = new StreamResult(file.getPath());
	transformer.transform(source, result);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in SaveButton");
		session = (Session)arg;
	    }
	}
    }
}


