/**
 * This class formats(indentation) an XML file
 */

package gov.nist.hl7maker.view.display;

// Swing
import javax.swing.JTextArea;

// SAX
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class XMLFormatter extends DefaultHandler {

    private JTextArea txt;
    private int indentLevel = 0;
    private boolean bText = false;

    /**
     * Set the JTextArea in where the XML file will be displayed
     * @param txt The JTextArea
     */
    public void setJTextArea(JTextArea txt) {
	this.txt = txt;
    }

    /**
     * Handle the indentation
     */
    public void nl() {
	txt.append("\n");
	for(int i=0;i<indentLevel;i++)
	    txt.append("  ");
    }

    public void startDocument() throws SAXException  {
	txt.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }

    public void endDocument() throws SAXException {
    }

    public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
	indentLevel++;
	nl();
	txt.append("<" + qName);
	if(attrs != null) {
	    for (int i=0;i<attrs.getLength();i++) {
		String aName = attrs.getQName(i);
		txt.append(" ");
		txt.append(aName + "=\"" + formatString(attrs.getValue(i)) + "\"");
	    }
	}
        txt.append(">");
    }

    public void endElement(String namespaceURI, String sName, String qName) throws SAXException {
	if(!bText)
	    nl();
	else
	    bText = false;
        txt.append("</" + qName + ">");
	indentLevel--;
    }

    public void characters(char[] ch, int start, int length) {
        String s = new String(ch, start, length);
        if (!s.trim().equals("")) {
	    txt.append(formatString(s));
	    bText = true;
	}
    }

    /**
     * Normalize a string, handles special characters
     * @src The source string
     * @return The normalized string
     */
    private String formatString(String src) {
	StringBuffer sb = new StringBuffer();

	for(int i=0;i<src.length();i++) {
	    if(src.charAt(i) == '&') {
		sb.append("&amp;");
	    }
	    else if(src.charAt(i) == '<') {
		sb.append("&lt;");
	    }
	    else if(src.charAt(i) == '>') {
		sb.append("&gt;");
	    }
	    else if(src.charAt(i) == '\'' || src.charAt(i) == '') {
		sb.append("&apos;");
	    }
	    else if(src.charAt(i) == '\"') {
		sb.append("&quot;");
	    }
	    else {
		sb.append(src.charAt(i));
	    }
	}
	return sb.toString();
    }
}
