/**
 * This class is a Panel for the XML View of a message
 */

package gov.nist.hl7maker.view.display;

// JDK
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
// Swing
import javax.swing.JTextArea;

// SAX
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// DOM
import org.w3c.dom.Document;

public class XMLView extends JTextArea {

    /**
     * Constructor
     */
    public XMLView() {
	setEditable(false);
    }

    /**
     * Display the message from a xml file 
     * @param xmlFile The xml filename
     */
    public void displayMessage(String xmlFile) throws ParserConfigurationException, SAXException, IOException {
	this.setText("");

	XMLFormatter handler = new XMLFormatter();
	handler.setJTextArea(this);

        SAXParserFactory factory = SAXParserFactory.newInstance();
	SAXParser saxParser = factory.newSAXParser();
	saxParser.parse(new File(xmlFile), handler);

	moveCaretPosition(0);
    }

    /**
     * Display the message from a Document object
     * @param doc The document
     */
    public void olddisplayMessage(Document doc) throws ParserConfigurationException, SAXException, IOException {
	try {
	    // Create a temporary file with this document
	    File fTmp = File.createTempFile("MessageMaker", ".xml", null);
	    // Save this doc into the temporary File
	    Transformer transformer = TransformerFactory.newInstance().newTransformer();
	    DOMSource source = new DOMSource(doc);
	    StreamResult result = new StreamResult(fTmp.getPath());
	    transformer.transform(source, result);
	    displayMessage(fTmp.getPath());
	    // Delete this temporary file on exit
	    fTmp.deleteOnExit();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
    }

    /**
     * Display the message from a Document object
     * @param doc The document
     */
    public void displayMessage(Document doc) {
	try {
	    Transformer transformer = TransformerFactory.newInstance().newTransformer();
	    DOMSource source = new DOMSource(doc);
	    StringWriter sw = new StringWriter();
	    StreamResult result = new StreamResult(sw);
	    transformer.setOutputProperty(OutputKeys.METHOD, "xml");
	    transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
	    transformer.setOutputProperty(OutputKeys.INDENT, "yes");
	    transformer.transform(source, result);
	    this.setText(sw.toString());
	    moveCaretPosition(0);
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
    }


    /**
     * Reset the display
     */
    public void reset() {
	setText("");
    }
}

