/**
 * This class transforms a XML message into a ER7 message
 */

package gov.nist.hl7maker.view.display;

// JDK
import java.io.File;
import java.io.FileInputStream;
//import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter;

// XML
import org.w3c.dom.Document;

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;

public class XMLtoER7 {

    private final static String XML2ER7 = "xslt" + File.separator + "xml2er7.xsl";

    /**
     * Transform a XML message into a ER7 message
     * @param xmlMessage The filename of the XML message
     * @return The message in ER7 format
     */
    public static String transform(String xmlMessage) {
	StringWriter sw = new StringWriter();
	try {
	    StreamSource xslt = new StreamSource(XML2ER7);
	    TransformerFactory tFactory = TransformerFactory.newInstance();
	    Transformer transformer = tFactory.newTransformer(xslt);
	    Source src = new StreamSource(new FileInputStream(xmlMessage));
	    Result dest = new StreamResult(sw);
	    //new StringWriter());//new FileOutputStream(xmlMessage.substring(0, xmlMessage.length()-4)  + ".er7"));
	    transformer.transform(src, dest);
	    System.out.println(sw);
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	return sw.toString();
    }

    /**
     * Transform a XML message into a ER7 message
     * @param xmlDoc The XML Message in DOM
     * @return The message in ER7 format
     */
    public static String transform(Document xmlDoc) {
	StringWriter sw = new StringWriter();
	try {
	    StreamSource xslt = new StreamSource(XML2ER7);
	    TransformerFactory tFactory = TransformerFactory.newInstance();
	    Transformer transformer = tFactory.newTransformer(xslt);
	    Source src = new DOMSource(xmlDoc);//new StreamSource(new FileInputStream(xmlMessage));
	    Result dest = new StreamResult(sw);
	    //new StringWriter());//new FileOutputStream(xmlMessage.substring(0, xmlMessage.length()-4)  + ".er7"));
	    transformer.transform(src, dest);
	    System.out.println(sw);
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
	/*catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	    }*/
	return sw.toString();
    }
}
