/**
 * This class is a dialog to configure the test
 */

package gov.nist.hl7maker.view.init;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;
import java.util.EventObject;
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.AbstractCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JCheckBox;
import javax.swing.SwingConstants;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeModel;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// DOM
import org.w3c.dom.Document;

// XSL
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.view.config.ProfileNode;
import gov.nist.hl7maker.view.config.ValueNode;
import gov.nist.hl7maker.view.config.TableValueNode;

public class ConfigureTestDialog extends JDialog implements Observer, ActionListener {

    private JComboBox cbType;

    private static JTable tableProfile;
    private TreeTableCellRenderer tree;
    private ProfileTableModel model;
    private Session session;

    private JLabel lblMsgVariant;
    private JButton btnSave, btnReset, btnClose, btnClear;

    /**
     * Constructor
     * @param frame A JFrame
     */
    public ConfigureTestDialog(JFrame frame) {
	super(frame, "Manual Test Selection", true);
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    closeDialog(evt);
		}
	    });

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);

	// Validity
	JPanel p = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.LINE_AXIS));
	JLabel l = new JLabel("Type of messages: ");
	p.add(l);
	cbType = new JComboBox();
	cbType.addActionListener(this);
	cbType.addItem("VALID");
	cbType.addItem("INVALID");
	//cbType.addItem("BOTH VALID and INVALID");
	p.add(cbType);
	p.add(Box.createRigidArea(new Dimension(550, 0)));

	// JTable
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	tableProfile = new JTable() {
		public boolean isCellEditable(int row, int col) {        
		    return true;
		}

		/* Workaround for BasicTableUI anomaly. Make sure the UI never tries to 
		 * paint the editor. The UI currently uses different techniques to 
		 * paint the renderers and editors and overriding setBounds() below 
		 * is not the right thing to do for an editor. Returning -1 for the 
		 * editing row in this case, ensures the editor is never painted. 
		 */
		public int getEditingRow() {
		    //		    return (model.getColumnClass(editingColumn) == MSHTableModel.class) ? -1 : editingRow;  
		    return -1;
		}

	    };
	model = new ProfileTableModel();
	tableProfile.setModel(model);
	JScrollPane tableScrollPane = new JScrollPane(tableProfile);
	tableScrollPane.setPreferredSize(new Dimension(800,550));
	tableScrollPane.setOpaque(true);
	p0.add(tableScrollPane);


	// Size of column
	TableColumn column = null;
	for(int i = 0; i < 2; i++) {
	    column = tableProfile.getColumnModel().getColumn(i);
	    if(i == 0) {
		column.setPreferredWidth(700);
	    } 
	    else if(i == 1){
		column.setPreferredWidth(75);
	    }
	    else if(i == 2){
		column.setPreferredWidth(20);
	    }
	}
	
        // Save / Reset / Close Buttons
	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
	// Label info on message variants
	lblMsgVariant = new JLabel("Number of Message Variants: 0");
	p6.add(lblMsgVariant);
	p6.add(Box.createRigidArea(new Dimension(25,0)));
	// Save
	btnSave = new JButton("Apply");
        btnSave.setBackground(new Color(204, 204, 255));
	btnSave.addActionListener(this);
	p6.add(btnSave);
	p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Reset
	btnReset = new JButton("Undo Modification");
        btnReset.setBackground(new Color(204, 204, 255));
	btnReset.addActionListener(this);
	p6.add(btnReset);
	p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Clear
	btnClear = new JButton("Clear All");
	btnClear.setBackground(new Color(204, 204, 255));
	btnClear.addActionListener(this);
	p6.add(btnClear);
	p6.add(Box.createRigidArea(new Dimension(25, 0)));
	// Close
	btnClose = new JButton("Close");
        btnClose.setBackground(new Color(204, 204, 255));
	btnClose.addActionListener(this);
	p6.add(btnClose);

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.PAGE_AXIS));
	p7.add(p);
	p7.add(Box.createRigidArea(new Dimension(0, 10)));
	p7.add(p0);
	p7.add(Box.createRigidArea(new Dimension(0,10)));
	p7.add(p6);
	p7.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p7);
    }

    /**
     * Set the session
     * @param session The session
     */
    public void setSession(Session session) {
	this.session = session;
	model.setSession(session);
	lblMsgVariant.setText("Number of Message Variants: " + session.countMsgVariants());
	btnClear.setEnabled(session.countMsgVariants() != 0);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureTestDialog");
		session = (Session)arg;
		model.setSession(session);
		//		lblMsgVariant.setText("Number of Message Variants: " + session.countMsgVariants());
		if(((Session)arg).getProfile().compareTo("") != 0) {
		    /*Document doc = null;
		    // TableModel
		    try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			doc = builder.parse(new File(((Session)arg).getProfile()));
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    model.setDocument(doc);*/
		    //model.set
		    // Create the tree. It will be used as a renderer and editor. 
		    tree = new TreeTableCellRenderer(model);
		    // Force the JTable and JTree to share their row selection models. 
		    tree.setSelectionModel(new DefaultTreeSelectionModel() { 
			    // Extend the implementation of the constructor, as if: 
			    /* public this() */ {
			tableProfile.setSelectionModel(listSelectionModel); 
			    } 
			}); 
		    // Make the tree and table row heights the same. 
		    tree.setRowHeight(tableProfile.getRowHeight()); // <= Permet d'afficher les sous arbres
		    
		    model.setTree(tree);
		    tableProfile.setDefaultRenderer(ProfileTableModel.class, tree); 
		    tableProfile.setDefaultEditor(ProfileTableModel.class, new TreeTableCellEditor());  
		    tableProfile.setShowGrid(false);
		    tableProfile.setIntercellSpacing(new Dimension(1, 0));

		    //tableProfile.setDefaultRenderer(Vector.class, new CurrentTableCellRenderer()); 
		    //tableProfile.setDefaultEditor(Vector.class, new CurrentTableCellEditor());  

		    tableProfile.setDefaultRenderer(Vector.class, new ValueTableCellRenderer());
		    tableProfile.setDefaultEditor(Vector.class, new ValueTableCellEditor());

		    //		    tableProfile.setDefaultRenderer(JButton.class, new SetTableCellRenderer());
		    //tableProfile.setDefaultEditor(JButton.class, new SetTableCellEditor());
		}
	    }
	    if(arg instanceof MVCArg) {
		if("profile".compareTo(((MVCArg)arg).getName()) == 0 && !((MVCArg)arg).getValue().equals("")) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in ConfigureTestDialog");
		    model.setSession((Session)o);

/*		    Document doc = null;
		    // TableModel
		    try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			doc = builder.parse(new File(((MVCArg)arg).getValue()));
			//lblMsgVariant.setText("Number of Message Variants: " + session.countMsgVariants());
		    }
		    catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    model.setDocument(doc);*/
		    // Create the tree. It will be used as a renderer and editor. 
		    tree = new TreeTableCellRenderer(model);
		    // Force the JTable and JTree to share their row selection models. 
		    tree.setSelectionModel(new DefaultTreeSelectionModel() { 
			    // Extend the implementation of the constructor, as if: 
			    /* public this() */ {
			tableProfile.setSelectionModel(listSelectionModel); 
			    } 
			}); 
		    // Make the tree and table row heights the same. 
		    tree.setRowHeight(tableProfile.getRowHeight()); // <= Permet d'afficher les sous arbres
		    
		    model.setTree(tree);
		    tableProfile.setDefaultRenderer(ProfileTableModel.class, tree); 
		    tableProfile.setDefaultEditor(ProfileTableModel.class, new TreeTableCellEditor());  
		    tableProfile.setShowGrid(false);
		    tableProfile.setIntercellSpacing(new Dimension(1, 0));

		    //tableProfile.setDefaultRenderer(Vector.class, new CurrentTableCellRenderer()); 
		    //tableProfile.setDefaultEditor(Vector.class, new CurrentTableCellEditor());  

		    tableProfile.setDefaultRenderer(Vector.class, new ValueTableCellRenderer());
		    tableProfile.setDefaultEditor(Vector.class, new ValueTableCellEditor());

		    //tableProfile.setDefaultRenderer(JButton.class, new SetTableCellRenderer());
		    //tableProfile.setDefaultEditor(JButton.class, new SetTableCellEditor());
		}
	    }
	}
    }

    /**
     * Handles clicks on JButton
     * @param evt The Event
     */
    public void actionPerformed(ActionEvent evt) {
	boolean needToSave = false;
	if(evt.getSource() == btnReset) {
	    session.setProfile(session.getProfile());
	    model.clearUpdate();
	    session.clearTest();
	    session.clearTempTest();
	    session.loadTest();
	    session.loadTempTest();
	    lblMsgVariant.setText("Number of Message Variants: " + session.countMsgVariants());
	    btnClear.setEnabled(session.countMsgVariants() != 0);
	}
	else if(evt.getSource() == btnClose) {
	    if(model.getUpdateElement().size() > 0) {
		Object[] options = {"Yes", "No"};
		JOptionPane optionPane = new JOptionPane("Apply Changes to Test Selections?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Yes");
		JDialog dialog = optionPane.createDialog(this, "Save the data");
		dialog.show();
		Object selectedValue = optionPane.getValue();
		
		if(selectedValue != null) {
		    if(((String)selectedValue).compareTo(options[0]) == 0) {
			// Save The data
			needToSave = true;
		    }
		    else if(((String)selectedValue).compareTo(options[1]) == 0) {
			setVisible(false);
			model.clearUpdate();
			session.clearTest();
			session.clearTempTest();
		    }
		}
	    }
	    else {
		setVisible(false);
		session.clearTest();
		session.clearTempTest();
	    }
	}
	if(evt.getSource() == btnSave || (evt.getSource() == btnClose && needToSave)) {
	    int oldType = cbType.getSelectedIndex();
	    for(int x=0;x<3;x++) {
		model.setType((short)x);
		Vector vUpdate = model.getUpdateElement();
		for(int i=0;i<vUpdate.size();i++) {
		    ProfileTableModel.UpdateElement ue = (ProfileTableModel.UpdateElement)vUpdate.get(i);
		    if(ue.selected) {
			// Cardinality
			if(ue.column == 1) {
			    switch(x) {
			    case 0:
				session.addTest("cardinalityValid", ue.pn);
				break;
			    case 1:
				session.addTest("cardinalityInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("cardinalityBoth", ue.pn);
				break;
			    }
			}
			// Usage
			else if(ue.column == 2) {
			    switch(x) {
			    case 0:
				session.addTest("usageValid", ue.pn);
				break;
			    case 1:
				session.addTest("usageInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("usageBoth", ue.pn);
				break;
			    }			    
			}
			// Error Codes
			else if(ue.column == 3) {
			    switch(x) {
			    case 0:
				session.addTest("errorCodesValid", ue.pn);
				break;
			    case 1:
				session.addTest("errorCodesInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("errorCodesBoth", ue.pn);
				break;
			    }			    
			}
		    }
		    else {
			// Cardinality
			if(ue.column == 1) {
			    switch(x) {
			    case 0:
				session.removeTest("cardinalityValid", ue.pn);
				break;
			    case 1:
				session.removeTest("cardinalityInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("cardinalityBoth", ue.pn);
				break;
			    }
			}
			// Usage
			else if(ue.column == 2) {
			    switch(x) {
			    case 0:
				session.removeTest("usageValid", ue.pn);
				break;
			    case 1:
				session.removeTest("usageInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("usageBoth", ue.pn);
				break;
			    }
			}
			// Error Codes
			else if(ue.column == 3) {
			    switch(x) {
			    case 0:
				session.removeTest("errorCodesValid", ue.pn);
				break;
			    case 1:
				session.removeTest("errorCodesInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("errorCodesBoth", ue.pn);
				break;
			    }			    
			}
		    }
		}
	    }
	    model.setType((short)oldType);
	    session.saveTest();
	    model.clearUpdate();
	    if(needToSave && evt.getSource() == btnClose) {
		setVisible(false);
		session.clearTest();
		session.clearTempTest();
	    }
	    session.setVolume(session.countMsgVariants(), true);
	}
	else if(evt.getSource() == btnClear) {
	    Object[] options = {"Yes", "No"};
	    JOptionPane optionPane = new JOptionPane("Clear All Test Selections?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Yes");
	    JDialog dialog = optionPane.createDialog(this, "Clear All");
	    dialog.show();
	    Object selectedValue = optionPane.getValue();
	    
	    if(selectedValue != null) {
		if(((String)selectedValue).compareTo(options[0]) == 0) {
		    session.setProfile(session.getProfile());
		    model.clearUpdate();
		    session.clearTest();
		    session.clearTempTest();
		    session.saveTest();
		    session.loadTest();
		    session.loadTempTest();
		    lblMsgVariant.setText("Number of Message Variants: " + session.countMsgVariants());
		    btnClear.setEnabled(session.countMsgVariants() != 0);
		}
	    }
	}
	else if(evt.getSource() == cbType) {
	    if(model != null) {
		model.setType((short)cbType.getSelectedIndex());
		model.fireTableDataChanged();
		tree.collapseRow(1);
	    }
	}
    }

    /**
     * Handle the event of closing the dialog
     * @param evt The Event
     */
    public void closeDialog(WindowEvent evt) {
	boolean needToSave = false;
	if(model.getUpdateElement().size() > 0) {
	    Object[] options = {"Yes", "No"};
	    JOptionPane optionPane = new JOptionPane("Apply Changes to Test Selections?",JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options, "Yes");
	    JDialog dialog = optionPane.createDialog(this, "Apply Changes and Close");
	    dialog.show();
	    Object selectedValue = optionPane.getValue();
	    
	    if(selectedValue != null && ((String)selectedValue).compareTo(options[0]) == 0) {
		// Save The data
		needToSave = true;
	    }
	}
	if(needToSave) {
	    int oldType = cbType.getSelectedIndex();
	    for(int x=0;x<3;x++) {
		model.setType((short)x);
		Vector vUpdate = model.getUpdateElement();
		for(int i=0;i<vUpdate.size();i++) {
		    ProfileTableModel.UpdateElement ue = (ProfileTableModel.UpdateElement)vUpdate.get(i);
		    if(ue.selected) {
			// Cardinality
			if(ue.column == 1) {
			    switch(x) {
			    case 0:
				session.addTest("cardinalityValid", ue.pn);
				break;
			    case 1:
				session.addTest("cardinalityInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("cardinalityBoth", ue.pn);
				break;
			    }
			}
			// Usage
			else if(ue.column == 2) {
			    switch(x) {
			    case 0:
				session.addTest("usageValid", ue.pn);
				break;
			    case 1:
				session.addTest("usageInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("usageBoth", ue.pn);
				break;
			    }
			}
			// Error Codes
			else if(ue.column == 3) {
			    switch(x) {
			    case 0:
				session.addTest("errorCodesValid", ue.pn);
				break;
			    case 1:
				session.addTest("errorCodesInvalid", ue.pn);
				break;
			    case 2:
				session.addTest("errorCodesBoth", ue.pn);
				break;
			    }			    
			}
		    }
		    else {
			// Cardinality
			if(ue.column == 1) {
			    switch(x) {
			    case 0:
				session.removeTest("cardinalityValid", ue.pn);
				break;
			    case 1:
				session.removeTest("cardinalityInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("cardinalityBoth", ue.pn);
				break;
			    }
			}
			// Usage
			else if(ue.column == 2) {
			    switch(x) {
			    case 0:
				session.removeTest("usageValid", ue.pn);
				break;
			    case 1:
				session.removeTest("usageInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("usageBoth", ue.pn);
				break;
			    }
			}
			// Error Codes
			else if(ue.column == 3) {
			    switch(x) {
			    case 0:
				session.removeTest("errorCodesValid", ue.pn);
				break;
			    case 1:
				session.removeTest("errorCodesInvalid", ue.pn);
				break;
			    case 2:
				session.removeTest("errorCodesBoth", ue.pn);
				break;
			    }			    
			}
		    }
		}
	    }
	    model.setType((short)oldType);
	    session.saveTest();
	    model.clearUpdate();
	    session.setVolume(session.countMsgVariants(), true);
	}
	model.clearUpdate();
	session.clearTest();
	session.clearTempTest();
    }

    /**
     * Get the message control id
     * @return The message control id
     */
   /* public String getMessageControlId() {
	return ((ConfigNode)configModel.getValues("Message Control ID", "MSH", "").get(0)).getValue();
	}*/

    // 
    // The renderer used to display the tree nodes, a JTree.  
    //
	
    public class TreeTableCellRenderer extends JTree implements TableCellRenderer {

	protected int visibleRow;
   
	public TreeTableCellRenderer(TreeModel model) { 
	    super(model);
	    collapseRow(1);
	}

	public void setBounds(int x, int y, int w, int h) {
	    super.setBounds(x, 0, w, ConfigureTestDialog.tableProfile.getHeight());
	}

	public void paint(Graphics g) {
	    g.translate(0, -visibleRow * getRowHeight());
	    super.paint(g);
	}

	public Component getTableCellRendererComponent(JTable table,
						       Object value,
						       boolean isSelected,
						       boolean hasFocus,
						       int row, int column) {
	    if(isSelected)
		setBackground(table.getSelectionBackground());
	    else
		setBackground(table.getBackground());

/*	    if(isSelected)
		setBackground(table.getSelectionBackground());
	    else {
		if(value instanceof Vector) {
		    Vector v = (Vector)value;
		    if(v.size() > 0) {
			if(((String)v.get(1)).endsWith(".SegmentNode"))
			    setBackground(new Color(255, 0, 0));
			else if(((String)v.get(1)).endsWith(".FieldNode"))
			    setBackground(new Color(255, 102, 0));
			else if(((String)v.get(1)).endsWith(".ComponentNode"))
			    setBackground(new Color(255, 255, 0));
			else if(((String)v.get(1)).endsWith(".SubComponentNode"))
			    setBackground(new Color(0, 255, 0));
			else
			    setBackground(Color.WHITE);
		    }
		}
		}*/

/*	    if(value != null)
	    System.out.println(value.getClass());*/
	    
	    visibleRow = row;
	    return this;
	}
    }

    // 
    // The editor used to interact with tree nodes, a JTree.  
    //

    public class TreeTableCellEditor extends AbstractCellEditor implements TableCellEditor {
 	public Component getTableCellEditorComponent(JTable table, Object value,
	 					     boolean isSelected, int r, int c) {
	    return tree;
	}
	
	public Object getCellEditorValue() { return null; }
	public boolean isCellEditable(EventObject e) { return true; }
	public boolean shouldSelectCell(EventObject anEvent) { return false; }
	public boolean stopCellEditing() { return true; }
	public void cancelCellEditing() {}
    }

    //
    // Renderer for the column "Value"
    //

    public class ValueTableCellRenderer extends JCheckBox implements TableCellRenderer {

	public Component getTableCellRendererComponent(JTable table,
						       Object value,
						       boolean isSelected,
						       boolean hasFocus,
						       int row, int column) {

	    setHorizontalAlignment(SwingConstants.CENTER); 
	    if(value instanceof Vector) {
		final Vector v = (Vector)value;
		if(v.size() > 1) {
		    setSelected(((Boolean)v.get(0)).booleanValue());
		    if(((String)v.get(1)).endsWith(".SegmentNode"))
			setBackground(new Color(204, 51, 51));
		    else if(((String)v.get(1)).endsWith(".FieldNode"))
			setBackground(new Color(102, 153, 204));
		    else if(((String)v.get(1)).endsWith(".ComponentNode"))
			setBackground(new Color(153, 204, 153));
		    else if(((String)v.get(1)).endsWith(".SubComponentNode"))
			setBackground(new Color(153, 102, 51));
		    else
			setBackground(Color.WHITE);
		    // Cardinality
		    if(column == 1) {
			setText((String)v.get(2));
			if(cbType.getSelectedIndex() == 0)
			    setToolTipText("Vary the cardinality setting of this element so that it is in the valid cardinality range.");
			else
			    setToolTipText("Vary the cardinality setting of this element so that it is outside the valid cardinality range.");
		    }
		    // Usage
		    else if(column == 2) {
			setText((String)v.get(3));
			String usage = (String)v.get(3);
			if(cbType.getSelectedIndex() == 0) {
			    if(usage.equals("RE"))
				setToolTipText("RE element is populated with data.");
			}
			else {
			    if(usage.equals("R"))
				setToolTipText("Required element is NOT populated with a data value.");
			    else if(usage.equals("X"))
				setToolTipText("Not supported element is populated with a data value.");
			}
		    }
		    // Error Codes
		    else if(column == 3) {
			setText((String)v.get(4));
			if(cbType.getSelectedIndex() == 1)
			    setToolTipText("Value set element is populated with a data value that is NOT in the code set.");
		    }
		}
		else if(v.size() == 1) {
		    return new JLabel("") {
			    public void paint(Graphics g) {
				Color bColor;
				if(((String)v.get(0)).endsWith(".SegmentNode"))
				    bColor = new Color(204, 51, 51);
				else if(((String)v.get(0)).endsWith(".FieldNode"))
				    bColor = new Color(102, 153, 204);
				else if(((String)v.get(0)).endsWith(".ComponentNode"))
				    bColor = new Color(153, 204, 153);
				else if(((String)v.get(0)).endsWith(".SubComponentNode"))
				    bColor = new Color(153, 102, 51);
				else
				    bColor = Color.WHITE;
				g.setColor(bColor);
				// Draw a rectangle in the background of the cell
				g.fillRect(0, 0, getWidth(), getHeight());
				super.paint(g);
			    }
			};
		}
	    }
	    else {
		return new JLabel("");
	    }
	    return this;
	}
    }

    // 
    // Editor for the column "Value"
    //

    public class ValueTableCellEditor extends AbstractCellEditor implements TableCellEditor, ActionListener {
	private JCheckBox chk;
	private int row, column;
	private JTable table;

	public ValueTableCellEditor() {
	    chk = new JCheckBox();
	    chk.setHorizontalAlignment(SwingConstants.CENTER); 
	    chk.setEnabled(true);
	    chk.addActionListener(this);
	}

 	public Component getTableCellEditorComponent(JTable table, Object value,
	 					     boolean isSelected, int r, int c) {
	    this.table = table;
	    row = r;
	    column = c;
	    if(value instanceof Vector) {
		final Vector v = (Vector)value;
		if(v.size() > 1) {
		    chk.setSelected(((Boolean)v.get(0)).booleanValue());
		    if(((String)v.get(1)).endsWith(".SegmentNode"))
			chk.setBackground(new Color(204, 51, 51));
		    else if(((String)v.get(1)).endsWith(".FieldNode"))
			chk.setBackground(new Color(102, 153, 204));
		    else if(((String)v.get(1)).endsWith(".ComponentNode"))
			chk.setBackground(new Color(153, 204, 153));
		    else if(((String)v.get(1)).endsWith(".SubComponentNode"))
			chk.setBackground(new Color(153, 102, 51));
		    else
			chk.setBackground(Color.WHITE);
		    // Cardinality
		    if(c == 1) {
			chk.setText((String)v.get(2));
			if(cbType.getSelectedIndex() == 0)
			    chk.setToolTipText("Vary the cardinality setting of this element so that it is in the valid cardinality range.");
			else
			    chk.setToolTipText("Vary the cardinality setting of this element so that it is outside the valid cardinality range.");
		    }
		    // Usage
		    else if(c == 2) {
			chk.setText((String)v.get(3));
			String usage = (String)v.get(3);
			if(cbType.getSelectedIndex() == 0) {
			    if(usage.equals("RE"))
				chk.setToolTipText("RE element is populated with data.");
			}
			else {
			    if(usage.equals("R"))
				chk.setToolTipText("Required element is NOT populated with a data value.");
			    else if(usage.equals("X"))
				chk.setToolTipText("Not supported element is populated with a data value.");
			}
		    }
		    // Error Codes
		    else if(c == 3) {
			chk.setText((String)v.get(4));
			if(cbType.getSelectedIndex() == 1)
			    chk.setToolTipText("Value set element is populated with a data value that is NOT in the code set.");
		    }
		    return chk;
		}
		else if(v.size() == 1) {
		    return new JLabel("") {
			    public void paint(Graphics g) {
				Color bColor;
				if(((String)v.get(0)).endsWith(".SegmentNode"))
				    bColor = new Color(204, 51, 51);
				else if(((String)v.get(0)).endsWith(".FieldNode"))
				    bColor = new Color(102, 153, 204);
				else if(((String)v.get(0)).endsWith(".ComponentNode"))
				    bColor = new Color(153, 204, 153);
				else if(((String)v.get(0)).endsWith(".SubComponentNode"))
				    bColor = new Color(153, 102, 51);
				else
				    bColor = Color.WHITE;
				g.setColor(bColor);
				// Draw a rectangle in the background of the cell
				g.fillRect(0, 0, getWidth(), getHeight());
				super.paint(g);
			    }
			};
		}
		
	    }
	    JLabel lblOther = new JLabel(" ") {
		    public void paint(Graphics g) {
			Color bColor;
			g.setColor(Color.WHITE);
			// Draw a rectangle in the background of the cell
			g.fillRect(0, 0, getWidth(), getHeight());
			super.paint(g);
		    }
		};
	    return lblOther;	    
	}

	public Object getCellEditorValue() { return null; }
	public boolean isCellEditable(EventObject e) { return true; }
	public boolean shouldSelectCell(EventObject anEvent) { return false; }

	public boolean stopCellEditing() {
	    fireEditingStopped();
	    return true;
	}
	
	public void cancelCellEditing() {
	    fireEditingCanceled();
	}

	public void addCellEditorListener(CellEditorListener l) {
	    listenerList.add(CellEditorListener.class, l);
	}
	
	public void removeCellEditorListener(CellEditorListener l) {
	    listenerList.remove(CellEditorListener.class, l);
	}

	/**
	 * Handles event
	 * @param evt The Event
	 */
	public void actionPerformed(ActionEvent evt) {
	    if(evt.getSource() == chk) {
		if(table != null) {
		    table.setValueAt(new Boolean(chk.isSelected()), row, column);
		    // Count the number of message variants
		    int oldType = cbType.getSelectedIndex();
		    for(int x=0;x<3;x++) {
			model.setType((short)x);
			Vector vUpdate = model.getUpdateElement();
			for(int i=0;i<vUpdate.size();i++) {
			    ProfileTableModel.UpdateElement ue = (ProfileTableModel.UpdateElement)vUpdate.get(i);
			    if(ue.selected) {
				// Cardinality
				if(ue.column == 1) {
				    switch(x) {
				    case 0:
					session.addTempTest("cardinalityValid", ue.pn);
					break;
				    case 1:
					session.addTempTest("cardinalityInvalid", ue.pn);
					break;
				    case 2:
					session.addTempTest("cardinalityBoth", ue.pn);
					break;
				    }
				}
				// Usage
				else if(ue.column == 2) {
				    switch(x) {
				    case 0:
					session.addTempTest("usageValid", ue.pn);
					break;
				    case 1:
					session.addTempTest("usageInvalid", ue.pn);
					break;
				    case 2:
					session.addTempTest("usageBoth", ue.pn);
					break;
				    }
				}
				// Error Codes
				else if(ue.column == 3) {
				    switch(x) {
				    case 0:
					session.addTempTest("errorCodesValid", ue.pn);
					break;
				    case 1:
					session.addTempTest("errorCodesInvalid", ue.pn);
					break;
				    case 2:
					session.addTempTest("errorCodesBoth", ue.pn);
					break;
				    }			    
				}
			    }
			    else {
				// Cardinality
				if(ue.column == 1) {
				    switch(x) {
				    case 0:
					session.removeTempTest("cardinalityValid", ue.pn);
					break;
				    case 1:
					session.removeTempTest("cardinalityInvalid", ue.pn);
					break;
				    case 2:
					session.removeTempTest("cardinalityBoth", ue.pn);
					break;
				    }
				}
				// Usage
				else if(ue.column == 2) {
				    switch(x) {
				    case 0:
					session.removeTempTest("usageValid", ue.pn);
					break;
				    case 1:
					session.removeTempTest("usageInvalid", ue.pn);
					break;
				    case 2:
					session.removeTempTest("usageBoth", ue.pn);
					break;
				    }
				}
				// Error Codes
				else if(ue.column == 3) {
				    switch(x) {
				    case 0:
					session.removeTempTest("errorCodesValid", ue.pn);
					break;
				    case 1:
					session.removeTempTest("errorCodesInvalid", ue.pn);
					break;
				    case 2:
					session.removeTempTest("errorCodesBoth", ue.pn);
					break;
				    }			    
				}
			    }
			}
		    }
		    model.setType((short)oldType);
		    lblMsgVariant.setText("Number of Message Variants: " + session.countTempMsgVariants());
		    btnClear.setEnabled(session.countTempMsgVariants() != 0);
		    //session.saveTest();
		    //model.clearUpdate();
		}
	    }
	}
    }
}
