/*
 * JPanel.java
 *
 * Created on June 15, 2004, 9:58 AM
 */

package gov.nist.hl7maker.view.init;
                                                                                

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Font;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSeparator;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JDialog;
import javax.swing.JCheckBox;
import javax.swing.ButtonGroup;
import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.border.EtchedBorder;
import javax.swing.text.PlainDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.AttributeSet;

// XML
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.ParserConfigurationException;

// DOM
import org.w3c.dom.Document;

// SAX
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

// Xerces
import org.apache.xerces.parsers.SAXParser;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.MMKFileFilter;
import gov.nist.hl7maker.util.WaitDialog;

/**
 *
 * @author  Rob Snelick
 *
 * Most of the GUI widgets have been created with the netbeans IDE. This code
 * uses the netbeans awtextra library for the absolute layout functionality.
 * This package is included with the distribution of the source code.
 * Modifications to the GUI should be made with the netbeans IDE.
 *
 */

/*
 * TODO: capture values of parameters when the user switches from this folder. For
 *       example, volume maybe set, but will not be recorded unless the user presses
 *       return on the UI. Volume maybe the only one. 
 */

public class MessageSetupPanel extends JPanel implements Observer {
    private Session session;
    private User user;
    private boolean visible;

    private JLabel heading1Label;
    private JSeparator jSeparator1;

    private JLabel profileLabel;
    private JTextField profileTextField;
    private JButton profileButton;
    private JButton profileClearButton;
    private JLabel nameLabel;
    private JTextField nameTextField;
    private JButton nameButton;
    private JButton nameClearButton;
    private JLabel pathLabel;
    private JTextField pathTextField;
    private JButton pathButton;
    private JButton pathClearButton;

    private JLabel heading2Label;
    private JSeparator jSeparator2;

    private JRadioButton rdValid;
    private JRadioButton rdInvalid;
    private JRadioButton rdBoth;
    private JRadioButton rdSampleRandom;
    private JRadioButton rdSampleMin;
    private JRadioButton rdSampleMax;
    private JRadioButton rdCustom;
    private JRadioButton rdManual;

    private JRadioButton rdIgnoreLength;
    private JRadioButton rdEnforceLength;

    private JLabel volumeLabel;
    private JTextField volumeTextField;
    private JButton btnOptions;
    private JButton btnManual;

    private JRadioButton rdUsageValid;
    private JRadioButton rdUsageInvalid;

    private JRadioButton rdCardinalityValidMin;
    private JRadioButton rdCardinalityValidRange;
    private JRadioButton rdCardinalityValidMax;
    private JRadioButton rdCardinalityInvalid;

    private JRadioButton rdCodedEntryValid;
    private JRadioButton rdCodedEntryInvalid;

    private JRadioButton rdLengthValid;
    private JRadioButton rdLengthIgnore;
    private JRadioButton rdLengthInvalid;

    private JComboBox usageTypeComboBox;
    private JCheckBox notSupportedUsageCheckBox;
    private JCheckBox requiredUsageCheckBox;
    private JCheckBox requiredEmptyUsageCheckBox;
    private JCheckBox conditionalUsageCheckBox;
    private JCheckBox conditionalEmptyUsageCheckBox;

    private JComboBox cardinalityTypeComboBox;
    private JCheckBox fieldCardinalityCheckBox;
    private JCheckBox segmentCardinalityCheckBox;

    private JLabel lblCodedEntryType;
    private JComboBox codedEntryTypeComboBox;

    private JComboBox lengthTypeComboBox;
    private JCheckBox compositeLengthCheckBox;
    private JCheckBox fieldLengthCheckBox;
    private JCheckBox componentLengthCheckBox;
    private JCheckBox subComponentLengthCheckBox;

    private JLabel lblContentComplexity;
    private JLabel lblContentComplexity2;
    //    private JComboBox contentTypeComboBox;
    private JComboBox contentComplexityComboBox;
    private JCheckBox dataTypeCheckBox;
    private JCheckBox outOfRangeCheckBox;
    private JCheckBox formatCheckBox;

    private JLabel lblHelp;

    private JPanel pFull;    
    private JPanel pEmpty;
    
    private JSeparator jSeparator3;
    private JButton resetDefaultsButton;

    private final int VOLUME_DEFAULT = 10;
    private int currentVolume = VOLUME_DEFAULT;

    private ConfigureTestDialog dlg;

    private static final String HL7_SCHEMA = "data/HL7RegistrationSchema.xsd";
    private static final String LOOSE_SCHEMA = "data/LooseHL7RegistrationSchema.xsd";
    private Validator handler;

    /** 
     * Creates new form JPanel 
     * @param ses A Session object
     */
    public MessageSetupPanel(Session ses, User user) {
	session = ses;      
	this.user = user;
	visible = false;
	// MVC
	session.addObserver(this);

	initComponents();
    }
    
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	setAlignmentX(Component.LEFT_ALIGNMENT);

	// Heading Pane

	JPanel heading1Pane = new JPanel();
	heading1Pane.setLayout(new BoxLayout(heading1Pane, BoxLayout.PAGE_AXIS));

	JPanel ph1 = new JPanel();
	ph1.setLayout(new BoxLayout(ph1, BoxLayout.LINE_AXIS));
	ph1.setAlignmentX(Component.LEFT_ALIGNMENT);

        heading1Label = new JLabel();
        heading1Label.setFont(new Font("SansSerif", 3, 14));
        heading1Label.setText("Test Message Set Initialization");
	ph1.add(Box.createRigidArea(new Dimension(100, 0)));
	ph1.add(heading1Label);
	heading1Pane.add(ph1);

	JPanel ph2 = new JPanel();
	ph2.setLayout(new BoxLayout(ph2, BoxLayout.LINE_AXIS));
	ph2.setAlignmentX(Component.LEFT_ALIGNMENT);

        jSeparator1 = new JSeparator();
	jSeparator1.setMaximumSize(new Dimension(665, 10));
	ph2.add(Box.createRigidArea(new Dimension(100, 0)));
	ph2.add(jSeparator1);
	heading1Pane.add(ph2);

	add(Box.createRigidArea(new Dimension(0, 10)));
	add(heading1Pane);

	// Profile Pane

	JPanel profilePane = new JPanel();
	profilePane.setLayout(new BoxLayout(profilePane, BoxLayout.LINE_AXIS));
	profilePane.setAlignmentX(Component.LEFT_ALIGNMENT);

	profilePane.add(Box.createRigidArea(new Dimension(146, 0)));

        profileLabel = new JLabel();
        profileLabel.setText("Profile: ");
        profileLabel.setToolTipText("Profile used in message generation");
	profilePane.add(profileLabel);

	profilePane.add(Box.createRigidArea(new Dimension(5, 0)));

        profileTextField = new JTextField();
	profileTextField.setEditable(false);
	profileTextField.setBackground(Color.WHITE);
        profileTextField.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    profileTextFieldActionPerformed(evt);
		}
	    });
	profilePane.add(profileTextField);

	profilePane.add(Box.createRigidArea(new Dimension(5, 0)));

        profileButton = new JButton();
        profileButton.setBackground(new Color(204, 204, 255));
        profileButton.setText("Browse ...");
        profileButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    profileButtonActionPerformed(evt);
		}
	    });
	profilePane.add(profileButton);

	profilePane.add(Box.createRigidArea(new Dimension(5, 0)));

        profileClearButton = new JButton();
        profileClearButton.setBackground(new Color(204, 204, 255));
        profileClearButton.setText("Clear");
        profileClearButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    //profileClearButtonActionPerformed(evt);
		    profileTextField.setText("");
		    session.setProfile("");
		}
	    });
	//profilePane.add(profileClearButton);

	profilePane.add(Box.createRigidArea(new Dimension(100, 0)));

	add(Box.createRigidArea(new Dimension(0, 10)));
	add(profilePane);

	// Session Name Pane

	JPanel namePane = new JPanel();
	namePane.setLayout(new BoxLayout(namePane, BoxLayout.LINE_AXIS));
	namePane.setAlignmentX(Component.LEFT_ALIGNMENT);

	namePane.add(Box.createRigidArea(new Dimension(101, 0)));

        nameLabel = new JLabel();
        nameLabel.setText("Session Name: ");
        nameLabel.setToolTipText("Name of the current session");
	namePane.add(nameLabel);

	namePane.add(Box.createRigidArea(new Dimension(5, 0)));

        nameTextField = new JTextField();
        nameTextField.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    nameTextFieldActionPerformed(evt);
		}
	    });
	namePane.add(nameTextField);

	namePane.add(Box.createRigidArea(new Dimension(5, 0)));

        nameButton = new JButton();
        nameButton.setBackground(new Color(204, 204, 255));
        nameButton.setText("Browse ...");
        nameButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    nameButtonActionPerformed(evt);
		}
	    });
	namePane.add(nameButton);

	namePane.add(Box.createRigidArea(new Dimension(5, 0)));

        nameClearButton = new JButton();
        nameClearButton.setBackground(new Color(204, 204, 255));
        nameClearButton.setText("Clear");
        nameClearButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    //nameButtonActionPerformed(evt);
		    nameTextField.setText("");
		}
	    });
	namePane.add(nameClearButton);

	namePane.add(Box.createRigidArea(new Dimension(100, 0)));

	add(Box.createRigidArea(new Dimension(0, 10)));
	//add(namePane);

	// Message Path Pane

	JPanel pathPane = new JPanel();
	pathPane.setLayout(new BoxLayout(pathPane, BoxLayout.LINE_AXIS));
	pathPane.setAlignmentX(Component.LEFT_ALIGNMENT);

	pathPane.add(Box.createRigidArea(new Dimension(102, 0)));

        pathLabel = new JLabel();
        pathLabel.setText("Message Path: ");
        pathLabel.setToolTipText("Directory location for messages");
	pathPane.add(pathLabel);

	pathPane.add(Box.createRigidArea(new Dimension(5, 0)));

        pathTextField = new JTextField();
	pathTextField.setEditable(false);
	pathTextField.setBackground(Color.WHITE);
        pathTextField.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    pathTextFieldActionPerformed(evt);
		}
	    });
	pathPane.add(pathTextField);

	pathPane.add(Box.createRigidArea(new Dimension(5, 0)));

        pathButton = new JButton();
        pathButton.setBackground(new Color(204, 204, 255));
        pathButton.setText("Browse ...");
        pathButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    pathButtonActionPerformed(evt);
		}
	    });
	pathPane.add(pathButton);

	pathPane.add(Box.createRigidArea(new Dimension(5, 0)));

        pathClearButton = new JButton();
        pathClearButton.setBackground(new Color(204, 204, 255));
        pathClearButton.setText("Clear");
        pathClearButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    //pathButtonActionPerformed(evt);
		    pathTextField.setText("");
		}
	    });
	//pathPane.add(pathClearButton);

	pathPane.add(Box.createRigidArea(new Dimension(100, 0)));

	add(Box.createRigidArea(new Dimension(0, 10)));
	add(pathPane);

	// Heading Pane

	JPanel heading2Pane = new JPanel();
	heading2Pane.setLayout(new BoxLayout(heading2Pane, BoxLayout.PAGE_AXIS));

	JPanel ph3 = new JPanel();
	ph3.setLayout(new BoxLayout(ph3, BoxLayout.LINE_AXIS));
	ph3.setAlignmentX(Component.LEFT_ALIGNMENT);

        heading2Label = new JLabel();
	heading2Label.setFont(new Font("SansSerif", 3, 14));
        heading2Label.setText("Test Message Options");
	ph3.add(Box.createRigidArea(new Dimension(100, 0)));
	ph3.add(heading2Label);
	heading2Pane.add(ph3);

	JPanel ph4 = new JPanel();
	ph4.setLayout(new BoxLayout(ph4, BoxLayout.LINE_AXIS));
	ph4.setAlignmentX(Component.LEFT_ALIGNMENT);

        jSeparator2 = new JSeparator();
	jSeparator2.setMaximumSize(new Dimension(665, 10));
	ph4.add(Box.createRigidArea(new Dimension(100, 0)));
	ph4.add(jSeparator2);
	heading2Pane.add(ph4);

	add(Box.createRigidArea(new Dimension(0, 25)));
	add(heading2Pane);

	// Volume
	JPanel testMsgVolumePane = new JPanel();
	testMsgVolumePane.setLayout(new BoxLayout(testMsgVolumePane, BoxLayout.LINE_AXIS));
	testMsgVolumePane.setAlignmentX(Component.LEFT_ALIGNMENT);

	testMsgVolumePane.add(Box.createRigidArea(new Dimension(133, 0)));

        volumeLabel = new JLabel();
        volumeLabel.setText(" Volume: ");
        volumeLabel.setToolTipText("Number of messages to be generated");
	testMsgVolumePane.add(volumeLabel);

	testMsgVolumePane.add(Box.createRigidArea(new Dimension(5, 0)));
	
        volumeTextField = new JTextField();
	volumeTextField.setMaximumSize(new Dimension(40, 30));
	volumeTextField.setDocument(new VolumeDocument());
	testMsgVolumePane.add(volumeTextField);

	//testMsgVolumePane.add(Box.createRigidArea(new Dimension(503, 0)));

	add(Box.createRigidArea(new Dimension(0, 10)));
	add(testMsgVolumePane);

	JPanel ph6 = new JPanel();
	ph6.setLayout(new BoxLayout(ph6, BoxLayout.LINE_AXIS));
	ph6.setAlignmentX(Component.LEFT_ALIGNMENT);

        jSeparator3 = new JSeparator();
	jSeparator3.setMaximumSize(new Dimension(635, 10));
	ph6.add(Box.createRigidArea(new Dimension(120, 0)));
	ph6.add(jSeparator3);
	heading2Pane.add(ph6);

	add(Box.createRigidArea(new Dimension(0, 5)));
	add(ph6);

	// Radio Button Pane

	JPanel testMsgRDPane = new JPanel();
	testMsgRDPane.setLayout(new BoxLayout(testMsgRDPane, BoxLayout.LINE_AXIS));
	testMsgRDPane.setAlignmentX(Component.LEFT_ALIGNMENT);

	testMsgRDPane.add(Box.createRigidArea(new Dimension(0, 0)));

	JPanel p30 = new JPanel();
	p30.setLayout(new BoxLayout(p30, BoxLayout.PAGE_AXIS));
	p30.setAlignmentX(Component.LEFT_ALIGNMENT);
	rdValid = new JRadioButton("Create a set of valid test messages");
	rdValid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdInvalid = new JRadioButton("Create a set of invalid test messages");
	rdInvalid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdBoth = new JRadioButton("Create a set of both valid and invalid test messages");
	rdBoth.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	p30.add(rdValid);
	p30.add(rdInvalid);
	p30.add(rdBoth);
	testMsgRDPane.add(p30);

	testMsgRDPane.add(Box.createRigidArea(new Dimension(115, 0)));

	JPanel p31 = new JPanel();
	p31.setLayout(new BoxLayout(p31, BoxLayout.PAGE_AXIS));
	p31.setAlignmentX(Component.LEFT_ALIGNMENT);
	rdSampleRandom = new JRadioButton("Create a sample message (randomly populated)");
	rdSampleRandom.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdSampleMin = new JRadioButton("Create a sample message (minimally populated)");
	rdSampleMin.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdSampleMax = new JRadioButton("Create a sample message (maximally populated)");
	rdSampleMax.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	p31.add(rdSampleRandom);
	p31.add(rdSampleMin);
	p31.add(rdSampleMax);
	testMsgRDPane.add(p31);

	ButtonGroup grpRD = new ButtonGroup();
	grpRD.add(rdValid);
	grpRD.add(rdInvalid);
	grpRD.add(rdBoth);
	grpRD.add(rdSampleRandom);
	grpRD.add(rdSampleMin);
	grpRD.add(rdSampleMax);
	testMsgRDPane.add(p30);
	testMsgRDPane.add(Box.createRigidArea(new Dimension(25, 0)));
	testMsgRDPane.add(p31);

	testMsgRDPane.add(Box.createRigidArea(new Dimension(115, 0)));

	add(Box.createRigidArea(new Dimension(0, 5)));
	add(testMsgRDPane);

	// Length global settings

	JPanel lengthSettingPane = new JPanel();
	lengthSettingPane.setLayout(new BoxLayout(lengthSettingPane, BoxLayout.LINE_AXIS));
	lengthSettingPane.setAlignmentX(Component.LEFT_ALIGNMENT);
	lengthSettingPane.add(Box.createRigidArea(new Dimension(115, 0)));

	JPanel p51 = new JPanel();
	p51.setLayout(new BoxLayout(p51, BoxLayout.LINE_AXIS));
	p51.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "When generating messages"));

	rdIgnoreLength = new JRadioButton("Ignore length constraints");
	rdIgnoreLength.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	p51.add(rdIgnoreLength);
	p51.add(Box.createRigidArea(new Dimension(5, 0)));
	rdEnforceLength = new JRadioButton("Enforce length constraints");
	rdEnforceLength.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	p51.add(rdEnforceLength);

	ButtonGroup grpLengthRD = new ButtonGroup();
	grpLengthRD.add(rdIgnoreLength);
	grpLengthRD.add(rdEnforceLength);

	lengthSettingPane.add(p51);

	add(Box.createRigidArea(new Dimension(0, 5)));
	add(lengthSettingPane);

	// Options Pane

	JPanel ph7 = new JPanel();
	ph7.setLayout(new BoxLayout(ph7, BoxLayout.LINE_AXIS));
	ph7.setAlignmentX(Component.LEFT_ALIGNMENT);

        JSeparator jSeparator4 = new JSeparator();
	jSeparator4.setMaximumSize(new Dimension(635, 10));
	ph7.add(Box.createRigidArea(new Dimension(120, 0)));
	ph7.add(jSeparator4);
	heading2Pane.add(ph7);

	add(Box.createRigidArea(new Dimension(0, 5)));
	add(ph7);

	JPanel testMsgOptPane = new JPanel();
	testMsgOptPane.setLayout(new BoxLayout(testMsgOptPane, BoxLayout.LINE_AXIS));
	testMsgOptPane.setAlignmentX(Component.LEFT_ALIGNMENT);

	testMsgOptPane.add(Box.createRigidArea(new Dimension(115, 0)));

	rdCustom = new JRadioButton("Customize Settings: ");
	grpRD.add(rdCustom);
	rdCustom.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	testMsgOptPane.add(rdCustom);

	testMsgOptPane.add(Box.createRigidArea(new Dimension(5, 0)));

	btnOptions = new JButton();
	btnOptions.setBackground(new Color(204, 204, 255));
	btnOptions.setText("More Options >>>");
	btnOptions.setEnabled(false);
        btnOptions.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    btnOptionsActionPerformed(evt);
		}
	    });
	testMsgOptPane.add(btnOptions);

	testMsgOptPane.add(Box.createRigidArea(new Dimension(62, 0)));

	rdManual = new JRadioButton("Manual test selection: ");
	grpRD.add(rdManual);
	rdManual.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	testMsgOptPane.add(rdManual);

	testMsgOptPane.add(Box.createRigidArea(new Dimension(5, 0)));

	btnManual = new JButton("Set");
	btnManual.setBackground(new Color(204, 204, 255));
	btnManual.setEnabled(false);
	btnManual.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if(!session.getProfile().equals("")) {
			if(dlg == null)
			    dlg = new ConfigureTestDialog(null);//new javax.swing.JFrame());
			dlg.setSession(session);
			session.addObserver(dlg);
			session.setProfile(session.getProfile());
			session.loadTest();
			session.loadTempTest();
			dlg.pack();
			// Center the window
			Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
			Dimension windowSize = dlg.getSize();
			dlg.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
					Math.max(0,(screenSize.height-windowSize.height)/2));
			dlg.setVisible(true);
		    }
		    else {
			JOptionPane.showMessageDialog(null, "You have to select a profile to use this feature", "No profile error", JOptionPane.ERROR_MESSAGE);
		    }

		}
	    });
	testMsgOptPane.add(btnManual);

	add(Box.createRigidArea(new Dimension(0, 5)));
	add(testMsgOptPane);

	// Cardinality

	JPanel cardinalityPane = new JPanel();
	cardinalityPane.setLayout(new BoxLayout(cardinalityPane, BoxLayout.PAGE_AXIS));
	cardinalityPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Cardinality"));


	JPanel px4 = new JPanel();
	px4.setLayout(new BoxLayout(px4, BoxLayout.PAGE_AXIS));
	rdCardinalityValidMin = new JRadioButton("Valid (Minimally populated)");
	rdCardinalityValidMin.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px4.add(rdCardinalityValidMin);
	rdCardinalityValidRange = new JRadioButton("Valid (In cardinality range)");
	rdCardinalityValidRange.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px4.add(rdCardinalityValidRange);
	rdCardinalityValidMax = new JRadioButton("Valid (Maximally populated)");
	rdCardinalityValidMax.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px4.add(rdCardinalityValidMax);
	rdCardinalityInvalid = new JRadioButton("Invalid (Some outside of cardinality range)");
	rdCardinalityInvalid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px4.add(rdCardinalityInvalid);
	ButtonGroup grpCardinality = new ButtonGroup();
	grpCardinality.add(rdCardinalityValidMin);
	grpCardinality.add(rdCardinalityValidRange);
	grpCardinality.add(rdCardinalityValidMax);
	grpCardinality.add(rdCardinalityInvalid);

	cardinalityPane.add(px4);
/*
	JPanel p20 = new JPanel();
	p20.setLayout(new BoxLayout(p20, BoxLayout.LINE_AXIS));
	JLabel lblCardinalityType = new JLabel(" Type: ");
	p20.add(lblCardinalityType);
	cardinalityTypeComboBox = new JComboBox();
	cardinalityTypeComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    typeComboBoxActionPerformed(evt);
		}
	    });        cardinalityTypeComboBox.setBackground(new Color(204, 204, 255));
	cardinalityTypeComboBox.addItem("Valid");
	cardinalityTypeComboBox.addItem("Invalid");
	p20.add(cardinalityTypeComboBox);
	p20.setAlignmentX(Component.LEFT_ALIGNMENT);
	//cardinalityPane.add(p20);

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
        segmentCardinalityCheckBox = new JCheckBox();
        segmentCardinalityCheckBox.setEnabled(false);
        segmentCardinalityCheckBox.setText("Segment");
        segmentCardinalityCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    cardinalityCheckBoxActionPerformed(evt);
		}
	    });
	segmentCardinalityCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p1.add(Box.createRigidArea(new Dimension(33, 0)));
	p1.add(segmentCardinalityCheckBox);
	p1.setAlignmentX(Component.LEFT_ALIGNMENT);
	//cardinalityPane.add(p1);
	
	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
        fieldCardinalityCheckBox = new JCheckBox();
        fieldCardinalityCheckBox.setEnabled(false);
        fieldCardinalityCheckBox.setText("Field");
        fieldCardinalityCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    cardinalityCheckBoxActionPerformed(evt);
		}
	    });
	fieldCardinalityCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p2.add(Box.createRigidArea(new Dimension(33, 0)));
	p2.add(fieldCardinalityCheckBox);
	p2.setAlignmentX(Component.LEFT_ALIGNMENT);
	//cardinalityPane.add(p2);
	*/
	// Usage
	
	JPanel usagePane = new JPanel();
	usagePane.setLayout(new BoxLayout(usagePane, BoxLayout.PAGE_AXIS));
	usagePane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Usage"));

	JPanel px1 = new JPanel();
	px1.setLayout(new BoxLayout(px1, BoxLayout.PAGE_AXIS));

	rdUsageValid = new JRadioButton("Valid                                                                      ");
	rdUsageValid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px1.add(rdUsageValid);
	rdUsageInvalid = new JRadioButton("Invalid");
	rdUsageInvalid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px1.add(rdUsageInvalid);
	ButtonGroup grpUsage = new ButtonGroup();
	grpUsage.add(rdUsageValid);
	grpUsage.add(rdUsageInvalid);

	usagePane.add(px1);

/*

	JPanel p21 = new JPanel();
	p21.setLayout(new BoxLayout(p21, BoxLayout.LINE_AXIS));
	JLabel lblUsageType = new JLabel(" Type: ");
	p21.add(lblUsageType);
	usageTypeComboBox = new JComboBox();
	usageTypeComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    typeComboBoxActionPerformed(evt);
		}
	    });        usageTypeComboBox.setBackground(new Color(204, 204, 255));
	usageTypeComboBox.addItem("Valid");
	usageTypeComboBox.addItem("Invalid");
	p21.add(usageTypeComboBox);
	p21.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p21);

	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
        requiredUsageCheckBox = new JCheckBox();
        requiredUsageCheckBox.setEnabled(false);
        requiredUsageCheckBox.setText("Required");
        requiredUsageCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    usageCheckBoxActionPerformed(evt);
		}
	    });
	requiredUsageCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p3.add(Box.createRigidArea(new Dimension(33, 0)));
	p3.add(requiredUsageCheckBox);
	p3.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p3);

	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
        requiredEmptyUsageCheckBox = new JCheckBox();
        requiredEmptyUsageCheckBox.setEnabled(false);
        requiredEmptyUsageCheckBox.setText("Required or Empty");
        requiredEmptyUsageCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    usageCheckBoxActionPerformed(evt);
		}
	    });
	requiredEmptyUsageCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p4.add(Box.createRigidArea(new Dimension(33, 0)));
	p4.add(requiredEmptyUsageCheckBox);
	p4.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p4);

	JPanel p5 = new JPanel();
	p5.setLayout(new BoxLayout(p5, BoxLayout.LINE_AXIS));
        conditionalUsageCheckBox = new JCheckBox();
        conditionalUsageCheckBox.setText("Conditional");
        conditionalUsageCheckBox.setEnabled(false);
        conditionalUsageCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    usageCheckBoxActionPerformed(evt);
		}
	    });
	conditionalUsageCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p5.add(Box.createRigidArea(new Dimension(33, 0)));
	p5.add(conditionalUsageCheckBox);
	p5.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p5);

	JPanel p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
        conditionalEmptyUsageCheckBox = new JCheckBox();
        conditionalEmptyUsageCheckBox.setText("Conditional or Empty");
        conditionalEmptyUsageCheckBox.setEnabled(false);
        conditionalEmptyUsageCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    usageCheckBoxActionPerformed(evt);
		}
	    });
	conditionalEmptyUsageCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p6.add(Box.createRigidArea(new Dimension(33, 0)));
	p6.add(conditionalEmptyUsageCheckBox);
	p6.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p6);

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.LINE_AXIS));
        notSupportedUsageCheckBox = new JCheckBox();
        notSupportedUsageCheckBox.setEnabled(false);
        notSupportedUsageCheckBox.setText("Not Supported");
        notSupportedUsageCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    usageCheckBoxActionPerformed(evt);
		}
	    });
	notSupportedUsageCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p7.add(Box.createRigidArea(new Dimension(33, 0)));
	p7.add(notSupportedUsageCheckBox);
	p7.setAlignmentX(Component.LEFT_ALIGNMENT);
	//usagePane.add(p7);
	*/
	// Coded Entry

	JPanel codedEntryPane = new JPanel();
	codedEntryPane.setLayout(new BoxLayout(codedEntryPane, BoxLayout.PAGE_AXIS));
	codedEntryPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Coded Entry"));

	JPanel px2 = new JPanel();
	px2.setLayout(new BoxLayout(px2, BoxLayout.PAGE_AXIS));

	rdCodedEntryValid = new JRadioButton("Valid (In code set)");
	rdCodedEntryValid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdCodedEntryValid.setEnabled(false);
	px2.add(rdCodedEntryValid);
	rdCodedEntryInvalid = new JRadioButton("Invalid (Some not in code set)        ");
	rdCodedEntryInvalid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	rdCodedEntryInvalid.setEnabled(false);
	px2.add(rdCodedEntryInvalid);
	ButtonGroup grpCodedEntry = new ButtonGroup();
	grpCodedEntry.add(rdCodedEntryValid);
	grpCodedEntry.add(rdCodedEntryInvalid);

	codedEntryPane.add(px2);

/*	JPanel p22 = new JPanel();
	p22.setLayout(new BoxLayout(p22, BoxLayout.LINE_AXIS));
	lblCodedEntryType = new JLabel(" Type: ");
	lblCodedEntryType.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p22.add(lblCodedEntryType);
	codedEntryTypeComboBox = new JComboBox();
	codedEntryTypeComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    typeComboBoxActionPerformed(evt);
		}
	    });
	codedEntryTypeComboBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
        codedEntryTypeComboBox.setBackground(new Color(204, 204, 255));
	codedEntryTypeComboBox.addItem("Valid");
	codedEntryTypeComboBox.addItem("Invalid");
	codedEntryTypeComboBox.setEnabled(false);
	p22.add(codedEntryTypeComboBox);
	p22.setAlignmentX(Component.LEFT_ALIGNMENT);
	//codedEntryPane.add(p22);
	//codedEntryPane.add(Box.createRigidArea(new Dimension(0, 47)));
	*/
	// Length

	JPanel lengthPane = new JPanel();
	lengthPane.setLayout(new BoxLayout(lengthPane, BoxLayout.PAGE_AXIS));
	lengthPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Length"));

	JPanel px3 = new JPanel();
	px3.setLayout(new BoxLayout(px3, BoxLayout.PAGE_AXIS));

	rdLengthValid = new JRadioButton("Valid (ensure valid length values)");
	rdLengthValid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px3.add(rdLengthValid);
	rdLengthIgnore = new JRadioButton("Ignore (disregard length attribute)");
	rdLengthIgnore.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px3.add(rdLengthIgnore);
	rdLengthInvalid = new JRadioButton("Invalid");
	rdLengthInvalid.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    rdActionPerformed(evt);
		}
	    });
	px3.add(rdLengthInvalid);
	ButtonGroup grpLength = new ButtonGroup();
	grpLength.add(rdLengthValid);
	grpLength.add(rdLengthIgnore);
	grpLength.add(rdLengthInvalid);

	lengthPane.add(px3);
	lengthPane.add(Box.createRigidArea(new Dimension(0, 23)));


/*
	JPanel p23 = new JPanel();
	p23.setLayout(new BoxLayout(p23, BoxLayout.LINE_AXIS));
	JLabel lblLengthType = new JLabel(" Type: ");
	p23.add(lblLengthType);
	lengthTypeComboBox = new JComboBox();
	lengthTypeComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    typeComboBoxActionPerformed(evt);
		}
	    });
        lengthTypeComboBox.setBackground(new Color(204, 204, 255));
	lengthTypeComboBox.addItem("Valid");
	lengthTypeComboBox.addItem("Ignore");
	p23.add(lengthTypeComboBox);
	p23.setAlignmentX(Component.LEFT_ALIGNMENT);
	//lengthPane.add(p23);

	JPanel p8bis = new JPanel();
	p8bis.setLayout(new BoxLayout(p8bis, BoxLayout.LINE_AXIS));
        compositeLengthCheckBox = new JCheckBox();
        compositeLengthCheckBox.setText("Composite");
	compositeLengthCheckBox.setEnabled(false);
        compositeLengthCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    lengthCheckBoxActionPerformed(evt);
		}
	    });
	compositeLengthCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p8bis.add(Box.createRigidArea(new Dimension(33, 0)));
	p8bis.add(compositeLengthCheckBox);
	p8bis.setAlignmentX(Component.LEFT_ALIGNMENT);
	//lengthPane.add(p8bis);

	JPanel p8 = new JPanel();
	p8.setLayout(new BoxLayout(p8, BoxLayout.LINE_AXIS));
        fieldLengthCheckBox = new JCheckBox();
        fieldLengthCheckBox.setText("Field");
	fieldLengthCheckBox.setEnabled(false);
        fieldLengthCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    lengthCheckBoxActionPerformed(evt);
		}
	    });
	fieldLengthCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p8.add(Box.createRigidArea(new Dimension(33, 0)));
	p8.add(fieldLengthCheckBox);
	p8.setAlignmentX(Component.LEFT_ALIGNMENT);
	//lengthPane.add(p8);

	JPanel p9 = new JPanel();
	p9.setLayout(new BoxLayout(p9, BoxLayout.LINE_AXIS));
        componentLengthCheckBox = new JCheckBox();
        componentLengthCheckBox.setText("Component");
	componentLengthCheckBox.setEnabled(false);
        componentLengthCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    lengthCheckBoxActionPerformed(evt);
		}
	    });
	componentLengthCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p9.add(Box.createRigidArea(new Dimension(33, 0)));
	p9.add(componentLengthCheckBox);
	p9.setAlignmentX(Component.LEFT_ALIGNMENT);
	//lengthPane.add(p9);

	JPanel p10 = new JPanel();
	p10.setLayout(new BoxLayout(p10, BoxLayout.LINE_AXIS));
        subComponentLengthCheckBox = new JCheckBox();
        subComponentLengthCheckBox.setText("SubComponent");
	subComponentLengthCheckBox.setEnabled(false);
        subComponentLengthCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    lengthCheckBoxActionPerformed(evt);
		}
	    });
	subComponentLengthCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p10.add(Box.createRigidArea(new Dimension(33, 0)));
	p10.add(subComponentLengthCheckBox);
	p10.setAlignmentX(Component.LEFT_ALIGNMENT);
	//lengthPane.add(p10);
	//lengthPane.add(Box.createRigidArea(new Dimension(0, 25)));
	*/
	// Content
	
	JPanel contentPane = new JPanel();
	contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
	contentPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Content"));

/*	JPanel p24 = new JPanel();
	p24.setLayout(new BoxLayout(p24, BoxLayout.LINE_AXIS));
	JLabel lblContentType = new JLabel(" Type: ");
	p24.add(lblContentType);
	contentTypeComboBox = new JComboBox();
	contentTypeComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    typeComboBoxActionPerformed(evt);
		}
	    });        contentTypeComboBox.setBackground(new Color(204, 204, 255));
	contentTypeComboBox.addItem("Valid");
	contentTypeComboBox.addItem("Invalid");
	p24.add(contentTypeComboBox);
	p24.setAlignmentX(Component.LEFT_ALIGNMENT);
	contentPane.add(p24);
*/

/*
	lblContentComplexity = new JLabel(" Include Valid");
	lblContentComplexity.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	//contentPane.add(lblContentComplexity);

	JPanel p25 = new JPanel();
	p25.setLayout(new BoxLayout(p25, BoxLayout.LINE_AXIS));
	p25.add(Box.createRigidArea(new Dimension(10, 0)));
	contentComplexityComboBox = new JComboBox();
	contentComplexityComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    complexityComboBoxActionPerformed(evt);
		}
	    });
	contentComplexityComboBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
        contentComplexityComboBox.setBackground(new Color(204, 204, 255));
	contentComplexityComboBox.addItem("Typical");
	contentComplexityComboBox.addItem("Atypical");
	contentComplexityComboBox.setEnabled(false);
	p25.add(contentComplexityComboBox);
	lblContentComplexity2 = new JLabel(" Data Values");
	lblContentComplexity2.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p25.add(lblContentComplexity2);
	p25.setAlignmentX(Component.LEFT_ALIGNMENT);
	//contentPane.add(p25);

	JPanel p24 = new JPanel();
	p24.setLayout(new BoxLayout(p24, BoxLayout.LINE_AXIS));
	p24.add(Box.createRigidArea(new Dimension(50, 0)));
	JSeparator js = new JSeparator();
	js.setMaximumSize(new Dimension(310, 10));
	p24.add(js);
	p24.add(Box.createRigidArea(new Dimension(50, 0)));
	p24.setAlignmentX(Component.LEFT_ALIGNMENT);
	//contentPane.add(Box.createRigidArea(new Dimension(0,7)));
	//contentPane.add(p24);
	//contentPane.add(Box.createRigidArea(new Dimension(0,7)));

	JLabel lblContentInvalid = new JLabel(" Include Invalid");
	//contentPane.add(lblContentInvalid);

	JPanel p12 = new JPanel();
	p12.setLayout(new BoxLayout(p12, BoxLayout.LINE_AXIS));
        dataTypeCheckBox = new JCheckBox();
        dataTypeCheckBox.setText("Data Type Values");
	dataTypeCheckBox.setEnabled(false);
        dataTypeCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    contentCheckBoxActionPerformed(evt);
		}
	    });
	dataTypeCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p12.add(Box.createRigidArea(new Dimension(33, 0)));
	p12.add(dataTypeCheckBox);
	p12.setAlignmentX(Component.LEFT_ALIGNMENT);
	//contentPane.add(p12);

	JPanel p13 = new JPanel();
	p13.setLayout(new BoxLayout(p13, BoxLayout.LINE_AXIS));
        outOfRangeCheckBox = new JCheckBox();
        outOfRangeCheckBox.setText("Out of Range Values");
	outOfRangeCheckBox.setEnabled(false);
        outOfRangeCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    contentCheckBoxActionPerformed(evt);
		}
	    });
	outOfRangeCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p13.add(Box.createRigidArea(new Dimension(33, 0)));
	p13.add(outOfRangeCheckBox);
	p13.setAlignmentX(Component.LEFT_ALIGNMENT);
	//contentPane.add(p13);

	JPanel p14 = new JPanel();
	p14.setLayout(new BoxLayout(p14, BoxLayout.LINE_AXIS));
        formatCheckBox = new JCheckBox();
        formatCheckBox.setText("Match Pattern Values");
	formatCheckBox.setEnabled(false);
        formatCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    contentCheckBoxActionPerformed(evt);
		}
	    });
	formatCheckBox.addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
	p14.add(Box.createRigidArea(new Dimension(33, 0)));
	p14.add(formatCheckBox);
	p14.setAlignmentX(Component.LEFT_ALIGNMENT);
	//contentPane.add(p14);
	//contentPane.add(Box.createRigidArea(new Dimension(0, 13)));
	*/
	// Help

	JPanel p26 = new JPanel();
	p26.setLayout(new BoxLayout(p26, BoxLayout.LINE_AXIS));
	lblHelp = new JLabel(" ", JLabel.CENTER);
	lblHelp.setBorder(BorderFactory.createLineBorder(Color.BLACK));
	lblHelp.setMaximumSize(new Dimension(700,20));
	p26.add(Box.createRigidArea(new Dimension(80,0)));
	p26.add(lblHelp);
	p26.add(Box.createRigidArea(new Dimension(80,0)));
	p26.setAlignmentX(Component.LEFT_ALIGNMENT);

	// Usage + Content + Length

	JPanel p15 = new JPanel();
	p15.setLayout(new BoxLayout(p15, BoxLayout.LINE_AXIS));
	p15.setAlignmentX(Component.LEFT_ALIGNMENT);
	p15.add(Box.createRigidArea(new Dimension(150, 0)));
	p15.add(usagePane);
	p15.add(Box.createRigidArea(new Dimension(35, 0)));
	p15.add(codedEntryPane);
	//p15.add(Box.createRigidArea(new Dimension(175, 0)));

	// Cardinality + Coded Entry

	JPanel p16 = new JPanel();
	p16.setLayout(new BoxLayout(p16, BoxLayout.LINE_AXIS));
	p16.setAlignmentX(Component.LEFT_ALIGNMENT);
	p16.add(Box.createRigidArea(new Dimension(150, 0)));
	p16.add(cardinalityPane);
	p16.add(Box.createRigidArea(new Dimension(35, 0)));
	p16.add(lengthPane);
	//p16.add(Box.createRigidArea(new Dimension(175, 0)));

	// resetDefaultsButton
	JPanel p18 = new JPanel();
	p18.setLayout(new BoxLayout(p18, BoxLayout.LINE_AXIS));
	p18.setAlignmentX(Component.LEFT_ALIGNMENT);
        resetDefaultsButton = new JButton();
        resetDefaultsButton.setBackground(new Color(204, 204, 255));
        resetDefaultsButton.setText("Reset Defaults");
        resetDefaultsButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    resetButtonActionPerformed(evt);
		}
	    });
	p18.add(Box.createRigidArea(new Dimension(370, 0)));
	p18.add(resetDefaultsButton);

	// Options + Help = pFull
	pFull = new JPanel();
	pFull.setLayout(new BoxLayout(pFull, BoxLayout.PAGE_AXIS));
	pFull.setAlignmentX(Component.LEFT_ALIGNMENT);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p15);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p16);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p18);
	pFull.add(Box.createRigidArea(new Dimension(0, 100)));

	/*pFull.add(p15);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p16);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p26);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));
	pFull.add(p18);
	pFull.add(Box.createRigidArea(new Dimension(0, 10)));*/

	add(Box.createRigidArea(new Dimension(0, 25)));
	//add(pFull);

	// Separator 
	JPanel ph5 = new JPanel();
	ph5.setLayout(new BoxLayout(ph5, BoxLayout.LINE_AXIS));
	ph5.setAlignmentX(Component.LEFT_ALIGNMENT);

/*        jSeparator3 = new JSeparator();
	jSeparator3.setMaximumSize(new Dimension(665, 10));
	ph5.add(Box.createRigidArea(new Dimension(100, 0)));
	ph5.add(jSeparator3);
	add(ph5);*/

	// Empty Panel
	pEmpty = new JPanel();
	pEmpty.setLayout(new BoxLayout(pEmpty, BoxLayout.PAGE_AXIS));
	pEmpty.setBackground(Color.WHITE);
	pEmpty.add(Box.createRigidArea(new Dimension(0, 356)));
	add(pEmpty);

	addMouseMotionListener(new MouseMotionListener() {
		public void mouseDragged(MouseEvent evt) {;}
		public void mouseMoved(MouseEvent evt) {
		    lblHelpMouseMoved(evt);
		}
	    });
    }

    /**
     * Handles action for btnOptions
     * @param evt The event
     */
    private void btnOptionsActionPerformed(ActionEvent evt) {
	if(visible) {
	    btnOptions.setText("More Options >>>");
	    remove(pFull);
	    add(pEmpty, getComponents().length);
	    visible = false;
	}
	else {
	    btnOptions.setText("<<< More Options");
	    remove(pEmpty);
	    add(pFull, getComponents().length-1);
	    visible = true;
	}
	updateUI();
    }

    /**
     * Handles the help label
     * @param evt The event
     */
    private void lblHelpMouseMoved(MouseEvent evt) {
	lblHelpMouseMoved(evt.getSource());
    }

    /**
     * Handles the help label
     * @param evt The source
     */
    private void lblHelpMouseMoved(Object evt) {
	if(evt == requiredUsageCheckBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" Required elements are populated.");
	    }
	    // Invalid
	    else {
		if(requiredUsageCheckBox.isSelected()) {
		    lblHelp.setText(" Some required elements are not populated.");
		}
		else {
		    lblHelp.setText(" Required elements will be populated.");
		}
	    }
	}
	else if(evt == requiredEmptyUsageCheckBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" Elements are sometimes populated.");
	    }
	    // Invalid
	    else {
		lblHelp.setText(" Elements are sometimes populated.");
	    }
	}
	else if(evt == conditionalUsageCheckBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" This option is not yet available.");
	    }
	    // Invalid
	    else {
		lblHelp.setText(" This option is not yet available.");
	    }
	}
	else if(evt == conditionalEmptyUsageCheckBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" This option is not yet available.");
	    }
	    // Invalid
	    else {
		lblHelp.setText(" This option is not yet available.");
	    }
	}
	else if(evt == notSupportedUsageCheckBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" Not Supported elements are empty.");
	    }
 	    // Invalid
	    else {
		if(notSupportedUsageCheckBox.isSelected()) {
		    lblHelp.setText(" Some Not Supported elements are populated.");
		}
		else {
		    lblHelp.setText(" Not Supported elements are empty.");
		}
	    }
	}
	else if(evt == fieldCardinalityCheckBox) {
	    // Valid
	    if(cardinalityTypeComboBox.getSelectedIndex() == 0) {
		if(fieldCardinalityCheckBox.isSelected()) {
		    lblHelp.setText(" Messages will contain field instances that are within the range defined by the cardinality attribute.");
		}
		else {
		    lblHelp.setText(" Message will contain the minimum number of fields.");
		}
	    }
	    // Invalid
	    else {
		lblHelp.setText(" Some messages will contain field instances that are not within the range defined by the cardinality attribute.");
	    }
	}
	else if(evt == segmentCardinalityCheckBox) {
	    // Valid
	    if(cardinalityTypeComboBox.getSelectedIndex() == 0) {
		if(segmentCardinalityCheckBox.isSelected()) {
		    lblHelp.setText(" Messages will contain segment instances that are within the range defined by the cardinality attribute.");
		}
		else {
		    lblHelp.setText(" Message will contain the minimum number of segments.");
		}
	    }
	    // Invalid
	    else {
		lblHelp.setText(" Some messages will contain segment instances that are not within the range defined by the cardinality attribute.");
	    }
	}
	else if(evt == codedEntryTypeComboBox || evt == lblCodedEntryType) {
	    // Valid
	    if(codedEntryTypeComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" All coded entry elements will be populated with values from a valid value set.");
	    }
	    // Invalid
	    else {
		lblHelp.setText(" Some coded entry elements will be populated with values not contained in a valid value set.");
	    }
	}
	else if(evt == compositeLengthCheckBox) {
	    // Ignore
	    if(lengthTypeComboBox.getSelectedIndex() == 1) {
		lblHelp.setText("Length of data may exceed specified element length.");
	    }
	    // Valid
	    else {
		if(compositeLengthCheckBox.isSelected()) {
		    lblHelp.setText("Length of composite data items adheres to the specified element length.");
		}
		else {
		    lblHelp.setText("Length of data may exceed specified element length.");
		}
	    }
	}
	else if(evt == fieldLengthCheckBox) {
	    // Ignore
	    if(lengthTypeComboBox.getSelectedIndex() == 1) {
		lblHelp.setText("Length of data may exceed specified element length.");
	    }
	    // Valid
	    else {
		if(fieldLengthCheckBox.isSelected()) {
		    lblHelp.setText("Length of data adheres to the length specified for the primitive field element.");
		}
		else {
		    lblHelp.setText("Length of data may exceed specified element length.");
		}
	    }
	}
	else if(evt == componentLengthCheckBox) {
	    // Ignore
	    if(lengthTypeComboBox.getSelectedIndex() == 1) {
		lblHelp.setText("Length of data may exceed specified element length.");
	    }
	    // Valid
	    else {
		if(componentLengthCheckBox.isSelected()) {
		    lblHelp.setText("Length of data adheres to the length specified for the primitive component element.");
		}
		else {
		    lblHelp.setText("Length of data may exceed specified element length.");
		}
	    }
	}
	else if(evt == subComponentLengthCheckBox) {
	    // Ignore
	    if(lengthTypeComboBox.getSelectedIndex() == 1) {
		lblHelp.setText("Length of data may exceed specified element length.");
	    }
	    // Valid
	    else {
		if(subComponentLengthCheckBox.isSelected()) {
		    lblHelp.setText("Length of data adheres to the length specified for the primitive subcomponent element.");
		}
		else {
		    lblHelp.setText("Length of data may exceed specified element length.");
		}
	    }
	}
	else if(evt == contentComplexityComboBox || evt == lblContentComplexity || evt == lblContentComplexity2) {
	    // Typical
	    if(contentComplexityComboBox.getSelectedIndex() == 0) {
		lblHelp.setText(" ");
	    }
	    // Atypical
	    else {
		lblHelp.setText(" ");
	    }
	}
	else if(evt == dataTypeCheckBox) {
	    // Invalid
	    if(dataTypeCheckBox.isSelected()) {
		lblHelp.setText("Data content does not adhere to data type (e.g., string in a numeric field).");
	    }
	    else {
		lblHelp.setText(" ");
	    }
	}
	else if(evt == outOfRangeCheckBox) {
	    if(outOfRangeCheckBox.isSelected()) {
		lblHelp.setText(" Data content is correct data type but the value is out of specified range.");
	    }
	    else {
		lblHelp.setText(" ");
	    }
	}
	else if(evt == formatCheckBox) {
	    if(formatCheckBox.isSelected()) {
		lblHelp.setText(" Data content does not match the given match pattern (regular expressions).");
	    }
	    else {
		lblHelp.setText(" ");
	    }
	}	
	else {
	    lblHelp.setText(" ");
	}
    }

    /**
     * Handles action for the radio buttons
     * @param evt The event
     */
    private void rdActionPerformed(ActionEvent evt) {
	volumeTextField.setEditable(true);
	btnManual.setEnabled(false);
	if(evt.getSource() == rdValid) {
	    session.setVolume(currentVolume, true);
	    session.setTypeMessageSet("Valid");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdInvalid) {
	    session.setVolume(currentVolume, true);
	    session.setTypeMessageSet("Invalid");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdBoth) {
	    session.setVolume(currentVolume, true);
	    session.setTypeMessageSet("Both");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdSampleRandom) {
	    session.setVolume(1, true);
	    session.setTypeMessageSet("SampleRandom");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    volumeTextField.setEditable(false);
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdSampleMin) {
	    session.setVolume(1, true);
	    session.setTypeMessageSet("SampleMin");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    volumeTextField.setEditable(false);
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdSampleMax) {
	    session.setVolume(1, true);
	    session.setTypeMessageSet("SampleMax");
	    btnOptions.setEnabled(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	    volumeTextField.setEditable(false);
	    setMessageSet(evt);
	}
	else if(evt.getSource() == rdCustom) {
	    session.setVolume(currentVolume, true);
	    session.setTypeMessageSet("Custom");
	    btnOptions.setEnabled(true);
	}
	else if(evt.getSource() == rdManual) {
	    session.setTypeMessageSet("Manual");
	    btnManual.setEnabled(true);
	    btnOptions.setEnabled(false);
	    if(session.getProfile().equals(""))
		session.setVolume(0, true);
	    else
		session.setVolume(session.countMsgVariants(), true);
	    volumeTextField.setEditable(false);
	    if(visible) {
		btnOptionsActionPerformed(null);
	    }
	}
	// Length Global
	else if(evt.getSource() == rdIgnoreLength) {
	    session.setLengthGlobal("Ignore");
	}
	else if(evt.getSource() == rdEnforceLength) {
	    session.setLengthGlobal("Enforce");
	}
	// Usage
	else if(evt.getSource() == rdUsageValid) {
	    session.setUsageType("Valid");
	}
	else if(evt.getSource() == rdUsageInvalid) {
	    session.setUsageType("Invalid");
	}
	// Cardinality
	else if(evt.getSource() == rdCardinalityValidMin) {
	    session.setCardinalityType("ValidMin");
	}
	else if(evt.getSource() == rdCardinalityValidRange) {
	    session.setCardinalityType("ValidRange");
	}
	else if(evt.getSource() == rdCardinalityValidMax) {
	    session.setCardinalityType("ValidMax");
	}
	else if(evt.getSource() == rdCardinalityInvalid) {
	    session.setCardinalityType("Invalid");
	}
	// Coded Entry
	else if(evt.getSource() == rdCodedEntryValid) {
	    session.setCodedEntryType("Valid");
	}
	else if(evt.getSource() == rdCodedEntryInvalid) {
	    session.setCodedEntryType("Invalid");
	}
	// Length
	else if(evt.getSource() == rdLengthValid) {
	    session.setLengthType("Valid");
	}
	else if(evt.getSource() == rdLengthIgnore) {
	    session.setLengthType("Ignore");
	}
	else if(evt.getSource() == rdLengthInvalid) {
	    session.setLengthType("Invalid");
	}
    }

    /**
     * Set the options for the different message set
     * @param evt An action event
     */
    private void setMessageSet(ActionEvent evt) {
	if(evt.getSource() == rdValid) {
/*	    usageTypeComboBox.setSelectedIndex(0);
	    cardinalityTypeComboBox.setSelectedIndex(0);
	    lengthTypeComboBox.setSelectedIndex(0);
	    codedEntryTypeComboBox.setSelectedIndex(0);

	    contentComplexityComboBox.setSelectedIndex(0);
	    dataTypeCheckBox.setEnabled(false);
	    outOfRangeCheckBox.setSelected(false);
	    formatCheckBox.setSelected(false);*/
	}
	else if(evt.getSource() == rdInvalid) {
/*	    usageTypeComboBox.setSelectedIndex(1);
	    cardinalityTypeComboBox.setSelectedIndex(1);
	    lengthTypeComboBox.setSelectedIndex(1);
	    codedEntryTypeComboBox.setSelectedIndex(1);

	    contentComplexityComboBox.setSelectedIndex(1);
	    dataTypeCheckBox.setEnabled(false);
	    outOfRangeCheckBox.setSelected(false);
	    formatCheckBox.setSelected(false);*/
	}
	else if(evt.getSource() == rdBoth) {
	}
	else if(evt.getSource() == rdSampleRandom) {
/*	    usageTypeComboBox.setSelectedIndex(0);
	    cardinalityTypeComboBox.setSelectedIndex(0);
	    lengthTypeComboBox.setSelectedIndex(0);
	    codedEntryTypeComboBox.setSelectedIndex(0);

	    contentComplexityComboBox.setSelectedIndex(0);
	    dataTypeCheckBox.setEnabled(false);
	    outOfRangeCheckBox.setSelected(false);
	    formatCheckBox.setSelected(false);*/
	}
    }

    /**
     * Handles action for pathTextField
     * @param evt The event
     */
    private void pathTextFieldActionPerformed(ActionEvent evt) {
	String p = pathTextField.getText();
	session.setMsgPath(p);
    }
    
    /**
     * Handles action for profileTextField
     * @param evt The event
     */
    private void profileTextFieldActionPerformed(ActionEvent evt) {
	String p = profileTextField.getText();
	session.setProfile(p);
    }
    
    /**
     * Handles action for nameTextField
     * @param evt The event
     */
    private void nameTextFieldActionPerformed(ActionEvent evt) {
	String n = nameTextField.getText();
	session.setName(n);
    }

    /**
     * Handles action for resetButton
     * @param evt The event
     */
    private void resetButtonActionPerformed(ActionEvent evt) {
	try {
	    if(session.getName().compareTo("") == 0) {
		session.loadOptions("config" + Constants.FS + "maker.dat");
	    }
	    else {
		session.loadOptions(session.getName());
	    }
	    if(visible) {
		//btnOptionsActionPerformed(null);
		//btnOptions.setEnabled(false);
	    }
	}
	catch(IOException ioe) {
	    try {
		session.loadOptions("config" + Constants.FS + "maker.dat");
	    }
	    catch(IOException ioe2) {
		System.err.println("IOException " + ioe2.getMessage());
		ioe2.printStackTrace();
	    }
	}	
    }

    /**
     * This event handler sets the states for the content test options. These types
     * of tests are independent. Therefore they are set individually.
     * @param evt The event
     */
    private void contentCheckBoxActionPerformed(ActionEvent evt) {
/*	if(evt.getSource() == dataTypeCheckBox) {
          session.setDataType(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == outOfRangeCheckBox) {
	    session.setOutOfRange(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == formatCheckBox) {
	    session.setFormat(((JCheckBox)evt.getSource()).isSelected());
	}
	lblHelpMouseMoved(evt.getSource());*/
    }

    /**
     * Handles action for lengthCheckBox
     * @param evt The event
     */
    private void lengthCheckBoxActionPerformed(ActionEvent evt) {
/*	if(evt.getSource() == fieldLengthCheckBox) {
	    session.setFieldLength(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == componentLengthCheckBox) {
	    session.setComponentLength(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == subComponentLengthCheckBox) {
	    session.setSubComponentLength(((JCheckBox)evt.getSource()).isSelected());
	}
	lblHelpMouseMoved(evt.getSource());*/
    }

    /**
     * Set the state for the usage parameters. If the "master" usage button
     * is not selected, then the other usage options are automatically set to 
     * inactive.
     *
     * Note is it possible for the master usage button to be selected and none of the
     * low level usage options. The effective meaning in this case is that no usage 
     * tests will be performed; essentially, it is the same as the master usage button 
     * not be selected.
     *
     * The conditional usage options are not yet implemented. Therefore, the parameters
     * for these are not enabled under any circumstance.
     * @param evt The event
     */
    private void usageCheckBoxActionPerformed(ActionEvent evt) {
/*	if(evt.getSource() == requiredUsageCheckBox) {
		session.setRequiredUsage(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == requiredEmptyUsageCheckBox) {
		session.setRequiredEmptyUsage(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == conditionalUsageCheckBox) {
	    session.setConditionalUsage(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == conditionalEmptyUsageCheckBox) {
	    session.setConditionalEmptyUsage(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == notSupportedUsageCheckBox) {
	    session.setNotSupportedUsage(((JCheckBox)evt.getSource()).isSelected());
	}
	lblHelpMouseMoved(evt.getSource());*/
    }

    /**
     * Set the state for the cardinality parameters. If the "master" cardinality button
     * is not selected, then the segment and field cardinality are automatically set to 
     * inactive.
     *
     * Note is it possible for the master cardinality button to be selected and both the
     * segment and field not selected. The effective meaning of these settings is that
     * no cardinality tests will be performed; essentially, it is the same as the master
     * cardinality button not be selected.
     * @param evt The event
     */
    private void cardinalityCheckBoxActionPerformed(ActionEvent evt) {
/*	if(evt.getSource() == segmentCardinalityCheckBox) {
		session.setSegmentCardinality(((JCheckBox)evt.getSource()).isSelected());
	}
	else if(evt.getSource() == fieldCardinalityCheckBox) {
		session.setFieldCardinality(((JCheckBox)evt.getSource()).isSelected());
	}
	lblHelpMouseMoved(evt.getSource());*/
    }

    /**
     * Handles action for complexityComboBox
     * @param evt The event
     */
    private void complexityComboBoxActionPerformed(ActionEvent evt) {
/*	String complexity = (String)((JComboBox)evt.getSource()).getSelectedItem();
	if(evt.getModifiers() != 0) {
	    if(evt.getSource() == contentComplexityComboBox) {
		session.setContentComplexity(complexity);
	    }
	    }*/
    }

    /**
     * This method handles the user actions associated with the types ComboBox
     * @param evt The event
     */
    private void typeComboBoxActionPerformed(ActionEvent evt) {
/*	if(evt.getModifiers() != 0) {
	    typeComboBoxActionPerformed(evt.getSource());
	    }*/
    }

    /**
     * This method handles the user actions associated with the types ComboBox
     * @param evt The source
     */
    private void typeComboBoxActionPerformed(Object evt) {
/*	String type = (String)((JComboBox)evt).getSelectedItem();
	if(evt == usageTypeComboBox) {
	    // Valid
	    if(usageTypeComboBox.getSelectedIndex() == 0) {
		requiredUsageCheckBox.setEnabled(false);
		requiredUsageCheckBox.setSelected(true);
		requiredEmptyUsageCheckBox.setEnabled(false);
		requiredEmptyUsageCheckBox.setSelected(true);
		conditionalUsageCheckBox.setEnabled(false);
		conditionalUsageCheckBox.setSelected(false);
		conditionalEmptyUsageCheckBox.setEnabled(false);
		conditionalEmptyUsageCheckBox.setSelected(false);
		notSupportedUsageCheckBox.setEnabled(false);
		notSupportedUsageCheckBox.setSelected(true);
	    }
	    // Invalid
	    else {
		requiredUsageCheckBox.setEnabled(true);
		requiredUsageCheckBox.setSelected(true);
		requiredEmptyUsageCheckBox.setEnabled(false);
		requiredEmptyUsageCheckBox.setSelected(true);
		conditionalUsageCheckBox.setEnabled(false);
		conditionalUsageCheckBox.setSelected(false);
		conditionalEmptyUsageCheckBox.setEnabled(false);
		conditionalEmptyUsageCheckBox.setSelected(false);
		notSupportedUsageCheckBox.setEnabled(true);
		notSupportedUsageCheckBox.setSelected(true);
	    }
	    session.setUsageType(type);
	    session.setRequiredUsage(requiredUsageCheckBox.isSelected());
	    session.setRequiredEmptyUsage(requiredEmptyUsageCheckBox.isSelected());
	    session.setConditionalUsage(conditionalUsageCheckBox.isSelected());
	    session.setConditionalEmptyUsage(conditionalEmptyUsageCheckBox.isSelected());
	    session.setNotSupportedUsage(notSupportedUsageCheckBox.isSelected());
	}
	else if(evt == contentTypeComboBox) {
	    // Valid
	    if(contentTypeComboBox.getSelectedIndex() == 0) {
		contentComplexityComboBox.setEnabled(true);
		dataTypeCheckBox.setEnabled(false);
		dataTypeCheckBox.setSelected(false);
		outOfRangeCheckBox.setEnabled(false);
		outOfRangeCheckBox.setSelected(false);
		formatCheckBox.setEnabled(false);
		formatCheckBox.setSelected(false);
	    }
	    // Invalid
	    else {
		contentComplexityComboBox.setEnabled(false);
		dataTypeCheckBox.setEnabled(true);
		dataTypeCheckBox.setSelected(false);
		outOfRangeCheckBox.setEnabled(true);
		outOfRangeCheckBox.setSelected(false);
		formatCheckBox.setEnabled(true);
		formatCheckBox.setSelected(false);
	    }
	    session.setContentType(type);
	    session.setContentComplexity((String)contentComplexityComboBox.getSelectedItem());
	    session.setDataType(dataTypeCheckBox.isSelected());
	    session.setOutOfRange(outOfRangeCheckBox.isSelected());
	    session.setFormat(formatCheckBox.isSelected());
	    }

	else if(evt == cardinalityTypeComboBox) {
	    // Valid
	    if(cardinalityTypeComboBox.getSelectedIndex() == 0) {
		fieldCardinalityCheckBox.setEnabled(true);
		fieldCardinalityCheckBox.setSelected(true);
		segmentCardinalityCheckBox.setEnabled(true);
		segmentCardinalityCheckBox.setSelected(true);
	    }
	    // Invalid
	    else {
		fieldCardinalityCheckBox.setEnabled(false);
		fieldCardinalityCheckBox.setSelected(true);
		segmentCardinalityCheckBox.setEnabled(false);
		segmentCardinalityCheckBox.setSelected(true);
	    }
	    session.setCardinalityType(type);
	    session.setFieldCardinality(fieldCardinalityCheckBox.isSelected());
	    session.setSegmentCardinality(segmentCardinalityCheckBox.isSelected());
	}
	else if(evt == codedEntryTypeComboBox) {
	    session.setCodedEntryType(type);
	    lblHelpMouseMoved(evt);
	}
	else if(evt == lengthTypeComboBox) {
	    // Valid
	    if(lengthTypeComboBox.getSelectedIndex() == 0) {
		compositeLengthCheckBox.setEnabled(false);
		compositeLengthCheckBox.setSelected(false);						   
		fieldLengthCheckBox.setEnabled(true);
		fieldLengthCheckBox.setSelected(true);
		componentLengthCheckBox.setEnabled(true);
		componentLengthCheckBox.setSelected(true);
		subComponentLengthCheckBox.setEnabled(true);
		subComponentLengthCheckBox.setSelected(true);						   
	    }
	    // Ignore
	    else {
		compositeLengthCheckBox.setEnabled(false);
		compositeLengthCheckBox.setSelected(false);						   
		fieldLengthCheckBox.setEnabled(false);
		fieldLengthCheckBox.setSelected(false);
		componentLengthCheckBox.setEnabled(false);
		componentLengthCheckBox.setSelected(false);
		subComponentLengthCheckBox.setEnabled(false);
		subComponentLengthCheckBox.setSelected(false);						   
	    }
	    session.setLengthType(type);
	    session.setCompositeLength(compositeLengthCheckBox.isSelected());
	    session.setFieldLength(fieldLengthCheckBox.isSelected());
	    session.setComponentLength(componentLengthCheckBox.isSelected());
	    session.setSubComponentLength(subComponentLengthCheckBox.isSelected());
	    }*/
    }

    /**
     * Handles action for pathButton
     * @param evt The event
     */
    private void pathButtonActionPerformed(ActionEvent evt) {
        // Add your handling code here:
	File file;
	String path;
	// Create an MMK file chooser
	final JFileChooser fc = new JFileChooser(user.getOpenedDir());
	
	// Set the chooser to accept only MMK files.
	fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	
	if (evt.getSource() == pathButton) {
	    int returnVal = fc.showOpenDialog(this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		if (fc.getSelectedFile().isDirectory()) {
		    file = fc.getSelectedFile();
		    path = file.getParent() + java.io.File.separator + file.getName();
		    session.setMsgPath(path);
		    pathTextField.setText(path);
		    user.setOpenedDir(path);
		} 
	    }
	}
    }

    /**
     * Handles action for profileButton
     * @param evt The event
     */
    private void profileButtonActionPerformed(ActionEvent evt) {
        // Add your handling code here:
	File profile;
	// Create an XML file chooser
	final JFileChooser fc = new JFileChooser(user.getOpenedDir());
	
	// Set the chooser to accept only XML files.
	fc.addChoosableFileFilter(new XMLFileFilter());
	fc.setAcceptAllFileFilterUsed(false);
	//Handle the load profile menu item action.
	
	if (evt.getSource() == profileButton) {
	    int returnVal = fc.showOpenDialog(this);
	    
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		profile = fc.getSelectedFile();
		//Open the file.
		String p = profile.getPath();
		// Test if the profile is correct against the Schema and well-formed
		int testP = testProfile(p);
		if(testP == 2) {
		    Object[] options = {"Load Anyway", "No"};
		    JOptionPane optionPane = new JOptionPane("The profile is not compliant with the HL7 profile schema.\n" + handler.error,JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_OPTION, null, options);
		    JDialog dialog = optionPane.createDialog(this, "ProfileError");
		    dialog.show();
		    Object selectedValue = optionPane.getValue();
		    if(selectedValue != null && ((String)selectedValue).equals("Load Anyway")) {
			testP = 0;
			session.setValidate(false);
			JOptionPane.showMessageDialog(this, "Messages will not be validated since profile in not HL7 compliant.", "Validation information", JOptionPane.INFORMATION_MESSAGE);
		    }
		}
		if(testP == 0) {
		    JDialog dlgWait = new WaitDialog("Load Profile and Generate the Data Repository ...");
		    // Center the window
		    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		    Dimension windowSize = dlgWait.getSize();
		    dlgWait.setLocation(Math.max(0,(screenSize.width -windowSize.width)/2), 
					Math.max(0,(screenSize.height-windowSize.height)/2));
		    
		    final JDialog fDlg = dlgWait;
		    final String fP = p;
		    Thread t = new Thread() {
			    public void run() {
				session.setProfile(fP);
				fDlg.dispose();
			    }
			};
		    t.start();
		    
		    dlgWait.show();

		    profileTextField.setText(p);
		    String name = p.substring(0, p.length()-3) + "mmk"; 
		    session.setName(name);
		    nameTextField.setText(name);
		    String path = profile.getParent() + java.io.File.separator + "messages";
		    session.setMsgPath(path);
		    pathTextField.setText(path);
		    user.setOpenedDir(profile.getParent());
		    // Set to a valid set
		    rdValid.setSelected(true);
		    session.setVolume(10, true);
		    volumeTextField.setEditable(true);
		    session.setVolume(currentVolume, true);
		    session.setTypeMessageSet("Valid");
		    btnOptions.setEnabled(false);
		    btnManual.setEnabled(false);
		    if(visible) {
			btnOptionsActionPerformed(null);
		    }
		    setMessageSet(evt);
		}
		// Not well-formed
		else if(testP == 1) {
		    JOptionPane.showMessageDialog(this, "The profile is not well-formed.", "Profile error", JOptionPane.ERROR_MESSAGE);
		}
	    } 
	}
    }

    /**
     * Capture the "name" that is identified with the current Message Test Set. This
     * file will have an ".mmk" extension.
     * @param evt The event
     */
    private void nameButtonActionPerformed(ActionEvent evt) {
	// Add your handling code here:
	File name;
	// Create an MMK file chooser
	final JFileChooser fc = new JFileChooser(user.getOpenedDir());
	
	// Set the chooser to accept only MMK files.
	fc.addChoosableFileFilter(new MMKFileFilter());
	fc.setAcceptAllFileFilterUsed(false);
	
	if (evt.getSource() == nameButton) {
	    int returnVal = fc.showOpenDialog(this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		name = fc.getSelectedFile();
		if(!name.getPath().endsWith(".mmk")) {
		    name = new File(name.getPath() + ".mmk");
		}
		session.setName(name.getPath());
		nameTextField.setText(name.getPath());
		user.setOpenedDir(name.getParent());
	    }
	}
    }
    
    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof Session) {
	    if(arg instanceof Session) {
		gov.nist.hl7maker.util.MVCLog.info("Event from Session in MessageSetupPanel");
		loadSession((Session)arg, false);
		// Set the focus on the init panel
		JTabbedPane tabbedPane = (JTabbedPane)this.getParent();
		tabbedPane.setSelectedIndex(0);
	    }
	    else if(arg instanceof MVCArg) {
		if("volume".compareTo(((MVCArg)arg).getName()) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in MessageSetupPanel");
		    volumeTextField.setText(((MVCArg)arg).getValue());
		}
		else if("msgPath".compareTo(((MVCArg)arg).getName()) == 0) {
		    gov.nist.hl7maker.util.MVCLog.info("Event from Session in MessageSetupPanel");
		    pathTextField.setText(((MVCArg)arg).getValue());
		}
	    }
	}
    }

    /**
     * Load the session
     * @param ses The session
     * @param onlyOptions true if only the options are updated
     */
    private void loadSession(Session ses, boolean onlyOptions) {
	session = ses;

	if(!onlyOptions) {
	    profileTextField.setText(session.getProfile());
	    nameTextField.setText(session.getName());
	    pathTextField.setText(session.getMsgPath());
	    btnOptions.setEnabled(false);
	    btnManual.setEnabled(false);
	    String typeMessageSet = session.getTypeMessageSet();
	    if("Valid".compareTo(typeMessageSet) == 0) {
		rdValid.setSelected(true);
	    }
	    else if("Invalid".compareTo(typeMessageSet) == 0) {
		rdInvalid.setSelected(true);
	    }
	    else if("Both".compareTo(typeMessageSet) == 0) {
		rdBoth.setSelected(true);
	    }
	    else if("SampleRandom".compareTo(typeMessageSet) == 0) {
		rdSampleRandom.setSelected(true);
	    }
	    else if("SampleMin".compareTo(typeMessageSet) == 0) {
		rdSampleMin.setSelected(true);
	    }
	    else if("SampleMax".compareTo(typeMessageSet) == 0) {
		rdSampleMax.setSelected(true);
	    }
	    else if("Custom".compareTo(typeMessageSet) == 0) {
		btnOptions.setEnabled(true);
		rdCustom.setSelected(true);
	    }
	    else if("Manual".compareTo(typeMessageSet) == 0) {
		btnManual.setEnabled(true);
		rdManual.setSelected(true);
	    }
	    volumeTextField.setText(String.valueOf(session.getVolume()));
	    String lengthGlobal = session.getLengthGlobal();
	    if("Ignore".compareTo(lengthGlobal) == 0) {
		rdIgnoreLength.setSelected(true);
	    }
	    else if("Enforce".compareTo(lengthGlobal) == 0) {
		rdEnforceLength.setSelected(true);
	    }
	}

	// Usage
	if(session.getUsageType().equals("Valid")) {
	    rdUsageValid.setSelected(true);
	}
	else if(session.getUsageType().equals("Invalid")) {
	    rdUsageInvalid.setSelected(true);
	}

	// Cardinality
	if(session.getCardinalityType().equals("ValidMin")) {
	    rdCardinalityValidMin.setSelected(true);
	}
	else if(session.getCardinalityType().equals("ValidRange")) {
	    rdCardinalityValidRange.setSelected(true);
	}
	else if(session.getCardinalityType().equals("ValidMax")) {
	    rdCardinalityValidMax.setSelected(true);
	}
	else if(session.getCardinalityType().equals("Invalid")) {
	    rdCardinalityInvalid.setSelected(true);
	}

	// Coded Entry
	if(session.getCodedEntryType().equals("Valid")) {
	    rdCodedEntryValid.setSelected(true);
	}
	else if(session.getCodedEntryType().equals("Invalid")) {
	    rdCodedEntryInvalid.setSelected(true);
	}

	// Length
	if(session.getLengthType().equals("Valid")) {
	    rdLengthValid.setSelected(true);
	}
	else if(session.getLengthType().equals("Ignore")) {
	    rdLengthIgnore.setSelected(true);
	}
	else if(session.getLengthType().equals("Invalid")) {
	    rdLengthInvalid.setSelected(true);
	}
/*
	cardinalityTypeComboBox.setSelectedItem(session.getCardinalityType());
	segmentCardinalityCheckBox.setSelected(session.getSegmentCardinality());
	fieldCardinalityCheckBox.setSelected(session.getFieldCardinality());
	typeComboBoxActionPerformed(cardinalityTypeComboBox);

	usageTypeComboBox.setSelectedItem(session.getUsageType());
	requiredUsageCheckBox.setSelected(session.getRequiredUsage());
	requiredEmptyUsageCheckBox.setSelected(session.getRequiredEmptyUsage());
	conditionalUsageCheckBox.setSelected(session.getConditionalUsage());
	conditionalEmptyUsageCheckBox.setSelected(session.getConditionalEmptyUsage());
	notSupportedUsageCheckBox.setSelected(session.getNotSupportedUsage());
	typeComboBoxActionPerformed(usageTypeComboBox);

	codedEntryTypeComboBox.setSelectedItem(session.getCodedEntryType());

	lengthTypeComboBox.setSelectedItem(session.getLengthType());
	fieldLengthCheckBox.setSelected(session.getFieldLength());
	componentLengthCheckBox.setSelected(session.getComponentLength());
	subComponentLengthCheckBox.setSelected(session.getSubComponentLength());
	typeComboBoxActionPerformed(lengthTypeComboBox);

	//	contentTypeComboBox.setSelectedItem(session.getContentType());
	contentComplexityComboBox.setSelectedItem(session.getContentComplexity());
	dataTypeCheckBox.setSelected(session.getDataType());
	outOfRangeCheckBox.setSelected(session.getOutOfRange());
	formatCheckBox.setSelected(session.getFormat());
	//	typeComboBoxActionPerformed(contentTypeComboBox);*/
    }

    /**
     * Test if the profile is well-formed and valid against the schema
     * @param xmlProfile The XML Profile
     * @return An integer 0 = correct 1 = not well-formed 2 = not HL7 compliant
     */
    private int testProfile(String xmlProfile) {
	// Try to parse it, if an exception is thrown, the profile is not correct
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc = builder.parse(new File(xmlProfile));
	    // HL7Compliant ?
	    if(!session.getValidateProfile().equals("donot")) {
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		SAXParser parser = new SAXParser();
		parser.setFeature("http://xml.org/sax/features/validation", true); 
		parser.setFeature("http://apache.org/xml/features/validation/schema", true);
		parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);
		if(session.getValidateProfile().equals("loose")) 
		    parser.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation", session.getProtocolPath(LOOSE_SCHEMA));
		else if(session.getValidateProfile().equals("strict"))
		    parser.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation", session.getProtocolPath(HL7_SCHEMA));

		System.out.println(session.getValidateProfile() + " " + parser.getProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation"));

		handler = new Validator();
		parser.setErrorHandler(handler);
		parser.parse(session.getProtocolPath(xmlProfile));
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		if(handler.validationError) {
		    return 2;
		}
	    }
	}
	catch(ParserConfigurationException pce) {
	    return 1;
	}
	catch(SAXException saxe) {
	    return 1;
	}
	catch(IOException ioe) {
	    System.out.println(ioe.getMessage());
	    return 1;
	}
	return 0;
    }

    /*
     * This class is used to validate the input for the message volume option. Invalid
     * entries are forbidden to be entered and will not be shown on the user interface.
     * This code will catch most invalid entries but is not exhaustive.
     */
    
    class VolumeDocument extends PlainDocument {
	
	protected VolumeDocument() {
	}
	
	/*
	 * This overrides the parent remove method. It reacts to the user removing a
	 * character via the user interface. The code prevents the user from entering
	 * invalid values for the volume option. It does allow the user to completely
	 * clear the value.
	 */
	
	public void remove(int offset, int length) throws BadLocationException {
	    int v = -1;
	    String s = getParagraphElement(0).getDocument().getText(offset, 1);

	    super.remove(offset, length);
	    int start = getParagraphElement(0).getDocument().getStartPosition().getOffset();
	    int len = getParagraphElement(0).getDocument().getLength();
	    
	    if (len > 0) {
		v = Integer.parseInt(getParagraphElement(0).getDocument().getText(start, len).trim());
	    } 
	    else {
		session.setVolume(0, false);
		session.fireEvent(new MVCArg("volumeMenu", String.valueOf(0)));
		return;
	    }
	    /* determine weather the removal of input is valid for the volume option */
	    try {
		isMinVolumeOk(v);
	    } 
	    catch (Exception e) {
		insertString(offset, s, null);
	    }
	    start = getParagraphElement(0).getDocument().getStartPosition().getOffset();
	    len = getParagraphElement(0).getDocument().getLength();
	    if (len > 0) {
		v = Integer.parseInt(getParagraphElement(0).getDocument().getText(start, len).trim());
	    }
	    session.setVolume(v, false);
	    session.fireEvent(new MVCArg("volumeMenu", String.valueOf(v)));
	    if(!rdSampleRandom.isSelected() && !rdSampleMin.isSelected() && !rdSampleMax.isSelected()) {
		currentVolume = v;
	    }
	}
	
	/*
	 * This overrides the parent insert method. It reacts to the user inserting a
	 * character via the user interface. The code prevents the user from entering
	 * invalid values for the volume option.
	 */
	
	public void insertString(int offset, String s, AttributeSet attributeSet) throws BadLocationException {
	    /* verify that the user entered a valid digit */
	    try {
		Integer.parseInt(s);
	    }
	    catch (Exception e) {
		return;
	    }
	    /* insert the valid digit into the string */
	    super.insertString(offset, s, attributeSet);
	    
	    int start = getParagraphElement(0).getDocument().getStartPosition().getOffset();
	    int len = getParagraphElement(0).getDocument().getLength();
	    int v = Integer.parseInt(getParagraphElement(0).getDocument().getText(start, len));
	    /* determine weather or not the input is valid for the volume option */
	    try {
		isMaxVolumeOk(v);
	    }
	    catch (Exception e) {
		super.remove(offset, 1);
	    }
	    
	    try {
		isMinVolumeOk(v);
	    } 
	    catch (Exception e) {
		super.remove(offset, 1);
	    }
	    start = getParagraphElement(0).getDocument().getStartPosition().getOffset();
	    len = getParagraphElement(0).getDocument().getLength();
	    v = Integer.parseInt(getParagraphElement(0).getDocument().getText(start, len));
	    session.setVolume(v, false);
	    session.fireEvent(new MVCArg("volumeMenu", String.valueOf(v)));
	    if(!rdSampleRandom.isSelected() && !rdSampleMin.isSelected() && !rdSampleMax.isSelected()) {
		currentVolume = v;
	    }
	}
	
	/* 
	 * isMaxVolumeOk is a helper method used to determine if the user input is valid.
	 */
	
	private void isMaxVolumeOk(int v) throws Exception {
	    //	    if (v > MESSAGE_VOLUME_MAX) {
	    if(v > session.getVolumeMax()) {
		throw new Exception ("Maximum message volume exceeded");
	    }
	}
	
	/* 
	 * isMinVolumeOk is a helper method used to determine if the user input is valid.
	 */
	
	private void isMinVolumeOk(int v) throws Exception {
	    //	    if (v < MESSAGE_VOLUME_MIN) {
	    if(v < session.getVolumeMin()) {
		throw new Exception ("Maximum message volume exceeded");
	    }
	}
    }

    /**
     * Private class to handle error from the validation of the profile against the XML Schema
     */
    private class Validator extends DefaultHandler {
	public boolean validationError = false;  
	public String error = "";

	public void error(SAXParseException exception) throws SAXException {
	    error += (exception.getMessage() + "\n");
	    validationError=true;
	}

	public void fatalError(SAXParseException exception) throws SAXException { 
	    error += (exception.getMessage() + "\n");
	    validationError = true;	    
	}		    
	
	public void warning(SAXParseException exception) throws SAXException {
	}	
    }
}
