/**
 * This class is a model for the Profile JTable
 */

package gov.nist.hl7maker.view.init;

// JDK
import java.util.Vector;

// Swing
import javax.swing.JTree;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.table.AbstractTableModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeModelListener;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

// MessageMaker
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.control.Profile;
import gov.nist.hl7maker.view.config.ProfileNode;
import gov.nist.hl7maker.view.config.SegmentGroupNode;
import gov.nist.hl7maker.view.config.TableValueNode;
import gov.nist.hl7maker.view.config.ValueNode;

public class ProfileTableModel extends AbstractTableModel implements TreeModel {

    private DefaultTreeModel treeModel;

    private Class[] classes = {ProfileTableModel.class, Vector.class, Vector.class, Vector.class};
    private String[] names = {"Element", "Cardinality", "Usage", "Value Set"};

    private Document doc;
    private JTree tree;
    private Session session;

    private Vector vUpdateValid, vUpdateInvalid, vUpdateBoth;
    private String messageType, versionID, triggerEvent, messageStructure;

    private short type;
    public static final short VALID = 0;
    public static final short INVALID = 1;
    public static final short BOTH = 2;

    /**
     * Constructor
     */
    public ProfileTableModel() {
	vUpdateValid = new Vector();
	vUpdateInvalid = new Vector();
	vUpdateBoth = new Vector();
    }
    
    /**
     * Set the document (the profile)
     * @param doc A Document object
     */
    public void oldsetDocument(Document doc) {
/*	removeEmptyTextElement(doc);
	createTreeModel(doc);*/
	vUpdateValid = new Vector();
	vUpdateInvalid = new Vector();
	vUpdateBoth = new Vector();
    }

    /**
     * Set the JTree
     * @param tree The JTree
     */
    public void setTree(JTree tree) {
	this.tree = tree;
    }

    /**
     * Set the session
     * @param session The session
     */
    public void setSession(Session session) {
	this.session = session;
	createTreeModel();
    }

    /**
     * Set the type of messages (VALID/INVALID/BOTH)
     * @type The type of messages
     */
    public void setType(short type) {
	this.type = type;
    }

    // Table Model

    /**
     * Return the number of rows
     * @return Number of rows
     */
    public int getRowCount() {
	if(tree == null)
	    return 0;
	return tree.getRowCount();
    }

    /**
     * Return the number of column
     * @return Number of columns
     */
    public int getColumnCount() {
	return classes.length;
    }

    /**
     * Get the value of the object at row i and column j
     * @param row Row position
     * @param column Column position
     */
    public Object getValueAt(int row, int column) {
	TreePath treePath = tree.getPathForRow(row);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    updateRowColumnProfileNode(pn, column, row);
	    Vector v = new Vector();
	    // Is it a special case (no variation)
	    if(isSpecialCase(pn, column)) {
		v.add(pn.getClass().getName());
	    }
	    else {
		// This is an updated element
		if(isUpdated(row, column))
		    v.add(new Boolean(isSelected(row, column)));
		// Check if there is a test in the session
		else {
		    // Cardinality
		    if(column == 1)
			v.add(new Boolean(session.isCardinalityTest(pn, type)));
		    // Usage
		    else if(column == 2)
			v.add(new Boolean(session.isUsageTest(pn, type)));
		    // Error Codes
		    else if(column == 3) 
			v.add(new Boolean(session.isErrorCodesTest(pn, type)));
		}
		v.add(pn.getClass().getName());
		if(pn.getCardMin() == null || pn.getCardMax() == null)
		    v.add("");
		else 
		    v.add(pn.getCardMin() + " .. " + pn.getCardMax());
		v.add(pn.getUsage());
		v.add(pn.getTable());
	    }
	    return v;
	}
	return null;
    }

    /**
     * Returns Object.class regardless of columnIndex
     * @param columnIndex The column index
     * @return The class
     */
    public Class getColumnClass(int columnIndex) {
	return classes[columnIndex];
    }

    /**
     * Returns a name for the column
     * @param column The column
     * @return The name
     */
    public String getColumnName(int column) {
	return names[column];
    }          

    /**
     * Handles the modification of the model
     * @param aValue The new value
     * @param rowIndex The row
     * @param columnIndex The column
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	if(aValue != null) {
	    TreePath treePath = tree.getPathForRow(rowIndex);
	    if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
		ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
		switch(type) {
		case VALID:
		    int idx = vUpdateValid.indexOf(new UpdateElement(pn, columnIndex));
		    // if aValue is a Integer remove the item
		    if(idx == -1) {
			vUpdateValid.add(new UpdateElement(pn, ((Boolean)aValue).booleanValue(), rowIndex, columnIndex));
		    }
		    else {
			((UpdateElement)vUpdateValid.get(idx)).selected = ((Boolean)aValue).booleanValue();
		    }
		    break;
		case INVALID:
		    idx = vUpdateInvalid.indexOf(new UpdateElement(pn, columnIndex));
		    // if aValue is a Integer remove the item
		    if(idx == -1) {
			vUpdateInvalid.add(new UpdateElement(pn, ((Boolean)aValue).booleanValue(), rowIndex, columnIndex));
		    }
		    else {
			((UpdateElement)vUpdateInvalid.get(idx)).selected = ((Boolean)aValue).booleanValue();
		    }
		    break;
		case BOTH:
		    idx = vUpdateBoth.indexOf(new UpdateElement(pn, columnIndex));
		    // if aValue is a Integer remove the item
		    if(idx == -1) {
			vUpdateBoth.add(new UpdateElement(pn, ((Boolean)aValue).booleanValue(), rowIndex, columnIndex));
		    }
		    else {
			((UpdateElement)vUpdateBoth.get(idx)).selected = ((Boolean)aValue).booleanValue();
		    }
		    break;
		}
	    }
	}
    }

    // TreeModel interface implementation

    public Object getRoot() {
	return treeModel.getRoot();
    }

    public int getChildCount(Object parent) {
 	return treeModel.getChildCount(parent);
    }

    public Object getChild(Object parent, int index) {
	return treeModel.getChild(parent, index);
    }

    public int getIndexOfChild(Object parent, Object child) {
	return treeModel.getIndexOfChild(parent, child);
    }

    /**
     * Return true if the node is a leaf
     * @param node The Node
     * @return is a leaf?
     */
    public boolean isLeaf(Object node) {
	return treeModel.isLeaf(node);
    }

    public void addTreeModelListener(TreeModelListener listener) {
    }

    public void removeTreeModelListener(TreeModelListener listener) {
    }
    
    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    /**
     * Create the tree structure
     */
    private void createTreeModel() { 
	Profile p = session.getRAMProfile();
	treeModel = new DefaultTreeModel(p.getTree(session.getProfile()));
    }

    /**
     * Is the checkbox of column c and row r selected?
     * @param r The row
     * @param c The column
     * @return true of false
     */
    public boolean isSelected(int r, int c) {
	TreePath treePath = tree.getPathForRow(r);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    switch(type) {
	    case VALID:
		int idx = vUpdateValid.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return ((UpdateElement)vUpdateValid.get(idx)).selected;
		}
		break;
	    case INVALID:
		idx = vUpdateInvalid.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return ((UpdateElement)vUpdateInvalid.get(idx)).selected;
		}
		break;
	    case BOTH:
		idx = vUpdateBoth.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return ((UpdateElement)vUpdateBoth.get(idx)).selected;
		}
		break;
	    }
	}
	return false;
    }

    /**
     * Is there a updated element for column c and row r?
     * @param r The row
     * @param c The column
     * @return true of false
     */
    public boolean isUpdated(int r, int c) {
	TreePath treePath = tree.getPathForRow(r);
	if(((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject() instanceof ProfileNode) {
	    ProfileNode pn = (ProfileNode)((DefaultMutableTreeNode)treePath.getLastPathComponent()).getUserObject();
	    switch(type) {
	    case VALID:
		int idx = vUpdateValid.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return true;
		}
		break;
	    case INVALID:
		idx = vUpdateInvalid.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return true;
		}
		break;
	    case BOTH:
		idx = vUpdateBoth.indexOf(new UpdateElement(pn, c));
		if(idx != -1) {
		    return true;
		}
		break;
	    }
	}
	return false;
    }

    /**
     * Is it a special case with no variation
     * @param pn The ProfileNode
     * @param column The column
     * @return true or false
     */
    public boolean isSpecialCase(ProfileNode pn, int column) {
	String usage = pn.getUsage();
	String card = pn.getCardMin() + ".." + pn.getCardMax();
	String tableId = pn.getTable();

	if(pn instanceof SegmentGroupNode || column == 1 && (pn.getCardMin() == null || pn.getCardMax() == null) || pn.hasXUsage())
	    return true;
	// No test for unsupported usage
	if(usage.equals("C") || usage.equals("CE") || usage.equals("U") || usage.equals("O") || usage.equals("NS"))
	    return true;
	// No Test for MSH MSH.1 MSH.2
	if(pn.getMsgLoc().equals("MSH") || pn.getMsgLoc().equals("MSH.1") || pn.getMsgLoc().equals("MSH.2"))
	    return true;
	switch(type) {
	case VALID:
	    // Case 1..1 and R
	    if(card.equals("1..1") && usage.equals("R"))
		return true;
	    // Case X and any cardinality
	    if(usage.equals("X"))
		return true;
	    // No cardinality test if usage RE and cardinality 0..1
	    if(column == 1 && usage.equals("RE") && card.equals("0..1"))
		return true;
	    // Usage R
	    if(column == 2 && usage.equals("R"))
		return true;
	    // No error codes for valid
	    if(column == 3)
		return true;
	    break;
	case INVALID:
	    // No cardinality error if usage X
	    if(column == 1 && usage.equals("X"))
		return true;
	    // No usage error for usage RE
	    if(column == 2 && usage.equals("RE"))
		return true;
	    // Usage RE, no cardinality if getMaxCard = "*"
	    if(column == 1 && usage.equals("RE") && pn.getCardMax().equals("*"))
		return true;
	    // Error Codes only for table
	    if(column == 3 && (usage.equals("X") || tableId == null))
		return true;
	    break;
	case BOTH:
	}
	return false;
    }

    /**
     * Update the row and column for a profileNode
     * @param pn The ProfileNode
     * @param column The column
     * @param row The row
     */
    public void updateRowColumnProfileNode(ProfileNode pn, int column, int row) {
	switch(type) {
	case VALID:
	    int idx = vUpdateValid.indexOf(new UpdateElement(pn, column));
	    if(idx != -1) {
		((UpdateElement)vUpdateValid.get(idx)).row = row;
	    }
	    break;
	case INVALID:
	    idx = vUpdateInvalid.indexOf(new UpdateElement(pn, column));
	    if(idx != -1) {
		((UpdateElement)vUpdateInvalid.get(idx)).row = row;
	    }
	    break;
	case BOTH:
	    idx = vUpdateBoth.indexOf(new UpdateElement(pn, column));
	    if(idx != -1) {
		((UpdateElement)vUpdateBoth.get(idx)).row = row;
	    }
	    break;
	}
    }

    /**
     * Get the ProfileNode for a row
     * @param row The row
     * @return The profile node
     */
/*    public ProfileNode getProfileNode(int row) {
	for(int i=0;i<vUpdate.size();i++) {
	    UpdateElement ue = (UpdateElement)vUpdate.get(i);
	    if(ue.row == row) {
		return ue.pn;
	    }
	}
	return null;
	}*/

    /**
     * Remove the item from the vUpdate vector after being saved.
     * @param row The row
     */
/*    public void removeUdpdatedItem(int row) {
	for(int i=0;i<vUpdate.size();i++) {
	    UpdateElement ue = (UpdateElement)vUpdate.get(i);
	    if(ue.row == row) {
		vUpdate.remove(ue);
	    }
	}
	}*/

    /**
     * Get the vector of updated element
     * @return The Vector
     */
    public Vector getUpdateElement() {
	switch(type) {
	case VALID:
	    return vUpdateValid;
	case INVALID:
	    return vUpdateInvalid;
	case BOTH:
	    return vUpdateBoth;
	}
	return null;
    }

    /**
     * Clear the update vector
     */
    public void clearUpdate() {
	vUpdateValid.clear();
	vUpdateInvalid.clear();
	vUpdateBoth.clear();
    }

    // This inner class handles the update of the model
    public class UpdateElement {

	public ProfileNode pn; // The profile node
	public boolean selected; // true or false if the checkbox is selected or not
	public int row; // The row of the cell modified
	public int column; // The column of the cell modified

	/**
	 * Constructor
	 * @param pn The profile node
	 * @param c The column of the cell modified
	 */
	public UpdateElement(ProfileNode pn, int c) {
	    this.pn = pn;
	    column = c;
	}
	
	/**
	 * Constructor
	 * @param pn The profile node
	 * @param selected is Selected?
	 * @param r The row of the cell modified
	 * @param c The column of the cell modified
	 */
	public UpdateElement(ProfileNode pn, boolean selected, int r, int c) {
	    this.pn = pn;
	    this.selected = selected;
	    row = r;
	    column = c;
	}

	public boolean equals(Object o) {
	    if(o instanceof UpdateElement) {
		if(pn.equals(((UpdateElement)o).pn) && column == ((UpdateElement)o).column) {
		    return true;
		}
	    }
	    return false;
	}

	public String toString() {
	    return pn + ": " + selected + " @row " + row + " @column " + column;
	}
    }
}
