/**
 * This dialog allows the user to add segments
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.util.StringTokenizer;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;

public class AddSegmentDialog extends JDialog implements ActionListener {

    private JComboBox cbPosition;
    private JComboBox cbSegment;

    private JButton btnAdd;
    private JButton btnClose;

    private BatchModel bModel;
    private boolean fhs = false, fts = false;

    private DefaultMutableTreeNode currentNode;

    /**
     * Constructor
     * @param frame A frame
     */
    public AddSegmentDialog(JFrame frame) {
	super(frame, "Add a segment", true);

	BoxLayout layout = new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS);
	getContentPane().setLayout(layout);
       
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	
	JLabel lbl1 = new JLabel("Choose the segment you want to add ");
	p0.add(lbl1);
	
	cbPosition = new JComboBox();
	cbPosition.addItem("Before");
	cbPosition.addItem("After");
	cbPosition.addActionListener(this);
	p0.add(cbPosition);
	
	JLabel lbl2 = new JLabel(" this position: ");
	p0.add(lbl2);

	cbSegment = new JComboBox();
	p0.add(cbSegment);

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	btnAdd = new JButton("Add");
	btnAdd.addActionListener(this);
        btnAdd.setBackground(new Color(204, 204, 255));
	p1.add(btnAdd);
	p1.add(Box.createRigidArea(new Dimension(5, 0)));
	btnClose = new JButton("Close");
	btnClose.addActionListener(this);
	btnClose.setBackground(new Color(204, 204, 255));
	p1.add(btnClose);

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.PAGE_AXIS));
	p2.add(p0);
	p2.add(Box.createRigidArea(new Dimension(0, 10)));
	p2.add(p1);
	p2.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	getContentPane().add(p2);

	init(null);
    }

    /**
     * Handles a click event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnAdd) {
	    bModel.addSegment((String)cbPosition.getSelectedItem(), (String)cbSegment.getSelectedItem(), currentNode);
	    setVisible(false);
	}
	else if(evt.getSource() == btnClose) {
	    setVisible(false);
	}
	else if(evt.getSource() == cbPosition) {
	    fillInSegment(((String)cbPosition.getSelectedItem()).toLowerCase(), BatchModel.getTypeNode(currentNode));
	}
    }

    /**
     * Set the model
     * @param model The model
     */
    public void setBatchModel(BatchModel model) {
	bModel = model;
    }

    /**
     * Init the panel
     * @param The selected node
     */
    public void init(DefaultMutableTreeNode node) {
	cbPosition.setEnabled(true);
	cbSegment.setEnabled(true);
	if(bModel != null && node != null) {
	    currentNode = node;
	    // Parse the structure	    
	    StringTokenizer tokens = new StringTokenizer(bModel.getStructure(), ",");
	    while(tokens.hasMoreTokens()) {
		String token = tokens.nextToken().trim();
		if(token.equalsIgnoreCase("FHS")) fhs = true;
		else if(token.equalsIgnoreCase("FTS")) fts = true;
	    }
	    if(BatchModel.isFHS(node)) {
		cbPosition.setSelectedItem("After");
		cbPosition.setEnabled(false);
		fillInSegment("after", "FHS");
	    }
	    else if(BatchModel.isBHS(node)) {
		cbPosition.setSelectedItem("Before");
		fillInSegment("before", "BHS");
	    }
	    else if(BatchModel.isMSG(node)) {
		cbPosition.setSelectedItem("Before");
		fillInSegment("before", "MSG");
	    }
	    else if(BatchModel.isBTS(node)) {
		cbPosition.setSelectedItem("Before");
		fillInSegment("before", "BTS");
	    }
	    else if(BatchModel.isFTS(node)) {
		cbPosition.setSelectedItem("Before");
		cbPosition.setEnabled(false);
		fillInSegment("before", "FTS");
	    }
	}
    }

    /**
     * Fill in the combobox with the good segment
     * @param pos The position Before/After
     * @param seg The segment
     */
    private void fillInSegment(String pos, String seg) {
	cbSegment.removeAllItems();
	if(seg.equals("FHS")) {
	    if(pos.equals("after")) {
		cbSegment.addItem("BHS");
		cbSegment.addItem("MSG");
		cbSegment.addItem("BTS");
		if(!fts) cbSegment.addItem("FTS");
	    }
	}
	else if(seg.equals("BHS")) {
	    if(fhs) { 
		cbPosition.setSelectedItem("After");
		cbPosition.setEnabled(false);
	    }
	    if(pos.equals("before")) {
		if(!fhs) cbSegment.addItem("FHS");
	    }
	    else if(pos.equals("after")) {
		cbSegment.removeAllItems();
		cbSegment.addItem("MSG");
		cbSegment.addItem("BTS");
		if(!fts) cbSegment.addItem("FTS");
	    }
	}
	else if(seg.equals("MSG")) {
	    if(pos.equals("before")) {
		if(!fhs) cbSegment.addItem("FHS");
		cbSegment.addItem("BHS");
	    }
	    else if(pos.equals("after")) {
		cbSegment.addItem("BTS");
		if(!fts) cbSegment.addItem("FTS");
	    }
	}
	else if(seg.equals("BTS")) {
	    if(fts) {
		cbPosition.setSelectedItem("Before");
		cbPosition.setEnabled(false);
	    }
	    if(pos.equals("before")) {
		cbSegment.removeAllItems();
		if(!fhs) cbSegment.addItem("FHS");
		cbSegment.addItem("BHS");
		cbSegment.addItem("MSG");
	    }
	    else if(pos.equals("after")) {
		if(!fts) cbSegment.addItem("FTS");
	    }
	}
	else if(seg.equals("FTS")) {
	    if(pos.equals("before")) {
		if(!fhs) cbSegment.addItem("FHS");
		cbSegment.addItem("BHS");
		cbSegment.addItem("MSG");
		cbSegment.addItem("BTS");
	    }
	}
    }
}
