/**
 * This panel displays XML and ER7 files which have been generated
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Dimension;

// Swing
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// DOM
import org.w3c.dom.Document;

// HAPI
//import ca.uhn.hl7v2.HL7Exception;

// MessageMaker
import gov.nist.hl7maker.view.display.XMLView;
import gov.nist.hl7maker.view.display.ER7View;
import gov.nist.hl7maker.util.MVCArg;

public class BatchDisplayPanel extends JPanel implements Observer {

    private XMLView xmlView;
    private ER7View er7View;

    /** 
     * Constructor
     */
    public BatchDisplayPanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	// XMLView
	xmlView = new XMLView();
	JScrollPane xmlScrollPane = new JScrollPane(xmlView);
	xmlScrollPane.setPreferredSize(new Dimension(100, 260));

	// ER7View
	er7View = new ER7View();
	JScrollPane er7ScrollPane = new JScrollPane(er7View);

	// SplitPane 
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, xmlScrollPane, er7ScrollPane);
	sp.setDividerLocation(405);
	p0.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	p0.add(sp);
	add(p0);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof BatchModel) {
	    if(arg instanceof MVCArg) {
		if("generateXML".equals(((MVCArg)arg).getName())) {
		    try {
			Document doc = (Document)((MVCArg)arg).getOValue();
			xmlView.displayMessage(doc);
			//er7View.displayMessage(doc);
			//er7View.displayMessage(doc, "2.4");
		    }
		    /*catch(ParserConfigurationException pce) {
			System.err.println("ParserConfigurationException " + pce.getMessage());
			pce.printStackTrace();
		    }
		    catch(SAXException saxe) {
			System.err.println("SAXException " + saxe.getMessage());
			saxe.printStackTrace();
		    }
		    catch(IOException ioe) {
			System.err.println("IOException " + ioe.getMessage());
			ioe.printStackTrace();
			}*/
		    catch(Exception e) { System.err.println("Exception: " + e.getMessage()); e.printStackTrace();}
		    /*catch(HL7Exception hl7e) {
			System.err.println("HL7Exception " + hl7e.getMessage());
			hl7e.printStackTrace();
			}*/
		}
		else if("generateER7".equals(((MVCArg)arg).getName())) {
		    //er7View.displayMessage(((MVCArg)arg).getValue());
		    try {
			Document doc = (Document)((MVCArg)arg).getOValue();
			er7View.displayMessage(doc);
		    }
		    catch(Exception e) { System.err.println("Exception: " + e.getMessage()); e.printStackTrace();}
		}
		else if("init".equals(((MVCArg)arg).getName())) {
		    init();
		}
	    }
	}
    }

    /**
     * Init the panel
     */
    public void init() {
	xmlView.reset();
	er7View.reset();
    }
}
