/**
 * This panel displays information on the current element and allows the user to modify some of the information
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Observer;
import java.util.Observable;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

// Swing
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.text.AttributeSet;
import javax.swing.text.PlainDocument;
import javax.swing.text.BadLocationException;

// MessageMaker
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.view.config.ProfileNode;

public class BatchElementValuePanel extends JPanel implements Observer, ActionListener, ListSelectionListener {

    private JTextField txtElementName;
    private JComboBox cbUsage;
    private JTextField txtDatatype;
    private JTextField txtLength;
    private JTextField txtItemNo;
    private JTextField txtValue;

    private JPanel pMultipleValue;
    private JPanel p6;

    private JList lstValue;
    private JScrollPane lstValueScrollPane;
    private JButton btnAdd;
    private JButton btnRemove;

    private Vector vListModel;
    private ProfileNode pn;

    private LengthDocument lengthDocument;
    private ValueDocument valueDocument;

    /**
     * Constructor
     */
    public BatchElementValuePanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JLabel lblElementName = new JLabel("Element Name: ");
	p0.add(Box.createRigidArea(new Dimension(100, 0)));
	p0.add(lblElementName);
	txtElementName = new JTextField();
	txtElementName.setEditable(false);
	txtElementName.setBackground(Color.WHITE);
	p0.add(txtElementName);
	p0.add(Box.createRigidArea(new Dimension(100, 0)));

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.LINE_AXIS));
	JLabel lblUsage = new JLabel("Usage: ");
	p1.add(Box.createRigidArea(new Dimension(100, 0)));
	p1.add(lblUsage);
	String[] sUsage = {"R","RE","X"};
	cbUsage = new JComboBox(sUsage);
	cbUsage.addActionListener(this);
	p1.add(cbUsage);
	p1.add(Box.createRigidArea(new Dimension(100, 0)));

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));
	JLabel lblDatatype = new JLabel("Datatype: ");
	p2.add(Box.createRigidArea(new Dimension(100, 0)));
	p2.add(lblDatatype);
	txtDatatype = new JTextField();
	txtDatatype.setEditable(false);
	txtDatatype.setBackground(Color.WHITE);
	p2.add(txtDatatype);
	p2.add(Box.createRigidArea(new Dimension(100, 0)));

	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));
	JLabel lblLength = new JLabel("Length: ");
	p3.add(Box.createRigidArea(new Dimension(100, 0)));
	p3.add(lblLength);
	txtLength = new JTextField();
	lengthDocument = new LengthDocument();
	txtLength.setDocument(lengthDocument);
	p3.add(txtLength);
	p3.add(Box.createRigidArea(new Dimension(100, 0)));

	JPanel p4 = new JPanel();
	p4.setLayout(new BoxLayout(p4, BoxLayout.LINE_AXIS));
	JLabel lblItemNo = new JLabel("Item Number: ");
	p4.add(Box.createRigidArea(new Dimension(100, 0)));
	p4.add(lblItemNo);
	txtItemNo = new JTextField();
	txtItemNo.setEditable(false);
	txtItemNo.setBackground(Color.WHITE);
	p4.add(txtItemNo);
	p4.add(Box.createRigidArea(new Dimension(100, 0)));

	p6 = new JPanel();
	p6.setLayout(new BoxLayout(p6, BoxLayout.LINE_AXIS));
	p6.add(Box.createRigidArea(new Dimension(100, 0)));
	JLabel lblValue = new JLabel("Value: ");
	p6.add(lblValue);
	txtValue = new JTextField();
	valueDocument = new ValueDocument();
	txtValue.setDocument(valueDocument);
	p6.add(txtValue);
	p6.add(Box.createRigidArea(new Dimension(100, 0)));

	add(p0);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(p1);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(p2);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(p3);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(p4);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(p6);
	add(Box.createRigidArea(new Dimension(0, 5)));
	add(Box.createRigidArea(new Dimension(0, 185)));
	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	pMultipleValue = new JPanel();
	pMultipleValue.setLayout(new BoxLayout(pMultipleValue, BoxLayout.LINE_AXIS));

	JPanel p7 = new JPanel();
	p7.setLayout(new BoxLayout(p7, BoxLayout.PAGE_AXIS));
	lstValue = new JList();
	lstValue.addListSelectionListener(this);
	lstValueScrollPane = new JScrollPane(lstValue);
	p7.add(lstValueScrollPane);
	p7.add(Box.createRigidArea(new Dimension(0, 5)));

	JPanel p8 = new JPanel();
	p8.setLayout(new BoxLayout(p8, BoxLayout.PAGE_AXIS));
	btnAdd = new JButton("Add");
	btnAdd.addActionListener(this);
	btnAdd.setBackground(new Color(204, 204, 255));
	p8.add(btnAdd);
	p8.add(Box.createRigidArea(new Dimension(0, 5)));
	btnRemove = new JButton("Remove");
	btnRemove.addActionListener(this);
	btnRemove.setBackground(new Color(204, 204, 255));
	p8.add(btnRemove);

	pMultipleValue.add(Box.createRigidArea(new Dimension(100, 0)));
	pMultipleValue.add(p7);
	pMultipleValue.add(Box.createRigidArea(new Dimension(10, 0)));
	pMultipleValue.add(p8);
	pMultipleValue.add(Box.createRigidArea(new Dimension(100, 0)));

	init();
    }

    /**
     * Handles a click event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnAdd) {
	    String error = "";
	    String value = txtValue.getText();
	    // Check length and datatype
	    try {
		if(value.length() > Integer.parseInt(txtLength.getText()))
		    error += "Length is incorrect.\n";
	    }
	    catch(NumberFormatException nfe) {
		error += "The length value must be an integer.\n";
	    }
	    if(pn.getDatatype().equals("NM")) {
		try {
		    Integer.parseInt(value);
		}
		catch(NumberFormatException nfe) {
		    error += "The value must be an integer.\n";
		}
	    }
	    if(error.equals("")) {
		vListModel.add(txtValue.getText());
		lstValue.setListData(vListModel);
		txtValue.setText("");
		ListModel lstModel = lstValue.getModel();
		String sValue = "";
		for(int i=0;i<lstModel.getSize();i++) {
		    sValue += (String)lstModel.getElementAt(i);
		    if(i != lstModel.getSize()-1)
			sValue += "||";
		}
		pn.setBatchValue(sValue);
	    }
	    else {
		JOptionPane.showMessageDialog(this, error, "Value error", JOptionPane.ERROR_MESSAGE);
	    }
	}
	else if(evt.getSource() == btnRemove) {
	    vListModel.remove(lstValue.getSelectedValue());
	    lstValue.setListData(vListModel);
	    btnRemove.setEnabled(false);
	    ListModel lstModel = lstValue.getModel();
	    String sValue = "";
	    for(int i=0;i<lstModel.getSize();i++) {
		sValue += (String)lstModel.getElementAt(i);
		if(i != lstModel.getSize()-1)
		    sValue += "||";
	    }
	    pn.setBatchValue(sValue);
	}
	else if(evt.getSource() == cbUsage) {
	    pn.setUsage((String)cbUsage.getSelectedItem());
	}
    }

    /**
     * Handles a list click event
     * @param evt The event
     */
    public void valueChanged(ListSelectionEvent evt) {
	btnRemove.setEnabled(true);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof BatchModel) {
	    if(arg instanceof MVCArg) {
		if("node".equals(((MVCArg)arg).getName())) {
		    Object obj = ((MVCArg)arg).getOValue();
		    if(obj instanceof ProfileNode) {
			pn = (ProfileNode)obj;
			// Fill in the form
			txtElementName.setText(pn.getName());
			cbUsage.setSelectedItem(pn.getUsage());
			txtDatatype.setText(pn.getDatatype());
			try {
			    lengthDocument.init();
			    valueDocument.init();
			}
			catch(BadLocationException ble) {
			    System.err.println("BadLocationException " + ble.getMessage());
			    ble.printStackTrace();
			}
			txtLength.setText(pn.getLength());
			txtItemNo.setText(pn.getItemNo());
			if(!pn.getCardMax().equals("*")) {
			    // Set the panel with one value
			    this.remove(pMultipleValue);
			    this.remove(this.getComponent(this.getComponents().length-1));
			    this.add(Box.createRigidArea(new Dimension(0, 185)));
			    txtValue.setText(pn.getBatchValue());
			}
			else {
			    // Set the panel with mutilple value
			    this.remove(pMultipleValue);
			    this.add(pMultipleValue, getComponents().length - 1);
			    this.remove(this.getComponent(this.getComponents().length-1));
			    this.add(Box.createRigidArea(new Dimension(0, 50)));
			    vListModel.clear();
			    if(pn.getBatchValue() != null) {
				StringTokenizer tokens = new StringTokenizer(pn.getBatchValue(), "||");
				while(tokens.hasMoreTokens())
				    vListModel.add(tokens.nextToken());
			    }
			    lstValue.setListData(vListModel);
			    txtValue.setText("");
			}
			updateUI();
		    }
		}
		else if("init".equals(((MVCArg)arg).getName())) {
		    init();
		}
	    }
	}
    }

    /**
     * Init the panel
     */
    private void init() {
	btnAdd.setEnabled(true);
	btnRemove.setEnabled(false);
	vListModel = new Vector();
    }

    /**
     * Set the values in the JList
     * @param values The list of values
     */
    private void setLstValue(String values) {
	Vector v = new Vector();
	if(values != null) {
	    StringTokenizer tokens = new StringTokenizer(values, "||");
	    while(tokens.hasMoreTokens()) {
		v.add(tokens.nextToken());
	    }
	}
	lstValue.setListData(v);
    }

    class LengthDocument extends PlainDocument {

	/** 
	 * Inserts some content into the document.
	 */
	public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
	    // Test if it is an integer
	    try {
		Integer.parseInt(str);
	    }
	    catch(Exception e) {
		return;
	    }
	    super.insertString(offs, str, a);
	    pn.setLength(getText(0, getLength()));
	}

	/**
	 * Removes some content from the document.
	 */
	public void remove(int offs, int len) throws BadLocationException {
	    super.remove(offs, len);
	    pn.setLength(getText(0, getLength()));
	}

	/**
	 * Init the document
	 */
	public void init() throws BadLocationException {
	    super.remove(0, getLength());
	}
    }

    class ValueDocument extends PlainDocument {

	/** 
	 * Inserts some content into the document.
	 */
	public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
	    String value = txtValue.getText();
	    // Check datatype
	    if(pn.getDatatype().equals("NM")) {
		try {
		    Integer.parseInt(str);
		}
		catch(NumberFormatException nfe) {
		    return;
		}
	    }
	    if(value.length() >= Integer.parseInt(txtLength.getText()))
		return;
	    super.insertString(offs, str, a);
	    if(pn.getCardMax().equals("1")) {
		if(pn.getCardMax().equals("1")) {
		    pn.setBatchValue(getText(0, getLength()));
		}
	    }
	}

	/**
	 * Removes some content from the document.
	 */
	public void remove(int offs, int len) throws BadLocationException {
	    String value = txtValue.getText();
	    super.remove(offs, len);
	    if(pn.getCardMax().equals("1")) {
		if(pn.getCardMax().equals("1")) {
		    pn.setBatchValue(value);
		}
	    }
	}

	/**
	 * Init the document
	 */
	public void init() throws BadLocationException {
	    super.remove(0, getLength());
	}
    }
}
