/**
 * This panel contains buttons to generate the XML Batch File and also for the ER7 format
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.io.File;
import java.io.IOException;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.StringTokenizer;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;

// AWT
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;

// XML
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

// XSL
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

// SAX
import org.xml.sax.SAXException;  

// HAPI
/*import ca.uhn.hl7v2.parser.DefaultXMLParser;
import ca.uhn.hl7v2.parser.PipeParser;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.HL7Exception;*/

// MessageMaker
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.XMLFileFilter;
import gov.nist.hl7maker.util.ER7FileFilter;
//import gov.nist.hl7maker.util.hapi.CustomHAPI;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.view.config.SegmentNode;
import gov.nist.hl7maker.view.config.FieldNode;
import gov.nist.hl7maker.view.config.ComponentNode;
import gov.nist.hl7maker.view.display.ER7View;
import gov.nist.hl7maker.view.display.XMLDirectory;
import gov.nist.hl7maker.view.display.XMLtoER7;

public class BatchGenerationPanel extends JPanel implements ActionListener, Observer {

    private BatchModel bModel;

    private JButton btnGenerate;
    private JButton btnExportXML;
    private JButton btnExportER7;

    private final String BATCH_DEFINITION_FILE = "data/BatchSegments.xml";

    private String batchER7;
    private Vector vProfile;

    /**
     * Constructor
     */
    public BatchGenerationPanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));

	JLabel lbl = new JLabel("Generate batch file: ");
	p0.add(lbl);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	btnGenerate = new JButton("Generate");
	btnGenerate.setBackground(new Color(204, 204, 255));
	btnGenerate.addActionListener(this);
	p0.add(btnGenerate);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));
	
	btnExportXML = new JButton("Export XML ...");
	btnExportXML.setBackground(new Color(204, 204, 255));
	btnExportXML.addActionListener(this);
	p0.add(btnExportXML);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));
	
	btnExportER7 = new JButton("Export ER7 ...");
	btnExportER7.setBackground(new Color(204, 204, 255));
	btnExportER7.addActionListener(this);
	p0.add(btnExportER7);

	add(p0);
	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	vProfile = new Vector();
	init();
    }

    /**
     * Handles a click event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnGenerate) {
	    generateXML();
	    if(errorXML.equals("")) {
		bModel.fireEvent(new MVCArg("generateXML", doc));
		generateER7();
		bModel.fireEvent(new MVCArg("generateER7", docER7));
		btnExportXML.setEnabled(true);
		btnExportER7.setEnabled(true);
	    }
	    else {
		JOptionPane.showMessageDialog(this, errorXML, "Value error", JOptionPane.ERROR_MESSAGE);
	    }
	}
	else if(evt.getSource() == btnExportXML) {
	    // Open a JFileChooser
	    JFileChooser fc = new JFileChooser(User.getStaticOpenedDir());
	    // Set the chooser to accept only XML files.
	    fc.addChoosableFileFilter(new XMLFileFilter());
	    fc.setAcceptAllFileFilterUsed(false);
	    fc.setSelectedFile(new File("Batch.xml"));
	    int returnVal = fc.showOpenDialog(this);
	    File f = null;
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		f = fc.getSelectedFile();
		if(!f.getPath().endsWith(".xml")) {
		    f = new File(f.getPath() + ".xml");
		}
		User.setStaticOpenedDir(f.getParent());
		// Write the DOM document to the file
		try {
		    Transformer transformer = TransformerFactory.newInstance().newTransformer();
		    DOMSource source = new DOMSource(doc);
		    StreamResult result = new StreamResult(f);
		    transformer.transform(source, result);
		}
		catch(TransformerConfigurationException tce) {
		    System.err.println("TransformerConfigurationException " + tce.getMessage());
		    tce.printStackTrace();
		}
		catch(TransformerException te) {
		    System.err.println("TransformerException " + te.getMessage());
		    te.printStackTrace();
		}
	    }
	}
	else if(evt.getSource() == btnExportER7) {
	    // Open a JFileChooser
	    JFileChooser fc = new JFileChooser(User.getStaticOpenedDir());
	    // Set the chooser to accept only XML files.
	    fc.addChoosableFileFilter(new ER7FileFilter());
	    fc.setAcceptAllFileFilterUsed(false);
	    fc.setSelectedFile(new File("Batch.er7"));
	    int returnVal = fc.showOpenDialog(this);
	    File f = null;
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		f = fc.getSelectedFile();
		if(!f.getPath().endsWith(".er7")) {
		    f = new File(f.getPath() + ".er7");
		}
		User.setStaticOpenedDir(f.getParent());
		// Write the DOM document to the file
		try {
		    BufferedWriter bw = new BufferedWriter(new FileWriter(f));
 		    bw.write(XMLtoER7.transform(docER7));
 		    bw.close();
		}
		catch(IOException ioe) {
		    System.err.println("IOException: " + ioe.getMessage());
		    ioe.printStackTrace();
		}
/*		try {
		    Transformer transformer = TransformerFactory.newInstance().newTransformer();
		    DOMSource source = new DOMSource(docER7);
		    StreamResult result = new StreamResult(f);
		    transformer.transform(source, result);
		}
		catch(TransformerConfigurationException tce) {
		    System.err.println("TransformerConfigurationException " + tce.getMessage());
		    tce.printStackTrace();
		}
		catch(TransformerException te) {
		    System.err.println("TransformerException " + te.getMessage());
		    te.printStackTrace();
		    }*/
	    }
	}
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof BatchModel) {
	    if(arg instanceof MVCArg) {
		if("structure".equals(((MVCArg)arg).getName())) {
		    if(!((MVCArg)arg).getValue().equals("")) {
			btnGenerate.setEnabled(true);
			btnExportXML.setEnabled(false);
			btnExportER7.setEnabled(false);
		    }
		    else {
			init();
		    }
		}
	    }
	}
    }

    /**
     * Set the model
     * @param model The model
     */
    public void setBatchModel(BatchModel model) {
	bModel = model;
    }

    /**
     * Init the panel
     */
    private void init() {
	btnGenerate.setEnabled(false);
	btnExportXML.setEnabled(false);
	btnExportER7.setEnabled(false);
    }

    private String segNodeName;
    private String errorXML;

    private Document doc;
    private Element xmlRoot;
    private Element xmlSeg;
    private Element xmlField;
    private Element xmlComponent;

    private Vector vBatchSeg, vBatchMsg;
    private Document docMsgBatch;
    private Element xmlMsgBatch;
    private Element xmlRootMsgBatch;

    private Document docER7;
    private Element xmlER7Root;

    /**
     * Generate the XML batch file
     */
    public void generateXML() {
	DefaultMutableTreeNode root = bModel.getTree();
	//Create a XML Document
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    doc = builder.newDocument();
	    docER7 = builder.newDocument();
	    docMsgBatch = builder.newDocument();
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	// Document to store the messages
	xmlRootMsgBatch = docMsgBatch.createElement("FAKEROOT");
	docMsgBatch.appendChild(xmlRootMsgBatch);
	
	// Create the root element
	vBatchSeg = new Vector();
	vBatchMsg = new Vector();
	xmlRoot = doc.createElement("BATCH");
	doc.appendChild(xmlRoot);
	xmlER7Root = docER7.createElement("ER7");
	docER7.appendChild(xmlER7Root);
	xmlMsgBatch = null;

	generateXMLNode(root);
    }
    
    private void generateXMLNode(DefaultMutableTreeNode node) {
	Object obj = node.getUserObject();
	
	if(obj instanceof SegmentNode) {
	    SegmentNode segNode = (SegmentNode)obj;
	    segNodeName = segNode.getName();
	    if(segNodeName.equals("BHS")) {
		xmlMsgBatch = doc.createElement("MESSAGEBATCH");
		xmlRoot.appendChild(xmlMsgBatch);
	    }
	    else if(segNodeName.equals("FTS")) {
		xmlMsgBatch = null;
	    }
	    xmlSeg = doc.createElement(segNodeName);
	    if(xmlMsgBatch == null)
		xmlRoot.appendChild(xmlSeg);
	    else
		xmlMsgBatch.appendChild(xmlSeg);
	    vBatchSeg.add(xmlSeg);
	    vBatchMsg.add(segNodeName);
	    vBatchMsg.add(null);
	    //System.out.println("<" + segNode.getName() + ">");
	}
	else if(obj instanceof FieldNode) {
	    FieldNode fieldNode = (FieldNode)obj;
	    String fieldNodeName = fieldNode.getName();
	    int idx = node.getParent().getIndex(node) + 1;
	    xmlField = doc.createElement(segNodeName + "." + idx);
	    if(node.isLeaf()) {
		// Check the validity of usage
		String usage = fieldNode.getUsage();
		String value = fieldNode.getBatchValue();
		if(usage.equals("R")) {
		    if(value == null || (value != null) && value.equals(""))
			errorXML += (segNodeName + ": " + fieldNodeName + " is required but no value is provided.\n");
		    else {
			if(fieldNode.getCardMax().equals("1")) {
			    xmlField.appendChild(doc.createTextNode(value));
			    xmlSeg.appendChild(xmlField);
			}
			else if(fieldNode.getCardMax().equals("*")) {
			    StringTokenizer tokens = new StringTokenizer(value, "||");
			    while(tokens.hasMoreTokens()) {
				xmlField = doc.createElement(segNodeName + "." + idx);
				xmlField.appendChild(doc.createTextNode(tokens.nextToken()));
				xmlSeg.appendChild(xmlField);
			    }
			}
		    }
		    //System.out.println("<" + segNodeName + "." + idx + ">" + value + "</" + segNodeName + "." + idx + ">");
		}
		else if(usage.equals("RE")) {
		    if(value != null  && !value.equals("")) {
			if(fieldNode.getCardMax().equals("1")) {
			    xmlField.appendChild(doc.createTextNode(value));
			    xmlSeg.appendChild(xmlField);
			}
			else if(fieldNode.getCardMax().equals("*")) {
			    StringTokenizer tokens = new StringTokenizer(value, "||");
			    while(tokens.hasMoreTokens()) {
				xmlField = doc.createElement(segNodeName + "." + idx);
				xmlField.appendChild(doc.createTextNode(tokens.nextToken()));
				xmlSeg.appendChild(xmlField);
			    }
			}
		    }
		    //			System.out.println("<" + segNodeName + "." + idx + ">" + value + "</" + segNodeName + "." + idx + ">");
		}
		idx++;
	    }
	}
	else if(obj instanceof ComponentNode) {
	    ComponentNode componentNode = (ComponentNode)obj;
	    String componentNodeName = ((FieldNode)(((DefaultMutableTreeNode)node.getParent()).getUserObject())).getDatatype();
	    int idx = node.getParent().getIndex(node) + 1;
	    xmlComponent = doc.createElement(componentNodeName + "." + idx);
	    // Check the validity of usage
	    String usage = componentNode.getUsage();
	    String value = componentNode.getBatchValue();
	    if(usage.equals("R")) {
		if(value == null || (value != null) && value.equals(""))
		    errorXML += (segNodeName + ": " + componentNode.getName() + " is required but no value is provided.\n");
		else {
		    xmlComponent.appendChild(doc.createTextNode(value));
		    xmlField.appendChild(xmlComponent);
		    xmlSeg.appendChild(xmlField);
		}
		    //System.out.println("<" + componentNodeName + "." + idx + ">" + value + "</" + componentNodeName + "." + idx + ">");
	    }
	    else if(usage.equals("RE")) {
		if(value != null  && !value.equals("")) {
		    xmlComponent.appendChild(doc.createTextNode(value));
		    xmlField.appendChild(xmlComponent);
		    xmlSeg.appendChild(xmlField);
		}
		    //		    System.out.println("<" + componentNodeName + "." + idx + ">" + value + "</" + componentNodeName + "." + idx + ">");
	    }
	}
	else if(obj instanceof String) {
	    String s = (String)obj;
	    if(s.equals("Batch")) {
		errorXML = "";
	    }
	}
	else if(obj instanceof MessageNode) {
	    if(xmlMsgBatch == null) {
		xmlMsgBatch = doc.createElement("MESSAGEBATCH");
		xmlRoot.appendChild(xmlMsgBatch);
	    }
	    MessageNode mn = (MessageNode)obj;
	    Vector vMsg = mn.getMessage();
	    String sMsgBatch = "";
	    for(int i=0;i<vMsg.size();i++) {
		try {
		    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		    DocumentBuilder builder = factory.newDocumentBuilder();
		    Document msgDoc = builder.parse(new File((String)vMsg.get(i)));
		    NodeList nl = (msgDoc.getDocumentElement().getChildNodes());
		    for(int j=0;j<nl.getLength();j++) {
			Node n = (Node)nl.item(j);
			if(n.getNodeType() == Node.ELEMENT_NODE)
			    xmlRootMsgBatch.appendChild(docMsgBatch.importNode(n, true));
			vBatchSeg.add(n);
		    }
		    //vBatchMsg.add(vMsg.get(i));
		    //vBatchMsg.add(msgDoc);
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}
	    }
	    sMsgBatch = getStringMsgBatch(docMsgBatch);
	    xmlMsgBatch.appendChild(doc.createCDATASection(sMsgBatch));
	}
	// Apply to children
	for(int i=0;i<node.getChildCount();i++) {
	    generateXMLNode((DefaultMutableTreeNode)node.getChildAt(i));
	}
    }

    /**
     * Get the messages as a string to put it in a CDATA section
     * @param doc The document
     * @return The messages in a string format
     */
    public String getStringMsgBatch(Document doc) {
	String sMsgBatch = "";
	try {
	    Transformer transformer = TransformerFactory.newInstance().newTransformer();
	    DOMSource source = new DOMSource(doc);
	    StringWriter sw = new StringWriter();
	    StreamResult result = new StreamResult(sw);
	    transformer.setOutputProperty(OutputKeys.METHOD, "xml");
	    transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
	    transformer.setOutputProperty(OutputKeys.INDENT, "yes");
	    transformer.transform(source, result);
	    sMsgBatch = sw.toString().replaceAll("<FAKEROOT>", "").replaceAll("</FAKEROOT>", "").substring(38);
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
	return sMsgBatch;
    }

    /**
     * Generate the ER7 file
     */
    public void generateER7() {
	for(int i=0;i<vBatchSeg.size();i++) {
	    xmlER7Root.appendChild(docER7.importNode((Node)vBatchSeg.get(i), true));
	}

/*	try {

	    CustomHAPI cHAPI = new CustomHAPI();
	    cHAPI.generateGroup(BATCH_DEFINITION_FILE, "");

	    // Create a fake DOM Object with only the batch segment FHS, FTS, BHS, BTS
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document batch = builder.newDocument();

	    Element xmlBatchRoot = batch.createElement("BATCH");
	    batch.appendChild(xmlBatchRoot);
	    for(int i=0;i<vBatchSeg.size();i++) {
		xmlBatchRoot.appendChild(batch.importNode((Node)vBatchSeg.get(i), true));
	    }

	    // Add a fake MSH segment
	    Element msh = (Element)batch.createElement("MSH");
	    Element msh1 = (Element)batch.createElement("MSH.1");
	    msh1.appendChild(batch.createTextNode("|"));
	    msh.appendChild(msh1);
	    Element msh2 = (Element)batch.createElement("MSH.2");
	    msh2.appendChild(batch.createTextNode("^~\\&"));
	    msh.appendChild(msh2);
	    batch.getDocumentElement().appendChild(msh);
	    batch = cHAPI.modifyDocument(batch, "");	

	    // Convert to ER7
	    DefaultXMLParser xmlParser = new DefaultXMLParser();
	    Message msg = xmlParser.parseDocument(batch, "2.4");
	    PipeParser pipeParser = new PipeParser();
	    String batchSeg = CustomHAPI.cleanER7(ER7View.formatER7(pipeParser.encode(msg)), "");

	    String[] msgSeg = new String[vBatchMsg.size()/2];
	    // Process the messages
	    for(int i=0;i<vBatchMsg.size()/2;i++) {
		if(vBatchMsg.get(i*2+1) != null) {
		    File f = new File((String)vBatchMsg.get(i*2));
		    XMLDirectory xmlDir = new XMLDirectory(f.getParent());
		    String profile = xmlDir.getProfileFromMessage(f.getPath());

		    // Check if the message has a Z-Segment or group segment
		    final boolean needCustom = CustomHAPI.needCustom(profile);
		    final boolean needGeneration = needCustom && !vProfile.contains(profile);
		    if(needGeneration) vProfile.add(profile);
		    
		    // Customize HAPI
		    if(needGeneration) {
			cHAPI.generate(profile);
		    }
		    factory = DocumentBuilderFactory.newInstance();
		    builder = factory.newDocumentBuilder();
		    //Document msgDoc = builder.parse(new File((String)vBatchMsg.get(i)));
		    Document msgDoc = (Document)vBatchMsg.get(i*2+1);
		    
		    String prefix = "";
		    if(needCustom) {
			prefix = new File(profile).getName();
			prefix = prefix.substring(0, prefix.length()-4).replaceAll("_","XX").replaceAll(" ","XX");
			msgDoc = cHAPI.modifyDocument(msgDoc, prefix);
		    }
		    
		    // Convert to ER7
		    xmlParser = new DefaultXMLParser();
		    msg = xmlParser.parseDocument(msgDoc, "2.4");
		    pipeParser = new PipeParser();
		    msgSeg[i] = CustomHAPI.cleanER7(ER7View.formatER7(pipeParser.encode(msg)), prefix);
		}
		else msgSeg[i] = (String)vBatchMsg.get(i*2);
	    }
	    batchER7 = cHAPI.getBatchER7(batchSeg, msgSeg);
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(TransformerConfigurationException tce) {
	    System.err.println("TransformerConfigurationException " + tce.getMessage());
	    tce.printStackTrace();
	}
	catch(TransformerException te) {
	    System.err.println("TransformerException " + te.getMessage());
	    te.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException: " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	catch(HL7Exception hl7e) {
	    System.err.println("HL7Exception: " + hl7e.getMessage());
	    hl7e.printStackTrace();
	    }*/
    }
}
