/**
 * This panel allows the user to select the messages, he wants to include in the batch
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.io.File;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;

//AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JFileChooser;
import javax.swing.BorderFactory;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

// XML
import javax.xml.parsers.ParserConfigurationException;

// SAX
import org.xml.sax.SAXException;  

// MessageMaker
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.util.Constants;
import gov.nist.hl7maker.control.User;
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.view.display.XMLDirectory;
import gov.nist.hl7maker.view.display.MetaData;

public class BatchMessageSelectionPanel extends JPanel implements Observer, ActionListener, TreeSelectionListener, ListSelectionListener {

    private BatchModel bModel;

    private JTree tree;
    private JScrollPane treeScrollPane;
    private JButton btnMount;

    private JList lstMsg;
    private JScrollPane lstMsgScrollPane;
    private JButton btnAdd;
    private JButton btnRemove;

    /**
     * Constructor
     */
    public BatchMessageSelectionPanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));

	tree = new JTree(new Vector());
	tree.addTreeSelectionListener(this);
	treeScrollPane = new JScrollPane(tree);
	treeScrollPane.setPreferredSize(new Dimension(250, 150));
	p0.add(treeScrollPane);
	p0.add(Box.createRigidArea(new Dimension(0, 5)));

	JPanel p3 = new JPanel();
	p3.setLayout(new BoxLayout(p3, BoxLayout.LINE_AXIS));

	btnMount = new JButton("Mount Directory");
	btnMount.addActionListener(this);
        btnMount.setBackground(new Color(204, 204, 255));
	p3.add(btnMount);

	p0.add(p3);

	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.PAGE_AXIS));

	lstMsg = new JList();
	lstMsg.addListSelectionListener(this);
	lstMsgScrollPane = new JScrollPane(lstMsg);
	p1.add(lstMsgScrollPane);
	p1.add(Box.createRigidArea(new Dimension(0, 5)));

	JPanel p2 = new JPanel();
	p2.setLayout(new BoxLayout(p2, BoxLayout.LINE_AXIS));

	btnAdd = new JButton("Add");
	btnAdd.addActionListener(this);
	btnAdd.setBackground(new Color(204, 204, 255));
	p2.add(btnAdd);
	p2.add(Box.createRigidArea(new Dimension(10, 0)));

	btnRemove = new JButton("Remove");
	btnRemove.addActionListener(this);
	btnRemove.setBackground(new Color(204, 204, 255));
	p2.add(btnRemove);

	p1.add(p2);

	add(p0);
	add(Box.createRigidArea(new Dimension(0, 10)));
	add(p1);
	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	init();
    }

    /**
     * Handles a click event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnMount) {
	    JFileChooser fc = new JFileChooser(User.getStaticOpenedDir());
	    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    int returnVal = fc.showOpenDialog(this);

	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fc.getSelectedFile();
		XMLDirectory x = new XMLDirectory(file.getPath());
		bModel.addDirectory(new XMLDirectory(file.getPath()));
		updateDir();
		User.setStaticOpenedDir(file.getPath());
	    }
	}
	else if(evt.getSource() == btnAdd) {
	    Vector vMsg = getMessageList();
	    for(int i=0;i<vMsg.size();i++) {
		bModel.addMessage((String)vMsg.get(i));
	    }
	    updateMsg();
	    btnAdd.setEnabled(false);
	}
	else if(evt.getSource() == btnRemove) {
	    Object[] o = lstMsg.getSelectedValues();
	    for(int i=0;i<o.length;i++) {
		bModel.removeMessage((String)o[i]);
	    }
	    updateMsg();
	    btnRemove.setEnabled(false);
	}
    }

    /**
     * Handles a tree click event
     * @param evt The event
     */
    public void valueChanged(TreeSelectionEvent evt) {
	btnAdd.setEnabled(true);
    }

    /**
     * Handles a list click event
     * @param evt The event
     */
    public void valueChanged(ListSelectionEvent evt) {
	btnRemove.setEnabled(true);
    }
 
    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof BatchModel) {
	    if(arg instanceof MVCArg) {
		if("node".equals(((MVCArg)arg).getName())) {
		    Object obj = ((MVCArg)arg).getOValue();
		    if(obj instanceof MessageNode) {
			updateDir();
			updateMsg();
		    }
		}
		else if("init".equals(((MVCArg)arg).getName())) {
		    init();
		}
	    }
	}
	else if(o instanceof Session) {
	    if(arg instanceof MVCArg) {
		if(((MVCArg)arg).getName().compareTo("msgPath") == 0) {
		    bModel.addDirectory(new XMLDirectory(((MVCArg)arg).getValue()));
		    updateDir();
		}
		else if(((MVCArg)arg).getName().compareTo("mountDir") == 0) {
		    bModel.addDirectory(new XMLDirectory(((MVCArg)arg).getValue()));
		    updateDir();
		}
	    }
	    else if(arg instanceof Session) {
		    bModel.addDirectory(new XMLDirectory(((Session)arg).getMsgPath()));
		    updateDir();
	    }
	}
    }

    /**
     * Set the model
     * @param model The model
     */
    public void setBatchModel(BatchModel model) {
	bModel = model;
    }

    /**
     * Update the tree of directories and messages
     */
    private void updateDir() {
	Vector vDir = bModel.getDirectory();

	DefaultMutableTreeNode top = new DefaultMutableTreeNode("Messages");
	DefaultMutableTreeNode dir = null;
	if(vDir.size() > 0) {
	    for(int i=0;i<vDir.size();i++) {
		XMLDirectory xmlDir = (XMLDirectory)vDir.get(i);
		try {
		    if(xmlDir.getTree() != null) {
			top.add(xmlDir.getTree());
		    }
		}
		catch(ParserConfigurationException pce) {
		    System.err.println("ParserConfigurationException " + pce.getMessage());
		    pce.printStackTrace();
		}
		catch(SAXException saxe) {
		    System.err.println("SAXException " + saxe.getMessage());
		    saxe.printStackTrace();
		}
		catch(IOException ioe) {
		    System.err.println("IOException " + ioe.getMessage());
		    ioe.printStackTrace();
		}
	    }
	}
	tree.setModel(new DefaultTreeModel(top));
    }
    
    /**
     * Update the list of messages
     */
    private void updateMsg() {
	Vector vMsg = bModel.getMessage();
	if(vMsg != null)
	    lstMsg.setListData(vMsg);
    }

    /**
     * List messages affected by the addition
     * @return List of messages
     */
    private Vector getMessageList() {
	try {
	    Vector v = new Vector();
	    DefaultMutableTreeNode curNode = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
	    if(curNode != null) {
		String val = (String)curNode.getUserObject();
		int idxSlash = val.indexOf(Constants.FS);
		int idxDotXML = val.indexOf(".xml");
		// Messages Node (Root)
		if(val.compareTo("Messages") == 0) {
		    // Do nothing
		}
		// Directory
		else if(idxSlash != -1 && idxDotXML == -1) {
		    // List all files (MetaData and files associated)
		    XMLDirectory xmlDir = new XMLDirectory((String)curNode.getUserObject());
		    Vector vTmp = xmlDir.getMessages();
		    for(int i=0;i<vTmp.size();i++) {
			String sMsg = (String)vTmp.get(i);
			if(!sMsg.startsWith("TD_")) {
			    v.add(curNode.getUserObject() + Constants.FS + vTmp.get(i));
			}
		    }
		}
		// Message Node
		else if (idxSlash == -1 && idxDotXML != -1) {
		    // Return only this message
		    v.add((String)((DefaultMutableTreeNode)curNode.getParent().getParent()).getUserObject() + Constants.FS + curNode.getUserObject());
		}
		// MetaData Node
		else if(idxSlash != -1 && idxDotXML != -1) {
		    // Return metadata and files associated
		    MetaData md = new MetaData((String)curNode.getUserObject());
		    Vector vTmp = md.getMessageList();
		    for(int i=0;i<vTmp.size();i++) {
			String sMsg = (String)vTmp.get(i);
			v.add((String)((DefaultMutableTreeNode)curNode.getParent()).getUserObject() + Constants.FS + sMsg);
		    }
		}
	    }
	    return v;
	}
	catch(ParserConfigurationException pce) {
	    System.err.println("ParserConfigurationException " + pce.getMessage());
	    pce.printStackTrace();
	}
	catch(SAXException saxe) {
	    System.err.println("SAXException " + saxe.getMessage());
	    saxe.printStackTrace();
	}
	catch(IOException ioe) {
	    System.err.println("IOException " + ioe.getMessage());
	    ioe.printStackTrace();
	}
	return null;
    }

    /**
     * Init the panel
     */
    private void init() {
	btnAdd.setEnabled(false);
	btnRemove.setEnabled(false);
	/*if(bModel != null) {
	    bModel.removeAllDirectory();
	    }*/
    }
}
