/**
 * This panel allows the user to specify a structure for batch messages
 */

package gov.nist.hl7maker.view.management;

// JDK
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Vector;

// AWT
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// Swing
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;

// MessageMaker
import gov.nist.hl7maker.util.MVCArg;

public class BatchStructurePanel extends JPanel implements ActionListener {

    private JTextField txtStructure;
    private JButton btnCompile;
    private JButton btnNew;

    private BatchModel bModel;

    /**
     * Constructor
     */
    public BatchStructurePanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));

	JLabel lblStructure = new JLabel("Structure: ");
	lblStructure.setToolTipText("Example: FHS, BHS, MSG, BTS, FTS");
	p0.add(lblStructure);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	txtStructure = new JTextField();
	txtStructure.setToolTipText("Example: FHS, BHS, MSG, BTS, FTS");
	txtStructure.setPreferredSize(new Dimension(300, 0));
	p0.add(txtStructure);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	btnCompile = new JButton("Compile");
	btnCompile.addActionListener(this);
        btnCompile.setBackground(new Color(204, 204, 255));
	p0.add(btnCompile);
	p0.add(Box.createRigidArea(new Dimension(10, 0)));

	btnNew = new JButton("New");
	btnNew.addActionListener(this);
        btnNew.setBackground(new Color(204, 204, 255));
	p0.add(btnNew);

	add(p0);
	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	init();
    }

    /**
     * Handles a click event
     * @param evt The event
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == btnCompile) {
	    String structure = txtStructure.getText().toUpperCase();
	    String correctedStructure = "";
	    // Check validity segment
	    String[] validSegment = {"FHS", "BHS", "MSG", "BTS", "FTS"};
	    StringTokenizer tokens = new StringTokenizer(structure, ",");
	    Vector vOrder = new Vector();
	    while(tokens.hasMoreTokens()) {
		String token = tokens.nextToken().trim();
		for(int i=0;i<validSegment.length;i++) {
		    if(token.equalsIgnoreCase(validSegment[i])) {
			vOrder.add(new Integer(i));
			i = validSegment.length;
		    }
		}
	    }
	    // Check the order
	    //Collections.sort(vOrder);
	    boolean fhs = false, fts = false;;
	    for(int i=0;i<vOrder.size();i++) {
		int position = ((Integer)vOrder.get(i)).intValue();
		if((position == 0 && !fhs) || (position == 4 && !fts) || (position > 0 && position < 4)) {
		    correctedStructure += validSegment[position];
		    correctedStructure += ",";
		}
		if(!fhs && position == 0) fhs = true;
		if(!fts && position == 4) fts = true;
	    }
	    if(correctedStructure.length() != 0) {
		correctedStructure = correctedStructure.substring(0, correctedStructure.length()-1);
		btnCompile.setEnabled(false);
		txtStructure.setEditable(false);
		btnNew.setEnabled(true);
	    }
	    txtStructure.setText(correctedStructure);
	    bModel.setStructure(correctedStructure);
	    bModel.fireEvent(new MVCArg("init", ""));
	}
	else if(evt.getSource() == btnNew) {
	    init();
	    txtStructure.setText("");
	    bModel.setStructure("");
	    bModel.fireEvent(new MVCArg("init", ""));
	}
    }

    /**
     * Set the model
     * @param model The model
     */
    public void setBatchModel(BatchModel model) {
	bModel = model;
    }

    /**
     * Init the panel
     */
    public void init() {
	btnCompile.setEnabled(true);
	txtStructure.setEditable(true);
	btnNew.setEnabled(false);
    }
}


