/**
 * This class is the panel to manage the messages (wrapping)
 */

package gov.nist.hl7maker.view.management;

// JDK
//import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

// AWT
import java.awt.CardLayout;
import java.awt.Dimension;
/*import java.awt.Component;
import java.awt.Color;*/

// Swing
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
/*import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.JLabel;
import javax.swing.JTextArea;*/

// MessageMaker
/*import gov.nist.hl7maker.control.Session;
  import gov.nist.hl7maker.control.User;*/
import gov.nist.hl7maker.control.Session;
import gov.nist.hl7maker.util.MVCArg;
import gov.nist.hl7maker.view.config.FieldNode;
import gov.nist.hl7maker.view.config.ComponentNode;

public class MessageManagementView extends JPanel implements Observer {

    private JPanel rightPane;
    private BatchMessageSelectionPanel bmsp;

    /**
     * Constructor
     */
    public MessageManagementView() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	BatchStructurePanel bsp = new BatchStructurePanel();
	add(bsp);

	BatchTreePanel btp = new BatchTreePanel();

	BatchElementValuePanel bevp = new BatchElementValuePanel();
	bmsp = new BatchMessageSelectionPanel();

	rightPane = new JPanel(new CardLayout());
	rightPane.add(new JPanel(), "Blank");
	rightPane.add(bevp, "ElementValue");
	rightPane.add(bmsp, "MessageSelection");

	// SplitPane 
	JPanel p0 = new JPanel();
	p0.setLayout(new BoxLayout(p0, BoxLayout.LINE_AXIS));
	JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, btp, rightPane);
	sp.setDividerLocation(310);
	p0.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	p0.add(sp);
	add(p0);

	BatchGenerationPanel bgp = new BatchGenerationPanel();
	add(bgp);
	//	add(Box.createRigidArea(new Dimension(0,10)));

	BatchDisplayPanel bdp = new BatchDisplayPanel();
	add(bdp);
	//	add(Box.createRigidArea(new Dimension(0,270)));

	// Create the Model
	BatchModel bm = new BatchModel();
	bm.addObserver(btp);
	bm.addObserver(this);
	bm.addObserver(bevp);
	bm.addObserver(bmsp);
	bm.addObserver(bgp);
	bm.addObserver(bdp);
	// Set the Model
	bsp.setBatchModel(bm);
	btp.setBatchModel(bm);
	bmsp.setBatchModel(bm);
	bgp.setBatchModel(bm);
    }

    /**
     * Set the Session
     * @param ses Session
     */
    public void setSession(Session ses) {
	ses.addObserver(bmsp);
    }

    /**
     * MVC
     * @param o The observable object
     * @param arg An argument passed to the notifyObservers method
     */
    public void update(Observable o, Object arg) {
	if(o instanceof BatchModel) {
	    if(arg instanceof MVCArg) {
		if("node".equals(((MVCArg)arg).getName())) {
		    Object obj = ((MVCArg)arg).getOValue();
		    if(obj instanceof FieldNode || obj instanceof ComponentNode) {
			CardLayout cl = (CardLayout)(rightPane.getLayout());
			cl.show(rightPane, "ElementValue");
		    }
		    else if(obj instanceof MessageNode) {
			CardLayout cl = (CardLayout)(rightPane.getLayout());
			cl.show(rightPane, "MessageSelection");
		    }
		}
		else if("nodeBlank".equals(((MVCArg)arg).getName())) {
		    CardLayout cl = (CardLayout)(rightPane.getLayout());
		    cl.show(rightPane, "Blank");
		}
		else if("init".equals(((MVCArg)arg).getName())) {
		    CardLayout cl = (CardLayout)(rightPane.getLayout());
		    cl.show(rightPane, "Blank");
		}
	    }
	}
    }
}
