<?xml version="1.0" encoding="UTF-8"?>
<!--
The contents of this file are subject to the Mozilla Public License Version 1.1 
(the "License"); you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the 
specific language governing rights and limitations under the License. 

The Original Code is "javaZ.1.0.xml".  Description: 
"Generates HAPI-based Z segment java classes for HL7 2.x using XSLT 1.0 transfomation."

The Initial Developer of the Original Code is Darius Kemeklis for U.S. Department Of Veteran Affairs   Veterans Health Administration.

Portions created by the Initial Developer are Copyright (C) 2005 to the Initial Developer. All Rights Reserved.

Contributor(s):

Alternatively, the contents of this file may be used under the terms of the 
GNU General Public License (the  ?GPL?), in which case the provisions of the GPL are 
applicable instead of those above.  If you wish to allow use of your version of this 
file only under the terms of the GPL and not to allow others to use your version 
of this file under the MPL, indicate your decision by deleting  the provisions above 
and replace  them with the notice and other provisions required by the GPL License.  
If you do not delete the provisions above, a recipient may use your version of 
this file under either the MPL or the GPL. 
-->
<!--
VA HL7 Code Generator
====================
Generates HAPI-based Z segment java classes for HL7 2.x using XSLT 1.0 

Input: HL7 Conformance Profile generated using HL7 modeling tools such as Messageing Workbench
Output: Specialized java classes for each Z segment within Conformance Profile.  Generates one concatenated file for each Z segment HAPI class.

@author: Darius Kemeklis 
@version: 0.0.3
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<xsl:output method="text"/>
<xsl:param name="prefix" select="''"/>
<xsl:template match="/">

  <xsl:variable name="version">
  <xsl:choose>    
  <xsl:when test="normalize-space(/HL7v2xConformanceProfile/@HL7Version) = ''">
     <xsl:value-of select="concat('24','')"/>
  </xsl:when>
  <xsl:otherwise>
     <xsl:value-of select="translate(/HL7v2xConformanceProfile/@HL7Version, '.', '')"/>    
  </xsl:otherwise>
  </xsl:choose>
  </xsl:variable>

        <!--<xsl:for-each select="HL7v2xConformanceProfile/HL7v2xStaticDef/Segment[starts-with(@Name,'Z')]">
		<xsl:variable name="name" select="@Name"/>
                <xsl:variable name="fullFileName" select="concat('ca/uhn/hl7v2/model/v',$version, '/message/',$prefix,"_",$name,'.java')"/>

////////// Start File: <xsl:value-of select="$fullFileName"/> //////////
-->

                <xsl:variable name="fullFileName" select="concat('ca/uhn/hl7v2/model/v',$version, '/message/',$prefix,'CUSTOM.java')"/>
  SplitHere<xsl:value-of select="$fullFileName"/>EndSplit

                <xsl:variable name="name"><xsl:value-of select="$prefix"/>CUSTOM</xsl:variable>
                <xsl:variable name="type">segment</xsl:variable>
		<xsl:call-template name="addMainClassDefinition">
			<xsl:with-param name="version" select="$version"/>
			<xsl:with-param name="name" select="$name"/>
                        <xsl:with-param name="type" select="$type"/>
                        <xsl:with-param name="prefix" select="concat($prefix, '')"/>
		</xsl:call-template>
		
<!--	</xsl:for-each> -->
<!--<xsl:for-each select="HL7v2xConformanceProfile/HL7v2xStaticDef/*/SegGroup">-->
<xsl:for-each select="//SegGroup">

  <xsl:variable name="fullFileName" select="concat('ca/uhn/hl7v2/model/v',$version, '/message/',$prefix, @Name, '.java')"/>
  SplitHere<xsl:value-of select="$fullFileName"/>EndSplit

  <xsl:variable name="version">
  <xsl:choose>    
  <xsl:when test="normalize-space(/HL7v2xConformanceProfile/@HL7Version) = ''">
     <xsl:value-of select="concat('24','')"/>
  </xsl:when>
  <xsl:otherwise>
     <xsl:value-of select="translate(/HL7v2xConformanceProfile/@HL7Version, '.', '')"/>    
  </xsl:otherwise>
  </xsl:choose>
  </xsl:variable>

  <xsl:variable name="name"><xsl:value-of select="$prefix"/><xsl:value-of select="@Name"/></xsl:variable>
  <xsl:variable name="type">seggroup</xsl:variable>
  <xsl:variable name="pos"><xsl:value-of select="position()"/></xsl:variable>
  <xsl:call-template name="addMainClassDefinition">
    <xsl:with-param name="version" select="$version"/>
    <xsl:with-param name="name" select="$name"/>
    <xsl:with-param name="type" select="$type"/>
    <xsl:with-param name="pos" select="$pos"/>
    <xsl:with-param name="prefix" select="concat($prefix, '')"/>
  </xsl:call-template>
  
</xsl:for-each>

</xsl:template>
<xsl:include href="javaGroup.core.xsl"/>

</xsl:stylesheet>
