<?xml version="1.0" encoding="UTF-8"?>
<!--
The contents of this file are subject to the Mozilla Public License Version 1.1 
(the "License"); you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the 
specific language governing rights and limitations under the License. 

The Original Code is "javaZ.1.0.xml".  Description: 
"Generates HAPI-based Z segment java classes for HL7 2.x using XSLT 1.0 transfomation."

The Initial Developer of the Original Code is Darius Kemeklis for U.S. Department Of Veteran Affairs   Veterans Health Administration.

Portions created by the Initial Developer are Copyright (C) 2005 to the Initial Developer. All Rights Reserved.

Contributor(s):

Alternatively, the contents of this file may be used under the terms of the 
GNU General Public License (the  ?GPL?), in which case the provisions of the GPL are 
applicable instead of those above.  If you wish to allow use of your version of this 
file only under the terms of the GPL and not to allow others to use your version 
of this file under the MPL, indicate your decision by deleting  the provisions above 
and replace  them with the notice and other provisions required by the GPL License.  
If you do not delete the provisions above, a recipient may use your version of 
this file under either the MPL or the GPL. 
-->
<!--
VA HL7 Code Generator
====================
Generates HAPI-based Z segment java classes for HL7 2.x.
Core library

@author: Darius Kemeklis 
@version: 0.0.3
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsd="http://www.w3.org/2001/XMLSchema">

<!-- Global Variables -->

	<xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
	<xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
	<xsl:variable name="symbolletters"><![CDATA[~`!@#$%^&*()_-+={[}]\|;:'",<.>/?</.>]]></xsl:variable>
	<xsl:variable name="spaceslatters">                                    </xsl:variable>


<!-- Global Templates -->

<!-- Template to creates Main Class definition -->
<xsl:template name="addMainClassDefinition">
	<xsl:param name="version"/>
	<xsl:param name="name"/>
        <xsl:param name="type"/>
        <xsl:param name="prefix"/>

package ca.uhn.hl7v2.model.v<xsl:value-of select="$version"/>.message;

import ca.uhn.log.HapiLogFactory;
import ca.uhn.hl7v2.model.v<xsl:value-of select="$version"/>.group.*;

import ca.uhn.hl7v2.model.v<xsl:value-of select="$version"/>.segment.*;

import ca.uhn.hl7v2.HL7Exception;

import ca.uhn.hl7v2.model.AbstractMessage;

public class <xsl:value-of select="$name"/> extends AbstractMessage  {

  /**
   * Creates a new <xsl:value-of select="$name"/> group
   */
  public <xsl:value-of select="$name"/>() {
    try {

        <xsl:if test="$type = 'segment'">
          <xsl:for-each select="/HL7v2xConformanceProfile/HL7v2xStaticDef/Segment | /HL7v2xConformanceProfile/HL7v2xStaticDef/SegGroup | /Specification/Message/Segment | /Specification/Message/SegGroup">
            <xsl:call-template name="addSegments">
              <xsl:with-param name="prefix" select="$prefix"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:if>
        <xsl:if test="$type = 'seggroup'">
          <xsl:for-each select="./Segment | ./SegGroup">
            <xsl:call-template name="addSegments">
              <xsl:with-param name="prefix" select="$prefix"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:if>

        } catch (HL7Exception he) {
        HapiLogFactory.getHapiLog(this.getClass()).error("Unexpected error creating <xsl:value-of select="$name"/> - this is probably a bug in the source code generator.", he);
        }
        }
        
        <xsl:if test="$type = 'segment'">
          <xsl:for-each select="/HL7v2xConformanceProfile/HL7v2xStaticDef/Segment | /HL7v2xConformanceProfile/HL7v2xStaticDef/SegGroup | /Specification/Message/Segment | /Specification/Message/SegGroup">
            <!--
            <xsl:for-each select="parent::*/preceding-sibling::Segment[starts-with(@Name,'Z')]">
            System.out.println(<xsl:value-of select="@Name"/>);              
            </xsl:for-each>
            System.out.println("Current" + <xsl:value-of select="current()/@Name"/>);
            System.out.println(Count: <xsl:value-of select=""/>);
-->

            <!--<xsl:if test="count(parent::*/preceding-sibling::Segment[./@Name = current()/@Name]) + count(parent::*/following-sibling::Segment[./@Name = current()/@Name]) = 0">         -->     
            <xsl:call-template name="addGetters">
              <xsl:with-param name="prefix" select="$prefix"/>
            </xsl:call-template>
            <!--</xsl:if>-->
          </xsl:for-each>
        </xsl:if>
        <xsl:if test="$type = 'seggroup'">
          <xsl:for-each select="./Segment | ./SegGroup">
            <xsl:call-template name="addGetters">
              <xsl:with-param name="prefix" select="$prefix"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:if>

}
      </xsl:template>


<!-- Template to add Fields to the segment in the constructor -->
<xsl:template name="addSegments">
  <xsl:param name="prefix"/>
  <!--<xsl:choose>
       <xsl:when test="@Length"></xsl:when>
       <xsl:otherwise>
         <xsl:message>
           Segment: <xsl:value-of select="../@Name"/>, Field: <xsl:value-of select="@Name"/> has unspecified Lenght, defaulting to 0.  Please fix your field definition.
       </xsl:message>
     </xsl:otherwise>
   </xsl:choose>-->
   this.add(<xsl:if test="local-name() = 'SegGroup'"><xsl:value-of select="$prefix"/></xsl:if><xsl:if test="self::node()[starts-with(@Name,'Z')]"><xsl:value-of select="$prefix"/></xsl:if><xsl:value-of select="@Name"/>.class, <xsl:choose>
   <xsl:when test="@Min='1'">true</xsl:when>
   <xsl:otherwise>false</xsl:otherwise>
 </xsl:choose>, <xsl:choose>
 <xsl:when test="@Max='1'">false</xsl:when>
 <xsl:otherwise>true</xsl:otherwise>
</xsl:choose><!--,  <xsl:choose>
<xsl:when test="@Length"><xsl:value-of select="@Length"/></xsl:when>
<xsl:otherwise>0</xsl:otherwise>
</xsl:choose>, null-->);
</xsl:template>




<!-- Template to add getters to the class -->
<xsl:template name="addGetters">
  <xsl:param name="prefix"/>
  <!--  <xsl:for-each select="/HL7v2xConformanceProfile/HL7v2xStaticDef/Segment | /HL7v2xConformanceProfile/HL7v2xStaticDef/SegGroup"> -->

		<xsl:variable name="segmentNameOrg" select="@Name"/>
		<xsl:variable name="segmentNameNorm">
                  <!--	<xsl:call-template name='convertpropercase'>
				<xsl:with-param name='toconvert' select='translate($segmentNameOrg,$symbolletters,$spaceslatters)' />
				<xsl:with-param name='index' select='0' />
			</xsl:call-template> -->
		</xsl:variable>
                <xsl:variable name="segmentName"><xsl:if test="local-name() = 'SegGroup'"><xsl:value-of select="$prefix"/></xsl:if><xsl:if test="self::node()[starts-with(@Name,'Z')]"><xsl:value-of select="$prefix"/></xsl:if><xsl:value-of select="@Name"/>
                  <!--		<xsl:call-template name='checksegmentname'>
				<xsl:with-param name='segmentNameNorm' select='$segmentNameNorm' />
			</xsl:call-template> -->
		</xsl:variable>

  /**
   * Returns <xsl:value-of select="$segmentName"/> - creates it if necessary
   */
  public <xsl:value-of select="$segmentName"/> get<xsl:value-of select="$segmentName"/>()  {
    <xsl:value-of select="$segmentName"/> ret = null;
    try {
    ret = (<xsl:value-of select="$segmentName"/>)this.get("<xsl:value-of select="$segmentName"/>");
    } catch (HL7Exception he) {
        HapiLogFactory.getHapiLog(this.getClass()).error("Unexpected error accessing data - this is probably a bug in the source code generator.", he);
    }
    return ret;
  }
    <!--	</xsl:for-each>  -->
      </xsl:template>


<!-- Template to convert string to proper cases   -->
<xsl:template name='convertpropercase'>
	<xsl:param name='toconvert' />
	<xsl:param name='index' />
	
	<xsl:if test="string-length($toconvert) > 0">
		<xsl:variable name='f' select='substring($toconvert, 1, 1)' />
		<xsl:variable name='s' select='substring($toconvert, 2)' />

		<xsl:if test="$f!=' '">
			<xsl:if test="$index=0">
				<xsl:value-of select="translate($f,$lcletters,$ucletters)" />
			</xsl:if>
			<xsl:if test="$index!=0">
				<xsl:value-of select="translate($f,$ucletters,$lcletters)" />
			</xsl:if>
			<xsl:variable name="nextF" select="substring($s,1,1)"/>
			
			<xsl:if test="$nextF=' '">
				<xsl:variable name="index2" select="0"/>
				<xsl:variable name='s2' select='substring($s, 2)' />
				
				<xsl:call-template name='convertpropercase'>
					<xsl:with-param name='toconvert' select='$s2' />
					<xsl:with-param name='index' select='$index2' />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="$nextF!=' '">
				<xsl:variable name="index2" select="$index+1"/>
				<xsl:variable name='s2' select='$s' />
				
				<xsl:call-template name='convertpropercase'>
					<xsl:with-param name='toconvert' select='$s2' />
					<xsl:with-param name='index' select='$index2' />
				</xsl:call-template>
			</xsl:if>
		</xsl:if>
		<xsl:if test="$f=' '">
			<xsl:call-template name='convertpropercase'>
				<xsl:with-param name='toconvert' select='$s' />
				<xsl:with-param name='index' select='0' />
			</xsl:call-template>
		</xsl:if>
	</xsl:if>
  </xsl:template>


<!-- Template to append "_" if segment getter conflicts with AbstractSegment getters -->
<xsl:template name='checksegmentname'>
	<xsl:param name='segmentNameNorm' />

	<xsl:choose>
		<xsl:when test="$segmentNameNorm='Name' or $segmentNameNorm='Segment' or $segmentNameNorm='Length' or $segmentNameNorm='Message' or $segmentNameNorm='Parent' or $segmentNameNorm='MaxCardinality'">
			<xsl:value-of select="concat($segmentNameNorm,'_')"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$segmentNameNorm"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

</xsl:stylesheet>
