<?xml version="1.0"?>
<!--
  Collect stats on the Segments, Fields, Components, and 
  SubComponents contained in the profile.  

	Output (from Meatadata):
  	Name					
    Organization
  Followed by Segment, Field, Comp, and Subcomp data, which
  looks like:
       nodeNumber'name'-M{R/RP/RE/X} ... nodeNumber'name'-M{R/RP/RE/X}  
       numOfSegs                    
       nodeNumber'name'-M{R/RP/RE/X} ... nodeNumber'name'-M{R/RP/RE/X}  
       numOfFields                 
       nodeNumber'name'-R/RP/RE/X ...   nodeNumber'name'-R/RP/RE/X  
       numOfComps                  
       nodeNumber'name'-R/RP/RE/X ...   nodeNumber'name'-R/RP/RE/X  
       numOfSubComps
    the names, other than segment names, include annotated ancestor
    names.  M{R/RP/RE/X} = M followed by R or RP, etc. 
    M = min cardinality > 0 & element required.
    R = required element.  
    RP = required primitive.
    RE = element Usage="RE".  
    X = element Usage="X".  
    
    Author:  Len Gebase, NIST
    -->  
<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="text"/>

<xsl:strip-space elements="*"/>

<xsl:variable name="quote">'</xsl:variable>
<xsl:variable name="dash">-</xsl:variable>

<xsl:variable name="newline"> 
<xsl:text>
</xsl:text>
</xsl:variable>

<xsl:template match="/">
  
  <!-- get the name and org name from Metadata first (empty for VA profile) -->
  <xsl:value-of select="/HL7v2xConformanceProfile/HL7v2xStaticDef/MetaData/@Name"/>
  <xsl:value-of select="$newline"/>
  <xsl:value-of select="/HL7v2xConformanceProfile/HL7v2xStaticDef/MetaData/@OrgName"/>
  <xsl:value-of select="$newline"/>
  
  <xsl:call-template name="segments"/>
  <xsl:call-template name="fields"/>
  <xsl:call-template name="components"/>
  <xsl:call-template name="subcomponents"/>  
</xsl:template>

<!-- for each segment, write out a string indicating if Min is positive 
    and the segment's Usage value.  also save the total num of segments 
    there may (or there may not be) a SegmentGroup between the
    static def and the Segment. -->
<xsl:template	name="segments">
<!--  <xsl:for-each select="/HL7v2xConformanceProfile/HL7v2xStaticDef//Segment">-->
  <xsl:for-each select="//Segment">
    <xsl:variable name="segNumber">	
      <xsl:number	level="multiple" count="Segment" />
    </xsl:variable>
    
    <xsl:value-of select="$segNumber"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="@Name"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="$dash"/>
  
    <!-- for now at least, when Min>0, only mark it with a M 
        if it's required -->
    <xsl:if test="@Min > 0 and @Usage='R'">
      <xsl:value-of select="'M'"/>
    </xsl:if>
    
    <xsl:value-of select="@Usage"/>
    <xsl:value-of select="' '"/>
    
    <xsl:if test="position() = last()">
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:for-each>

  <!-- 
  <xsl:for-each select="/HL7v2xConformanceProfile/HL7v2xStaticDef/Segment">
    <xsl:if test="@Min > 0">
      <xsl:value-of select="position()"/>
      <xsl:value-of select="' '"/>
    </xsl:if>
    <xsl:if test="position() = last()">
      <xsl:value-of select="'END '"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="' END '"/>
    </xsl:if>
  </xsl:for-each>
-->
</xsl:template>

<!-- for each field, write out a string indicating if Min is positive 
    and the field's Usage value.  also save the total num of fields -->
<xsl:template	name="fields">
<!--      select="/HL7v2xConformanceProfile/HL7v2xStaticDef//Segment/Field">-->
  <xsl:for-each select="//Segment/Field">
    
    <xsl:variable name="fieldNumber">	
      <xsl:number	level="multiple" count="Segment | Field" />
    </xsl:variable>
    
    <xsl:value-of select="$fieldNumber"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="'#SEG# '"/>
    <xsl:value-of select="parent::*/@Name"/>
    <xsl:value-of select="' #FIELD# '"/>
    <xsl:value-of select="@Name"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="$dash"/>

    <xsl:if test="@Min > 0 and @Usage='R'">
      <xsl:value-of select="'M'"/>
    </xsl:if>

    <xsl:variable name="primitive">
      <xsl:call-template name="isPrimitive">
        <xsl:with-param name="dataType" select="@Datatype"/>
      </xsl:call-template>
    </xsl:variable>
  
  	<!-- we don't want to add Fields with usage="X" unless the parent
      	is required.  we want to use these for generating errors.  if
        the parent is included though, we'll never get to the child
        and create a usage error for X-elements. -->
    <xsl:choose>
	    <xsl:when test="@Usage='X'">
	    	<xsl:if test="parent::*/@Usage='R'">
		      <xsl:value-of select="@Usage"/>
        </xsl:if>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="@Usage"/>
	    </xsl:otherwise>
    </xsl:choose>
      
    <xsl:if test="$primitive='true'">
      <xsl:value-of select="'P'"/>
    </xsl:if>
    <xsl:value-of select="' '"/>
    
    <!-- <xsl:value-of select="$newline"/> -->
    
    <xsl:if test="position() = last()">
<!--       <xsl:value-of select="'END '"/> -->
<!--       <xsl:value-of select="-1"/> -->
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="$newline"/>
<!--       <xsl:value-of select="' '"/> -->
    </xsl:if>
  </xsl:for-each>
<!-- 
  <xsl:for-each
      select="/HL7v2xConformanceProfile/HL7v2xStaticDef/Segment/Field">
    <xsl:if test="@Min > 0">
      <xsl:value-of select="position()"/>
      <xsl:value-of select="' '"/>
    </xsl:if>
    <xsl:if test="position() = last()">
      <xsl:value-of select="'END '"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="' END'"/>
    </xsl:if>
  </xsl:for-each>
 -->
</xsl:template>

<xsl:template	name="components">
<!--      select="/HL7v2xConformanceProfile/HL7v2xStaticDef//Segment/Field/Component">     -->
  <xsl:for-each select="//Segment/Field/Component">     
    
    <xsl:variable name="compNumber">	
      <xsl:number	level="multiple" count="Segment | Field | Component" />
    </xsl:variable>
    
    <xsl:value-of select="$compNumber"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="'#SEG# '"/>      
    <xsl:value-of select="parent::*/parent::*/@Name"/>
    <xsl:value-of select="' #FIELD# '"/>
    <xsl:value-of select="parent::*/@Name"/>
    <xsl:value-of select="' #COMP# '"/>
    <xsl:value-of select="@Name"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="$dash"/>

    <xsl:variable name="primitive">
      <xsl:call-template name="isPrimitive">
        <xsl:with-param name="dataType" select="@Datatype"/>
      </xsl:call-template>
    </xsl:variable>

  	<!-- we don't want to add Components with usage="X" unless the parent
      	is required.  we want to use these for generating errors.  if
        the parent is included though, we'll never get to the child
        and create a usage error for X-elements. -->
    <xsl:choose>
	    <xsl:when test="@Usage='X'">
	    	<xsl:if test="parent::*/@Usage='R'">
		      <xsl:value-of select="@Usage"/>
        </xsl:if>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="@Usage"/>
	    </xsl:otherwise>
    </xsl:choose>
      
    <xsl:if test="$primitive='true'">
      <xsl:value-of select="'P'"/>
    </xsl:if>
    <xsl:value-of select="' '"/>
 
    <!-- <xsl:value-of select="$newline"/> -->
    
    <xsl:if test="position() = last()">
<!--      <xsl:value-of select="'END '"/> -->
<!--       <xsl:value-of select="-1"/> -->
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="$newline"/>
 <!--       <xsl:value-of select="' '"/> -->
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template	name="subcomponents">
<!--      select="/HL7v2xConformanceProfile/HL7v2xStaticDef//Segment/Field/Component/SubComponent">-->
  <xsl:for-each select="//Segment/Field/Component/SubComponent">
    
    <xsl:variable name="subNumber">	
      <xsl:number	level="multiple" count="Segment | Field | Component 
                        | SubComponent" />
    </xsl:variable>
    
    <xsl:value-of select="$subNumber"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="'#SEG# '"/>      
    <xsl:value-of select="parent::*/parent::*/parent::*/@Name"/>
    <xsl:value-of select="' #FIELD# '"/>
    <xsl:value-of select="parent::*/parent::*/@Name"/>
    <xsl:value-of select="' #COMP# '"/>
    <xsl:value-of select="parent::*/@Name"/>
    <xsl:value-of select="' #SUBCOMP# '"/>
    <xsl:value-of select="@Name"/>
    <xsl:value-of select="$quote"/>
    <xsl:value-of select="$dash"/>

    <xsl:variable name="primitive">
      <xsl:call-template name="isPrimitive">
        <xsl:with-param name="dataType" select="@Datatype"/>
      </xsl:call-template>
    </xsl:variable>

  	<!-- we don't want to add SubComps with usage="X" unless the parent
      	is required.  we want to use these for generating errors.  if
        the parent is included though, we'll never get to the child
        and create a usage error for X-elements. -->
    <xsl:choose>
	    <xsl:when test="@Usage='X'">
	    	<xsl:if test="parent::*/@Usage='R'">
		      <xsl:value-of select="@Usage"/>
        </xsl:if>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="@Usage"/>
	    </xsl:otherwise>
    </xsl:choose>
      
    <xsl:if test="$primitive='true'">
      <xsl:value-of select="'P'"/>
    </xsl:if>
    <xsl:value-of select="' '"/>
    
    <!-- <xsl:value-of select="$newline"/> -->
    
    <xsl:if test="position() = last()">
<!--       <xsl:value-of select="'END '"/> -->
<!--       <xsl:value-of select="-1"/> -->
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="last()"/>
      <xsl:value-of select="$newline"/>
<!--       <xsl:value-of select="' '"/> -->
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<!-- ............................................................. -->
<!-- our list of primitive types is very limited here.  the reason it
    is is that in general these 2 types (ST & NM) are the only ones we
    can produce a value length error for without also producing a 2nd 
    error, e.g., to produce a length error for an ID type, you have to
    also create an invalid ID.  -->
<xsl:template	name="isPrimitive">
  <xsl:param name="dataType"/>
  
  <xsl:choose>	
    <xsl:when test="$dataType='ST' or $dataType='NM'">
      <xsl:value-of select="'true'"/>
    </xsl:when>
    <xsl:otherwise>	
      <xsl:value-of select="'false'"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
